searchData={"items":[{"type":"module","title":"Explorer","doc":"Explorer brings series (one-dimensional) and dataframes (two-dimensional) for fast\ndata exploration to Elixir.","ref":"Explorer.html"},{"type":"module","title":"Features and design - Explorer","doc":"Explorer high-level features are:\n\n- Simply typed series: `:binary`, `:boolean`, `:category`, `:date`, `:datetime`,\n  `:duration`, floats of 32 and 64 bits (`{:f, size}`), integers of 8, 16, 32\n  and 64 bits (`{:s, size}`, `{:u, size}`), `:null`, `:string`, `:time`, `:list`,\n  and `:struct`.\n\n- A powerful but constrained and opinionated API, so you spend less time looking\n  for the right function and more time doing data manipulation.\n\n- Support for CSV, Parquet, NDJSON, and Arrow IPC formats\n\n- Integration with external databases via [ADBC](https://github.com/elixir-explorer/adbc)\n  and direct connection to file storages such as S3\n\n- Pluggable backends, providing a uniform API whether you're working in-memory\n  or (forthcoming) on remote databases or even Spark dataframes.\n\n- The first (and default) backend is based on NIF bindings to the blazing-fast\n  [polars](https://docs.rs/polars) library.\n\nThe API is heavily influenced by [Tidy Data](https://vita.had.co.nz/papers/tidy-data.pdf)\nand borrows much of its design from [dplyr](https://dplyr.tidyverse.org). The philosophy\nis heavily influenced by this passage from `dplyr`'s documentation:\n\n> - By constraining your options, it helps you think about your data manipulation\n>   challenges.\n>\n> - It provides simple “verbs”, functions that correspond to the most common data\n>   manipulation tasks, to help you translate your thoughts into code.\n>\n> - It uses efficient backends, so you spend less time waiting for the computer.\n\nThe aim here isn't to have the fastest dataframe library around (though it certainly\nhelps that [we're building on Polars, one of the fastest](https://h2oai.github.io/db-benchmark/)).\nInstead, we're aiming to bridge the best of many worlds:\n\n- the elegance of `dplyr`\n- the speed of `polars`\n- the joy of Elixir\n\nThat means you can expect the guiding principles to be 'Elixir-ish'. For example,\nyou won't see the underlying data mutated, even if that's the most efficient implementation.\nExplorer functions will always return a new dataframe or series.","ref":"Explorer.html#module-features-and-design"},{"type":"module","title":"Getting started - Explorer","doc":"Inside an Elixir script or [Livebook](https://livebook.dev):\n\n```elixir\nMix.install([\n  {:explorer, \"~> 0.9.0\"}\n])\n```\n\nOr in the `mix.exs` file of your application:\n\n```elixir\ndef deps do\n  [\n    {:explorer, \"~> 0.9.0\"}\n  ]\nend\n```\n\nExplorer will download a precompiled version of its native code upon installation. You can force a local build by setting the environment variable `EXPLORER_BUILD=1` and including `:rustler` as a dependency:\n\n```elixir\n  {:explorer, \"~> 0.9.0\", system_env: %{\"EXPLORER_BUILD\" => \"1\"}},\n  {:rustler, \">= 0.0.0\"}\n```\n\nIf necessary, clean up before rebuilding with `mix deps.clean explorer`.","ref":"Explorer.html#module-getting-started"},{"type":"module","title":"A glimpse of the API - Explorer","doc":"We have two ways to represent data with Explorer:\n\n- using a series, that is similar to a list, but is guaranteed to contain items\n  of one data type only - or one *dtype* for short. Notice that nil values are\n  permitted in series of any dtype.\n\n- using a dataframe, that is just a way to represent one or more series together,\n  and work with them as a whole. The only restriction is that all the series shares\n  the same size.\n\nA series can be created from a list:\n\n```elixir\nfruits = Explorer.Series.from_list([\"apple\", \"mango\", \"banana\", \"orange\"])\n```\n\nYour newly created series is going to look like:\n\n```\n#Explorer.Series \n```\n\nAnd you can, for example, sort that series:\n\n```elixir\nExplorer.Series.sort(fruits)\n```\n\nResulting in the following:\n\n```\n#Explorer.Series \n```","ref":"Explorer.html#module-a-glimpse-of-the-api"},{"type":"module","title":"Dataframes - Explorer","doc":"Dataframes can be created in two ways:\n\n- by reading from files or memory using the\n  [IO functions](https://hexdocs.pm/explorer/Explorer.DataFrame.html#module-io-operations).\n  This is by far the most common way to load dataframes in Explorer.\n  We accept Parquet, IPC, CSV, and NDJSON files.\n\n- by using the `Explorer.DataFrame.new/2` function, that is neat for small experiments.\n  We are going to use this function here.\n\nYou can pass either series or lists to it:\n\n```elixir\nmountains = Explorer.DataFrame.new(name: [\"Everest\", \"K2\", \"Aconcagua\"], elevation: [8848, 8611, 6962])\n```\n\nYour dataframe is going to look like this:\n\n```\n#Explorer.DataFrame \n```\n\nIt's also possible to see a dataframe like a table, using the `Explorer.DataFrame.print/2`\nfunction:\n\n```elixir\nExplorer.DataFrame.print(mountains)\n```\n\nPrints:\n\n```\n+-------------------------------------------+\n| Explorer DataFrame: [rows: 3, columns: 2] |\n+---------------------+---------------------+\n|        name         |      elevation      |\n|              |                 |\n+=====================+=====================+\n| Everest             | 8848                |\n+---------------------+---------------------+\n| K2                  | 8611                |\n+---------------------+---------------------+\n| Aconcagua           | 6962                |\n+---------------------+---------------------+\n```\n\nAnd now I want to show you how to filter our dataframe. But first, let's require\nthe `Explorer.DataFrame` module and give a short name to it:\n\n```elixir\nrequire Explorer.DataFrame, as: DF\n```\n\nThe \"require\" is needed to load the macro features of that module.\nWe give it a shorter name to simplify our examples.\n\nNow let's go to the filter. I want to filter the mountains that are above\nthe mean elevation in our dataframe:\n\n```elixir\nDF.filter(mountains, elevation > mean(elevation))\n```\n\nYou can see that we can refer to the columns using their names, and use functions\nwithout define them. This is possible due the powerful `Explorer.Query` features,\nand it's the main reason we need to \"require\" the `Explorer.DataFrame` module.\n\nThe result is going to look like this:\n\n```\n#Explorer.DataFrame \n```\n\nThere is an extensive guide that you can play with Livebook:\n[Ten Minutes to Explorer](https://hexdocs.pm/explorer/exploring_explorer.html)\n\nYou can also check the `Explorer.DataFrame` and `Explorer.Series` docs for further\ndetails.","ref":"Explorer.html#module-dataframes"},{"type":"module","title":"Explorer.DataFrame","doc":"The DataFrame struct and API.\n\nDataframes are two-dimensional tabular data structures similar to a spreadsheet.\nFor example, the Iris dataset:\n\n    iex> Explorer.Datasets.iris()\n    #Explorer.DataFrame \n\nThis dataframe has 150 rows and five columns. Each column is an `Explorer.Series`\nof the same size (150):\n\n    iex> df = Explorer.Datasets.iris()\n    iex> df[\"sepal_length\"]\n    #Explorer.Series","ref":"Explorer.DataFrame.html"},{"type":"module","title":"Creating dataframes - Explorer.DataFrame","doc":"Dataframes can be created from normal Elixir terms. The main way you might do this is\nwith the `new/1` function. For example:\n\n    iex> Explorer.DataFrame.new(a: [\"a\", \"b\"], b: [1, 2])\n    #Explorer.DataFrame \n\nOr with a list of maps:\n\n    iex> Explorer.DataFrame.new([%{\"col1\" => \"a\", \"col2\" => 1}, %{\"col1\" => \"b\", \"col2\" => 2}])\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#module-creating-dataframes"},{"type":"module","title":"Verbs - Explorer.DataFrame","doc":"Explorer uses the idea of a consistent set of SQL-like `verbs` like\n[`dplyr`](https://dplyr.tidyverse.org) which can help solve common\ndata manipulation challenges. These are split into single table verbs,\nmultiple table verbs, and row-based verbs:","ref":"Explorer.DataFrame.html#module-verbs"},{"type":"module","title":"Single table verbs - Explorer.DataFrame","doc":"Single table verbs are (unsurprisingly) used for manipulating a single dataframe.\nThose operations typically driven by column names. These are:\n\n- `select/2` for picking columns and `discard/2` to discard them\n- `filter/2` for picking rows based on predicates\n- `mutate/2` for adding or replacing columns that are functions of existing columns\n- `sort_by/2` for changing the ordering of rows\n- `distinct/2` for picking unique rows\n- `summarise/2` for reducing multiple rows down to a single summary\n- `pivot_longer/3` and `pivot_wider/4` for massaging dataframes into longer or\n  wider forms, respectively\n\nEach of these combine with `Explorer.DataFrame.group_by/2` for operating by group.","ref":"Explorer.DataFrame.html#module-single-table-verbs"},{"type":"module","title":"Multiple table verbs - Explorer.DataFrame","doc":"Multiple table verbs are used for combining tables. These are:\n\n- `join/3` for performing SQL-like joins\n- `concat_columns/1` for horizontally \"stacking\" dataframes\n- `concat_rows/1` for vertically \"stacking\" dataframes","ref":"Explorer.DataFrame.html#module-multiple-table-verbs"},{"type":"module","title":"Row-based verbs - Explorer.DataFrame","doc":"Those operations are driven by the row index. These are:\n\n- `head/2` for picking the first rows\n- `tail/2` for picking the last rows\n- `slice/2` for slicing the dataframe by row indexes or a range\n- `slice/3` for slicing a section by an offset\n- `sample/2` for sampling the data-frame by row","ref":"Explorer.DataFrame.html#module-row-based-verbs"},{"type":"module","title":"IO operations - Explorer.DataFrame","doc":"Explorer supports reading and writing of:\n\n- delimited files (such as CSV or TSV)\n- [Parquet](https://databricks.com/glossary/what-is-parquet)\n- [Arrow IPC](https://arrow.apache.org/docs/format/Columnar.html#ipc-file-format)\n- [Arrow Streaming IPC](https://arrow.apache.org/docs/format/Columnar.html#ipc-streaming-format)\n- [Newline Delimited JSON](https://github.com/ndjson/ndjson-spec)\n- Databases via `Adbc` in `from_query/3`\n\nThe convention Explorer uses is to have `from_*` and `to_*` functions to read and write\nto files in the formats above. `load_*` and `dump_*` versions are also available to read\nand write those formats directly in memory.\n\nFiles can be fetched from local or remote file system, such as S3, using the following formats:\n\n    # path to a file in disk\n    Explorer.DataFrame.from_parquet(\"/path/to/file.parquet\")\n\n    # path to a URL schema (with optional configuration)\n    Explorer.DataFrame.from_parquet(\"s3://bucket/file.parquet\", config: FSS.S3.config_from_system_env())\n\n    # it's possible to configure using keyword lists\n    Explorer.DataFrame.from_parquet(\"s3://bucket/file.parquet\", config: [access_key_id: \"my-key\", secret_access_key: \"my-secret\"])\n\n    # a FSS entry (it already includes its config)\n    Explorer.DataFrame.from_parquet(FSS.S3.parse(\"s3://bucket/file.parquet\"))\n\nThe `:config` option of `from_*` functions is only required if the filename is a path\nto a remote resource. In case it's a FSS entry, the requirement is that the config is passed\ninside the entry struct.\n\nFor more details about the options, see the [FSS docs](https://hexdocs.pm/fss).","ref":"Explorer.DataFrame.html#module-io-operations"},{"type":"module","title":"Selecting columns and access - Explorer.DataFrame","doc":"Several functions in this module, such as `select/2`, `discard/2`, `drop_nil/2`, and so\nforth accept a single or multiple columns as arguments. The columns can be specified in\na variety of formats, which we describe below.\n\n`Explorer.DataFrame` also implements the `Access` behaviour (also known as the brackets\nsyntax). This should be familiar for users coming from other language with dataframes\nsuch as R or Python. For example:\n\n    iex> df = Explorer.Datasets.wine()\n    iex> df[\"class\"]\n    #Explorer.Series \n\nAccessing the dataframe with a column name either as a string or an atom, will return\nthe column. You can also pass an integer representing the column order:\n\n    iex> df = Explorer.Datasets.wine()\n    iex> df[0]\n    #Explorer.Series \n\nYou can also pass a list, a range, or a regex to return a dataframe matching\nthe given data type. For example, by passing a list:\n\n    iex> df = Explorer.Datasets.wine()\n    iex> df[[\"class\", \"hue\"]]\n    #Explorer.DataFrame \n\nOr a range for the given positions:\n\n    iex> df = Explorer.Datasets.wine()\n    iex> df[0..2]\n    #Explorer.DataFrame \n\nOr a regex to keep only columns matching a given pattern:\n\n    iex> df = Explorer.Datasets.wine()\n    iex> df[~r/(class|hue)/]\n    #Explorer.DataFrame \n\nGiven you can also access a series using its index, you can use\nmultiple accesses to select a column and row at the same time:\n\n    iex> df = Explorer.Datasets.wine()\n    iex> df[\"class\"][3]\n    1","ref":"Explorer.DataFrame.html#module-selecting-columns-and-access"},{"type":"function","title":"Explorer.DataFrame.collect/1","doc":"Computes and collects a data frame to the current node.\n\nIf the data frame is already in the current node, it is computed\nbut no transfer happens.","ref":"Explorer.DataFrame.html#collect/1"},{"type":"function","title":"Examples - Explorer.DataFrame.collect/1","doc":"series = Explorer.Series.from_list([1, 2, 3], node: :some@node)\n    Explorer.Series.collect(series)","ref":"Explorer.DataFrame.html#collect/1-examples"},{"type":"function","title":"Explorer.DataFrame.compute/1","doc":"Computes the lazy data frame into an eager one, executing the query.\n\nIt is the opposite of `lazy/1`. If it is not a lazy data frame,\nthis is a noop.\n\nCollecting a grouped dataframe should return a grouped dataframe.","ref":"Explorer.DataFrame.html#compute/1"},{"type":"function","title":"Explorer.DataFrame.concat_columns/1","doc":"Combine two or more dataframes column-wise.\n\nThis function expects the dataframes to have the same number of rows,\notherwise rows may be silently discarded. Eager backends may check\nwhenever this happens and raise instead of silently fail. But this may not\nbe possible for lazy dataframes as the number of rows is not known upfront.\n\nWhen working with grouped dataframes, be aware that only groups from the first\ndataframe are kept in the resultant dataframe.","ref":"Explorer.DataFrame.html#concat_columns/1"},{"type":"function","title":"Examples - Explorer.DataFrame.concat_columns/1","doc":"iex> df1 = Explorer.DataFrame.new(x: [1, 2, 3], y: [\"a\", \"b\", \"c\"])\n    iex> df2 = Explorer.DataFrame.new(z: [4, 5, 6], a: [\"d\", \"e\", \"f\"])\n    iex> Explorer.DataFrame.concat_columns([df1, df2])\n    #Explorer.DataFrame \n\nConflicting names are suffixed with the index of the dataframe in the array:\n\n    iex> df1 = Explorer.DataFrame.new(x: [1, 2, 3], y: [\"a\", \"b\", \"c\"])\n    iex> df2 = Explorer.DataFrame.new(x: [4, 5, 6], a: [\"d\", \"e\", \"f\"])\n    iex> Explorer.DataFrame.concat_columns([df1, df2])\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#concat_columns/1-examples"},{"type":"function","title":"Explorer.DataFrame.concat_columns/2","doc":"Combine two dataframes column-wise.\n\nWhen working with grouped dataframes, be aware that only groups from the left-hand side\ndataframe are kept in the resultant dataframe.\n\n`concat_columns(df1, df2)` is equivalent to `concat_columns([df1, df2])`.","ref":"Explorer.DataFrame.html#concat_columns/2"},{"type":"function","title":"Explorer.DataFrame.concat_rows/1","doc":"Combine two or more dataframes row-wise (stack).\n\nColumn names and dtypes must match. The only exception is for numeric\ncolumns that can be mixed together, and casted automatically to the\n\"highest\" dtype.\n\nFor example, if a column is represented by `{:u, 16}` and `{:u, 32}`,\nit will be casted to the highest unsigned integer dtype between the two,\nwhich is `{:u, 32}`.\nIf it is mixing signed and unsigned integers, it will be casted to\nthe highest signed integer possible.\nAnd if floats and integers are mixed together, Explorer will cast\nthem to the float dtype `{:f, 64}`.\n\nWhen working with grouped dataframes, be aware that only groups from the first\ndataframe are kept in the resultant dataframe.","ref":"Explorer.DataFrame.html#concat_rows/1"},{"type":"function","title":"Examples - Explorer.DataFrame.concat_rows/1","doc":"iex> df1 = Explorer.DataFrame.new(x: [1, 2, 3], y: [\"a\", \"b\", \"c\"])\n    iex> df2 = Explorer.DataFrame.new(x: [4, 5, 6], y: [\"d\", \"e\", \"f\"])\n    iex> Explorer.DataFrame.concat_rows([df1, df2])\n    #Explorer.DataFrame \n\n    iex> df1 = Explorer.DataFrame.new(x: [1, 2, 3], y: [\"a\", \"b\", \"c\"])\n    iex> df2 = Explorer.DataFrame.new(x: [4.2, 5.3, 6.4], y: [\"d\", \"e\", \"f\"])\n    iex> Explorer.DataFrame.concat_rows([df1, df2])\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#concat_rows/1-examples"},{"type":"function","title":"Explorer.DataFrame.concat_rows/2","doc":"Combine two dataframes row-wise.\n\n`concat_rows(df1, df2)` is equivalent to `concat_rows([df1, df2])`.\n\nWhen working with grouped dataframes, be aware that only groups from the left-hand side\ndataframe are kept in the resultant dataframe.","ref":"Explorer.DataFrame.html#concat_rows/2"},{"type":"function","title":"Explorer.DataFrame.correlation/2","doc":"Calculates the pairwise correlation of numeric columns.\n\nThe returned dataframe is the correlation matrix.","ref":"Explorer.DataFrame.html#correlation/2"},{"type":"function","title":"Supported dtypes - Explorer.DataFrame.correlation/2","doc":"Only columns with the following dtypes are taken into account.\n\n* floats: `{:f, 32}` and `{:f, 64}`\n* integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`\n\nThe resultant columns are always `{:f, 64}`.","ref":"Explorer.DataFrame.html#correlation/2-supported-dtypes"},{"type":"function","title":"Options - Explorer.DataFrame.correlation/2","doc":"* `:columns` - the selection of columns to calculate. Defaults to all numeric columns.\n* `:column_name` - the name of the column with column names. Defaults to \"names\".\n* `:ddof` - the 'delta degrees of freedom' - the divisor used in the correlation\n  calculation. Defaults to 1.\n* `:method` refers to the correlation method. The following methods are available:\n  - `:pearson` : Standard correlation coefficient. (default)\n  - `:spearman` : Spearman rank correlation.","ref":"Explorer.DataFrame.html#correlation/2-options"},{"type":"function","title":"Examples - Explorer.DataFrame.correlation/2","doc":"iex> df = Explorer.DataFrame.new(dogs: [1, 8, 3], cats: [4, 5, 2])\n    iex> Explorer.DataFrame.correlation(df)\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#correlation/2-examples"},{"type":"function","title":"Explorer.DataFrame.covariance/2","doc":"Calculates the pairwise covariance of numeric columns.\n\nThe returned dataframe is the covariance matrix.","ref":"Explorer.DataFrame.html#covariance/2"},{"type":"function","title":"Supported dtypes - Explorer.DataFrame.covariance/2","doc":"Only columns with the following dtypes are taken into account.\n\n* floats: `{:f, 32}` and `{:f, 64}`\n* integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`\n\nThe resultant columns are always `{:f, 64}`.","ref":"Explorer.DataFrame.html#covariance/2-supported-dtypes"},{"type":"function","title":"Options - Explorer.DataFrame.covariance/2","doc":"* `:columns` - the selection of columns to calculate. Defaults to all numeric columns.\n* `:column_name` - the name of the column with column names. Defaults to \"names\".\n* `:ddof` - the 'delta degrees of freedom' - the divisor used in the covariance\n  calculation. Defaults to 1.","ref":"Explorer.DataFrame.html#covariance/2-options"},{"type":"function","title":"Examples - Explorer.DataFrame.covariance/2","doc":"iex> df = Explorer.DataFrame.new(dogs: [1, 0, 2, 1], cats: [2, 3, 0, 1])\n    iex> Explorer.DataFrame.covariance(df)\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#covariance/2-examples"},{"type":"function","title":"Explorer.DataFrame.describe/2","doc":"Describe numeric columns of a DataFrame.\n\nGroups are ignored if the dataframe is using any.","ref":"Explorer.DataFrame.html#describe/2"},{"type":"function","title":"Options - Explorer.DataFrame.describe/2","doc":"* `:percentiles` - Floating point list with the percentiles to be calculated. (default: `[0.25, 0.5, 0.75]`)","ref":"Explorer.DataFrame.html#describe/2-options"},{"type":"function","title":"Examples - Explorer.DataFrame.describe/2","doc":"iex> df = Explorer.DataFrame.new(a: [\"d\", nil, \"f\"], b: [1, 2, 3], c: [\"a\", \"b\", \"c\"])\n    iex> Explorer.DataFrame.describe(df)\n    #Explorer.DataFrame \n\n    iex> df = Explorer.DataFrame.new(a: [\"d\", nil, \"f\"], b: [1, 2, 3], c: [\"a\", \"b\", \"c\"])\n    iex> Explorer.DataFrame.describe(df, percentiles: [0.3, 0.5, 0.8])\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#describe/2-examples"},{"type":"function","title":"Explorer.DataFrame.discard/2","doc":"Discards a subset of columns by name.\n\nIt's important to notice that groups are kept:\nyou can't discard grouping columns.","ref":"Explorer.DataFrame.html#discard/2"},{"type":"function","title":"Examples - Explorer.DataFrame.discard/2","doc":"iex> df = Explorer.DataFrame.new(a: [\"a\", \"b\", \"c\"], b: [1, 2, 3])\n    iex> Explorer.DataFrame.discard(df, [\"b\"])\n    #Explorer.DataFrame \n\n    iex> df = Explorer.DataFrame.new(a: [\"a\", \"b\", \"c\"], b: [1, 2, 3], c: [4, 5, 6])\n    iex> Explorer.DataFrame.discard(df, [\"a\", \"b\"])\n    #Explorer.DataFrame \n\nRanges, regexes, and functions are also accepted in column names, as in `select/2`.","ref":"Explorer.DataFrame.html#discard/2-examples"},{"type":"function","title":"Grouped examples - Explorer.DataFrame.discard/2","doc":"You cannot discard grouped columns. You need to ungroup before removing them:\n\n    iex> df = Explorer.Datasets.iris()\n    iex> grouped = Explorer.DataFrame.group_by(df, \"species\")\n    iex> Explorer.DataFrame.discard(grouped, [\"species\"])\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#discard/2-grouped-examples"},{"type":"function","title":"Explorer.DataFrame.distinct/3","doc":"Takes distinct rows by a selection of columns.\n\nDistinct is not affected by groups, although groups are kept in the\ncolumns selection if `keep_all` option is false (the default).","ref":"Explorer.DataFrame.html#distinct/3"},{"type":"function","title":"Options - Explorer.DataFrame.distinct/3","doc":"* `:keep_all` - If set to `true`, keep all columns. Default is `false`.","ref":"Explorer.DataFrame.html#distinct/3-options"},{"type":"function","title":"Examples - Explorer.DataFrame.distinct/3","doc":"By default will return unique values of the requested columns:\n\n    iex> df = Explorer.Datasets.fossil_fuels()\n    iex> Explorer.DataFrame.distinct(df, [\"year\", \"country\"])\n    #Explorer.DataFrame \n\nIf `keep_all` is set to `true`, then the first value of each column not in the requested\ncolumns will be returned:\n\n    iex> df = Explorer.Datasets.fossil_fuels()\n    iex> Explorer.DataFrame.distinct(df, [\"year\", \"country\"], keep_all: true)\n    #Explorer.DataFrame \n\nA callback on the dataframe's names can be passed instead of a list (like `select/2`):\n\n    iex> df = Explorer.DataFrame.new(x1: [1, 3, 3], x2: [\"a\", \"c\", \"c\"], y1: [1, 2, 3])\n    iex> Explorer.DataFrame.distinct(df, &String.starts_with?(&1, \"x\"))\n    #Explorer.DataFrame \n\nIf the dataframe has groups, then the columns of each group will be added to the distinct columns:\n\n    iex> df = Explorer.DataFrame.new(x1: [1, 3, 3], x2: [\"a\", \"c\", \"c\"], y1: [1, 2, 3])\n    iex> df = Explorer.DataFrame.group_by(df, \"x1\")\n    iex> Explorer.DataFrame.distinct(df, [\"x2\"])\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#distinct/3-examples"},{"type":"function","title":"Explorer.DataFrame.drop_nil/2","doc":"Drop nil values.\n\nOptionally accepts a subset of columns.","ref":"Explorer.DataFrame.html#drop_nil/2"},{"type":"function","title":"Examples - Explorer.DataFrame.drop_nil/2","doc":"To drop nils on all columns:\n\n    iex> df = Explorer.DataFrame.new(a: [1, 2, nil], b: [1, nil, 3])\n    iex> Explorer.DataFrame.drop_nil(df)\n    #Explorer.DataFrame \n\n To drop nils on a single column:\n\n    iex> df = Explorer.DataFrame.new(a: [1, 2, nil], b: [1, nil, 3])\n    iex> Explorer.DataFrame.drop_nil(df, :a)\n    #Explorer.DataFrame \n\nTo drop some columns:\n\n    iex> df = Explorer.DataFrame.new(a: [1, 2, nil], b: [1, nil, 3], c: [nil, 5, 6])\n    iex> Explorer.DataFrame.drop_nil(df, [:a, :c])\n    #Explorer.DataFrame \n\nRanges, regexes, and functions are also accepted in column names, as in `select/2`.","ref":"Explorer.DataFrame.html#drop_nil/2-examples"},{"type":"function","title":"Explorer.DataFrame.dtypes/1","doc":"Gets the dtypes of the dataframe columns.","ref":"Explorer.DataFrame.html#dtypes/1"},{"type":"function","title":"Examples - Explorer.DataFrame.dtypes/1","doc":"iex> df = Explorer.DataFrame.new(floats: [1.0, 2.0], ints: [1, 2])\n    iex> Explorer.DataFrame.dtypes(df)\n    %{\"floats\" => {:f, 64}, \"ints\" => {:s, 64}}","ref":"Explorer.DataFrame.html#dtypes/1-examples"},{"type":"function","title":"Explorer.DataFrame.dummies/2","doc":"Turns a set of columns to dummy variables.\n\nIn case the dataframe is using groups, all groups will be removed.","ref":"Explorer.DataFrame.html#dummies/2"},{"type":"function","title":"Examples - Explorer.DataFrame.dummies/2","doc":"To mark a single column as dummy:\n\n    iex> df = Explorer.DataFrame.new(col_x: [\"a\", \"b\", \"a\", \"c\"], col_y: [\"b\", \"a\", \"b\", \"d\"])\n    iex> Explorer.DataFrame.dummies(df, \"col_x\")\n    #Explorer.DataFrame \n\nOr multiple columns:\n\n    iex> df = Explorer.DataFrame.new(col_x: [\"a\", \"b\", \"a\", \"c\"], col_y: [\"b\", \"a\", \"b\", \"d\"])\n    iex> Explorer.DataFrame.dummies(df, [\"col_x\", \"col_y\"])\n    #Explorer.DataFrame \n\nOr all string columns:\n\n    iex> df = Explorer.DataFrame.new(num: [1, 2, 3, 4], col_y: [\"b\", \"a\", \"b\", \"d\"])\n    iex> Explorer.DataFrame.dummies(df, fn _name, type -> type == :string end)\n    #Explorer.DataFrame \n\nRanges, regexes, and functions are also accepted in column names, as in `select/2`.","ref":"Explorer.DataFrame.html#dummies/2-examples"},{"type":"function","title":"Explorer.DataFrame.dump_csv/2","doc":"Writes a dataframe to a binary representation of a delimited file.","ref":"Explorer.DataFrame.html#dump_csv/2"},{"type":"function","title":"Options - Explorer.DataFrame.dump_csv/2","doc":"* `:header` - Should the column names be written as the first line of the file? (default: `true`)\n  * `:delimiter` - A single character used to separate fields within a record. (default: `\",\"`)","ref":"Explorer.DataFrame.html#dump_csv/2-options"},{"type":"function","title":"Examples - Explorer.DataFrame.dump_csv/2","doc":"iex> df = Explorer.Datasets.fossil_fuels() |> Explorer.DataFrame.head(2)\n    iex> Explorer.DataFrame.dump_csv(df)\n    {:ok, \"year,country,total,solid_fuel,liquid_fuel,gas_fuel,cement,gas_flaring,per_capita,bunker_fuels\\n2010,AFGHANISTAN,2308,627,1601,74,5,0,0.08,9\\n2010,ALBANIA,1254,117,953,7,177,0,0.43,7\\n\"}","ref":"Explorer.DataFrame.html#dump_csv/2-examples"},{"type":"function","title":"Explorer.DataFrame.dump_csv!/2","doc":"Similar to `dump_csv/2`, but raises in case of error.","ref":"Explorer.DataFrame.html#dump_csv!/2"},{"type":"function","title":"Explorer.DataFrame.dump_ipc/2","doc":"Writes a dataframe to a binary representation of an IPC file.\n\nGroups are ignored if the dataframe is using any.","ref":"Explorer.DataFrame.html#dump_ipc/2"},{"type":"function","title":"Options - Explorer.DataFrame.dump_ipc/2","doc":"* `:compression` - The compression algorithm to use when writing files.\n    Supported options are:\n\n      * `nil` (uncompressed, default)\n      * `:zstd`\n      * `:lz4`.","ref":"Explorer.DataFrame.html#dump_ipc/2-options"},{"type":"function","title":"Explorer.DataFrame.dump_ipc!/2","doc":"Similar to `dump_ipc/2`, but raises in case of error.","ref":"Explorer.DataFrame.html#dump_ipc!/2"},{"type":"function","title":"Explorer.DataFrame.dump_ipc_stream/2","doc":"Writes a dataframe to a binary representation of an IPC Stream file.\n\nGroups are ignored if the dataframe is using any.","ref":"Explorer.DataFrame.html#dump_ipc_stream/2"},{"type":"function","title":"Options - Explorer.DataFrame.dump_ipc_stream/2","doc":"* `:compression` - The compression algorithm to use when writing files.\n    Supported options are:\n\n      * `nil` (uncompressed, default)\n      * `:zstd`\n      * `:lz4`.","ref":"Explorer.DataFrame.html#dump_ipc_stream/2-options"},{"type":"function","title":"Explorer.DataFrame.dump_ipc_stream!/2","doc":"Similar to `dump_ipc_stream/2`, but raises in case of error.","ref":"Explorer.DataFrame.html#dump_ipc_stream!/2"},{"type":"function","title":"Explorer.DataFrame.dump_ndjson/1","doc":"Writes a dataframe to a binary representation of a NDJSON file.\n\nGroups are ignored if the dataframe is using any.","ref":"Explorer.DataFrame.html#dump_ndjson/1"},{"type":"function","title":"Examples - Explorer.DataFrame.dump_ndjson/1","doc":"iex> df = Explorer.DataFrame.new(col_a: [1, 2], col_b: [5.1, 5.2])\n    iex> Explorer.DataFrame.dump_ndjson(df)\n    {:ok, ~s({\"col_a\":1,\"col_b\":5.1}\\n{\"col_a\":2,\"col_b\":5.2}\\n)}","ref":"Explorer.DataFrame.html#dump_ndjson/1-examples"},{"type":"function","title":"Explorer.DataFrame.dump_ndjson!/1","doc":"Similar to `dump_ndjson!/2`, but raises in case of error.","ref":"Explorer.DataFrame.html#dump_ndjson!/1"},{"type":"function","title":"Explorer.DataFrame.dump_parquet/2","doc":"Writes a dataframe to a binary representation of a Parquet file.\n\nGroups are ignored if the dataframe is using any.","ref":"Explorer.DataFrame.html#dump_parquet/2"},{"type":"function","title":"Options - Explorer.DataFrame.dump_parquet/2","doc":"* `:compression` - The compression algorithm to use when writing files.\n    Where a compression level is available, this can be passed as a tuple,\n    such as `{:zstd, 3}`. Supported options are:\n\n      * `nil` (uncompressed, default)\n      * `:snappy`\n      * `:gzip` (with levels 1-9)\n      * `:brotli` (with levels 1-11)\n      * `:zstd` (with levels -7-22)\n      * `:lz4raw`.","ref":"Explorer.DataFrame.html#dump_parquet/2-options"},{"type":"function","title":"Explorer.DataFrame.dump_parquet!/2","doc":"Similar to `dump_parquet/2`, but raises in case of error.","ref":"Explorer.DataFrame.html#dump_parquet!/2"},{"type":"function","title":"Explorer.DataFrame.explode/2","doc":"Explodes one or multiple list column into multiple rows.\n\nWhen exploding multiple columns, the number of list elements in each row for\nthe exploded columns must be the same.","ref":"Explorer.DataFrame.html#explode/2"},{"type":"function","title":"Examples - Explorer.DataFrame.explode/2","doc":"iex> df = Explorer.DataFrame.new(a: [[1, 2], [3, 4]], b: [[5, 6], [7, 8]], c: [\"a\", \"b\"])\n    iex> Explorer.DataFrame.explode(df, :a)\n    #Explorer.DataFrame \n    iex> Explorer.DataFrame.explode(df, [:a, :b])\n    #Explorer.DataFrame \n\nYou can think of exploding multiple list columns as being the inverse of\naggregating the elements the exploded columns into lists:\n\n    iex> df = Explorer.DataFrame.new(a: [1, 2, 3, 4], b: [5, 6, 7, 8], c: [\"a\", \"a\", \"b\", \"b\"])\n    iex> df = df |> Explorer.DataFrame.group_by(:c) |> Explorer.DataFrame.summarise(a: a, b: b)\n    #Explorer.DataFrame \n    iex> Explorer.DataFrame.explode(df, [:a, :b]) # we are back where we started\n    #Explorer.DataFrame \n\nIf you want to perform the cartesian product of two list columns, you must\ncall `explode/2` once for each column:\n\n    iex> df = Explorer.DataFrame.new(a: [[1, 2], [3, 4]], b: [[5, 6], [7, 8]], c: [\"a\", \"b\"])\n    iex> df |> Explorer.DataFrame.explode(:a) |> Explorer.DataFrame.explode(:b)\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#explode/2-examples"},{"type":"macro","title":"Explorer.DataFrame.filter/2","doc":"Picks rows based on `Explorer.Query`.\n\nThe query is compiled and runs efficiently against the dataframe.\nThe query must return a boolean expression or a list of boolean expressions.\nWhen a list is returned, they are joined as `and` expressions.\n\n> #### Notice {: .notice}\n>\n> This is a macro. You must `require  Explorer.DataFrame` before using it.\n\nBesides element-wise series operations, you can also use window functions\nand aggregations inside comparisons. In such cases, grouped dataframes\nmay have different results than ungrouped ones, because the filtering\nis computed withing groups. See examples below.\n\nSee `filter_with/2` for a callback version of this function without\n`Explorer.Query`.","ref":"Explorer.DataFrame.html#filter/2"},{"type":"macro","title":"Examples - Explorer.DataFrame.filter/2","doc":"iex> df = Explorer.DataFrame.new(col1: [\"a\", \"b\", \"c\"], col2: [1, 2, 3])\n    iex> Explorer.DataFrame.filter(df, col2 > 2)\n    #Explorer.DataFrame \n\n    iex> df = Explorer.DataFrame.new(col1: [\"a\", \"b\", \"c\"], col2: [1, 2, 3])\n    iex> Explorer.DataFrame.filter(df, col1 == \"b\")\n    #Explorer.DataFrame \n\n    iex> df = Explorer.DataFrame.new(col1: [5, 4, 3], col2: [1, 2, 3])\n    iex> Explorer.DataFrame.filter(df, [col1 > 3, col2 < 3])\n    #Explorer.DataFrame \n\nReturning a non-boolean expression errors:\n\n    iex> df = Explorer.DataFrame.new(col1: [\"a\", \"b\", \"c\"], col2: [1, 2, 3])\n    iex> Explorer.DataFrame.filter(df, cumulative_max(col2))\n    ** (ArgumentError) expecting the function to return a boolean LazySeries, but instead it returned a LazySeries of type {:s, 64}\n\nWhich can be addressed by converting it to boolean:\n\n    iex> df = Explorer.DataFrame.new(col1: [\"a\", \"b\", \"c\"], col2: [1, 2, 3])\n    iex> Explorer.DataFrame.filter(df, cumulative_max(col2) == 1)\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#filter/2-examples"},{"type":"macro","title":"Grouped examples - Explorer.DataFrame.filter/2","doc":"In a grouped dataframe, the aggregation is calculated within each group.\n\nIn the following example we select the flowers of the Iris dataset that have the \"petal length\"\nabove the average of each species group.\n\n    iex> df = Explorer.Datasets.iris()\n    iex> grouped = Explorer.DataFrame.group_by(df, \"species\")\n    iex> Explorer.DataFrame.filter(grouped, petal_length > mean(petal_length))\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#filter/2-grouped-examples"},{"type":"function","title":"Explorer.DataFrame.filter_with/2","doc":"Picks rows based on a callback function.\n\nThe callback receives a lazy dataframe. A lazy dataframe does\nnot hold any values, instead it stores all operations in order to\nexecute all filtering performantly.\n\nThis is a callback version of `filter/2`.","ref":"Explorer.DataFrame.html#filter_with/2"},{"type":"function","title":"Examples - Explorer.DataFrame.filter_with/2","doc":"iex> df = Explorer.DataFrame.new(col1: [\"a\", \"b\", \"c\"], col2: [1, 2, 3])\n    iex> Explorer.DataFrame.filter_with(df, &Explorer.Series.greater(&1[\"col2\"], 2))\n    #Explorer.DataFrame \n\n    iex> df = Explorer.DataFrame.new(col1: [\"a\", \"b\", \"c\"], col2: [1, 2, 3])\n    iex> Explorer.DataFrame.filter_with(df, fn df -> Explorer.Series.equal(df[\"col1\"], \"b\") end)\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#filter_with/2-examples"},{"type":"function","title":"Grouped examples - Explorer.DataFrame.filter_with/2","doc":"In a grouped dataframe, the aggregation is calculated within each group.\n\nIn the following example we select the flowers of the Iris dataset that have the \"petal length\"\nabove the average of each species group.\n\n    iex> df = Explorer.Datasets.iris()\n    iex> grouped = Explorer.DataFrame.group_by(df, \"species\")\n    iex> Explorer.DataFrame.filter_with(grouped, &Explorer.Series.greater(&1[\"petal_length\"], Explorer.Series.mean(&1[\"petal_length\"])))\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#filter_with/2-grouped-examples"},{"type":"function","title":"Explorer.DataFrame.frequencies/2","doc":"Creates a new dataframe with unique rows and the frequencies of each.","ref":"Explorer.DataFrame.html#frequencies/2"},{"type":"function","title":"Examples - Explorer.DataFrame.frequencies/2","doc":"iex> df = Explorer.DataFrame.new(a: [\"a\", \"a\", \"b\"], b: [1, 1, nil])\n    iex> Explorer.DataFrame.frequencies(df, [:a, :b])\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#frequencies/2-examples"},{"type":"function","title":"Explorer.DataFrame.from_csv/2","doc":"Reads a delimited file into a dataframe.\n\nIt accepts a filename that can be a local file, a \"s3://\" schema, or\na `FSS` entry like `FSS.S3.Entry`.\n\nIf the CSV is compressed, it is automatically decompressed.","ref":"Explorer.DataFrame.html#from_csv/2"},{"type":"function","title":"Options - Explorer.DataFrame.from_csv/2","doc":"* `:delimiter` - A single character used to separate fields within a record. (default: `\",\"`)\n\n  * `:dtypes` - A list/map of `{\"column_name\", dtype}` tuples. Any non-specified column has its type\n    imputed from the first 1000 rows. (default: `[]`)\n\n  * `:header` - Does the file have a header of column names as the first row or not? (default: `true`)\n\n  * `:max_rows` - Maximum number of lines to read. (default: `nil`)\n\n  * `:nil_values` - A list of strings that should be interpreted as a nil values. (default: `[]`)\n\n  * `:skip_rows` - The number of lines to skip at the beginning of the file. (default: `0`)\n\n  * `:skip_rows_after_header` - The number of lines to skip after the header row. (default: `0`)\n\n  * `:columns` - A list of column names or indexes to keep.\n    If present, only these columns are read into the dataframe. (default: `nil`)\n\n  * `:infer_schema_length` Maximum number of rows read for schema inference.\n    Setting this to nil will do a full table scan and will be slow (default: `1000`).\n\n  * `:parse_dates` - Automatically try to parse dates/ datetimes and time.\n    If parsing fails, columns remain of dtype `string`\n\n  * `:eol_delimiter` - A single character used to represent new lines. (default: `\"\n\"`)\n\n  * `:config` - An optional struct, keyword list or map, normally associated with remote\n    file systems. See [IO section](#module-io-operations) for more details. (default: `nil`)\n\n  * `:backend` - The Explorer backend to use. Defaults to the value returned by `Explorer.Backend.get/0`.\n\n  * `:lazy` - force the results into the lazy version of the current backend.\n\n  * `:node` - The Erlang node to allocate the data frame on.\n\n  * `:encoding` - Encoding to use when reading the file. For now, the only possible values are `utf8` and `utf8-lossy`.\n    The utf8-lossy option means that invalid utf8 values are replaced with � characters. (default: `\"utf8\"`)","ref":"Explorer.DataFrame.html#from_csv/2-options"},{"type":"function","title":"Explorer.DataFrame.from_csv!/2","doc":"Similar to `from_csv/2` but raises if there is a problem reading the CSV.","ref":"Explorer.DataFrame.html#from_csv!/2"},{"type":"function","title":"Explorer.DataFrame.from_ipc/2","doc":"Reads an IPC file into a dataframe.\n\nIt accepts a filename that can be a local file, a \"s3://\" schema, or\na `FSS` entry like `FSS.S3.Entry`.","ref":"Explorer.DataFrame.html#from_ipc/2"},{"type":"function","title":"Options - Explorer.DataFrame.from_ipc/2","doc":"* `:columns` - List with the name or index of columns to be selected.\n    Defaults to all columns.\n\n  * `:config` - An optional struct, keyword list or map, normally associated with remote\n    file systems. See [IO section](#module-io-operations) for more details. (default: `nil`)\n\n  * `:backend` - The Explorer backend to use. Defaults to the value returned by `Explorer.Backend.get/0`.\n\n  * `:lazy` - force the results into the lazy version of the current backend.\n\n  * `:node` - The Erlang node to allocate the data frame on.","ref":"Explorer.DataFrame.html#from_ipc/2-options"},{"type":"function","title":"Explorer.DataFrame.from_ipc!/2","doc":"Similar to `from_ipc/2` but raises if there is a problem reading the IPC file.","ref":"Explorer.DataFrame.html#from_ipc!/2"},{"type":"function","title":"Explorer.DataFrame.from_ipc_stream/2","doc":"Reads an IPC Streaming file into a dataframe.\n\nIt's possible to read from an IPC Streaming file using the lazy Polars\nbackend, but the implementation is not truly lazy. We are going to read it\nfirst using the eager backend, and then convert the dataframe to lazy.","ref":"Explorer.DataFrame.html#from_ipc_stream/2"},{"type":"function","title":"Options - Explorer.DataFrame.from_ipc_stream/2","doc":"* `:columns` - List with the name or index of columns to be selected.\n    Defaults to all columns.\n\n  * `:backend` - The Explorer backend to use. Defaults to the value returned by `Explorer.Backend.get/0`.\n\n  * `:lazy` - force the results into the lazy version of the current backend.\n\n  * `:node` - The Erlang node to allocate the data frame on.\n\n  * `:config` - An optional struct, keyword list or map, normally associated with remote\n    file systems. See [IO section](#module-io-operations) for more details. (default: `nil`)","ref":"Explorer.DataFrame.html#from_ipc_stream/2-options"},{"type":"function","title":"Explorer.DataFrame.from_ipc_stream!/2","doc":"Similar to `from_ipc_stream/2` but raises if there is a problem reading the IPC Stream file.","ref":"Explorer.DataFrame.html#from_ipc_stream!/2"},{"type":"function","title":"Explorer.DataFrame.from_ndjson/2","doc":"Read a file of JSON objects or lists separated by new lines","ref":"Explorer.DataFrame.html#from_ndjson/2"},{"type":"function","title":"Options - Explorer.DataFrame.from_ndjson/2","doc":"* `:batch_size` - Sets the batch size for reading rows.\n    This value may have significant impact in performance,\n    so adjust it for your needs (default: `1000`).\n\n  * `:infer_schema_length` - Maximum number of rows read for schema inference.\n    Setting this to nil will do a full table scan and will be slow (default: `1000`).\n\n  * `:backend` - The Explorer backend to use. Defaults to the value returned by `Explorer.Backend.get/0`.\n\n  * `:lazy` - force the results into the lazy version of the current backend.\n\n  * `:node` - The Erlang node to allocate the data frame on.\n\n  * `:config` - An optional struct, keyword list or map, normally associated with remote\n    file systems. See [IO section](#module-io-operations) for more details. (default: `nil`)","ref":"Explorer.DataFrame.html#from_ndjson/2-options"},{"type":"function","title":"Explorer.DataFrame.from_ndjson!/2","doc":"Similar to `from_ndjson/2`, but raises in case of error.","ref":"Explorer.DataFrame.html#from_ndjson!/2"},{"type":"function","title":"Explorer.DataFrame.from_parquet/2","doc":"Reads a parquet file into a dataframe.\n\nIt accepts a filename that can be a local file, a \"s3://\" schema, or\na `FSS` entry like `FSS.S3.Entry`.","ref":"Explorer.DataFrame.html#from_parquet/2"},{"type":"function","title":"Options - Explorer.DataFrame.from_parquet/2","doc":"* `:max_rows` - Maximum number of lines to read. (default: `nil`)\n\n  * `:columns` - A list of column names or indexes to keep. If present,\n    only these columns are read into the dataframe. (default: `nil`)\n\n  * `:rechunk` - Make sure that all columns are contiguous in memory\n    by aggregating the chunks into a single array. (default: `false`)\n\n  * `:config` - An optional struct, keyword list or map, normally associated with remote\n    file systems. See [IO section](#module-io-operations) for more details. (default: `nil`)\n\n  * `:backend` - The Explorer backend to use. Defaults to the value returned by `Explorer.Backend.get/0`.\n\n  * `:lazy` - force the results into the lazy version of the current backend.\n\n  * `:node` - The Erlang node to allocate the data frame on.","ref":"Explorer.DataFrame.html#from_parquet/2-options"},{"type":"function","title":"Explorer.DataFrame.from_parquet!/2","doc":"Similar to `from_parquet/2` but raises if there is a problem reading the Parquet file.","ref":"Explorer.DataFrame.html#from_parquet!/2"},{"type":"function","title":"Explorer.DataFrame.from_query/4","doc":"Reads data from a query.\n\n`conn` must be a `Adbc.Connection` process. `sql` is a string representing\nthe sql query and `params` is the list of query parameters. See `Adbc`\nfor more information.","ref":"Explorer.DataFrame.html#from_query/4"},{"type":"function","title":"Example - Explorer.DataFrame.from_query/4","doc":"In order to read data from a database, you must list `:adbc` as a dependency,\ndownload the relevant driver, and start both database and connection processes\nin your supervision tree.\n\nFirst, add `:adbc` as a dependency in your `mix.exs`:\n\n    {:adbc, \"~> 0.1\"}\n\nNow, in your config/config.exs, configure the drivers you are going to use\n(see `Adbc` module docs for more information on supported drivers):\n\n    config :adbc, :drivers, [:sqlite]\n\nIf you are using a notebook or scripting, you can also use `Adbc.download_driver!/1`\nto dynamically download one.\n\nThen start the database and the relevant connection processes in your\nsupervision tree:\n\n    children = [\n      {Adbc.Database,\n       driver: :sqlite,\n       process_options: [name: MyApp.DB]},\n      {Adbc.Connection,\n       database: MyApp.DB,\n       process_options: [name: MyApp.Conn]}\n    ]\n\n    Supervisor.start_link(children, strategy: :one_for_one)\n\nIn a notebook, the above would look like this:\n\n    db = Kino.start_child!({Adbc.Database, driver: :sqlite})\n    conn = Kino.start_child!({Adbc.Connection, database: db})\n\nAnd now you can make queries with:\n\n    # For named connections\n    {:ok, _} = Explorer.DataFrame.from_query(MyApp.Conn, \"SELECT 123\", [])\n\n    # When using the conn PID directly\n    {:ok, _} = Explorer.DataFrame.from_query(conn, \"SELECT 123\", [])\n\nParameterized queries are possible by passing a list of parameters as the third argument. The\nquery string will use a different placeholder depending on the driver. For example, `?` for\nSQLite and `$1`, `$2`, etc. for PostgreSQL.\n\n    # For SQLite\n    {:ok, _} = Explorer.DataFrame.from_query(conn, \"SELECT ?\", [123])\n\n    # When using PostgreSQL\n    {:ok, _} = Explorer.DataFrame.from_query(conn, \"SELECT $1\", [123])","ref":"Explorer.DataFrame.html#from_query/4-example"},{"type":"function","title":"Options - Explorer.DataFrame.from_query/4","doc":"* `:backend` - The Explorer backend to use. Defaults to the value returned by `Explorer.Backend.get/0`.\n\n  * `:lazy` - force the results into the lazy version of the current backend.","ref":"Explorer.DataFrame.html#from_query/4-options"},{"type":"function","title":"Explorer.DataFrame.from_query!/4","doc":"Similar to `from_query/4` but raises if there is an error.","ref":"Explorer.DataFrame.html#from_query!/4"},{"type":"function","title":"Explorer.DataFrame.group_by/2","doc":"Group the dataframe by one or more variables.\n\nWhen the dataframe has grouping variables, operations are performed per group.\n`Explorer.DataFrame.ungroup/2` removes grouping.","ref":"Explorer.DataFrame.html#group_by/2"},{"type":"function","title":"Examples - Explorer.DataFrame.group_by/2","doc":"You can group by a single variable:\n\n    iex> df = Explorer.Datasets.fossil_fuels()\n    iex> Explorer.DataFrame.group_by(df, \"country\")\n    #Explorer.DataFrame \n\nOr you can group by multiple columns in a given list:\n\n    iex> df = Explorer.Datasets.fossil_fuels()\n    iex> Explorer.DataFrame.group_by(df, [\"country\", \"year\"])\n    #Explorer.DataFrame \n\nOr by a range:\n\n    iex> df = Explorer.Datasets.fossil_fuels()\n    iex> Explorer.DataFrame.group_by(df, 0..1)\n    #Explorer.DataFrame \n\nRegexes and functions are also accepted in column names, as in `select/2`.","ref":"Explorer.DataFrame.html#group_by/2-examples"},{"type":"function","title":"Explorer.DataFrame.groups/1","doc":"Returns the groups of a dataframe.","ref":"Explorer.DataFrame.html#groups/1"},{"type":"function","title":"Examples - Explorer.DataFrame.groups/1","doc":"iex> df = Explorer.Datasets.fossil_fuels()\n    iex> df = Explorer.DataFrame.group_by(df, \"country\")\n    iex> Explorer.DataFrame.groups(df)\n    [\"country\"]\n\n    iex> df = Explorer.Datasets.iris()\n    iex> Explorer.DataFrame.groups(df)\n    []","ref":"Explorer.DataFrame.html#groups/1-examples"},{"type":"function","title":"Explorer.DataFrame.head/2","doc":"Returns the first *n* rows of the dataframe.\n\nBy default it returns the first 5 rows.\n\nIf the dataframe is using groups, then the first *n* rows of each group is\nreturned.","ref":"Explorer.DataFrame.html#head/2"},{"type":"function","title":"Examples - Explorer.DataFrame.head/2","doc":"iex> df = Explorer.Datasets.fossil_fuels()\n    iex> Explorer.DataFrame.head(df)\n    #Explorer.DataFrame \n\n    iex> df = Explorer.Datasets.fossil_fuels()\n    iex> Explorer.DataFrame.head(df, 2)\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#head/2-examples"},{"type":"function","title":"Grouped examples - Explorer.DataFrame.head/2","doc":"Using grouped dataframes makes `head/2` return *n* rows from each group.\nHere is an example using the Iris dataset, and returning two rows from each group:\n\n    iex> df = Explorer.Datasets.iris()\n    iex> grouped = Explorer.DataFrame.group_by(df, \"species\")\n    iex> Explorer.DataFrame.head(grouped, 2)\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#head/2-grouped-examples"},{"type":"function","title":"Explorer.DataFrame.join/3","doc":"Join two tables.","ref":"Explorer.DataFrame.html#join/3"},{"type":"function","title":"Join types - Explorer.DataFrame.join/3","doc":"* `:inner` - Returns all rows from `left` where there are matching values in `right`, and all columns from `left` and `right`.\n  * `:left` - Returns all rows from `left` and all columns from `left` and `right`. Rows in `left` with no match in `right` will have `nil` values in the new columns.\n  * `:right` - Returns all rows from `right` and all columns from `left` and `right`. Rows in `right` with no match in `left` will have `nil` values in the new columns.\n  * `:outer` - Returns all rows and all columns from both `left` and `right`. Where there are not matching values, returns `nil` for the one missing.\n  * `:cross` - Also known as a cartesian join. Returns all combinations of `left` and `right`. Can be very computationally expensive.","ref":"Explorer.DataFrame.html#join/3-join-types"},{"type":"function","title":"Options - Explorer.DataFrame.join/3","doc":"* `:on` - The column(s) to join on. Defaults to overlapping columns. Does not apply to cross join.\n  * `:how` - One of the join types (as an atom) described above. Defaults to `:inner`.","ref":"Explorer.DataFrame.html#join/3-options"},{"type":"function","title":"Examples - Explorer.DataFrame.join/3","doc":"Inner join:\n\n    iex> left = Explorer.DataFrame.new(a: [1, 2, 3], b: [\"a\", \"b\", \"c\"])\n    iex> right = Explorer.DataFrame.new(a: [1, 2, 2], c: [\"d\", \"e\", \"f\"])\n    iex> Explorer.DataFrame.join(left, right)\n    #Explorer.DataFrame \n\nLeft join:\n\n    iex> left = Explorer.DataFrame.new(a: [1, 2, 3], b: [\"a\", \"b\", \"c\"])\n    iex> right = Explorer.DataFrame.new(a: [1, 2, 2], c: [\"d\", \"e\", \"f\"])\n    iex> Explorer.DataFrame.join(left, right, how: :left)\n    #Explorer.DataFrame \n\nRight join:\n\n    iex> left = Explorer.DataFrame.new(a: [1, 2, 3], b: [\"a\", \"b\", \"c\"])\n    iex> right = Explorer.DataFrame.new(a: [1, 2, 4], c: [\"d\", \"e\", \"f\"])\n    iex> Explorer.DataFrame.join(left, right, how: :right)\n    #Explorer.DataFrame \n\nOuter join:\n\n    iex> left = Explorer.DataFrame.new(a: [1, 2, 3], b: [\"a\", \"b\", \"c\"])\n    iex> right = Explorer.DataFrame.new(a: [1, 2, 4], c: [\"d\", \"e\", \"f\"])\n    iex> Explorer.DataFrame.join(left, right, how: :outer)\n    #Explorer.DataFrame \n\n\nCross join:\n\n    iex> left = Explorer.DataFrame.new(a: [1, 2, 3], b: [\"a\", \"b\", \"c\"])\n    iex> right = Explorer.DataFrame.new(a: [1, 2, 4], c: [\"d\", \"e\", \"f\"])\n    iex> Explorer.DataFrame.join(left, right, how: :cross)\n    #Explorer.DataFrame \n\nInner join with different names:\n\n    iex> left = Explorer.DataFrame.new(a: [1, 2, 3], b: [\"a\", \"b\", \"c\"])\n    iex> right = Explorer.DataFrame.new(d: [1, 2, 2], c: [\"d\", \"e\", \"f\"])\n    iex> Explorer.DataFrame.join(left, right, on: [{\"a\", \"d\"}])\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#join/3-examples"},{"type":"function","title":"Grouped examples - Explorer.DataFrame.join/3","doc":"When doing a join operation with grouped dataframes, the joined dataframe\nmay keep the groups from only one side.\n\nAn inner join operation will keep the groups from the left-hand side dataframe:\n\n    iex> left = Explorer.DataFrame.new(a: [1, 2, 3], b: [\"a\", \"b\", \"c\"])\n    iex> right = Explorer.DataFrame.new(a: [1, 2, 2], c: [\"d\", \"e\", \"f\"])\n    iex> grouped_left = Explorer.DataFrame.group_by(left, \"b\")\n    iex> grouped_right = Explorer.DataFrame.group_by(right, \"c\")\n    iex> Explorer.DataFrame.join(grouped_left, grouped_right)\n    #Explorer.DataFrame \n\nA left join operation will keep the groups from the left-hand side dataframe:\n\n    iex> left = Explorer.DataFrame.new(a: [1, 2, 3], b: [\"a\", \"b\", \"c\"])\n    iex> right = Explorer.DataFrame.new(a: [1, 2, 2], c: [\"d\", \"e\", \"f\"])\n    iex> grouped_left = Explorer.DataFrame.group_by(left, \"b\")\n    iex> grouped_right = Explorer.DataFrame.group_by(right, \"c\")\n    iex> Explorer.DataFrame.join(grouped_left, grouped_right, how: :left)\n    #Explorer.DataFrame \n\nA right join operation will keep the groups from the right-hand side dataframe:\n\n    iex> left = Explorer.DataFrame.new(a: [1, 2, 3], b: [\"a\", \"b\", \"c\"])\n    iex> right = Explorer.DataFrame.new(a: [1, 2, 4], c: [\"d\", \"e\", \"f\"])\n    iex> grouped_left = Explorer.DataFrame.group_by(left, \"b\")\n    iex> grouped_right = Explorer.DataFrame.group_by(right, \"c\")\n    iex> Explorer.DataFrame.join(grouped_left, grouped_right, how: :right)\n    #Explorer.DataFrame \n\nAn outer join operation is going to keep the groups from the left-hand side dataframe:\n\n    iex> left = Explorer.DataFrame.new(a: [1, 2, 3], b: [\"a\", \"b\", \"c\"])\n    iex> right = Explorer.DataFrame.new(a: [1, 2, 4], c: [\"d\", \"e\", \"f\"])\n    iex> grouped_left = Explorer.DataFrame.group_by(left, \"b\")\n    iex> grouped_right = Explorer.DataFrame.group_by(right, \"c\")\n    iex> Explorer.DataFrame.join(grouped_left, grouped_right, how: :outer)\n    #Explorer.DataFrame \n\n\n\nA cross join operation is going to keep the groups from the left-hand side dataframe:\n\n    iex> left = Explorer.DataFrame.new(a: [1, 2, 3], b: [\"a\", \"b\", \"c\"])\n    iex> right = Explorer.DataFrame.new(a: [1, 2, 4], c: [\"d\", \"e\", \"f\"])\n    iex> grouped_left = Explorer.DataFrame.group_by(left, \"b\")\n    iex> grouped_right = Explorer.DataFrame.group_by(right, \"c\")\n    iex> Explorer.DataFrame.join(grouped_left, grouped_right, how: :cross)\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#join/3-grouped-examples"},{"type":"function","title":"Explorer.DataFrame.lazy/1","doc":"Converts the dataframe to the lazy version of the current backend.\n\nOperations on a lazy dataframe are not executed immediately, instead\nthey are batched together, and performed when `collect/1` is invoked.\nThis allows for query optimizations, parallelism, leading to better\nperformance.\n\nIf already lazy, this is a noop. Converting a grouped dataframe should\nreturn a lazy dataframe with groups.","ref":"Explorer.DataFrame.html#lazy/1"},{"type":"function","title":"Explorer.DataFrame.lazy?/1","doc":"Returns true if the data frame is a lazy one.","ref":"Explorer.DataFrame.html#lazy?/1"},{"type":"function","title":"Explorer.DataFrame.load_csv/2","doc":"Reads a representation of a CSV file into a dataframe.\n\nIf the CSV is compressed, it is automatically decompressed.\n\nAccepts the same options as `from_csv/2`.","ref":"Explorer.DataFrame.html#load_csv/2"},{"type":"function","title":"Explorer.DataFrame.load_csv!/2","doc":"Similar to `load_csv/2` but raises if there is a problem reading the CSV.","ref":"Explorer.DataFrame.html#load_csv!/2"},{"type":"function","title":"Explorer.DataFrame.load_ipc/2","doc":"Reads a binary representing an IPC file into a dataframe.\n\nAccepts the same options as `from_ipc/2`.","ref":"Explorer.DataFrame.html#load_ipc/2"},{"type":"function","title":"Explorer.DataFrame.load_ipc!/2","doc":"Similar to `load_ipc/2` but raises if there is a problem reading the IPC file.","ref":"Explorer.DataFrame.html#load_ipc!/2"},{"type":"function","title":"Explorer.DataFrame.load_ipc_stream/2","doc":"Reads a binary representing an IPC Stream file into a dataframe.\n\nAccepts the same options as `from_ipc_stream/2`.","ref":"Explorer.DataFrame.html#load_ipc_stream/2"},{"type":"function","title":"Explorer.DataFrame.load_ipc_stream!/2","doc":"Similar to `load_ipc_stream/2` but raises if there is a problem.","ref":"Explorer.DataFrame.html#load_ipc_stream!/2"},{"type":"function","title":"Explorer.DataFrame.load_ndjson/2","doc":"Reads a representation of a NDJSON file into a dataframe.\n\nAccepts the same options as `from_ndjson/2`.","ref":"Explorer.DataFrame.html#load_ndjson/2"},{"type":"function","title":"Explorer.DataFrame.load_ndjson!/2","doc":"Similar to `load_ndjson/2`, but raises in case of error.","ref":"Explorer.DataFrame.html#load_ndjson!/2"},{"type":"function","title":"Examples - Explorer.DataFrame.load_ndjson!/2","doc":"iex> contents = ~s({\"col_a\":1,\"col_b\":5.1}\\n{\"col_a\":2,\"col_b\":5.2}\\n)\n    iex> Explorer.DataFrame.load_ndjson!(contents)\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#load_ndjson!/2-examples"},{"type":"function","title":"Explorer.DataFrame.load_parquet/2","doc":"Reads a binary representation of a parquet file into a dataframe.\n\nAccepts the same options as `from_parquet/2`.","ref":"Explorer.DataFrame.html#load_parquet/2"},{"type":"function","title":"Explorer.DataFrame.load_parquet!/2","doc":"Similar to `load_parquet/2` but raises if there is a problem reading the Parquet file.","ref":"Explorer.DataFrame.html#load_parquet!/2"},{"type":"function","title":"Explorer.DataFrame.mask/2","doc":"Picks rows based on a list or series of values.","ref":"Explorer.DataFrame.html#mask/2"},{"type":"function","title":"Examples - Explorer.DataFrame.mask/2","doc":"This function must only be used when you need to select rows based\non external values that are not available to the dataframe. For example,\nyou can pass a list:\n\n    iex> df = Explorer.DataFrame.new(col1: [\"a\", \"b\", \"c\"], col2: [1, 2, 3])\n    iex> Explorer.DataFrame.mask(df, [false, true, false])\n    #Explorer.DataFrame \n\nYou must avoid using masks when the masks themselves are computed from\nother columns. For example, DO NOT do this:\n\n    iex> df = Explorer.DataFrame.new(col1: [\"a\", \"b\", \"c\"], col2: [1, 2, 3])\n    iex> Explorer.DataFrame.mask(df, Explorer.Series.greater(df[\"col2\"], 1))\n    #Explorer.DataFrame \n\nInstead, do this:\n\n    iex> df = Explorer.DataFrame.new(col1: [\"a\", \"b\", \"c\"], col2: [1, 2, 3])\n    iex> Explorer.DataFrame.filter_with(df, fn df -> Explorer.Series.greater(df[\"col2\"], 1) end)\n    #Explorer.DataFrame \n\nThe `filter_with/2` version is much more efficient because it doesn't need\nto create intermediate series representations to apply the mask.","ref":"Explorer.DataFrame.html#mask/2-examples"},{"type":"macro","title":"Explorer.DataFrame.mutate/2","doc":"Creates or modifies columns based on `Explorer.Query`.\n\nThe query is compiled and runs efficiently against the dataframe.\nNew variables overwrite existing variables of the same name.\nColumn names are coerced from atoms to strings.\n\n> #### Notice {: .notice}\n>\n> This is a macro. You must `require  Explorer.DataFrame` before using it.\n\nBesides element-wise series operations, you can also use window functions\nand aggregations inside mutations. In such cases, grouped dataframes\nmay have different results than ungrouped ones, because the mutation\nis computed withing groups. See examples below.\n\nSee `mutate_with/2` for a callback version of this function without\n`Explorer.Query`. If your mutation cannot be expressed with queries,\nyou may compute the values using the `Explorer.Series` API directly\nand then add it to the dataframe using `put/3`.","ref":"Explorer.DataFrame.html#mutate/2"},{"type":"macro","title":"Examples - Explorer.DataFrame.mutate/2","doc":"Mutations are useful to add or modify columns in your dataframe:\n\n    iex> df = Explorer.DataFrame.new(a: [\"a\", \"b\", \"c\"], b: [1, 2, 3])\n    iex> Explorer.DataFrame.mutate(df, c: b + 1)\n    #Explorer.DataFrame \n\nIt's also possible to overwrite existing columns:\n\n    iex> df = Explorer.DataFrame.new(a: [\"a\", \"b\", \"c\"], b: [1, 2, 3])\n    iex> Explorer.DataFrame.mutate(df, a: b * 2)\n    #Explorer.DataFrame \n\nScalar values are repeated to fill the series:\n\n    iex> df = Explorer.DataFrame.new(a: [\"a\", \"b\", \"c\"], b: [1, 2, 3])\n    iex> Explorer.DataFrame.mutate(df, a: 4)\n    #Explorer.DataFrame \n\nIt's also possible to use functions from the Series module, like `Explorer.Series.window_sum/3`:\n\n    iex> df = Explorer.DataFrame.new(a: [1, 2, 3])\n    iex> Explorer.DataFrame.mutate(df, b: window_sum(a, 2))\n    #Explorer.DataFrame \n\nAlternatively, all of the above works with a map instead of a keyword list:\n\n    iex> df = Explorer.DataFrame.new(a: [\"a\", \"b\", \"c\"], b: [1, 2, 3])\n    iex> Explorer.DataFrame.mutate(df, %{\"c\" => cast(b, {:f, 64})})\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#mutate/2-examples"},{"type":"macro","title":"Grouped examples - Explorer.DataFrame.mutate/2","doc":"Mutations in grouped dataframes takes the context of the group.\nThis enables some aggregations to be made considering each group. It's almost like `summarise/2`,\nbut repeating the results for each member in the group.\nFor example, if we want to count how many elements of a given group, we can add a new\ncolumn with that aggregation:\n\n    iex> df = Explorer.DataFrame.new(id: [\"a\", \"a\", \"b\"], b: [1, 2, 3])\n    iex> grouped = Explorer.DataFrame.group_by(df, :id)\n    iex> Explorer.DataFrame.mutate(grouped, count: count(b))\n    #Explorer.DataFrame \n\nIn case we want to get the average size of the petal length from the Iris dataset, we can:\n\n    iex> df = Explorer.Datasets.iris()\n    iex> grouped = Explorer.DataFrame.group_by(df, \"species\")\n    iex> Explorer.DataFrame.mutate(grouped, petal_length_avg: mean(petal_length))\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#mutate/2-grouped-examples"},{"type":"function","title":"Explorer.DataFrame.mutate_with/2","doc":"Creates or modifies columns using a callback function.\n\nThe callback receives a lazy dataframe. A lazy dataframe doesn't\nhold any values, instead it stores all operations in order to\nexecute all mutations performantly.\n\nThis is a callback version of `mutate/2`. If your mutation\ncannot be expressed with lazy dataframes, you may compute the\nvalues using the `Explorer.Series` API directly and then add\nit to the dataframe using `put/3`.","ref":"Explorer.DataFrame.html#mutate_with/2"},{"type":"function","title":"Examples - Explorer.DataFrame.mutate_with/2","doc":"Here is an example of a new column that sums the value of two other columns:\n\n    iex> df = Explorer.DataFrame.new(a: [4, 5, 6], b: [1, 2, 3])\n    iex> Explorer.DataFrame.mutate_with(df, &[c: Explorer.Series.add(&1[\"a\"], &1[\"b\"])])\n    #Explorer.DataFrame \n\nYou can overwrite existing columns as well:\n\n    iex> df = Explorer.DataFrame.new(a: [\"a\", \"b\", \"c\"], b: [1, 2, 3])\n    iex> Explorer.DataFrame.mutate_with(df, &[b: Explorer.Series.pow(&1[\"b\"], 2)])\n    #Explorer.DataFrame \n\nIt's possible to \"reuse\" a variable for different computations:\n\n    iex> df = Explorer.DataFrame.new(a: [4, 5, 6], b: [1, 2, 3])\n    iex> Explorer.DataFrame.mutate_with(df, fn ldf ->\n    iex>   c = Explorer.Series.add(ldf[\"a\"], ldf[\"b\"])\n    iex>   [c: c, d: Explorer.Series.window_sum(c, 2)]\n    iex> end)\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#mutate_with/2-examples"},{"type":"function","title":"Grouped examples - Explorer.DataFrame.mutate_with/2","doc":"Mutations in grouped dataframes takes the context of the group.\nFor example, if we want to count how many elements of a given group,\nwe can add a new column with that aggregation:\n\n    iex> df = Explorer.DataFrame.new(id: [\"a\", \"a\", \"b\"], b: [1, 2, 3])\n    iex> grouped = Explorer.DataFrame.group_by(df, :id)\n    iex> Explorer.DataFrame.mutate_with(grouped, &[count: Explorer.Series.count(&1[\"b\"])])\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#mutate_with/2-grouped-examples"},{"type":"function","title":"Explorer.DataFrame.n_columns/1","doc":"Returns the number of columns in the dataframe.\n\nThis function works the same way for grouped dataframes.","ref":"Explorer.DataFrame.html#n_columns/1"},{"type":"function","title":"Examples - Explorer.DataFrame.n_columns/1","doc":"iex> df = Explorer.Datasets.fossil_fuels()\n    iex> Explorer.DataFrame.n_columns(df)\n    10","ref":"Explorer.DataFrame.html#n_columns/1-examples"},{"type":"function","title":"Explorer.DataFrame.n_rows/1","doc":"Returns the number of rows in the dataframe.\n\nThis function works the same way for grouped dataframes, considering the entire\ndataframe in the counting of rows.","ref":"Explorer.DataFrame.html#n_rows/1"},{"type":"function","title":"Examples - Explorer.DataFrame.n_rows/1","doc":"iex> df = Explorer.Datasets.fossil_fuels()\n    iex> Explorer.DataFrame.n_rows(df)\n    1094","ref":"Explorer.DataFrame.html#n_rows/1-examples"},{"type":"function","title":"Explorer.DataFrame.names/1","doc":"Gets the names of the dataframe columns.","ref":"Explorer.DataFrame.html#names/1"},{"type":"function","title":"Examples - Explorer.DataFrame.names/1","doc":"iex> df = Explorer.DataFrame.new(floats: [1.0, 2.0], ints: [1, 2])\n    iex> Explorer.DataFrame.names(df)\n    [\"floats\", \"ints\"]","ref":"Explorer.DataFrame.html#names/1-examples"},{"type":"function","title":"Explorer.DataFrame.new/2","doc":"Creates a new dataframe.\n\nIt accepts any of:\n\n  * a map or keyword list of string/atom keys and series as values\n  * a map or keyword list of string/atom keys and tensors as values\n  * any data structure adhering to the `Table.Reader` protocol","ref":"Explorer.DataFrame.html#new/2"},{"type":"function","title":"Options - Explorer.DataFrame.new/2","doc":"* `:backend` - The Explorer backend to use. Defaults to the value returned by `Explorer.Backend.get/0`.\n  * `:dtypes` - A list/map of `{column_name, dtype}` pairs. (default: `[]`)\n  * `:lazy` - force the results into the lazy version of the current backend.","ref":"Explorer.DataFrame.html#new/2-options"},{"type":"function","title":"Examples - Explorer.DataFrame.new/2","doc":"","ref":"Explorer.DataFrame.html#new/2-examples"},{"type":"function","title":"From series - Explorer.DataFrame.new/2","doc":"Series can be given either as keyword lists or maps\nwhere the keys are the name and the values are series:\n\n    iex> Explorer.DataFrame.new(%{\n    ...>   floats: Explorer.Series.from_list([1.0, 2.0]),\n    ...>   ints: Explorer.Series.from_list([1, nil])\n    ...> })\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#new/2-from-series"},{"type":"function","title":"From tensors - Explorer.DataFrame.new/2","doc":"To create dataframe from tensors, you can pass a matrix as argument.\nEach matrix column becomes a dataframe column with names x1, x2, x3,\netc:\n\n    iex> Explorer.DataFrame.new(Nx.tensor([\n    ...>   [1, 2, 3],\n    ...>   [4, 5, 6]\n    ...> ]))\n    #Explorer.DataFrame \n\nExplorer expects tensors to have certain types, so you may need to cast\nthe data accordingly. See `Explorer.Series.from_tensor/2` for more info.\n\nYou can also pass a keyword list or maps of vectors (rank 1 tensors):\n\n    iex> Explorer.DataFrame.new(%{\n    ...>   floats: Nx.tensor([1.0, 2.0], type: :f64),\n    ...>   ints: Nx.tensor([3, 4])\n    ...> })\n    #Explorer.DataFrame \n\nUse dtypes to force a particular representation:\n\n    iex> Explorer.DataFrame.new([\n    ...>   floats: Nx.tensor([1.0, 2.0], type: :f64),\n    ...>   times: Nx.tensor([3_000, 4_000])\n    ...> ], dtypes: [times: :time])\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#new/2-from-tensors"},{"type":"function","title":"From tabular - Explorer.DataFrame.new/2","doc":"Tabular data can be either columnar or row-based.\nLet's start with column data:\n\n    iex> Explorer.DataFrame.new(%{floats: [1.0, 2.0], ints: [1, nil]})\n    #Explorer.DataFrame \n\n    iex> Explorer.DataFrame.new(floats: [1.0, 2.0], ints: [1, nil])\n    #Explorer.DataFrame \n\n    iex> Explorer.DataFrame.new([floats: [1.0, 2.0], ints: [1, nil], binaries: [<<239, 191, 19>>, nil]], dtypes: [{:binaries, :binary}])\n    #Explorer.DataFrame >, nil]\n    >\n\n    iex> Explorer.DataFrame.new(%{floats: [1.0, 2.0], ints: [1, \"wrong\"]})\n    ** (ArgumentError) cannot create series \"ints\": the value \"wrong\" does not match the inferred dtype {:s, 64}\n\nFrom row data:\n\n    iex> rows = [%{id: 1, name: \"José\"}, %{id: 2, name: \"Christopher\"}, %{id: 3, name: \"Cristine\"}]\n    iex> Explorer.DataFrame.new(rows)\n    #Explorer.DataFrame \n\n    iex> rows = [[id: 1, name: \"José\"], [id: 2, name: \"Christopher\"], [id: 3, name: \"Cristine\"]]\n    iex> Explorer.DataFrame.new(rows)\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#new/2-from-tabular"},{"type":"function","title":"Explorer.DataFrame.nil_count/1","doc":"Counts the number of `nil` elements in each column.","ref":"Explorer.DataFrame.html#nil_count/1"},{"type":"function","title":"Examples - Explorer.DataFrame.nil_count/1","doc":"iex> df = Explorer.DataFrame.new(a: [\"d\", nil, \"f\"], b: [nil, 2, nil], c: [\"a\", \"b\", \"c\"])\n    iex> Explorer.DataFrame.nil_count(df)\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#nil_count/1-examples"},{"type":"function","title":"Explorer.DataFrame.pivot_longer/3","doc":"Pivot data from wide to long.\n\n`pivot_longer/3` \"lengthens\" data, increasing the number of rows and\ndecreasing the number of columns. The inverse transformation is `pivot_wider/4`.\n\nThe second argument, `columns_to_pivot`, can be either list of column names to pivot.\nRanges, regexes, and functions are also accepted in column names, as in `select/2`.\nThe selected columns must always have the same data type.\n\nIn case the dataframe is using groups, the groups that are also in the list of columns\nto pivot will be removed from the resultant dataframe. See the examples below.","ref":"Explorer.DataFrame.html#pivot_longer/3"},{"type":"function","title":"Options - Explorer.DataFrame.pivot_longer/3","doc":"* `:select` - Columns that are not in the list of pivot and should be kept in the dataframe.\n    Ranges, regexes, and functions are also accepted in column names, as in `select/2`.\n    Defaults to all columns except the ones to pivot.\n\n  * `:discard` - Columns that are not in the list of pivot and should be dropped from the dataframe.\n    Ranges, regexes, and functions are also accepted in column names, as in `select/2`.\n    This list of columns is going to be subtracted from the list of `select`.\n    Defaults to an empty list.\n\n  * `:names_to` - A string specifying the name of the column to create from the data stored\n    in the column names of the dataframe. Defaults to `\"variable\"`.\n\n  * `:values_to` - A string specifying the name of the column to create from the data stored\n    in series element values. Defaults to `\"value\"`.","ref":"Explorer.DataFrame.html#pivot_longer/3-options"},{"type":"function","title":"Examples - Explorer.DataFrame.pivot_longer/3","doc":"iex> df = Explorer.Datasets.fossil_fuels()\n    iex> Explorer.DataFrame.pivot_longer(df, &String.ends_with?(&1, \"fuel\"))\n    #Explorer.DataFrame \n\n    iex> df = Explorer.Datasets.fossil_fuels()\n    iex> Explorer.DataFrame.pivot_longer(df, &String.ends_with?(&1, \"fuel\"), select: [\"year\", \"country\"])\n    #Explorer.DataFrame \n\n    iex> df = Explorer.Datasets.fossil_fuels()\n    iex> Explorer.DataFrame.pivot_longer(df, [\"total\"], select: [\"year\", \"country\"], discard: [\"country\"])\n    #Explorer.DataFrame \n\n    iex> df = Explorer.Datasets.fossil_fuels()\n    iex> Explorer.DataFrame.pivot_longer(df, [\"total\"], select: [], names_to: \"my_var\", values_to: \"my_value\")\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#pivot_longer/3-examples"},{"type":"function","title":"Grouped examples - Explorer.DataFrame.pivot_longer/3","doc":"In the following example we want to take the Iris dataset and increase the number of rows by\npivoting the \"sepal_length\" column. This dataset is grouped by \"species\", so the resultant\ndataframe is going to keep the \"species\" group:\n\n    iex> df = Explorer.Datasets.iris()\n    iex> grouped = Explorer.DataFrame.group_by(df, \"species\")\n    iex> Explorer.DataFrame.pivot_longer(grouped, [\"sepal_length\"])\n    #Explorer.DataFrame \n\nNow we want to do something different: we want to pivot the \"species\" column that is also a group.\nThis is going to remove the group in the resultant dataframe:\n\n    iex> df = Explorer.Datasets.iris()\n    iex> grouped = Explorer.DataFrame.group_by(df, \"species\")\n    iex> Explorer.DataFrame.pivot_longer(grouped, [\"species\"])\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#pivot_longer/3-grouped-examples"},{"type":"function","title":"Explorer.DataFrame.pivot_wider/4","doc":"Pivot data from long to wide.\n\n`pivot_wider/4` \"widens\" data, increasing the number of columns and decreasing the number of rows.\nThe inverse transformation is `pivot_longer/3`.\n\nIn case the dataframe is using groups, the groups that are also in the list of columns\nto pivot will be removed from the resultant dataframe. See the examples below.","ref":"Explorer.DataFrame.html#pivot_wider/4"},{"type":"function","title":"Options - Explorer.DataFrame.pivot_wider/4","doc":"* `:id_columns` - A set of columns that uniquely identifies each observation.\n\n  Defaults to all columns in data except for the columns specified in `names_from` and `values_from`,\n  and columns that are of the `{:f, 64}` dtype.\n\n  Typically used when you have redundant variables, i.e. variables whose values are perfectly correlated\n  with existing variables. May accept a filter callback, a list or a range of column names.\n  If an empty list is passed, or a range that results in a empty list of column names, it raises an error.\n\n  ID columns cannot be of the float type and any columns of this dtype is discarded.\n  If you need to use float columns as IDs, you must carefully consider rounding\n  or truncating the column and converting it to integer, as long as doing so\n  preserves the properties of the column.\n\n* `:names_prefix` - String added to the start of every variable name.\n  This is particularly useful if `names_from` is a numeric vector and you want to create syntactic variable names.","ref":"Explorer.DataFrame.html#pivot_wider/4-options"},{"type":"function","title":"Examples - Explorer.DataFrame.pivot_wider/4","doc":"Suppose we have a basketball court and multiple teams that want to train in that court. They need\nto share a schedule with the hours each team is going to use it. Here is a dataframe representing\nthat schedule:\n\n    iex> Explorer.DataFrame.new(\n    iex>   weekday: [\"Monday\", \"Tuesday\", \"Wednesday\", \"Thursday\", \"Friday\", \"Monday\", \"Tuesday\", \"Wednesday\", \"Thursday\", \"Friday\"],\n    iex>   team: [\"A\", \"B\", \"C\", \"A\", \"B\", \"C\", \"A\", \"B\", \"C\", \"A\"],\n    iex>   hour: [10, 9, 10, 10, 11, 15, 14, 16, 14, 16]\n    iex> )\n\nThis dataframe is going to look like this - using `print/2`:\n\n      +----------------------------------------------+\n      |  Explorer DataFrame: [rows: 10, columns: 3]  |\n      +---------------+--------------+---------------+\n      |    weekday    |     team     |     hour      |\n      |        |       |       |\n      +===============+==============+===============+\n      | Monday        | A            | 10            |\n      +---------------+--------------+---------------+\n      | Tuesday       | B            | 9             |\n      +---------------+--------------+---------------+\n      | Wednesday     | C            | 10            |\n      +---------------+--------------+---------------+\n      | Thursday      | A            | 10            |\n      +---------------+--------------+---------------+\n      | Friday        | B            | 11            |\n      +---------------+--------------+---------------+\n      | Monday        | C            | 15            |\n      +---------------+--------------+---------------+\n      | Tuesday       | A            | 14            |\n      +---------------+--------------+---------------+\n      | Wednesday     | B            | 16            |\n      +---------------+--------------+---------------+\n      | Thursday      | C            | 14            |\n      +---------------+--------------+---------------+\n      | Friday        | A            | 16            |\n      +---------------+--------------+---------------+\n\nYou can see that the \"weekday\" repeats, and it's not clear how free the agenda is.\nWe can solve that by pivoting the \"weekday\" column in multiple columns, making each weekday\na new column in the resultant dataframe.\n\n    iex> df = Explorer.DataFrame.new(\n    iex>   weekday: [\"Monday\", \"Tuesday\", \"Wednesday\", \"Thursday\", \"Friday\", \"Monday\", \"Tuesday\", \"Wednesday\", \"Thursday\", \"Friday\"],\n    iex>   team: [\"A\", \"B\", \"C\", \"A\", \"B\", \"C\", \"A\", \"B\", \"C\", \"A\"],\n    iex>   hour: [10, 9, 10, 10, 11, 15, 14, 16, 14, 16]\n    iex> )\n    iex> Explorer.DataFrame.pivot_wider(df, \"weekday\", \"hour\")\n    #Explorer.DataFrame \n\nNow if we print that same dataframe with `print/2`, we get a better picture of the schedule:\n\n      +----------------------------------------------------------------------+\n      |              Explorer DataFrame: [rows: 3, columns: 6]               |\n      +----------+-----------+-----------+-----------+-----------+-----------+\n      |   team   |  Monday   |  Tuesday  | Wednesday | Thursday  |  Friday   |\n      |   |   |   |   |   |   |\n      +==========+===========+===========+===========+===========+===========+\n      | A        | 10        | 14        |           | 10        | 16        |\n      +----------+-----------+-----------+-----------+-----------+-----------+\n      | B        |           | 9         | 16        |           | 11        |\n      +----------+-----------+-----------+-----------+-----------+-----------+\n      | C        | 15        |           | 10        | 14        |           |\n      +----------+-----------+-----------+-----------+-----------+-----------+\n\nPivot wider can create unpredictable column names, and sometimes they can conflict with ID columns.\nIn that scenario, we add a number as suffix to duplicated column names. Here is an example:\n\n    iex> df = Explorer.DataFrame.new(\n    iex>   product_id: [1, 1, 1, 1, 2, 2, 2, 2],\n    iex>   property: [\"product_id\", \"width_cm\", \"height_cm\", \"length_cm\", \"product_id\", \"width_cm\", \"height_cm\", \"length_cm\"],\n    iex>   property_value: [1, 42, 40, 64, 2, 35, 20, 40]\n    iex> )\n    iex> Explorer.DataFrame.pivot_wider(df, \"property\", \"property_value\")\n    #Explorer.DataFrame \n\nBut if the option `:names_prefix` is used, that suffix is not added:\n\n    iex> df = Explorer.DataFrame.new(\n    iex>   product_id: [1, 1, 1, 1, 2, 2, 2, 2],\n    iex>   property: [\"product_id\", \"width_cm\", \"height_cm\", \"length_cm\", \"product_id\", \"width_cm\", \"height_cm\", \"length_cm\"],\n    iex>   property_value: [1, 42, 40, 64, 2, 35, 20, 40]\n    iex> )\n    iex> Explorer.DataFrame.pivot_wider(df, \"property\", \"property_value\", names_prefix: \"col_\")\n    #Explorer.DataFrame \n\nMultiple columns are accepted for the `values_from` parameter, but the behaviour is slightly\ndifferent for the naming of new columns in the resultant dataframe. The new columns are going\nto be prefixed by the name of the original value column, followed by the name of the variable.\n\n    iex> df = Explorer.DataFrame.new(\n    iex>   product_id: [1, 1, 1, 1, 2, 2, 2, 2],\n    iex>   property: [\"product_id\", \"width_cm\", \"height_cm\", \"length_cm\", \"product_id\", \"width_cm\", \"height_cm\", \"length_cm\"],\n    iex>   property_value: [1, 42, 40, 64, 2, 35, 20, 40],\n    iex>   another_value: [1, 43, 41, 65, 2, 36, 21, 42]\n    iex> )\n    iex> Explorer.DataFrame.pivot_wider(df, \"property\", [\"property_value\", \"another_value\"])\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#pivot_wider/4-examples"},{"type":"function","title":"Grouped examples - Explorer.DataFrame.pivot_wider/4","doc":"Now using the same idea, we can see that there is not much difference for grouped dataframes.\nThe only detail is that groups that are not ID columns are discarded.\n\n    iex> df = Explorer.DataFrame.new(\n    iex>   weekday: [\"Monday\", \"Tuesday\", \"Wednesday\", \"Thursday\", \"Friday\", \"Monday\", \"Tuesday\", \"Wednesday\", \"Thursday\", \"Friday\"],\n    iex>   team: [\"A\", \"B\", \"C\", \"A\", \"B\", \"C\", \"A\", \"B\", \"C\", \"A\"],\n    iex>   hour: [10, 9, 10, 10, 11, 15, 14, 16, 14, 16]\n    iex> )\n    iex> grouped = Explorer.DataFrame.group_by(df, \"team\")\n    iex> Explorer.DataFrame.pivot_wider(grouped, \"weekday\", \"hour\")\n    #Explorer.DataFrame \n\nIn the following example the group \"weekday\" is going to be removed, because the column is going\nto be pivoted in multiple columns:\n\n    iex> df = Explorer.DataFrame.new(\n    iex>   weekday: [\"Monday\", \"Tuesday\", \"Wednesday\", \"Thursday\", \"Friday\", \"Monday\", \"Tuesday\", \"Wednesday\", \"Thursday\", \"Friday\"],\n    iex>   team: [\"A\", \"B\", \"C\", \"A\", \"B\", \"C\", \"A\", \"B\", \"C\", \"A\"],\n    iex>   hour: [10, 9, 10, 10, 11, 15, 14, 16, 14, 16]\n    iex> )\n    iex> grouped = Explorer.DataFrame.group_by(df, \"weekday\")\n    iex> Explorer.DataFrame.pivot_wider(grouped, \"weekday\", \"hour\")\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#pivot_wider/4-grouped-examples"},{"type":"function","title":"Explorer.DataFrame.print/2","doc":"Prints the DataFrame in a tabular fashion.","ref":"Explorer.DataFrame.html#print/2"},{"type":"function","title":"Examples - Explorer.DataFrame.print/2","doc":"df = Explorer.Datasets.iris()\n    Explorer.DataFrame.print(df)\n    Explorer.DataFrame.print(df, limit: 1)\n    Explorer.DataFrame.print(df, limit: :infinity)","ref":"Explorer.DataFrame.html#print/2-examples"},{"type":"function","title":"Explorer.DataFrame.pull/2","doc":"Extracts a single column as a series.\n\nThis is equivalent to `df[field]` for retrieving a single field.\nThe returned series will have its `:name` field set to the column name.","ref":"Explorer.DataFrame.html#pull/2"},{"type":"function","title":"Examples - Explorer.DataFrame.pull/2","doc":"iex> df = Explorer.Datasets.fossil_fuels()\n    iex> Explorer.DataFrame.pull(df, \"total\")\n    #Explorer.Series \n\n    iex> df = Explorer.Datasets.fossil_fuels()\n    iex> Explorer.DataFrame.pull(df, 2)\n    #Explorer.Series","ref":"Explorer.DataFrame.html#pull/2-examples"},{"type":"function","title":"Explorer.DataFrame.put/4","doc":"Creates or modifies a single column.\n\nThis is a simplified way to add or modify one column,\naccepting a series, tensor, or a list.\n\nIf you are computing a series, it is preferrable to use\n`mutate/2` or `mutate_with/2` to compute the series and\nmodify it in a single step, as it is more powerful and\nit handles both expressions and scalar values accordingly.\n\nIf you are passing tensors or lists, they will be automatically\nconverted to a series. By default, the new series will have the\nsame dtype as the existing series, unless the `:dtype` option\nis given. If there is no existing series, one is inferred from\nthe tensor/list.","ref":"Explorer.DataFrame.html#put/4"},{"type":"function","title":"Examples - Explorer.DataFrame.put/4","doc":"iex> df = Explorer.DataFrame.new(a: [1, 2, 3])\n    iex> Explorer.DataFrame.put(df, :b, Explorer.Series.transform(df[:a], fn n -> n * 2 end))\n    #Explorer.DataFrame \n\n    iex> df = Explorer.DataFrame.new(a: [1, 2, 3])\n    iex> Explorer.DataFrame.put(df, :b, Explorer.Series.from_list([4, 5, 6]))\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#put/4-examples"},{"type":"function","title":"Grouped examples - Explorer.DataFrame.put/4","doc":"If the dataframe is grouped, `put/3` is going to ignore the groups.\nSo the series must be of the same size of the entire dataframe.\n\n    iex> df = Explorer.DataFrame.new(a: [1, 2, 3])\n    iex> grouped = Explorer.DataFrame.group_by(df, \"a\")\n    iex> series = Explorer.Series.from_list([9, 8, 7])\n    iex> Explorer.DataFrame.put(grouped, :b, series)\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#put/4-grouped-examples"},{"type":"function","title":"Tensor examples - Explorer.DataFrame.put/4","doc":"You can also put tensors into the dataframe:\n\n    iex> df = Explorer.DataFrame.new([])\n    iex> Explorer.DataFrame.put(df, :a, Nx.tensor([1, 2, 3]))\n    #Explorer.DataFrame \n\nYou can specify which dtype the tensor represents.\nFor example, a tensor of s64 represents integers\nby default, but it may also represent timestamps\nin microseconds from the Unix epoch:\n\n    iex> df = Explorer.DataFrame.new([])\n    iex> Explorer.DataFrame.put(df, :a, Nx.tensor([1, 2, 3]), dtype: {:naive_datetime, :microsecond})\n    #Explorer.DataFrame \n\nIf there is already a column where we want to place the tensor,\nthe column dtype will be automatically used, this means that\nupdating dataframes in place while preserving their types is\nstraight-forward:\n\n    iex> df = Explorer.DataFrame.new(a: [~N[1970-01-01 00:00:00]])\n    iex> Explorer.DataFrame.put(df, :a, Nx.tensor(529550625987654))\n    #Explorer.DataFrame \n\nThis is particularly useful for categorical columns:\n\n    iex> cat = Explorer.Series.from_list([\"foo\", \"bar\", \"baz\"], dtype: :category)\n    iex> df = Explorer.DataFrame.new(a: cat)\n    iex> Explorer.DataFrame.put(df, :a, Nx.tensor([2, 1, 0]))\n    #Explorer.DataFrame \n\nOn the other hand, if you try to put a floating tensor on\nan integer column, an error will be raised unless a dtype\nor `dtype: :infer` is given:\n\n    iex> df = Explorer.DataFrame.new(a: [1, 2, 3])\n    iex> Explorer.DataFrame.put(df, :a, Nx.tensor(1.0, type: :f64))\n    ** (ArgumentError) dtype {:s, 64} expects a tensor of type {:s, 64} but got type {:f, 64}\n\n    iex> df = Explorer.DataFrame.new(a: [1, 2, 3])\n    iex> Explorer.DataFrame.put(df, :a, Nx.tensor(1.0, type: :f64), dtype: {:f, 64})\n    #Explorer.DataFrame \n\n    iex> df = Explorer.DataFrame.new(a: [1, 2, 3])\n    iex> Explorer.DataFrame.put(df, :a, Nx.tensor(1.0, type: :f64), dtype: :infer)\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#put/4-tensor-examples"},{"type":"function","title":"List examples - Explorer.DataFrame.put/4","doc":"Similar to tensors, we can also put lists in the dataframe:\n\n    iex> df = Explorer.DataFrame.new([])\n    iex> Explorer.DataFrame.put(df, :a, [1, 2, 3])\n    #Explorer.DataFrame \n\nThe same considerations as above apply.","ref":"Explorer.DataFrame.html#put/4-list-examples"},{"type":"function","title":"Explorer.DataFrame.relocate/3","doc":"Relocates columns.\n\nChange column order within a DataFrame. The `before` and `after` options are mutually exclusive.\nProviding no options will relocate the columns to beginning of the DataFrame.","ref":"Explorer.DataFrame.html#relocate/3"},{"type":"function","title":"Options - Explorer.DataFrame.relocate/3","doc":"* `:before` - Specifies to relocate before the given column.\n    It can be an index or a column name.\n\n  * `:after` - Specifies to relocate after the given column.\n    It can be an index or a column name.","ref":"Explorer.DataFrame.html#relocate/3-options"},{"type":"function","title":"Examples - Explorer.DataFrame.relocate/3","doc":"Relocate a single column\n\n    iex> df = Explorer.DataFrame.new(a: [\"a\", \"b\", \"a\"], b: [1, 3, 1], c: [nil, 5, 6])\n    iex> Explorer.DataFrame.relocate(df, \"a\", after: \"c\")\n    #Explorer.DataFrame \n\nRelocate (and reorder) multiple columns to the beginning\n\n    iex> df = Explorer.DataFrame.new(a: [1, 2], b: [5.1, 5.2], c: [4, 5], d: [\"yes\", \"no\"])\n    iex> Explorer.DataFrame.relocate(df, [\"d\", 1], before: 0)\n    #Explorer.DataFrame \n\nRelocate before another column\n\n    iex> df = Explorer.DataFrame.new(a: [1, 2], b: [5.1, 5.2], c: [4, 5], d: [\"yes\", \"no\"])\n    iex> Explorer.DataFrame.relocate(df, [\"a\", \"c\"], before: \"b\")\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#relocate/3-examples"},{"type":"function","title":"Explorer.DataFrame.rename/2","doc":"Renames columns.\n\nRenaming a column that is also a group is going to rename the group as well.\nTo apply a function to a subset of columns, see `rename_with/3`.","ref":"Explorer.DataFrame.html#rename/2"},{"type":"function","title":"Examples - Explorer.DataFrame.rename/2","doc":"You can pass in a list of new names:\n\n    iex> df = Explorer.DataFrame.new(a: [\"a\", \"b\", \"a\"], b: [1, 3, 1])\n    iex> Explorer.DataFrame.rename(df, [\"c\", \"d\"])\n    #Explorer.DataFrame \n\nOr you can rename individual columns using keyword args:\n\n    iex> df = Explorer.DataFrame.new(a: [\"a\", \"b\", \"a\"], b: [1, 3, 1])\n    iex> Explorer.DataFrame.rename(df, a: \"first\")\n    #Explorer.DataFrame \n\nOr you can rename individual columns using a map:\n\n    iex> df = Explorer.DataFrame.new(a: [\"a\", \"b\", \"a\"], b: [1, 3, 1])\n    iex> Explorer.DataFrame.rename(df, %{\"a\" => \"first\"})\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#rename/2-examples"},{"type":"function","title":"Explorer.DataFrame.rename_with/3","doc":"Renames columns with a function.\n\nRenaming a column that is also a group is going to rename the group as well.","ref":"Explorer.DataFrame.html#rename_with/3"},{"type":"function","title":"Examples - Explorer.DataFrame.rename_with/3","doc":"If no columns are specified, it will apply the function to all column names:\n\n    iex> df = Explorer.Datasets.fossil_fuels()\n    iex> Explorer.DataFrame.rename_with(df, &String.upcase/1)\n    #Explorer.DataFrame \n\nA callback can be used to filter the column names that will be renamed, similarly to `select/2`:\n\n    iex> df = Explorer.Datasets.fossil_fuels()\n    iex> Explorer.DataFrame.rename_with(df, &String.ends_with?(&1, \"_fuel\"), &String.trim_trailing(&1, \"_fuel\"))\n    #Explorer.DataFrame \n\nOr you can just pass in the list of column names you'd like to apply the function to:\n\n    iex> df = Explorer.Datasets.fossil_fuels()\n    iex> Explorer.DataFrame.rename_with(df, [\"total\", \"cement\"], &String.upcase/1)\n    #Explorer.DataFrame \n\nRanges, regexes, and functions are also accepted in column names, as in `select/2`.","ref":"Explorer.DataFrame.html#rename_with/3-examples"},{"type":"function","title":"Explorer.DataFrame.sample/3","doc":"Sample rows from a dataframe.\n\nIf given an integer as the second argument, it will return N samples. If given a float, it will\nreturn that proportion of the series.\n\nCan sample with or without replacement.\n\nFor grouped dataframes, sample will take into account the rows of each group, meaning that\nif you try to get N samples and you have G groups, you will get N * G rows. See the examples\nbelow.","ref":"Explorer.DataFrame.html#sample/3"},{"type":"function","title":"Options - Explorer.DataFrame.sample/3","doc":"* `:replace` - If set to `true`, each sample will be independent and therefore\n    values may repeat. Required to be `true` for `n` greater then the number of rows\n    in the dataframe or `frac` > 1.0. (default: `false`)\n\n  * `:seed` - An integer to be used as a random seed. If nil, a random value between 0\n    and 2^64 - 1 will be used. (default: `nil`)\n\n  * `:shuffle` - If set to `true`, the resultant dataframe is going to be shuffle\n    if the sample is equal to the size of the dataframe. (default: `false`)","ref":"Explorer.DataFrame.html#sample/3-options"},{"type":"function","title":"Examples - Explorer.DataFrame.sample/3","doc":"You can sample N rows:\n\n    iex> df = Explorer.Datasets.fossil_fuels()\n    iex> Explorer.DataFrame.sample(df, 3, seed: 100)\n    #Explorer.DataFrame \n\nOr you can sample a proportion of rows:\n\n    iex> df = Explorer.Datasets.fossil_fuels()\n    iex> Explorer.DataFrame.sample(df, 0.03, seed: 100)\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#sample/3-examples"},{"type":"function","title":"Grouped examples - Explorer.DataFrame.sample/3","doc":"In the following example we have the Iris dataset grouped by species, and we want\nto take a sample of two plants from each group. Since we have three species, the\nresultant dataframe is going to have six rows (2 * 3).\n\n    iex> df = Explorer.Datasets.iris()\n    iex> grouped = Explorer.DataFrame.group_by(df, \"species\")\n    iex> Explorer.DataFrame.sample(grouped, 2, seed: 100)\n    #Explorer.DataFrame \n\nThe behaviour is similar when you want to take a fraction of the rows from each group. The main\ndifference is that each group can have more or less rows, depending on its size.\n\n    iex> df = Explorer.Datasets.iris()\n    iex> grouped = Explorer.DataFrame.group_by(df, \"species\")\n    iex> Explorer.DataFrame.sample(grouped, 0.1, seed: 100)\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#sample/3-grouped-examples"},{"type":"function","title":"Explorer.DataFrame.select/2","doc":"Selects a subset of columns by name.\n\nIt's important to notice that groups are kept:\nyou can't select off grouping columns.","ref":"Explorer.DataFrame.html#select/2"},{"type":"function","title":"Examples - Explorer.DataFrame.select/2","doc":"You can select a single column:\n\n    iex> df = Explorer.DataFrame.new(a: [\"a\", \"b\", \"c\"], b: [1, 2, 3])\n    iex> Explorer.DataFrame.select(df, \"a\")\n    #Explorer.DataFrame \n\nOr a list of names:\n\n    iex> df = Explorer.DataFrame.new(a: [\"a\", \"b\", \"c\"], b: [1, 2, 3])\n    iex> Explorer.DataFrame.select(df, [\"a\"])\n    #Explorer.DataFrame \n\nYou can also use a range or a list of integers:\n\n    iex> df = Explorer.DataFrame.new(a: [\"a\", \"b\", \"c\"], b: [1, 2, 3], c: [4, 5, 6])\n    iex> Explorer.DataFrame.select(df, [0, 1])\n    #Explorer.DataFrame \n\n    iex> df = Explorer.DataFrame.new(a: [\"a\", \"b\", \"c\"], b: [1, 2, 3], c: [4, 5, 6])\n    iex> Explorer.DataFrame.select(df, 0..1)\n    #Explorer.DataFrame \n\nOr you can use a callback function that takes the dataframe's names as its first argument:\n\n    iex> df = Explorer.DataFrame.new(a: [\"a\", \"b\", \"c\"], b: [1, 2, 3])\n    iex> Explorer.DataFrame.select(df, &String.starts_with?(&1, \"b\"))\n    #Explorer.DataFrame \n\nOr, if you prefer, a regex:\n\n    iex> df = Explorer.DataFrame.new(a: [\"a\", \"b\", \"c\"], b: [1, 2, 3])\n    iex> Explorer.DataFrame.select(df, ~r/^b$/)\n    #Explorer.DataFrame \n\nOr a callback function that takes names and types:\n\n    iex> df = Explorer.DataFrame.new(a: [\"a\", \"b\", \"c\"], b: [1, 2, 3])\n    iex> Explorer.DataFrame.select(df, fn _name, type -> type == {:s, 64} end)\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#select/2-examples"},{"type":"function","title":"Grouped examples - Explorer.DataFrame.select/2","doc":"Columns that are also groups cannot be removed,\nyou need to ungroup before removing these columns.\n\n    iex> df = Explorer.Datasets.iris()\n    iex> grouped = Explorer.DataFrame.group_by(df, \"species\")\n    iex> Explorer.DataFrame.select(grouped, [\"sepal_width\"])\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#select/2-grouped-examples"},{"type":"function","title":"Explorer.DataFrame.shape/1","doc":"Gets the shape of the dataframe as a `{height, width}` tuple.\n\nThis function works the same way for grouped dataframes, considering the entire\ndataframe in the counting of rows.","ref":"Explorer.DataFrame.html#shape/1"},{"type":"function","title":"Examples - Explorer.DataFrame.shape/1","doc":"iex> df = Explorer.DataFrame.new(floats: [1.0, 2.0, 3.0], ints: [1, 2, 3])\n    iex> Explorer.DataFrame.shape(df)\n    {3, 2}","ref":"Explorer.DataFrame.html#shape/1-examples"},{"type":"function","title":"Explorer.DataFrame.shuffle/2","doc":"Change the order of the rows of a dataframe randomly.\n\nThis function is going to ignore groups.","ref":"Explorer.DataFrame.html#shuffle/2"},{"type":"function","title":"Options - Explorer.DataFrame.shuffle/2","doc":"* `:seed` - An integer to be used as a random seed. If nil, a random value between 0\n    and 2^64 - 1 will be used. (default: `nil`)","ref":"Explorer.DataFrame.html#shuffle/2-options"},{"type":"function","title":"Examples - Explorer.DataFrame.shuffle/2","doc":"iex> df = Explorer.Datasets.fossil_fuels()\n    iex> Explorer.DataFrame.shuffle(df, seed: 100)\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#shuffle/2-examples"},{"type":"function","title":"Explorer.DataFrame.slice/2","doc":"Slices rows at the given indices as a new dataframe.\n\nThe indices may be a list or series of indices, or a range.\nA list of indices does not support negative numbers.\nRanges may be negative on either end, which are then\nnormalized. Note ranges in Elixir are inclusive.\n\nSlice works differently when a dataframe is grouped.\nIt is going to consider the indices of each group\ninstead of the entire dataframe. See the examples below.\n\nIf your intention is to grab a portion of each group,\nprefer to use `sample/3` instead.","ref":"Explorer.DataFrame.html#slice/2"},{"type":"function","title":"Examples - Explorer.DataFrame.slice/2","doc":"iex> df = Explorer.DataFrame.new(a: [1, 2, 3], b: [\"a\", \"b\", \"c\"])\n    iex> Explorer.DataFrame.slice(df, [0, 2])\n    #Explorer.DataFrame \n\nWith a series\n\n    iex> df = Explorer.DataFrame.new(a: [1, 2, 3], b: [\"a\", \"b\", \"c\"])\n    iex> Explorer.DataFrame.slice(df, Explorer.Series.from_list([0, 2]))\n    #Explorer.DataFrame \n\nWith a range:\n\n    iex> df = Explorer.DataFrame.new(a: [1, 2, 3], b: [\"a\", \"b\", \"c\"])\n    iex> Explorer.DataFrame.slice(df, 1..2)\n    #Explorer.DataFrame \n\nWith a range with negative first and last:\n\n    iex> df = Explorer.DataFrame.new(a: [1, 2, 3], b: [\"a\", \"b\", \"c\"])\n    iex> Explorer.DataFrame.slice(df, -2..-1)\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#slice/2-examples"},{"type":"function","title":"Grouped examples - Explorer.DataFrame.slice/2","doc":"We are going to once again use the Iris dataset.\nIn this example we want to take elements at indexes\n0 and 2:\n\n    iex> df = Explorer.Datasets.iris()\n    iex> grouped = Explorer.DataFrame.group_by(df, \"species\")\n    iex> Explorer.DataFrame.slice(grouped, [0, 2])\n    #Explorer.DataFrame \n\nNow we want to take the first 3 rows of each group.\nThis is going to work with the range `0..2`:\n\n    iex> df = Explorer.Datasets.iris()\n    iex> grouped = Explorer.DataFrame.group_by(df, \"species\")\n    iex> Explorer.DataFrame.slice(grouped, 0..2)\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#slice/2-grouped-examples"},{"type":"function","title":"Explorer.DataFrame.slice/3","doc":"Subset a continuous set of rows.","ref":"Explorer.DataFrame.html#slice/3"},{"type":"function","title":"Examples - Explorer.DataFrame.slice/3","doc":"iex> df = Explorer.Datasets.fossil_fuels()\n    iex> Explorer.DataFrame.slice(df, 1, 2)\n    #Explorer.DataFrame \n\nNegative offsets count from the end of the series:\n\n    iex> df = Explorer.Datasets.fossil_fuels()\n    iex> Explorer.DataFrame.slice(df, -10, 2)\n    #Explorer.DataFrame \n\nIf the length would run past the end of the dataframe, the result may be shorter than the length:\n\n    iex> df = Explorer.Datasets.fossil_fuels()\n    iex> Explorer.DataFrame.slice(df, -10, 20)\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#slice/3-examples"},{"type":"function","title":"Grouped examples - Explorer.DataFrame.slice/3","doc":"We want to take the first 3 rows of each group. We need the offset 0 and the length 3:\n\n    iex> df = Explorer.Datasets.iris()\n    iex> grouped = Explorer.DataFrame.group_by(df, \"species\")\n    iex> Explorer.DataFrame.slice(grouped, 0, 3)\n    #Explorer.DataFrame \n\nWe can also pass a negative offset:\n\n    iex> df = Explorer.Datasets.iris()\n    iex> grouped = Explorer.DataFrame.group_by(df, \"species\")\n    iex> Explorer.DataFrame.slice(grouped, -6, 3)\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#slice/3-grouped-examples"},{"type":"macro","title":"Explorer.DataFrame.sort_by/3","doc":"Sorts rows by columns using `Explorer.Query`.\n\n> #### Notice {: .notice}\n>\n> This is a macro. You must `require  Explorer.DataFrame` before using it.\n\nSee `sort_with/2` for a callback version of this function without\n`Explorer.Query`.","ref":"Explorer.DataFrame.html#sort_by/3"},{"type":"macro","title":"Options - Explorer.DataFrame.sort_by/3","doc":"* `:nils` - `:first` or `:last`.\n    By default it is `:last` if direction is `:asc`, and `:first` otherwise.\n\n  * `:parallel` - boolean.\n    Whether to parallelize the sorting.\n    By default it is `true`.\n\n    Parallel sort isn't available on certain lazy operations.\n    In those situations this option is ignored.\n\n  * `:stable` - boolean.\n    Determines if the sorting is stable (ties are guaranteed to maintain their order) or not.\n    Unstable sorting may be more performant.\n    By default it is `false`.","ref":"Explorer.DataFrame.html#sort_by/3-options"},{"type":"macro","title":"Examples - Explorer.DataFrame.sort_by/3","doc":"A single column name will sort ascending by that column:\n\n    iex> df = Explorer.DataFrame.new(a: [\"b\", \"c\", \"a\"], b: [1, 2, 3])\n    iex> Explorer.DataFrame.sort_by(df, a)\n    #Explorer.DataFrame \n\nYou can also sort descending:\n\n    iex> df = Explorer.DataFrame.new(a: [\"b\", \"c\", \"a\"], b: [1, 2, 3])\n    iex> Explorer.DataFrame.sort_by(df, desc: a)\n    #Explorer.DataFrame \n\nYou can specify how `nil`s are sorted:\n\n    iex> df = Explorer.DataFrame.new(a: [\"b\", \"c\", nil, \"a\"])\n    iex> Explorer.DataFrame.sort_by(df, [desc: a], nils: :first)\n    #Explorer.DataFrame \n\nSorting by more than one column sorts them in the order they are entered:\n\n    iex> df = Explorer.Datasets.fossil_fuels()\n    iex> Explorer.DataFrame.sort_by(df, asc: total, desc: country)\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#sort_by/3-examples"},{"type":"macro","title":"Grouped examples - Explorer.DataFrame.sort_by/3","doc":"When used in a grouped dataframe, sort_by is going to sort each group individually and\nthen return the entire dataframe with the existing groups. If one of the sort_by columns\nis also a group, the sorting for that column is not going to work. It is necessary to\nfirst summarise the desired column and then sort_by it.\n\nHere is an example using the Iris dataset. We group by species and then we try to sort\nthe dataframe by species and petal length, but only \"petal length\" is taken into account\nbecause \"species\" is a group.\n\n    iex> df = Explorer.Datasets.iris()\n    iex> grouped = Explorer.DataFrame.group_by(df, \"species\")\n    iex> Explorer.DataFrame.sort_by(grouped, desc: species, asc: sepal_width)\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#sort_by/3-grouped-examples"},{"type":"function","title":"Explorer.DataFrame.sort_with/3","doc":"Sorts rows by columns using a callback function.\n\nThe callback receives a lazy dataframe which stores\noperations instead of values for efficient sorting.\n\nThis is a callback version of `sort_by/2`.","ref":"Explorer.DataFrame.html#sort_with/3"},{"type":"function","title":"Options - Explorer.DataFrame.sort_with/3","doc":"* `:nils` - `:first` or `:last`.\n    By default it is `:last` if direction is `:asc`, and `:first` otherwise.\n\n  * `:parallel` - boolean.\n    Whether to parallelize the sorting.\n    By default it is `true`.\n\n    Parallel sort isn't available on certain lazy operations.\n    In those situations this option is ignored.\n\n  * `:stable` - boolean.\n    Determines if the sorting is stable (ties are guaranteed to maintain their order) or not.\n    Unstable sorting may be more performant.\n    By default it is `false`.","ref":"Explorer.DataFrame.html#sort_with/3-options"},{"type":"function","title":"Examples - Explorer.DataFrame.sort_with/3","doc":"A single column name will sort ascending by that column:\n\n    iex> df = Explorer.DataFrame.new(a: [\"b\", \"c\", \"a\"], b: [1, 2, 3])\n    iex> Explorer.DataFrame.sort_with(df, &(&1[\"a\"]))\n    #Explorer.DataFrame \n\nYou can also sort descending:\n\n    iex> df = Explorer.DataFrame.new(a: [\"b\", \"c\", \"a\"], b: [1, 2, 3])\n    iex> Explorer.DataFrame.sort_with(df, &[desc: &1[\"a\"]])\n    #Explorer.DataFrame \n\nYou can specify how `nil`s are sorted:\n\n    iex> df = Explorer.DataFrame.new(a: [\"b\", \"c\", nil, \"a\"])\n    iex> Explorer.DataFrame.sort_with(df, &[desc: &1[\"a\"]], nils: :first)\n    #Explorer.DataFrame \n\nSorting by more than one column sorts them in the order they are entered:\n\n    iex> df = Explorer.DataFrame.new(a: [3, 1, 3], b: [2, 1, 3])\n    iex> Explorer.DataFrame.sort_with(df, &[desc: &1[\"a\"], asc: &1[\"b\"]])\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#sort_with/3-examples"},{"type":"function","title":"Grouped examples - Explorer.DataFrame.sort_with/3","doc":"iex> df = Explorer.Datasets.iris()\n    iex> grouped = Explorer.DataFrame.group_by(df, \"species\")\n    iex> Explorer.DataFrame.sort_with(grouped, &[desc: &1[\"species\"], asc: &1[\"sepal_width\"]])\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#sort_with/3-grouped-examples"},{"type":"function","title":"Explorer.DataFrame.sql/3","doc":"Create a dataframe from the result of a SQL query on an existing dataframe.\n\n>","ref":"Explorer.DataFrame.html#sql/3"},{"type":"function","title":"SQL Query is Unvalidated {: .warning} - Explorer.DataFrame.sql/3","doc":">\n> Explorer does not validate the SQL query string provided. Instead it passes\n> it directly to the backend. As such, the SQL dialect will be backend\n> dependent and any errors will come directly from the backend.\n\n## `from` Clause\n\nThe `from` clause of the SQL query should reference a chosen table name. The\ndefault name is `\"df\"`. See the examples for a custom table name.","ref":"Explorer.DataFrame.html#sql/3-sql-query-is-unvalidated-warning"},{"type":"function","title":"Examples - Explorer.DataFrame.sql/3","doc":"Basic example:\n\n    iex> df = Explorer.DataFrame.new(a: [1, 2, 3], b: [\"x\", \"y\", \"y\"])\n    iex> Explorer.DataFrame.sql(df, \"select ARRAY_AGG(a), b from df group by b order by b\")\n    #Explorer.DataFrame \n\nCustom table name:\n\n    iex> df = Explorer.DataFrame.new(a: [1, 2, 3])\n    iex> Explorer.DataFrame.sql(df, \"select a + 1 from my_table\", table_name: \"my_table\")\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#sql/3-examples"},{"type":"macro","title":"Explorer.DataFrame.summarise/2","doc":"Summarise each group to a single row using `Explorer.Query`.\n\nTo summarise, you must either perform an aggregation (defined in `Explorer.Series`)\non the desired columns, or select plain columns to retrieve all elements of a group.\n\nThe query is compiled and runs efficiently against the dataframe. This function\nperforms aggregations based on groups and it implicitly ungroups the resultant dataframe.\n\n> #### Notice {: .notice}\n>\n> This is a macro. You must `require  Explorer.DataFrame` before using it.\n\nSee `summarise_with/2` for a callback version of this function without\n`Explorer.Query`.","ref":"Explorer.DataFrame.html#summarise/2"},{"type":"macro","title":"Examples - Explorer.DataFrame.summarise/2","doc":"iex> df = Explorer.DataFrame.new(letters: ~w(a b c d e), is_vowel: [true, false, false, false, true])\n    iex> grouped_df = Explorer.DataFrame.group_by(df, :is_vowel)\n    iex> Explorer.DataFrame.summarise(grouped_df, letters: letters)\n    #Explorer.DataFrame \n\n    iex> df = Explorer.Datasets.fossil_fuels()\n    iex> grouped_df = Explorer.DataFrame.group_by(df, \"year\")\n    iex> Explorer.DataFrame.summarise(grouped_df, total_max: max(total), total_min: min(total))\n    #Explorer.DataFrame \n\nSuppose you want to get the mean sepal width of each Iris species. You could do something\nlike this:\n\n    iex> df = Explorer.Datasets.iris()\n    iex> grouped_df = Explorer.DataFrame.group_by(df, \"species\")\n    iex> Explorer.DataFrame.summarise(grouped_df, mean_sepal_width: round(mean(sepal_width), 3))\n    #Explorer.DataFrame \n\nIn case aggregations for all the dataframe is what you want, you can use ungrouped\ndataframes:\n\n    iex> df = Explorer.Datasets.iris()\n    iex> Explorer.DataFrame.summarise(df, mean_petal_length: round(mean(petal_length), 2))\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#summarise/2-examples"},{"type":"function","title":"Explorer.DataFrame.summarise_with/2","doc":"Summarise each group to a single row using a callback function.\n\nIn case no group is set, the entire dataframe will be considered.\nThe callback receives a lazy dataframe. A lazy dataframe does not\nhold any values, instead it stores all operations in order to\nexecute all summarizations performantly.\n\nThis is a callback version of `summarise/2`.","ref":"Explorer.DataFrame.html#summarise_with/2"},{"type":"function","title":"Examples - Explorer.DataFrame.summarise_with/2","doc":"iex> alias Explorer.{DataFrame, Series}\n    iex> df = Explorer.Datasets.fossil_fuels() |> DataFrame.group_by(\"year\")\n    iex> DataFrame.summarise_with(df, &[total_max: Series.max(&1[\"total\"]), countries: Series.n_distinct(&1[\"country\"])])\n    #Explorer.DataFrame \n\n    iex> alias Explorer.{DataFrame, Series}\n    iex> df = Explorer.Datasets.fossil_fuels()\n    iex> DataFrame.summarise_with(df, &[total_max: Series.max(&1[\"total\"]), countries: Series.n_distinct(&1[\"country\"])])\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#summarise_with/2-examples"},{"type":"function","title":"Explorer.DataFrame.tail/2","doc":"Returns the last *n* rows of the dataframe.\n\nBy default it returns the last 5 rows.\n\nIf the dataframe is using groups, then the last *n* rows of each group is\nreturned.","ref":"Explorer.DataFrame.html#tail/2"},{"type":"function","title":"Examples - Explorer.DataFrame.tail/2","doc":"iex> df = Explorer.Datasets.fossil_fuels()\n    iex> Explorer.DataFrame.tail(df)\n    #Explorer.DataFrame \n\n    iex> df = Explorer.Datasets.fossil_fuels()\n    iex> Explorer.DataFrame.tail(df, 2)\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#tail/2-examples"},{"type":"function","title":"Grouped examples - Explorer.DataFrame.tail/2","doc":"Using grouped dataframes makes `tail/2` return **n rows** from each group.\nHere is an example using the Iris dataset, and returning two rows from each group:\n\n    iex> df = Explorer.Datasets.iris()\n    iex> grouped = Explorer.DataFrame.group_by(df, \"species\")\n    iex> Explorer.DataFrame.tail(grouped, 2)\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#tail/2-grouped-examples"},{"type":"function","title":"Explorer.DataFrame.to_columns/2","doc":"Converts a dataframe to a list of columns with lists as values.\n\nSee `to_series/2` if you want a list of columns with series as values.\nNote that this function does not take into account groups.\n\n> #### Warning {: .warning}\n>\n> This is an expensive operation since it converts series to lists and doing\n> so will copy the whole dataframe. Prefer to use the operations in this and\n> the `Explorer.Series` module rather than the ones in `Enum` whenever possible,\n> as Explorer is optimized for large series.","ref":"Explorer.DataFrame.html#to_columns/2"},{"type":"function","title":"Options - Explorer.DataFrame.to_columns/2","doc":"* `:atom_keys` - Configure if the resultant map should have atom keys. (default: `false`)","ref":"Explorer.DataFrame.html#to_columns/2-options"},{"type":"function","title":"Examples - Explorer.DataFrame.to_columns/2","doc":"iex> df = Explorer.DataFrame.new(ints: [1, nil], floats: [1.0, 2.0])\n    iex> Explorer.DataFrame.to_columns(df)\n    %{\"floats\" => [1.0, 2.0], \"ints\" => [1, nil]}\n\n    iex> df = Explorer.DataFrame.new(floats: [1.0, 2.0], ints: [1, nil])\n    iex> Explorer.DataFrame.to_columns(df, atom_keys: true)\n    %{floats: [1.0, 2.0], ints: [1, nil]}","ref":"Explorer.DataFrame.html#to_columns/2-examples"},{"type":"function","title":"Explorer.DataFrame.to_csv/3","doc":"Writes a dataframe to a delimited file.\n\nGroups are ignored if the dataframe is using any.","ref":"Explorer.DataFrame.html#to_csv/3"},{"type":"function","title":"Options - Explorer.DataFrame.to_csv/3","doc":"* `:header` - Should the column names be written as the first line of the file? (default: `true`)\n\n  * `:delimiter` - A single character used to separate fields within a record. (default: `\",\"`)\n\n  * `:config` - An optional struct, keyword list or map, normally associated with remote\n    file systems. See [IO section](#module-io-operations) for more details. (default: `nil`)\n\n  * `:streaming` - Tells the backend if it should use streaming, which means\n    that the dataframe is not loaded to the memory at once, and instead it is\n    written in chunks from a lazy dataframe.  Defaults to true for local filesystems,\n    ignored on all others.","ref":"Explorer.DataFrame.html#to_csv/3-options"},{"type":"function","title":"Explorer.DataFrame.to_csv!/3","doc":"Similar to `to_csv/3` but raises if there is a problem reading the CSV.","ref":"Explorer.DataFrame.html#to_csv!/3"},{"type":"function","title":"Explorer.DataFrame.to_ipc/3","doc":"Writes a dataframe to an IPC file.\n\nApache IPC is a language-agnostic columnar data structure that can be used to store dataframes.\nIt excels as a format for quickly exchange data between different programming languages.\n\nGroups are ignored if the dataframe is using any.","ref":"Explorer.DataFrame.html#to_ipc/3"},{"type":"function","title":"Options - Explorer.DataFrame.to_ipc/3","doc":"* `:compression` - The compression algorithm to use when writing files.\n    Supported options are:\n\n      * `nil` (uncompressed, default)\n      * `:zstd`\n      * `:lz4`.\n\n  * `:streaming` - Tells the backend if it should use streaming, which means\n    that the dataframe is not loaded to the memory at once, and instead it is\n    written in chunks from a lazy dataframe.\n\n    This option has no effect on eager - the default - dataframes.\n    It defaults to `true`.\n\n  * `:config` - An optional struct, keyword list or map, normally associated with remote\n    file systems. See [IO section](#module-io-operations) for more details. (default: `nil`)","ref":"Explorer.DataFrame.html#to_ipc/3-options"},{"type":"function","title":"Explorer.DataFrame.to_ipc!/3","doc":"Similar to `to_ipc/3`, but raises in case of error.","ref":"Explorer.DataFrame.html#to_ipc!/3"},{"type":"function","title":"Explorer.DataFrame.to_ipc_stream/3","doc":"Writes a dataframe to an IPC Stream file.\n\nArrow IPC Streams provide a streaming protocol or “format\" for sending an arbitrary\nlength sequence of record batches.\nThe format must be processed from start to end, and does not support random access.","ref":"Explorer.DataFrame.html#to_ipc_stream/3"},{"type":"function","title":"Options - Explorer.DataFrame.to_ipc_stream/3","doc":"* `:compression` - The compression algorithm to use when writing files.\n    Supported options are:\n\n      * `nil` (uncompressed, default)\n      * `:zstd`\n      * `:lz4`.\n\n  * `:config` - An optional struct, keyword list or map, normally associated with remote\n    file systems. See [IO section](#module-io-operations) for more details. (default: `nil`)","ref":"Explorer.DataFrame.html#to_ipc_stream/3-options"},{"type":"function","title":"Explorer.DataFrame.to_ipc_stream!/3","doc":"Similar to `to_ipc_stream/3`, but raises in case of error.","ref":"Explorer.DataFrame.html#to_ipc_stream!/3"},{"type":"function","title":"Explorer.DataFrame.to_ndjson/3","doc":"Writes a dataframe to a ndjson file.\n\nGroups are ignored if the dataframe is using any.\n\nNDJSON are files that contains JSON files separated by new lines.\nThey are often used as structured logs.","ref":"Explorer.DataFrame.html#to_ndjson/3"},{"type":"function","title":"Options - Explorer.DataFrame.to_ndjson/3","doc":"* `:config` - An optional struct, keyword list or map, normally associated with remote\n    file systems. See [IO section](#module-io-operations) for more details. (default: `nil`)","ref":"Explorer.DataFrame.html#to_ndjson/3-options"},{"type":"function","title":"Explorer.DataFrame.to_ndjson!/3","doc":"Similar to `to_ndjson/3`, but raises in case of error.","ref":"Explorer.DataFrame.html#to_ndjson!/3"},{"type":"function","title":"Explorer.DataFrame.to_parquet/3","doc":"Writes a dataframe to a parquet file.\n\nGroups are ignored if the dataframe is using any.","ref":"Explorer.DataFrame.html#to_parquet/3"},{"type":"function","title":"Options - Explorer.DataFrame.to_parquet/3","doc":"* `:compression` - The compression algorithm to use when writing files.\n    Where a compression level is available, this can be passed as a tuple,\n    such as `{:zstd, 3}`. Supported options are:\n\n      * `nil` (uncompressed, default)\n      * `:snappy`\n      * `:gzip` (with levels 1-9)\n      * `:brotli` (with levels 1-11)\n      * `:zstd` (with levels -7-22)\n      * `:lz4raw`.\n\n  * `:streaming` - Tells the backend if it should use streaming, which means\n    that the dataframe is not loaded to the memory at once, and instead it is\n    written in chunks from a lazy dataframe.\n\n    This option has no effect on eager - the default - dataframes.\n    It defaults to `true`.\n\n  * `:config` - An optional struct, keyword list or map, normally associated with remote\n    file systems. See [IO section](#module-io-operations) for more details. (default: `nil`)","ref":"Explorer.DataFrame.html#to_parquet/3-options"},{"type":"function","title":"Explorer.DataFrame.to_parquet!/3","doc":"Similar to `to_parquet/3`, but raises in case of error.","ref":"Explorer.DataFrame.html#to_parquet!/3"},{"type":"function","title":"Explorer.DataFrame.to_rows/2","doc":"Converts a dataframe to a list of maps (rows).\n\n> #### Warning {: .warning}\n>\n> This is an expensive operation since data is stored in a columnar format.\n> You must avoid converting a dataframe to rows, as that will transform and\n> copy the whole dataframe in memory. Prefer to use the operations in this\n> module rather than the ones in `Enum` whenever possible, as this module is\n> optimized for large series.","ref":"Explorer.DataFrame.html#to_rows/2"},{"type":"function","title":"Options - Explorer.DataFrame.to_rows/2","doc":"* `:atom_keys` - Configure if the resultant maps should have atom keys. (default: `false`)","ref":"Explorer.DataFrame.html#to_rows/2-options"},{"type":"function","title":"Examples - Explorer.DataFrame.to_rows/2","doc":"iex> df = Explorer.DataFrame.new(floats: [1.0, 2.0], ints: [1, nil])\n    iex> Explorer.DataFrame.to_rows(df)\n    [%{\"floats\" => 1.0, \"ints\" => 1}, %{\"floats\" => 2.0 ,\"ints\" => nil}]\n\n    iex> df = Explorer.DataFrame.new(floats: [1.0, 2.0], ints: [1, nil])\n    iex> Explorer.DataFrame.to_rows(df, atom_keys: true)\n    [%{floats: 1.0, ints: 1}, %{floats: 2.0, ints: nil}]","ref":"Explorer.DataFrame.html#to_rows/2-examples"},{"type":"function","title":"Explorer.DataFrame.to_rows_stream/2","doc":"Converts a dataframe to a stream of maps (rows).\n\n> #### Warning {: .warning}\n>\n> This is an expensive operation since data is stored in a columnar format.\n> Prefer to use the operations in this module rather than the ones in `Enum`\n> whenever possible, as this module is optimized for large series.","ref":"Explorer.DataFrame.html#to_rows_stream/2"},{"type":"function","title":"Options - Explorer.DataFrame.to_rows_stream/2","doc":"* `:atom_keys` - Configure if the resultant maps should have atom keys. (default: `false`)\n  * `:chunk_size` - Number of rows passed to `to_rows/2` while streaming over the data. (default: `1000`)","ref":"Explorer.DataFrame.html#to_rows_stream/2-options"},{"type":"function","title":"Examples - Explorer.DataFrame.to_rows_stream/2","doc":"iex> df = Explorer.DataFrame.new(floats: [1.0, 2.0], ints: [1, nil])\n    iex> Explorer.DataFrame.to_rows_stream(df) |> Enum.map(& &1)\n    [%{\"floats\" => 1.0, \"ints\" => 1}, %{\"floats\" => 2.0 ,\"ints\" => nil}]\n\n    iex> df = Explorer.DataFrame.new(floats: [1.0, 2.0], ints: [1, nil])\n    iex> Explorer.DataFrame.to_rows_stream(df, atom_keys: true) |> Enum.map(& &1)\n    [%{floats: 1.0, ints: 1}, %{floats: 2.0, ints: nil}]","ref":"Explorer.DataFrame.html#to_rows_stream/2-examples"},{"type":"function","title":"Explorer.DataFrame.to_series/2","doc":"Converts a dataframe to a list of columns with series as values.\n\nSee `to_columns/2` if you want a list of columns with lists as values.\nNote that this function does not take into account groups.","ref":"Explorer.DataFrame.html#to_series/2"},{"type":"function","title":"Options - Explorer.DataFrame.to_series/2","doc":"* `:atom_keys` - Configure if the resultant map should have atom keys. (default: `false`)","ref":"Explorer.DataFrame.html#to_series/2-options"},{"type":"function","title":"Examples - Explorer.DataFrame.to_series/2","doc":"iex> df = Explorer.DataFrame.new(ints: [1, nil], floats: [1.0, 2.0])\n    iex> map = Explorer.DataFrame.to_series(df)\n    iex> Explorer.Series.to_list(map[\"floats\"])\n    [1.0, 2.0]\n    iex> Explorer.Series.to_list(map[\"ints\"])\n    [1, nil]","ref":"Explorer.DataFrame.html#to_series/2-examples"},{"type":"function","title":"Explorer.DataFrame.transform/3","doc":"Make new columns by applying a native Elixir function to each row.\n\nThe native Elixir function should take a map (an element from\n`to_rows_stream/2`) as input and return a map where the keys are the desired\nnew columns and the values are the values for that row.\n\nSee also the version of this function for a single series:\n\n  * `Explorer.Series.transform/2`\n\n>","ref":"Explorer.DataFrame.html#transform/3"},{"type":"function","title":"Warning {: .warning} - Explorer.DataFrame.transform/3","doc":">\n> This is an expensive operation since data is stored in a columnar format.\n> You should avoid converting a dataframe to rows where possible as it\n> will incur significant overhead. Prefer instead to use the operations in\n> this module rather than native Elixir ones. This function should only be\n> used as a fallback when no equivalent DataFrame operation is available.","ref":"Explorer.DataFrame.html#transform/3-warning-warning"},{"type":"function","title":"Options - Explorer.DataFrame.transform/3","doc":"* `:names` - Column names to select for serializing.\n    By default all columns will be returned.\n\nAlso, all options which are valid for `to_rows_stream` are valid here.\n\n>","ref":"Explorer.DataFrame.html#transform/3-options"},{"type":"function","title":"The `:names` field is recommended {: .info} - Explorer.DataFrame.transform/3","doc":">\n> It's recommended that you provide the `:names` option and to set it to only\n> those column names needed by your function. Otherwise, you'll be serializing\n> unnecessary information.","ref":"Explorer.DataFrame.html#transform/3-the-names-field-is-recommended-info"},{"type":"function","title":"Examples - Explorer.DataFrame.transform/3","doc":"Pre-selecting with `:names`:\n\n    iex> alias Explorer.DataFrame, as: DF\n    iex> df = DF.new(\n    ...>   numbers: [1, 2],\n    ...>   datetime_local: [~N[2024-01-01 00:00:00], ~N[2024-01-01 00:00:00]],\n    ...>   timezone: [\"Etc/UTC\", \"America/New_York\"]\n    ...> )\n    iex> DF.transform(df, [names: [\"datetime_local\", \"timezone\"]], fn row ->\n    ...>   datetime_utc =\n    ...>     row[\"datetime_local\"]\n    ...>     |> DateTime.from_naive!(row[\"timezone\"])\n    ...>     |> DateTime.shift_zone!(\"Etc/UTC\")\n    ...>\n    ...>   %{datetime_utc: datetime_utc}\n    ...> end)\n    #Explorer.DataFrame \n\nConverting to atoms with `:atom_keys`:\n\n    iex> alias Explorer.DataFrame, as: DF\n    iex> df = DF.new(\n    ...>   major: [1, 2, 3],\n    ...>   minor: [11, 12, 13],\n    ...>   patch: [0, 0, 0]\n    ...> )\n    iex> DF.transform(df, [atom_keys: true], fn row ->\n    ...>   version = Version.parse!(\"#{row.major}.#{row.minor}.#{row.patch}\")\n    ...>   %{meets_requirements: Version.match?(version, \"> 2.0.0\")}\n    ...> end)\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#transform/3-examples"},{"type":"function","title":"Explorer.DataFrame.transpose/2","doc":"Transpose a DataFrame.\n\n> #### Warning {: .warning}\n>\n> This is an expensive operation since data is stored in a columnar format.","ref":"Explorer.DataFrame.html#transpose/2"},{"type":"function","title":"Options - Explorer.DataFrame.transpose/2","doc":"* `:header` - When a string is passed, name of the header column is set to the value. When `header` is `true`, name is set to `column` (default: `false`)\n  * `:columns` - names for non header columns. Length of column_names should match the row count of data frame. (default: `nil`)","ref":"Explorer.DataFrame.html#transpose/2-options"},{"type":"function","title":"Examples - Explorer.DataFrame.transpose/2","doc":"iex> df = Explorer.DataFrame.new(a: [\"d\", nil], b: [1, 2], c: [\"a\", \"b\"])\n    iex> Explorer.DataFrame.transpose(df)\n    #Explorer.DataFrame \n\n    iex> df = Explorer.DataFrame.new(a: [\"d\", nil], b: [1, 2], c: [\"a\", \"b\"])\n    iex> Explorer.DataFrame.transpose(df, columns: [\"x\", \"y\"])\n    #Explorer.DataFrame \n\n    iex> df = Explorer.DataFrame.new(a: [\"d\", nil], b: [1, 2], c: [\"a\", \"b\"])\n    iex> Explorer.DataFrame.transpose(df, header: \"name\")\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#transpose/2-examples"},{"type":"function","title":"Explorer.DataFrame.ungroup/2","doc":"Removes grouping variables.\n\nAccepts a list of group names. If groups is not specified, then all groups are\nremoved.","ref":"Explorer.DataFrame.html#ungroup/2"},{"type":"function","title":"Examples - Explorer.DataFrame.ungroup/2","doc":"Ungroups all by default:\n\n    iex> df = Explorer.Datasets.fossil_fuels()\n    iex> df = Explorer.DataFrame.group_by(df, [\"country\", \"year\"])\n    iex> Explorer.DataFrame.ungroup(df)\n    #Explorer.DataFrame \n\nUngrouping a single column:\n\n    iex> df = Explorer.Datasets.fossil_fuels()\n    iex> df = Explorer.DataFrame.group_by(df, [\"country\", \"year\"])\n    iex> Explorer.DataFrame.ungroup(df, \"country\")\n    #Explorer.DataFrame \n\nLists, ranges, regexes, and functions are also accepted in column names, as in `select/2`.","ref":"Explorer.DataFrame.html#ungroup/2-examples"},{"type":"function","title":"Explorer.DataFrame.unnest/2","doc":"Unnests one or multiple struct columns into individual columns.\n\nThe field names in a unnested column must not exist in the dataframe\nor in any other unnested columns.","ref":"Explorer.DataFrame.html#unnest/2"},{"type":"function","title":"Examples - Explorer.DataFrame.unnest/2","doc":"iex> df = Explorer.DataFrame.new(before: [1, 2], struct: [%{x: 1, y: 2}, %{x: 3, y: 4}], after: [3, 4])\n    iex> Explorer.DataFrame.unnest(df, :struct)\n    #Explorer.DataFrame \n\n    iex> df = Explorer.DataFrame.new(struct1: [%{x: 1, y: 2}, %{x: 3, y: 4}], struct2: [%{z: 5}, %{z: 6}])\n    iex> Explorer.DataFrame.unnest(df, [:struct1, :struct2])\n    #Explorer.DataFrame","ref":"Explorer.DataFrame.html#unnest/2-examples"},{"type":"type","title":"Explorer.DataFrame.column/0","doc":"Represents a column name or its index.","ref":"Explorer.DataFrame.html#t:column/0"},{"type":"type","title":"Explorer.DataFrame.column_name/0","doc":"Represents a column name as atom or string.","ref":"Explorer.DataFrame.html#t:column_name/0"},{"type":"type","title":"Explorer.DataFrame.column_names/0","doc":"Represents multiple column names as atoms or strings.","ref":"Explorer.DataFrame.html#t:column_names/0"},{"type":"type","title":"Explorer.DataFrame.column_pairs/1","doc":"Represents a column pair where the value is a column name or\na column index, and the value is of type `value`.","ref":"Explorer.DataFrame.html#t:column_pairs/1"},{"type":"type","title":"Explorer.DataFrame.columns/0","doc":"Represents multiple columns.\n\nThe columns may be specified as one of:\n\n  * a list of columns indexes or names as atoms and strings\n\n  * a range\n\n  * a regex that keeps only the names matching the regex\n\n  * a one-arity function that receives column names and returns\n    true for column names to keep\n\n  * a two-arity function that receives column names and types and\n    returns true for column names to keep","ref":"Explorer.DataFrame.html#t:columns/0"},{"type":"type","title":"Explorer.DataFrame.fs_entry/0","doc":"Represents a filesystem entry, that can be local, S3 or URL.","ref":"Explorer.DataFrame.html#t:fs_entry/0"},{"type":"type","title":"Explorer.DataFrame.t/0","doc":"Represents a dataframe.","ref":"Explorer.DataFrame.html#t:t/0"},{"type":"module","title":"Explorer.Datasets","doc":"Datasets used in examples and exploration.\n\nNote those datasets are not available inside Elixir releases\n(see `mix release`), which is the usual way to deploy Elixir\nin production. Therefore, if you need one of those datasets\nin production, you must download the source files to your\nown application `priv` directory and load them yourself.\nFor example:\n\n    Explorer.DataFrame.from_csv!(Application.app_dir(:my_app, \"priv/iris.csv\"))","ref":"Explorer.Datasets.html"},{"type":"function","title":"Explorer.Datasets.fossil_fuels/0","doc":"CO2 emissions from fossil fuels since 2010, by country","ref":"Explorer.Datasets.html#fossil_fuels/0"},{"type":"function","title":"Citation - Explorer.Datasets.fossil_fuels/0","doc":"Boden, T.A., G. Marland, and R.J. Andres. 2013. Global, Regional, and National Fossil-Fuel CO2\n    Emissions. Carbon Dioxide Information Analysis Center, Oak Ridge National Laboratory, U.S.\n    Department of Energy, Oak Ridge, Tenn., U.S.A. doi 10.3334/CDIAC/00001_V2013","ref":"Explorer.Datasets.html#fossil_fuels/0-citation"},{"type":"function","title":"Explorer.Datasets.iris/0","doc":"Iris Dataset.\n\nThis classic dataset was collected by Edgar Anderson in 1936\nand made famous by R. A. Fisher's 1936 paper. It consists of\nseveral measurements of three species of Iris (Iris setosa,\nIris virginica and Iris versicolor).\n\nDownloaded and modified from: https://archive.ics.uci.edu/ml/machine-learning-databases/iris/iris.data","ref":"Explorer.Datasets.html#iris/0"},{"type":"function","title":"Citation - Explorer.Datasets.iris/0","doc":"Fisher,R. A.. (1988). Iris. UCI Machine Learning Repository. https://doi.org/10.24432/C56C76.","ref":"Explorer.Datasets.html#iris/0-citation"},{"type":"function","title":"Explorer.Datasets.wine/0","doc":"Wine Dataset.\n\nThe data is the result of a chemical analysis of wines grown in the same\nregion in Italy but derived from three different cultivars. The analysis\ndetermined the quantities of 13 constituents found in each of the three\ntypes of wines.\n\nDownloaded and modified from: https://archive.ics.uci.edu/ml/machine-learning-databases/wine/wine.data","ref":"Explorer.Datasets.html#wine/0"},{"type":"function","title":"Citation - Explorer.Datasets.wine/0","doc":"Aeberhard,Stefan and Forina,M.. (1991). Wine. UCI Machine Learning Repository. https://doi.org/10.24432/C5PC7J.","ref":"Explorer.Datasets.html#wine/0-citation"},{"type":"module","title":"Explorer.Query","doc":"High-level query for Explorer.\n\n> #### Explorer.DataFrame vs DF {: .tip}\n>\n> All examples below assume you have defined aliased\n> `Explorer.DataFrame` to `DF` as shown below:\n>\n>     require Explorer.DataFrame, as: DF\n>\n\nQueries convert regular Elixir code which compile to efficient\ndataframes operations. Inside a query, only the limited set of\nSeries operations are available and identifiers, such as `strs`\nand `nums`, represent dataframe column names:\n\n    iex> df = DF.new(strs: [\"a\", \"b\", \"c\"], nums: [1, 2, 3])\n    iex> DF.filter(df, nums > 2)\n    #Explorer.DataFrame \n\nIf a column has unusual format, you can either rename it before-hand,\nor use `col/1` inside queries:\n\n    iex> df = DF.new(\"unusual nums\": [1, 2, 3])\n    iex> DF.filter(df, col(\"unusual nums\") > 2)\n    #Explorer.DataFrame \n\nAll operations from `Explorer.Series` are imported inside queries.\nThis module also provides operators to use in queries, which are\nalso imported into queries.","ref":"Explorer.Query.html"},{"type":"module","title":"Supported operations - Explorer.Query","doc":"Queries are supported in the following operations:\n\n  * `Explorer.DataFrame.sort_by/2`\n  * `Explorer.DataFrame.filter/2`\n  * `Explorer.DataFrame.mutate/2`\n  * `Explorer.DataFrame.summarise/2`","ref":"Explorer.Query.html#module-supported-operations"},{"type":"module","title":"Interpolation - Explorer.Query","doc":"If you want to access variables defined outside of the query\nor get access to all Elixir constructs, you must use `^`:\n\n    iex> min = 2\n    iex> df = DF.new(strs: [\"a\", \"b\", \"c\"], nums: [1, 2, 3])\n    iex> DF.filter(df, nums > ^min)\n    #Explorer.DataFrame \n\n    iex> min = 2\n    iex> df = DF.new(strs: [\"a\", \"b\", \"c\"], nums: [1, 2, 3])\n    iex> DF.filter(df, nums < ^if(min > 0, do: 10, else: -10))\n    #Explorer.DataFrame \n\n`^` can be used with `col` to access columns dynamically:\n\n    iex> df = DF.new(\"unusual nums\": [1, 2, 3])\n    iex> name = \"unusual nums\"\n    iex> DF.filter(df, col(^name) > 2)\n    #Explorer.DataFrame","ref":"Explorer.Query.html#module-interpolation"},{"type":"module","title":"Conditionals - Explorer.Query","doc":"Queries support both `if/2` and `unless/2` operations inside queries.\n\n`cond/1` can be used to write multi-clause conditions:\n\n    iex> df = DF.new(a: [10, 4, 6])\n    iex> DF.mutate(df,\n    ...>   b:\n    ...>     cond do\n    ...>       a > 9 -> \"Exceptional\"\n    ...>       a > 5 -> \"Passed\"\n    ...>       true -> \"Failed\"\n    ...>     end\n    ...> )\n    #Explorer.DataFrame","ref":"Explorer.Query.html#module-conditionals"},{"type":"module","title":"Across and comprehensions - Explorer.Query","doc":"`Explorer.Query` leverages the power behind Elixir for-comprehensions\nto provide a powerful syntax for traversing several columns in a dataframe\nat once. For example, imagine you want to standardize the data on the\niris dataset, you could write this:\n\n    iex> iris = Explorer.Datasets.iris()\n    iex> DF.mutate(iris,\n    ...>   sepal_width: (sepal_width - mean(sepal_width)) / variance(sepal_width),\n    ...>   sepal_length: (sepal_length - mean(sepal_length)) / variance(sepal_length),\n    ...>   petal_length: (petal_length - mean(petal_length)) / variance(petal_length),\n    ...>   petal_width: (petal_width - mean(petal_width)) / variance(petal_width)\n    ...> )\n    #Explorer.DataFrame \n\nWhile the code above does its job, it is quite repetitive. With across and for-comprehensions,\nwe could instead write:\n\n    iex> iris = Explorer.Datasets.iris()\n    iex> DF.mutate(iris,\n    ...>   for col <- across([\"sepal_width\", \"sepal_length\", \"petal_length\", \"petal_width\"]) do\n    ...>     {col.name, (col - mean(col)) / variance(col)}\n    ...>   end\n    ...> )\n    #Explorer.DataFrame \n\nWhich achieves the same result in a more concise and maintainable way.\n`across/1` may receive any of the following input as arguments:\n\n  * a list of columns indexes or names as atoms and strings\n\n  * a range\n\n  * a regex that keeps only the names matching the regex\n\nFor example, since we know the width and length columns are the first four,\nwe could also have written (remember ranges in Elixir are inclusive):\n\n    DF.mutate(iris,\n      for col <- across(0..3) do\n        {col.name, (col - mean(col)) / variance(col)}\n      end\n    )\n\nOr using a regex:\n\n    DF.mutate(iris,\n      for col <- across(~r/(sepal|petal)_(length|width)/) do\n        {col.name, (col - mean(col)) / variance(col)}\n      end\n    )\n\nFor those new to Elixir, for-comprehensions have the following format:\n\n    for PATTERN <- GENERATOR, FILTER do\n      EXPR\n    end\n\nA comprehension filter is a mechanism that allows us to keep only columns\nbased on additional properties, such as its `dtype`. A for-comprehension can\nhave multiple generators and filters. For instance, if you want to apply\nstandardization to all float columns, we can use `across/0` to access all\ncolumns and then use a filter to keep only the float ones:\n\n    iex> iris = Explorer.Datasets.iris()\n    iex> DF.mutate(iris,\n    ...>   for col <- across(), col.dtype == {:f, 64} do\n    ...>     {col.name, (col - mean(col)) / variance(col)}\n    ...>   end\n    ...> )\n    #Explorer.DataFrame \n\nFor-comprehensions works with all dataframe verbs. As we have seen\nabove, for mutations we must return tuples as pair with the mutation\nname and its value. `summarise` works similarly. Note in both cases\nthe name could also be generated dynamically. For example, to compute\nthe mean per species, you could write:\n\n    iex> Explorer.Datasets.iris()\n    ...> |> DF.group_by(\"species\")\n    ...> |> DF.summarise(\n    ...>   for col <- across(), col.dtype == {:f, 64} do\n    ...>     {\"#{col.name}_mean\", round(mean(col), 3)}\n    ...>   end\n    ...> )\n    #Explorer.DataFrame \n\n`sort_by` expects a list of columns to sort by, while for-comprehensions\nin `filter` generate a list of conditions, which are joined using `and`.\nFor example, to filter all entries have both sepal and petal length above\naverage, using a filter on the column name, one could write:\n\n    iex> iris = Explorer.Datasets.iris()\n    iex> DF.filter(iris,\n    ...>   for col <- across(), String.ends_with?(col.name, \"_length\") do\n    ...>     col > mean(col)\n    ...>   end\n    ...> )\n    #Explorer.DataFrame \n\n> #### Do not mix comprehension and queries {: .warning}\n>\n> The filter inside a for-comprehension works at the meta level:\n> it can only filter columns based on their names and dtypes, but\n> not on their values. For example, this code does not make any\n> sense and it will fail to compile:\n>\n>     |> DF.filter(\n>       for col <- across(), col > mean(col) do\n>         col\n>       end\n>     end)\n>\n> Another way to think about it, the comprehensions traverse on the\n> columns themselves, the contents inside the comprehension do-block\n> traverse on the values inside the columns.","ref":"Explorer.Query.html#module-across-and-comprehensions"},{"type":"module","title":"Implementation details - Explorer.Query","doc":"Queries simply become lazy dataframe operations at runtime.\nFor example, the following query\n\n    Explorer.DataFrame.filter(df, nums > 2)\n\nis equivalent to\n\n    Explorer.DataFrame.filter_with(df, fn df -> Explorer.Series.greater(df[\"nums\"], 2) end)\n\nThis means that, whenever you want to generate queries programatically,\nyou can fallback to the regular `_with` APIs.","ref":"Explorer.Query.html#module-implementation-details"},{"type":"function","title":"Explorer.Query.**/2","doc":"Delegate to `Explorer.Series.pow/2`.","ref":"Explorer.Query.html#**/2"},{"type":"function","title":"Explorer.Query.*/2","doc":"Delegate to `Explorer.Series.multiply/2`.","ref":"Explorer.Query.html#*/2"},{"type":"function","title":"Explorer.Query.+/1","doc":"Unary plus operator.\n\nWorks with numbers and series.","ref":"Explorer.Query.html#+/1"},{"type":"function","title":"Explorer.Query.+/2","doc":"Delegate to `Explorer.Series.add/2`.","ref":"Explorer.Query.html#+/2"},{"type":"function","title":"Explorer.Query.-/1","doc":"Unary minus operator.\n\nWorks with numbers and series.","ref":"Explorer.Query.html#-/1"},{"type":"function","title":"Explorer.Query.-/2","doc":"Delegate to `Explorer.Series.subtract/2`.","ref":"Explorer.Query.html#-/2"},{"type":"function","title":"Explorer.Query.//2","doc":"Delegate to `Explorer.Series.divide/2`.","ref":"Explorer.Query.html#//2"},{"type":"function","title":"Explorer.Query.!=/2","doc":"Delegate to `Explorer.Series.not_equal/2`.","ref":"Explorer.Query.html#!=/2"},{"type":"function","title":"Explorer.Query.</2","doc":"Delegate to `Explorer.Series.less/2`.","ref":"Explorer.Query.html#%3C/2"},{"type":"function","title":"Explorer.Query.<=/2","doc":"Delegate to `Explorer.Series.less_equal/2`.","ref":"Explorer.Query.html#%3C=/2"},{"type":"macro","title":"Explorer.Query.<>/2","doc":"String concatenation operator.\n\nWorks with strings and series of strings.","ref":"Explorer.Query.html#%3C%3E/2"},{"type":"macro","title":"Examples - Explorer.Query.<>/2","doc":"DF.mutate(df, name: first_name <> \" \" <> last_name)\n\nIf you want to convert concatenate non-string\nseries, you can explicitly cast them to string\nbefore:\n\n    DF.mutate(df, name: cast(year, :string) <> \"-\" <> cast(month, :string))\n\nOr use format:\n\n    DF.mutate(df, name: format([year, \"-\", month]))","ref":"Explorer.Query.html#%3C%3E/2-examples"},{"type":"function","title":"Explorer.Query.==/2","doc":"Delegate to `Explorer.Series.equal/2`.","ref":"Explorer.Query.html#==/2"},{"type":"function","title":"Explorer.Query.>/2","doc":"Delegate to `Explorer.Series.greater/2`.","ref":"Explorer.Query.html#%3E/2"},{"type":"function","title":"Explorer.Query.>=/2","doc":"Delegate to `Explorer.Series.greater_equal/2`.","ref":"Explorer.Query.html#%3E=/2"},{"type":"macro","title":"Explorer.Query.across/0","doc":"Accesses all columns in the dataframe.\n\nThis is the equivalent to `across(..)`.\n\nSee the module docs for more information.","ref":"Explorer.Query.html#across/0"},{"type":"macro","title":"Explorer.Query.across/1","doc":"Accesses the columns given by `selector` in the dataframe.\n\n`across/1` is used as the generator inside for-comprehensions.\n\nSee the module docs for more information.","ref":"Explorer.Query.html#across/1"},{"type":"function","title":"Explorer.Query.and/2","doc":"Binary and operator.\n\nWorks with boolean and series.","ref":"Explorer.Query.html#and/2"},{"type":"macro","title":"Explorer.Query.col/1","doc":"Accesses a column by name.\n\nIf your column name contains whitespace or start with\nuppercase letters, you can still access its name by\nusing this macro:\n\n    iex> df = Explorer.DataFrame.new(\"unusual nums\": [1, 2, 3])\n    iex> Explorer.DataFrame.filter(df, col(\"unusual nums\") > 2)\n    #Explorer.DataFrame \n\n`name` must be an atom, a string, or an integer.\nIt is equivalent to `df[name]` but inside a query.\n\nThis can also be used if you want to access a column\nprogrammatically, for example:\n\n    iex> df = Explorer.DataFrame.new(nums: [1, 2, 3])\n    iex> name = :nums\n    iex> Explorer.DataFrame.filter(df, col(^name) > 2)\n    #Explorer.DataFrame \n\nFor traversing multiple columns programmatically,\nsee `across/0` and `across/1`.","ref":"Explorer.Query.html#col/1"},{"type":"function","title":"Explorer.Query.if/2","doc":"Provides `if/2` conditionals inside queries.","ref":"Explorer.Query.html#if/2"},{"type":"function","title":"Explorer.Query.not/1","doc":"Unary not operator.\n\nWorks with boolean and series.","ref":"Explorer.Query.html#not/1"},{"type":"function","title":"Explorer.Query.or/2","doc":"Binary or operator.\n\nWorks with boolean and series.","ref":"Explorer.Query.html#or/2"},{"type":"macro","title":"Explorer.Query.query/1","doc":"Builds an anonymous function from a query.\n\nThis is the entry point used by `Explorer.DataFrame.filter/2`\nand friends to convert queries into anonymous functions.\nSee the moduledoc for more information.","ref":"Explorer.Query.html#query/1"},{"type":"function","title":"Explorer.Query.unless/2","doc":"Provides `unless/2` conditionals inside queries.","ref":"Explorer.Query.html#unless/2"},{"type":"module","title":"Explorer.Remote","doc":"A module responsible for placing remote dataframes and\ngarbage collect them.\n\nThe functions in `Explorer.DataFrame` and `Explorer.Series`\nwill automatically move operations on remote dataframes to\nthe nodes they belong to. `Explorer` also integrates with\n`FLAME` and automatically tracks remote dataframes and\nseries returned from `FLAME` calls when the `:track_resources`\noption is enabled.\n\nThis module provides additional conveniences for manual placement.","ref":"Explorer.Remote.html"},{"type":"module","title":"Implementation details - Explorer.Remote","doc":"In order to understand what this module does, we need\nto understand the challenges in working with remote series\nand dataframes.\n\nSeries and dataframes are actually NIF resources: they are\npointers to blobs of memory operated by low-level libraries.\nThose are represented in Erlang/Elixir as references (the\nsame as the one returned by `make_ref/0`). Once the reference\nis garbage collected (based on refcounting), those NIF\nresources are garbage collected and the memory is reclaimed.\n\nWhen using Distributed Erlang, you may write this code:\n\n    remote_series = :erpc.call(node, Explorer.Series, :from_list, [[1, 2, 3]])\n\nHowever, the code above will not work, because the series\nwill be allocated in the remote node and the remote node\nwon't hold a reference to said series! This means the series\nis garbage collected and if we attempt to read it later on,\nfrom the caller node, it will no longer exist. Therefore,\nwe must explicitly place these resources in remote nodes\nby spawning processes to hold these refernces. That's what\nthe `place/2` function in this module does.\n\nWe also need to guarantee these resources are not kept\nforever by these remote nodes, so `place/2` creates a\nlocal NIF resource that notifies the remote resources\nthey have been GCed, effectively implementing a remote\ngarbage collector.","ref":"Explorer.Remote.html#module-implementation-details"},{"type":"function","title":"Explorer.Remote.place/2","doc":"Receives a data structure and traverses it looking\nfor remote dataframes and series.\n\nIf any is found, it spawns a process on the remote node\nand sets up a distributed garbage collector. This function\nonly traverses maps, lists, and tuples, it does not support\narbitrary structs (such as map sets).\n\nIt returns the updated term and a list of remote PIDs\nspawned.","ref":"Explorer.Remote.html#place/2"},{"type":"module","title":"Explorer.Series","doc":"The Series struct and API.\n\nA series can be of the following data types:\n\n  * `:binary` - Binaries (sequences of bytes)\n  * `:boolean` - Boolean\n  * `:category` - Strings but represented internally as integers\n  * `:date` - Date type that unwraps to `Elixir.Date`\n  * `{:naive_datetime, precision}` - Naive DateTime type with millisecond/microsecond/nanosecond\n    precision that unwraps to `Elixir.NaiveDateTime`\n  * `{:datetime, precision, time_zone}` - DateTime type with millisecond/microsecond/nanosecond\n    precision that unwraps to `Elixir.DateTime`\n  * `{:duration, precision}` - Duration type with millisecond/microsecond/nanosecond\n    precision that unwraps to `Explorer.Duration`\n  * `{:f, size}` - a 64-bit or 32-bit floating point number\n  * `{:s, size}` - a 8-bit or 16-bit or 32-bit or 64-bit signed integer number.\n  * `{:u, size}` - a 8-bit or 16-bit or 32-bit or 64-bit unsigned integer number.\n  * `:null` - `nil`s exclusively\n  * `:string` - UTF-8 encoded binary\n  * `:time` - Time type that unwraps to `Elixir.Time`\n  * `{:list, dtype}` - A recursive dtype that can store lists. Examples: `{:list, :boolean}` or\n    a nested list dtype like `{:list, {:list, :boolean}}`.\n  * `{:struct, [{key, dtype}]}` - A recursive dtype that can store Arrow/Polars structs (not to be\n    confused with Elixir's struct). This type unwraps to Elixir maps with string keys. Examples:\n    `{:struct, [{\"a\", :string}]}` or a nested struct dtype like `{:struct, [{\"a\", {:struct, [{\"b\", :string}]}}]}`.\n\nWhen passing a dtype as argument, aliases are supported for convenience\nand compatibility with the Elixir ecosystem:\n\n  * All numeric dtypes (signed integer, unsigned integer, and floats) can\n    be specified as an atom in the form of `:s32`, `:u8`, `:f32` and so on\n  * The atom `:float` as an alias for `{:f, 64}` to mirror Elixir's floats\n  * The atom `:integer` as an alias for `{:s, 64}` to mirror Elixir's integers\n\nA series must consist of a single data type only. Series may have `nil` values in them.\nThe series `dtype` can be retrieved via the `dtype/1` function or directly accessed as\n`series.dtype`. A `series.name` field is also available, but it is always `nil` unless\nthe series is retrieved from a dataframe.\n\nMany functions only apply to certain dtypes. These functions may appear on distinct\ncategories on the sidebar. Other functions may work on several datatypes, such as\ncomparison functions. In such cases, a \"Supported dtypes\" section will be available\nin the function documentation.","ref":"Explorer.Series.html"},{"type":"module","title":"Creating series - Explorer.Series","doc":"Series can be created using `from_list/2`, `from_binary/3`, and friends:\n\nSeries can be made of numbers:\n\n    iex> Explorer.Series.from_list([1, 2, 3])\n    #Explorer.Series \n\nSeries are nullable, so you may also include nils:\n\n    iex> Explorer.Series.from_list([1.0, nil, 2.5, 3.1])\n    #Explorer.Series \n\nAny of the dtypes above are supported, such as strings:\n\n    iex> Explorer.Series.from_list([\"foo\", \"bar\", \"baz\"])\n    #Explorer.Series","ref":"Explorer.Series.html#module-creating-series"},{"type":"module","title":"Casting numeric series (type promotion) - Explorer.Series","doc":"Series of integers and floats are automatically cast when executing certain\noperations. For example, adding a series of s64 with f64 will return a list\nof f64.\n\nNumeric casting works like this:\n\n  * when working with the same numeric type but of different precisions,\n    the higher precision wins\n\n  * when working with unsigned integers and signed integers, unsigned integers\n    are cast to signed integers using double of its precision (maximum of 64 bits)\n\n  * when working with integers and floats, integers are always cast floats,\n    keep the floating number precision","ref":"Explorer.Series.html#module-casting-numeric-series-type-promotion"},{"type":"module","title":"Series queries - Explorer.Series","doc":"DataFrames have named columns, so their queries use column names as variables:\n\n    iex> require Explorer.DataFrame\n    iex> df = Explorer.DataFrame.new(col_name: [1, 2, 3])\n    iex> Explorer.DataFrame.filter(df, col_name > 2)\n    #Explorer.DataFrame \n\nSeries have no named columns (a series constitutes a single column,\nso no name is required). This means their queries can't use column\nnames as variables. Instead, series queries use the special `_` variable like so:\n\n    iex> s = Explorer.Series.from_list([1, 2, 3])\n    iex> Explorer.Series.filter(s, _ > 2)\n    #Explorer.Series","ref":"Explorer.Series.html#module-series-queries"},{"type":"function","title":"Explorer.Series.abs/1","doc":"Gets the series absolute values.","ref":"Explorer.Series.html#abs/1"},{"type":"function","title":"Supported dtypes - Explorer.Series.abs/1","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`","ref":"Explorer.Series.html#abs/1-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.abs/1","doc":"iex> s = Explorer.Series.from_list([1, 2, -1, -3])\n    iex> Explorer.Series.abs(s)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([1.0, 2.0, -1.0, -3.0])\n    iex> Explorer.Series.abs(s)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([1.0, 2.0, nil, -3.0])\n    iex> Explorer.Series.abs(s)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([\"a\", \"b\", \"c\"])\n    iex> Explorer.Series.abs(s)\n    ** (ArgumentError) Explorer.Series.abs/1 not implemented for dtype :string. Valid dtypes are {:f, 32}, {:f, 64}, {:s, 8}, {:s, 16}, {:s, 32}, {:s, 64}, {:u, 8}, {:u, 16}, {:u, 32} and {:u, 64}","ref":"Explorer.Series.html#abs/1-examples"},{"type":"function","title":"Explorer.Series.acos/1","doc":"Computes the the arccosine of a number.\nThe resultant series is going to be of dtype `{:f, 64}`, in radians, with values between 0 and pi.","ref":"Explorer.Series.html#acos/1"},{"type":"function","title":"Supported dtype - Explorer.Series.acos/1","doc":"* `{:f, 32}`\n  * `{:f, 64}`","ref":"Explorer.Series.html#acos/1-supported-dtype"},{"type":"function","title":"Examples - Explorer.Series.acos/1","doc":"iex> s = [1.0, 0.0, -1.0, -0.7071067811865475, 0.7071067811865475] |> Explorer.Series.from_list()\n    iex> Explorer.Series.acos(s)\n    #Explorer.Series","ref":"Explorer.Series.html#acos/1-examples"},{"type":"function","title":"Explorer.Series.add/2","doc":"Adds right to left, element-wise.\n\nWhen mixing floats and integers, the resulting series will have dtype `{:f, 64}`.\n\nAt least one of the arguments must be a series. If both\nsizes are series, the series must have the same size or\nat last one of them must have size of 1.","ref":"Explorer.Series.html#add/2"},{"type":"function","title":"Supported dtypes - Explorer.Series.add/2","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`","ref":"Explorer.Series.html#add/2-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.add/2","doc":"iex> s1 = Explorer.Series.from_list([1, 2, 3])\n    iex> s2 = Explorer.Series.from_list([4, 5, 6])\n    iex> Explorer.Series.add(s1, s2)\n    #Explorer.Series \n\nYou can also use scalar values on both sides:\n\n    iex> s1 = Explorer.Series.from_list([1, 2, 3])\n    iex> Explorer.Series.add(s1, 2)\n    #Explorer.Series \n\n    iex> s1 = Explorer.Series.from_list([1, 2, 3])\n    iex> Explorer.Series.add(2, s1)\n    #Explorer.Series","ref":"Explorer.Series.html#add/2-examples"},{"type":"function","title":"Explorer.Series.all?/1","doc":"Returns if all the values in a boolean series are true.","ref":"Explorer.Series.html#all?/1"},{"type":"function","title":"Supported dtypes - Explorer.Series.all?/1","doc":"* `:boolean`","ref":"Explorer.Series.html#all?/1-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.all?/1","doc":"iex> s = Series.from_list([true, true, true])\n    iex> Series.all?(s)\n    true\n\n    iex> s = Series.from_list([true, false, true])\n    iex> Series.all?(s)\n    false\n\n    iex> s = Series.from_list([1, 2, 3])\n    iex> Series.all?(s)\n    ** (ArgumentError) Explorer.Series.all?/1 not implemented for dtype {:s, 64}. Valid dtype is :boolean\n\nAn empty series will always return true:\n\n    iex> s = Series.from_list([], dtype: :boolean)\n    iex> Series.all?(s)\n    true\n\nOpposite to Elixir but similar to databases, `nil` values are ignored:\n\n    iex> s = Series.from_list([nil, true, true])\n    iex> Series.all?(s)\n    true\n\n    iex> s = Series.from_list([nil, nil, nil], dtype: :boolean)\n    iex> Series.all?(s)\n    true","ref":"Explorer.Series.html#all?/1-examples"},{"type":"function","title":"Explorer.Series.all_equal/2","doc":"Checks equality between two entire series.","ref":"Explorer.Series.html#all_equal/2"},{"type":"function","title":"Examples - Explorer.Series.all_equal/2","doc":"iex> s1 = Explorer.Series.from_list([\"a\", \"b\"])\n    iex> s2 = Explorer.Series.from_list([\"a\", \"b\"])\n    iex> Explorer.Series.all_equal(s1, s2)\n    true\n\n    iex> s1 = Explorer.Series.from_list([\"a\", \"b\"])\n    iex> s2 = Explorer.Series.from_list([\"a\", \"c\"])\n    iex> Explorer.Series.all_equal(s1, s2)\n    false\n\n    iex> s1 = Explorer.Series.from_list([\"a\", \"b\"])\n    iex> s2 = Explorer.Series.from_list([1, 2])\n    iex> Explorer.Series.all_equal(s1, s2)\n    false","ref":"Explorer.Series.html#all_equal/2-examples"},{"type":"function","title":"Explorer.Series.and/2","doc":"Returns a boolean mask of `left and right`, element-wise.\n\nBoth sizes must be series, the series must have the same\nsize or at last one of them must have size of 1.","ref":"Explorer.Series.html#and/2"},{"type":"function","title":"Examples - Explorer.Series.and/2","doc":"iex> s1 = Explorer.Series.from_list([1, 2, 3])\n    iex> mask1 = Explorer.Series.greater(s1, 1)\n    iex> mask2 = Explorer.Series.less(s1, 3)\n    iex> Explorer.Series.and(mask1, mask2)\n    #Explorer.Series","ref":"Explorer.Series.html#and/2-examples"},{"type":"function","title":"Explorer.Series.any?/1","doc":"Returns if any of the values in a boolean series are true.","ref":"Explorer.Series.html#any?/1"},{"type":"function","title":"Supported dtypes - Explorer.Series.any?/1","doc":"* `:boolean`","ref":"Explorer.Series.html#any?/1-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.any?/1","doc":"iex> s = Series.from_list([true, true, true])\n    iex> Series.any?(s)\n    true\n\n    iex> s = Series.from_list([true, false, true])\n    iex> Series.any?(s)\n    true\n\n    iex> s = Series.from_list([1, 2, 3])\n    iex> Series.any?(s)\n    ** (ArgumentError) Explorer.Series.any?/1 not implemented for dtype {:s, 64}. Valid dtype is :boolean\n\nAn empty series will always return `false`:\n\n    iex> s = Series.from_list([], dtype: :boolean)\n    iex> Series.any?(s)\n    false\n\nOpposite to Elixir but similar to databases, `nil` values are ignored:\n\n    iex> s = Series.from_list([nil, true, true])\n    iex> Series.any?(s)\n    true\n\n    iex> s = Series.from_list([nil, nil, nil], dtype: :boolean)\n    iex> Series.any?(s)\n    false","ref":"Explorer.Series.html#any?/1-examples"},{"type":"function","title":"Explorer.Series.argmax/1","doc":"Gets the index of the maximum value of the series.","ref":"Explorer.Series.html#argmax/1"},{"type":"function","title":"Supported dtypes - Explorer.Series.argmax/1","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`\n  * `:date`\n  * `:time`\n  * `:datetime`\n  * `:duration`","ref":"Explorer.Series.html#argmax/1-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.argmax/1","doc":"iex> s = Explorer.Series.from_list([1, 2, nil, 3])\n    iex> Explorer.Series.argmax(s)\n    3\n\n    iex> s = Explorer.Series.from_list([1.0, 2.0, nil, 3.0])\n    iex> Explorer.Series.argmax(s)\n    3\n\n    iex> s = Explorer.Series.from_list([~D[2021-01-01], ~D[1999-12-31]])\n    iex> Explorer.Series.argmax(s)\n    0\n\n    iex> s = Explorer.Series.from_list([~N[2021-01-01 00:00:00], ~N[1999-12-31 00:00:00]])\n    iex> Explorer.Series.argmax(s)\n    0\n\n    iex> s = Explorer.Series.from_list([~T[00:02:03.000212], ~T[00:05:04.000456]])\n    iex> Explorer.Series.argmax(s)\n    1\n\n    iex> s = Explorer.Series.from_list([], dtype: :integer)\n    iex> Explorer.Series.argmax(s)\n    nil\n\n    iex> s = Explorer.Series.from_list([\"a\", \"b\", \"c\"])\n    iex> Explorer.Series.argmax(s)\n    ** (ArgumentError) Explorer.Series.argmax/1 not implemented for dtype :string. Valid dtypes are :date, :time, {:duration, :microsecond}, {:duration, :millisecond}, {:duration, :nanosecond}, {:f, 32}, {:f, 64}, {:naive_datetime, :microsecond}, {:naive_datetime, :millisecond}, {:naive_datetime, :nanosecond}, {:s, 8}, {:s, 16}, {:s, 32}, {:s, 64}, {:u, 8}, {:u, 16}, {:u, 32} and {:u, 64}","ref":"Explorer.Series.html#argmax/1-examples"},{"type":"function","title":"Explorer.Series.argmin/1","doc":"Gets the index of the minimum value of the series.\n\nNote that `nil` is ignored. In case an empty list\nor a series whose all elements are `nil` is used,\nthe result will be `nil`.","ref":"Explorer.Series.html#argmin/1"},{"type":"function","title":"Supported dtypes - Explorer.Series.argmin/1","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`\n  * `:date`\n  * `:time`\n  * `:datetime`\n  * `:duration`","ref":"Explorer.Series.html#argmin/1-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.argmin/1","doc":"iex> s = Explorer.Series.from_list([1, 2, nil, 3])\n    iex> Explorer.Series.argmin(s)\n    0\n\n    iex> s = Explorer.Series.from_list([1.0, 2.0, nil, 3.0])\n    iex> Explorer.Series.argmin(s)\n    0\n\n    iex> s = Explorer.Series.from_list([~D[2021-01-01], ~D[1999-12-31]])\n    iex> Explorer.Series.argmin(s)\n    1\n\n    iex> s = Explorer.Series.from_list([~N[2021-01-01 00:00:00], ~N[1999-12-31 00:00:00]])\n    iex> Explorer.Series.argmin(s)\n    1\n\n    iex> s = Explorer.Series.from_list([~T[00:02:03.000212], ~T[00:05:04.000456]])\n    iex> Explorer.Series.argmin(s)\n    0\n\n    iex> s = Explorer.Series.from_list([], dtype: :integer)\n    iex> Explorer.Series.argmin(s)\n    nil\n\n    iex> s = Explorer.Series.from_list([nil], dtype: :integer)\n    iex> Explorer.Series.argmin(s)\n    nil\n\n    iex> s = Explorer.Series.from_list([\"a\", \"b\", \"c\"])\n    iex> Explorer.Series.argmin(s)\n    ** (ArgumentError) Explorer.Series.argmin/1 not implemented for dtype :string. Valid dtypes are :date, :time, {:duration, :microsecond}, {:duration, :millisecond}, {:duration, :nanosecond}, {:f, 32}, {:f, 64}, {:naive_datetime, :microsecond}, {:naive_datetime, :millisecond}, {:naive_datetime, :nanosecond}, {:s, 8}, {:s, 16}, {:s, 32}, {:s, 64}, {:u, 8}, {:u, 16}, {:u, 32} and {:u, 64}","ref":"Explorer.Series.html#argmin/1-examples"},{"type":"function","title":"Explorer.Series.argsort/2","doc":"Returns the indices that would sort the series.\n\nThe resultant series is going to have the `{:u, 32}` dtype.","ref":"Explorer.Series.html#argsort/2"},{"type":"function","title":"Options - Explorer.Series.argsort/2","doc":"* `:direction` - `:asc` or `:desc`, meaning \"ascending\" or \"descending\", respectively.\n    By default it sorts in ascending order.\n\n  * `:nils` - `:first` or `:last`.\n    By default it is `:last` if direction is `:asc`, and `:first` otherwise.\n\n  * `:parallel` - boolean.\n    Whether to parallelize the sorting.\n    By default it is `true`.\n\n  * `:stable` - boolean.\n    Determines if the sorting is stable (ties are guaranteed to maintain their order) or not.\n    Unstable sorting may be more performant.\n    By default it is `false`.","ref":"Explorer.Series.html#argsort/2-options"},{"type":"function","title":"Examples - Explorer.Series.argsort/2","doc":"iex> s = Explorer.Series.from_list([9, 3, 7, 1])\n    iex> Explorer.Series.argsort(s)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([9, 3, 7, 1])\n    iex> Explorer.Series.argsort(s, direction: :desc)\n    #Explorer.Series","ref":"Explorer.Series.html#argsort/2-examples"},{"type":"function","title":"Explorer.Series.asin/1","doc":"Computes the the arcsine of a number.\nThe resultant series is going to be of dtype `{:f, 64}`, in radians, with values between -pi/2 and pi/2.","ref":"Explorer.Series.html#asin/1"},{"type":"function","title":"Supported dtype - Explorer.Series.asin/1","doc":"* `{:f, 32}`\n  * `{:f, 64}`","ref":"Explorer.Series.html#asin/1-supported-dtype"},{"type":"function","title":"Examples - Explorer.Series.asin/1","doc":"iex> s = [1.0, 0.0, -1.0, -0.7071067811865475, 0.7071067811865475] |> Explorer.Series.from_list()\n    iex> Explorer.Series.asin(s)\n    #Explorer.Series","ref":"Explorer.Series.html#asin/1-examples"},{"type":"function","title":"Explorer.Series.at/2","doc":"Returns the value of the series at the given index.\n\nThis function will raise an error in case the index\nis out of bounds.","ref":"Explorer.Series.html#at/2"},{"type":"function","title":"Examples - Explorer.Series.at/2","doc":"iex> s = Explorer.Series.from_list([\"a\", \"b\", \"c\"])\n    iex> Explorer.Series.at(s, 2)\n    \"c\"\n\n    iex> s = Explorer.Series.from_list([\"a\", \"b\", \"c\"])\n    iex> Explorer.Series.at(s, 4)\n    ** (ArgumentError) index 4 out of bounds for series of size 3","ref":"Explorer.Series.html#at/2-examples"},{"type":"function","title":"Explorer.Series.at_every/2","doc":"Takes every *n*th value in this series, returned as a new series.","ref":"Explorer.Series.html#at_every/2"},{"type":"function","title":"Examples - Explorer.Series.at_every/2","doc":"iex> s = 1..10 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.at_every(s, 2)\n    #Explorer.Series \n\nIf *n* is bigger than the size of the series, the result is a new series with only the first value of the supplied series.\n\n    iex> s = 1..10 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.at_every(s, 20)\n    #Explorer.Series","ref":"Explorer.Series.html#at_every/2-examples"},{"type":"function","title":"Explorer.Series.atan/1","doc":"Computes the the arctangent of a number.\nThe resultant series is going to be of dtype `{:f, 64}`, in radians, with values between -pi/2 and pi/2.","ref":"Explorer.Series.html#atan/1"},{"type":"function","title":"Supported dtype - Explorer.Series.atan/1","doc":"* `{:f, 32}`\n  * `{:f, 64}`","ref":"Explorer.Series.html#atan/1-supported-dtype"},{"type":"function","title":"Examples - Explorer.Series.atan/1","doc":"iex> s = [1.0, 0.0, -1.0, -0.7071067811865475, 0.7071067811865475] |> Explorer.Series.from_list()\n    iex> Explorer.Series.atan(s)\n    #Explorer.Series","ref":"Explorer.Series.html#atan/1-examples"},{"type":"function","title":"Explorer.Series.cast/2","doc":"Cast the series to another type.","ref":"Explorer.Series.html#cast/2"},{"type":"function","title":"Examples - Explorer.Series.cast/2","doc":"iex> s = Explorer.Series.from_list([1, 2, 3])\n    iex> Explorer.Series.cast(s, :string)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([1, 2, 3])\n    iex> Explorer.Series.cast(s, {:f, 64})\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([1, 2, 3])\n    iex> Explorer.Series.cast(s, :date)\n    #Explorer.Series \n\nNote that `time` is represented as an integer of nanoseconds since midnight.\nIn Elixir we can't represent nanoseconds, only microseconds. So be aware that\ninformation can be lost if a conversion is needed (e.g. calling `to_list/1`).\n\n    iex> s = Explorer.Series.from_list([1_000, 2_000, 3_000])\n    iex> Explorer.Series.cast(s, :time)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([86399 * 1_000 * 1_000 * 1_000])\n    iex> Explorer.Series.cast(s, :time)\n    #Explorer.Series \n\nNote that `datetime` is represented as an integer of microseconds since Unix Epoch (1970-01-01 00:00:00).\n\n    iex> s = Explorer.Series.from_list([1, 2, 3])\n    iex> Explorer.Series.cast(s, {:naive_datetime, :microsecond})\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([1649883642 * 1_000 * 1_000])\n    iex> Explorer.Series.cast(s, {:naive_datetime, :microsecond})\n    #Explorer.Series \n\nYou can also use `cast/2` to categorise a string:\n\n    iex> s = Explorer.Series.from_list([\"apple\", \"banana\",  \"apple\", \"lemon\"])\n    iex> Explorer.Series.cast(s, :category)\n    #Explorer.Series \n\n`cast/2` will return the series as a no-op if you try to cast to the same dtype.\n\n    iex> s = Explorer.Series.from_list([1, 2, 3])\n    iex> Explorer.Series.cast(s, :integer)\n    #Explorer.Series","ref":"Explorer.Series.html#cast/2-examples"},{"type":"function","title":"Explorer.Series.categories/1","doc":"Return a series with the category names of a categorical series.\n\nEach category has the index equal to its position.\nNo order for the categories is guaranteed.","ref":"Explorer.Series.html#categories/1"},{"type":"function","title":"Examples - Explorer.Series.categories/1","doc":"iex> s = Explorer.Series.from_list([\"a\", \"b\", \"c\", nil, \"a\", \"c\"], dtype: :category)\n    iex> Explorer.Series.categories(s)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([\"c\", \"a\", \"b\"], dtype: :category)\n    iex> Explorer.Series.categories(s)\n    #Explorer.Series","ref":"Explorer.Series.html#categories/1-examples"},{"type":"function","title":"Explorer.Series.categorise/2","doc":"Categorise a series of integers or strings according to `categories`.\n\nThis function receives a series of integers or strings and convert them\ninto the categories specified by the second argument.\nThe second argument can be one of:\n\n  * a series with dtype `:category`. The integers will be indexes into\n    the categories of the given series (returned by `categories/1`)\n\n  * a series with dtype `:string`. The integers will be indexes into\n    the series itself\n\n  * a list of strings. The integers will be indexes into the list\n\nThis is going to essentially \"copy\" the source of categories from the left series\nto the right. All members from the left that are not present in the right hand-side\nare going to be `nil`.\n\nIf you have a series of strings and you want to convert them into categories,\ninvoke `cast(series, :category)` instead.","ref":"Explorer.Series.html#categorise/2"},{"type":"function","title":"Examples - Explorer.Series.categorise/2","doc":"If a categorical series is given as second argument, we will extract its\ncategories and map the integers into it:\n\n    iex> categories = Explorer.Series.from_list([\"a\", \"b\", \"c\", nil, \"a\"], dtype: :category)\n    iex> indexes = Explorer.Series.from_list([0, 2, 1, 0, 2])\n    iex> Explorer.Series.categorise(indexes, categories)\n    #Explorer.Series \n\nOtherwise, if a list of strings or a series of strings is given, they are\nconsidered to be the categories series itself:\n\n    iex> categories = Explorer.Series.from_list([\"a\", \"b\", \"c\"])\n    iex> indexes = Explorer.Series.from_list([0, 2, 1, 0, 2])\n    iex> Explorer.Series.categorise(indexes, categories)\n    #Explorer.Series \n\n    iex> indexes = Explorer.Series.from_list([0, 2, 1, 0, 2])\n    iex> Explorer.Series.categorise(indexes, [\"a\", \"b\", \"c\"])\n    #Explorer.Series \n\nElements that are not mapped to a category will become `nil`:\n\n    iex> indexes = Explorer.Series.from_list([0, 2, nil, 0, 2, 7])\n    iex> Explorer.Series.categorise(indexes, [\"a\", \"b\", \"c\"])\n    #Explorer.Series \n\nStrings can be used as \"indexes\" to create a categorical series\nwith the intersection of members:\n\n    iex> strings = Explorer.Series.from_list([\"a\", \"c\", nil, \"c\", \"b\", \"d\"])\n    iex> Explorer.Series.categorise(strings, [\"a\", \"b\", \"c\"])\n    #Explorer.Series","ref":"Explorer.Series.html#categorise/2-examples"},{"type":"function","title":"Explorer.Series.ceil/1","doc":"Ceil floating point series to highest integers smaller or equal to the float value.","ref":"Explorer.Series.html#ceil/1"},{"type":"function","title":"Examples - Explorer.Series.ceil/1","doc":"iex> s = Explorer.Series.from_list([1.124993, 2.555321, 3.995001])\n    iex> Explorer.Series.ceil(s)\n    #Explorer.Series","ref":"Explorer.Series.html#ceil/1-examples"},{"type":"function","title":"Explorer.Series.clip/3","doc":"Clip (or clamp) the values in a series.\n\nValues that fall outside of the interval defined by the `min` and `max`\nbounds are clipped to the bounds.","ref":"Explorer.Series.html#clip/3"},{"type":"function","title":"Supported dtypes - Explorer.Series.clip/3","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`\n\nClipping other dtypes are possible using `select/3`.","ref":"Explorer.Series.html#clip/3-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.clip/3","doc":"iex> s = Explorer.Series.from_list([-50, 5, nil, 50])\n    iex> Explorer.Series.clip(s, 1, 10)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([-50, 5, nil, 50])\n    iex> Explorer.Series.clip(s, 1.5, 10.5)\n    #Explorer.Series","ref":"Explorer.Series.html#clip/3-examples"},{"type":"function","title":"Explorer.Series.coalesce/1","doc":"Finds the first non-missing element at each position.","ref":"Explorer.Series.html#coalesce/1"},{"type":"function","title":"Examples - Explorer.Series.coalesce/1","doc":"iex> s1 = Explorer.Series.from_list([1, 2, nil, nil])\n    iex> s2 = Explorer.Series.from_list([1, 2, nil, 4])\n    iex> s3 = Explorer.Series.from_list([nil, nil, 3, 4])\n    iex> Explorer.Series.coalesce([s1, s2, s3])\n    #Explorer.Series","ref":"Explorer.Series.html#coalesce/1-examples"},{"type":"function","title":"Explorer.Series.coalesce/2","doc":"Finds the first non-missing element at each position.\n\n`coalesce(s1, s2)` is equivalent to `coalesce([s1, s2])`.","ref":"Explorer.Series.html#coalesce/2"},{"type":"function","title":"Examples - Explorer.Series.coalesce/2","doc":"iex> s1 = Explorer.Series.from_list([1, nil, 3, nil])\n    iex> s2 = Explorer.Series.from_list([1, 2, nil, 4])\n    iex> Explorer.Series.coalesce(s1, s2)\n    #Explorer.Series \n\n    iex> s1 = Explorer.Series.from_list([\"foo\", nil, \"bar\", nil])\n    iex> s2 = Explorer.Series.from_list([1, 2, nil, 4])\n    iex> Explorer.Series.coalesce(s1, s2)\n    ** (ArgumentError) cannot invoke Explorer.Series.coalesce/2 with mismatched dtypes: :string and {:s, 64}","ref":"Explorer.Series.html#coalesce/2-examples"},{"type":"function","title":"Explorer.Series.collect/1","doc":"Collects a series to the current node.\n\nIf the series is already in the current node, it works as a no-op.","ref":"Explorer.Series.html#collect/1"},{"type":"function","title":"Examples - Explorer.Series.collect/1","doc":"series = Explorer.Series.from_list([1, 2, 3], node: :some@node)\n    Explorer.Series.collect(series)","ref":"Explorer.Series.html#collect/1-examples"},{"type":"function","title":"Explorer.Series.concat/1","doc":"Concatenate one or more series.\n\nType promotion may happen between numeric series or\nnull series. All other dtypes must match.","ref":"Explorer.Series.html#concat/1"},{"type":"function","title":"Examples - Explorer.Series.concat/1","doc":"iex> s1 = Explorer.Series.from_list([1, 2, 3])\n    iex> s2 = Explorer.Series.from_list([4, 5, 6])\n    iex> Explorer.Series.concat([s1, s2])\n    #Explorer.Series \n\n    iex> s1 = Explorer.Series.from_list([1, 2, 3])\n    iex> s2 = Explorer.Series.from_list([4.0, 5.0, 6.4])\n    iex> Explorer.Series.concat([s1, s2])\n    #Explorer.Series","ref":"Explorer.Series.html#concat/1-examples"},{"type":"function","title":"Explorer.Series.concat/2","doc":"Concatenate two series.\n\n`concat(s1, s2)` is equivalent to `concat([s1, s2])`.","ref":"Explorer.Series.html#concat/2"},{"type":"function","title":"Explorer.Series.contains/2","doc":"Detects whether a string contains a substring.\n\n>","ref":"Explorer.Series.html#contains/2"},{"type":"function","title":"Notice {: .warning} - Explorer.Series.contains/2","doc":">\n> This function detects only literal strings. For regular expressions, see `re_contains/2`.","ref":"Explorer.Series.html#contains/2-notice-warning"},{"type":"function","title":"Examples - Explorer.Series.contains/2","doc":"iex> s = Explorer.Series.from_list([\"abc\", \"def\", \"bcd\"])\n    iex> Explorer.Series.contains(s, \"bc\")\n    #Explorer.Series","ref":"Explorer.Series.html#contains/2-examples"},{"type":"function","title":"Explorer.Series.correlation/3","doc":"Compute the correlation between two series.\n\nThe parameter `ddof` refers to the 'delta degrees of freedom' - the divisor\nused in the correlation calculation. Defaults to 1.\n\nThe parameter `:method` refers to the correlation method. The following methods are available:\n  - `:pearson` : Standard correlation coefficient. (default)\n  - `:spearman` : Spearman rank correlation.","ref":"Explorer.Series.html#correlation/3"},{"type":"function","title":"Supported dtypes - Explorer.Series.correlation/3","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`","ref":"Explorer.Series.html#correlation/3-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.correlation/3","doc":"iex> s1 = Series.from_list([1, 8, 3])\n    iex> s2 = Series.from_list([4, 5, 2])\n    iex> Series.correlation(s1, s2)\n    0.5447047794019219","ref":"Explorer.Series.html#correlation/3-examples"},{"type":"function","title":"Explorer.Series.cos/1","doc":"Computes the the cosine of a number (in radians).\nThe resultant series is going to be of dtype `{:f, 64}`, with values between 1 and -1.","ref":"Explorer.Series.html#cos/1"},{"type":"function","title":"Supported dtype - Explorer.Series.cos/1","doc":"* `{:f, 32}`\n  * `{:f, 64}`","ref":"Explorer.Series.html#cos/1-supported-dtype"},{"type":"function","title":"Examples - Explorer.Series.cos/1","doc":"iex> pi = :math.pi()\n    iex> s = [-pi * 3/2, -pi, -pi / 2, -pi / 4, 0, pi / 4, pi / 2, pi, pi * 3/2] |> Explorer.Series.from_list()\n    iex> Explorer.Series.cos(s)\n    #Explorer.Series","ref":"Explorer.Series.html#cos/1-examples"},{"type":"function","title":"Explorer.Series.count/1","doc":"Counts the number of non-`nil` elements in a series.\n\nSee also:\n\n  * `count_nil/1` - counts only the `nil` elements.\n  * `size/1` - counts all elements.","ref":"Explorer.Series.html#count/1"},{"type":"function","title":"Examples - Explorer.Series.count/1","doc":"Without `nil`:\n\n    iex> s = Explorer.Series.from_list([\"a\", \"b\", \"c\"])\n    iex> Explorer.Series.count(s)\n    3\n\nWith `nil`:\n\n    iex> s = Explorer.Series.from_list([\"a\", nil, \"c\"])\n    iex> Explorer.Series.count(s)\n    2\n\nWith `:nan` (`:nan` does not count as `nil`):\n\n    iex> s = Explorer.Series.from_list([1, :nan, 3])\n    iex> Explorer.Series.count(s)\n    3","ref":"Explorer.Series.html#count/1-examples"},{"type":"function","title":"Explorer.Series.count_matches/2","doc":"Count how many times a substring appears in a string.\n\n>","ref":"Explorer.Series.html#count_matches/2"},{"type":"function","title":"Notice {: .warning} - Explorer.Series.count_matches/2","doc":">\n> This function detects only literal strings. For regular expressions, see `re_count_matches/2`.","ref":"Explorer.Series.html#count_matches/2-notice-warning"},{"type":"function","title":"Examples - Explorer.Series.count_matches/2","doc":"iex> s = Explorer.Series.from_list([\"abc\", \"def\", \"bcd\", nil])\n    iex> Explorer.Series.count_matches(s, \"bc\")\n    #Explorer.Series","ref":"Explorer.Series.html#count_matches/2-examples"},{"type":"function","title":"Explorer.Series.covariance/3","doc":"Compute the covariance between two series.","ref":"Explorer.Series.html#covariance/3"},{"type":"function","title":"Supported dtypes - Explorer.Series.covariance/3","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`","ref":"Explorer.Series.html#covariance/3-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.covariance/3","doc":"iex> s1 = Series.from_list([1, 8, 3])\n    iex> s2 = Series.from_list([4, 5, 2])\n    iex> Series.covariance(s1, s2)\n    3.0","ref":"Explorer.Series.html#covariance/3-examples"},{"type":"function","title":"Explorer.Series.cumulative_max/2","doc":"Calculates the cumulative maximum of the series.\n\nOptionally, can accumulate in reverse.\n\nDoes not fill nil values. See `fill_missing/2`.","ref":"Explorer.Series.html#cumulative_max/2"},{"type":"function","title":"Supported dtypes - Explorer.Series.cumulative_max/2","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`\n  * `:date`\n  * `:time`\n  * `:datetime`\n  * `:duration`","ref":"Explorer.Series.html#cumulative_max/2-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.cumulative_max/2","doc":"iex> s = [1, 2, 3, 4] |> Explorer.Series.from_list()\n    iex> Explorer.Series.cumulative_max(s)\n    #Explorer.Series \n\n    iex> s = [1, 2, nil, 4] |> Explorer.Series.from_list()\n    iex> Explorer.Series.cumulative_max(s)\n    #Explorer.Series \n\n    iex> s = [~T[03:00:02.000000], ~T[02:04:19.000000], nil, ~T[13:24:56.000000]] |> Explorer.Series.from_list()\n    iex> Explorer.Series.cumulative_max(s)\n    #Explorer.Series","ref":"Explorer.Series.html#cumulative_max/2-examples"},{"type":"function","title":"Explorer.Series.cumulative_min/2","doc":"Calculates the cumulative minimum of the series.\n\nOptionally, can accumulate in reverse.\n\nDoes not fill nil values. See `fill_missing/2`.","ref":"Explorer.Series.html#cumulative_min/2"},{"type":"function","title":"Supported dtypes - Explorer.Series.cumulative_min/2","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`\n  * `:date`\n  * `:time`\n  * `:datetime`\n  * `:duration`","ref":"Explorer.Series.html#cumulative_min/2-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.cumulative_min/2","doc":"iex> s = [1, 2, 3, 4] |> Explorer.Series.from_list()\n    iex> Explorer.Series.cumulative_min(s)\n    #Explorer.Series \n\n    iex> s = [1, 2, nil, 4] |> Explorer.Series.from_list()\n    iex> Explorer.Series.cumulative_min(s)\n    #Explorer.Series \n\n    iex> s = [~T[03:00:02.000000], ~T[02:04:19.000000], nil, ~T[13:24:56.000000]] |> Explorer.Series.from_list()\n    iex> Explorer.Series.cumulative_min(s)\n    #Explorer.Series","ref":"Explorer.Series.html#cumulative_min/2-examples"},{"type":"function","title":"Explorer.Series.cumulative_product/2","doc":"Calculates the cumulative product of the series.\n\nOptionally, can accumulate in reverse.\n\nDoes not fill nil values. See `fill_missing/2`.","ref":"Explorer.Series.html#cumulative_product/2"},{"type":"function","title":"Supported dtypes - Explorer.Series.cumulative_product/2","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`","ref":"Explorer.Series.html#cumulative_product/2-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.cumulative_product/2","doc":"iex> s = [1, 2, 3, 2] |> Explorer.Series.from_list()\n    iex> Explorer.Series.cumulative_product(s)\n    #Explorer.Series \n\n    iex> s = [1, 2, nil, 4] |> Explorer.Series.from_list()\n    iex> Explorer.Series.cumulative_product(s)\n    #Explorer.Series","ref":"Explorer.Series.html#cumulative_product/2-examples"},{"type":"function","title":"Explorer.Series.cumulative_sum/2","doc":"Calculates the cumulative sum of the series.\n\nOptionally, can accumulate in reverse.\n\nDoes not fill nil values. See `fill_missing/2`.","ref":"Explorer.Series.html#cumulative_sum/2"},{"type":"function","title":"Supported dtypes - Explorer.Series.cumulative_sum/2","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`\n  * `:boolean`","ref":"Explorer.Series.html#cumulative_sum/2-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.cumulative_sum/2","doc":"iex> s = [1, 2, 3, 4] |> Explorer.Series.from_list()\n    iex> Explorer.Series.cumulative_sum(s)\n    #Explorer.Series \n\n    iex> s = [1, 2, nil, 4] |> Explorer.Series.from_list()\n    iex> Explorer.Series.cumulative_sum(s)\n    #Explorer.Series","ref":"Explorer.Series.html#cumulative_sum/2-examples"},{"type":"function","title":"Explorer.Series.cut/3","doc":"Bins values into discrete values.\n\nGiven a `bins` length of N, there will be N+1 categories.","ref":"Explorer.Series.html#cut/3"},{"type":"function","title":"Options - Explorer.Series.cut/3","doc":"* `:labels` - The labels assigned to the bins. Given `bins` of\n    length N, `:labels` must be of length N+1. Defaults to the bin\n    bounds (e.g. `(-inf -1.0]`, `(-1.0, 1.0]`, `(1.0, inf]`)\n\n  * `:break_point_label` - The name given to the breakpoint column.\n    Defaults to `break_point`.\n\n  * `:category_label` - The name given to the category column.\n    Defaults to `category`.","ref":"Explorer.Series.html#cut/3-options"},{"type":"function","title":"Examples - Explorer.Series.cut/3","doc":"iex> s = Explorer.Series.from_list([1.0, 2.0, 3.0])\n    iex> Explorer.Series.cut(s, [1.5, 2.5])\n    #Explorer.DataFrame","ref":"Explorer.Series.html#cut/3-examples"},{"type":"function","title":"Explorer.Series.day_of_week/1","doc":"Returns a day-of-week number starting from Monday = 1. (ISO 8601 weekday number)","ref":"Explorer.Series.html#day_of_week/1"},{"type":"function","title":"Examples - Explorer.Series.day_of_week/1","doc":"iex> s = Explorer.Series.from_list([~D[2023-01-15], ~D[2023-01-16], ~D[2023-01-20], nil])\n    iex> Explorer.Series.day_of_week(s)\n    #Explorer.Series \n\nIt can also be called on a datetime series.\n\n    iex> s = Explorer.Series.from_list([~N[2023-01-15 00:00:00], ~N[2023-01-16 23:59:59.999999], ~N[2023-01-20 12:00:00], nil])\n    iex> Explorer.Series.day_of_week(s)\n    #Explorer.Series","ref":"Explorer.Series.html#day_of_week/1-examples"},{"type":"function","title":"Explorer.Series.day_of_year/1","doc":"Returns the day-of-year number starting from 1.\n\nThe return value ranges from 1 to 366 (the last day of year differs by years).","ref":"Explorer.Series.html#day_of_year/1"},{"type":"function","title":"Examples - Explorer.Series.day_of_year/1","doc":"iex> s = Explorer.Series.from_list([~D[2023-01-01], ~D[2023-01-02], ~D[2023-02-01], nil])\n    iex> Explorer.Series.day_of_year(s)\n    #Explorer.Series \n\nIt can also be called on a datetime series.\n\n    iex> s = Explorer.Series.from_list(\n    ...>   [~N[2023-01-01 00:00:00], ~N[2023-01-02 00:00:00], ~N[2023-02-01 23:59:59], nil]\n    ...> )\n    iex> Explorer.Series.day_of_year(s)\n    #Explorer.Series","ref":"Explorer.Series.html#day_of_year/1-examples"},{"type":"function","title":"Explorer.Series.distinct/1","doc":"Returns the unique values of the series.","ref":"Explorer.Series.html#distinct/1"},{"type":"function","title":"Examples - Explorer.Series.distinct/1","doc":"iex> s = [1, 1, 2, 2, 3, 3] |> Explorer.Series.from_list()\n    iex> Explorer.Series.distinct(s)\n    #Explorer.Series","ref":"Explorer.Series.html#distinct/1-examples"},{"type":"function","title":"Explorer.Series.divide/2","doc":"Divides left by right, element-wise.\n\nThe resulting series will have the dtype as `{:f, 64}`.\n\nAt least one of the arguments must be a series. If both\nsizes are series, the series must have the same size or\nat last one of them must have size of 1.","ref":"Explorer.Series.html#divide/2"},{"type":"function","title":"Supported dtypes - Explorer.Series.divide/2","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`","ref":"Explorer.Series.html#divide/2-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.divide/2","doc":"iex> s1 = [10, 10, 10] |> Explorer.Series.from_list()\n    iex> s2 = [2, 2, 2] |> Explorer.Series.from_list()\n    iex> Explorer.Series.divide(s1, s2)\n    #Explorer.Series \n\n    iex> s1 = [10, 10, 10] |> Explorer.Series.from_list()\n    iex> Explorer.Series.divide(s1, 2)\n    #Explorer.Series \n\n    iex> s1 = [10, 52 ,10] |> Explorer.Series.from_list()\n    iex> Explorer.Series.divide(s1, 2.5)\n    #Explorer.Series \n\n    iex> s1 = [10, 10, 10] |> Explorer.Series.from_list()\n    iex> s2 = [2, 0, 2] |> Explorer.Series.from_list()\n    iex> Explorer.Series.divide(s1, s2)\n    #Explorer.Series","ref":"Explorer.Series.html#divide/2-examples"},{"type":"function","title":"Explorer.Series.downcase/1","doc":"Converts all characters to lowercase.","ref":"Explorer.Series.html#downcase/1"},{"type":"function","title":"Examples - Explorer.Series.downcase/1","doc":"iex> s = Explorer.Series.from_list([\"ABC\", \"DEF\", \"BCD\"])\n    iex> Explorer.Series.downcase(s)\n    #Explorer.Series","ref":"Explorer.Series.html#downcase/1-examples"},{"type":"function","title":"Explorer.Series.dtype/1","doc":"Returns the data type of the series.\n\nSee the moduledoc for all supported dtypes.","ref":"Explorer.Series.html#dtype/1"},{"type":"function","title":"Examples - Explorer.Series.dtype/1","doc":"iex> s = Explorer.Series.from_list([1, 2, 3])\n    iex> Explorer.Series.dtype(s)\n    {:s, 64}\n\n    iex> s = Explorer.Series.from_list([\"a\", nil, \"b\", \"c\"])\n    iex> Explorer.Series.dtype(s)\n    :string","ref":"Explorer.Series.html#dtype/1-examples"},{"type":"function","title":"Explorer.Series.equal/2","doc":"Returns boolean mask of `left == right`, element-wise.\n\nAt least one of the arguments must be a series. If both\nsizes are series, the series must have the same size or\nat last one of them must have size of 1.","ref":"Explorer.Series.html#equal/2"},{"type":"function","title":"Examples - Explorer.Series.equal/2","doc":"iex> s1 = Explorer.Series.from_list([1, 2, 3])\n    iex> s2 = Explorer.Series.from_list([1, 2, 4])\n    iex> Explorer.Series.equal(s1, s2)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([1, 2, 3])\n    iex> Explorer.Series.equal(s, 1)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([true, true, false])\n    iex> Explorer.Series.equal(s, true)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([\"a\", \"b\", \"c\"])\n    iex> Explorer.Series.equal(s, \"a\")\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([~D[2021-01-01], ~D[1999-12-31]])\n    iex> Explorer.Series.equal(s, ~D[1999-12-31])\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([~N[2022-01-01 00:00:00], ~N[2022-01-01 23:00:00]])\n    iex> Explorer.Series.equal(s, ~N[2022-01-01 00:00:00])\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([\"a\", \"b\", \"c\"])\n    iex> Explorer.Series.equal(s, false)\n    ** (ArgumentError) cannot invoke Explorer.Series.equal/2 with mismatched dtypes: :string and false","ref":"Explorer.Series.html#equal/2-examples"},{"type":"function","title":"Explorer.Series.ewm_mean/2","doc":"Calculate the exponentially weighted moving average, given smoothing factor alpha.","ref":"Explorer.Series.html#ewm_mean/2"},{"type":"function","title":"Options - Explorer.Series.ewm_mean/2","doc":"* `:alpha` - Optional smoothing factor which specifies the importance given\n    to most recent observations. It is a value such that, 0 < alpha <= 1. Defaults to 0.5.\n\n  * `:adjust` - If set to true, it corrects the bias introduced by smoothing process.\n    Defaults to `true`.\n\n  * `:min_periods` - The number of values in the window that should be non-nil\n    before computing a result. Defaults to `1`.\n\n  * `:ignore_nils` - If set to true, it ignore nulls in the calculation. Defaults to `true`.","ref":"Explorer.Series.html#ewm_mean/2-options"},{"type":"function","title":"Examples - Explorer.Series.ewm_mean/2","doc":"iex> s = 1..5 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.ewm_mean(s)\n    #Explorer.Series \n\n    iex> s = 1..5 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.ewm_mean(s, alpha: 0.1)\n    #Explorer.Series","ref":"Explorer.Series.html#ewm_mean/2-examples"},{"type":"function","title":"Explorer.Series.ewm_standard_deviation/2","doc":"Calculate the exponentially weighted moving standard deviation, given smoothing factor alpha.","ref":"Explorer.Series.html#ewm_standard_deviation/2"},{"type":"function","title":"Options - Explorer.Series.ewm_standard_deviation/2","doc":"* `:alpha` - Optional smoothing factor which specifies the importance given\n    to most recent observations. It is a value such that, 0 < alpha <= 1. Defaults to 0.5.\n\n  * `:adjust` - If set to true, it corrects the bias introduced by smoothing process.\n    Defaults to `true`.\n\n  * `:bias` - If set to false, it corrects the estimate to be statistically unbiased.\n    Defaults to `false`.\n\n  * `:min_periods` - The number of values in the window that should be non-nil\n    before computing a result. Defaults to `1`.\n\n  * `:ignore_nils` - If set to true, it ignore nulls in the calculation. Defaults to `true`.","ref":"Explorer.Series.html#ewm_standard_deviation/2-options"},{"type":"function","title":"Examples - Explorer.Series.ewm_standard_deviation/2","doc":"iex> s = 1..5 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.ewm_standard_deviation(s)\n    #Explorer.Series \n\n    iex> s = 1..5 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.ewm_standard_deviation(s, alpha: 0.1)\n    #Explorer.Series","ref":"Explorer.Series.html#ewm_standard_deviation/2-examples"},{"type":"function","title":"Explorer.Series.ewm_variance/2","doc":"Calculate the exponentially weighted moving variance, given smoothing factor alpha.","ref":"Explorer.Series.html#ewm_variance/2"},{"type":"function","title":"Options - Explorer.Series.ewm_variance/2","doc":"* `:alpha` - Optional smoothing factor which specifies the importance given\n    to most recent observations. It is a value such that, 0 < alpha <= 1. Defaults to 0.5.\n\n  * `:adjust` - If set to true, it corrects the bias introduced by smoothing process.\n    Defaults to `true`.\n\n  * `:bias` - If set to false, it corrects the estimate to be statistically unbiased.\n    Defaults to `false`.\n\n  * `:min_periods` - The number of values in the window that should be non-nil\n    before computing a result. Defaults to `1`.\n\n  * `:ignore_nils` - If set to true, it ignore nulls in the calculation. Defaults to `true`.","ref":"Explorer.Series.html#ewm_variance/2-options"},{"type":"function","title":"Examples - Explorer.Series.ewm_variance/2","doc":"iex> s = 1..5 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.ewm_variance(s)\n    #Explorer.Series \n\n    iex> s = 1..5 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.ewm_variance(s, alpha: 0.1)\n    #Explorer.Series","ref":"Explorer.Series.html#ewm_variance/2-examples"},{"type":"function","title":"Explorer.Series.exp/1","doc":"Calculates the exponential of all elements.","ref":"Explorer.Series.html#exp/1"},{"type":"function","title":"Explorer.Series.field/2","doc":"Extracts a field from a struct series\n\nThe field can be an atom or string.","ref":"Explorer.Series.html#field/2"},{"type":"function","title":"Examples - Explorer.Series.field/2","doc":"iex> s = Series.from_list([%{a: 1}, %{a: 2}])\n    iex> Series.field(s, :a)\n    #Explorer.Series \n\n    iex> s = Series.from_list([%{a: 1}, %{a: 2}])\n    iex> Series.field(s, \"a\")\n    #Explorer.Series","ref":"Explorer.Series.html#field/2-examples"},{"type":"function","title":"Explorer.Series.fill_missing/2","doc":"Fill missing values with the given strategy. If a scalar value is provided instead of a strategy\natom, `nil` will be replaced with that value. It must be of the same `dtype` as the series.","ref":"Explorer.Series.html#fill_missing/2"},{"type":"function","title":"Strategies - Explorer.Series.fill_missing/2","doc":"* `:forward` - replace nil with the previous value\n  * `:backward` - replace nil with the next value\n  * `:max` - replace nil with the series maximum\n  * `:min` - replace nil with the series minimum\n  * `:mean` - replace nil with the series mean\n  * `:nan` (float only) - replace nil with `NaN`","ref":"Explorer.Series.html#fill_missing/2-strategies"},{"type":"function","title":"Examples - Explorer.Series.fill_missing/2","doc":"iex> s = Explorer.Series.from_list([1, 2, nil, 4])\n    iex> Explorer.Series.fill_missing(s, :forward)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([1, 2, nil, 4])\n    iex> Explorer.Series.fill_missing(s, :backward)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([1, 2, nil, 4])\n    iex> Explorer.Series.fill_missing(s, :max)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([1, 2, nil, 4])\n    iex> Explorer.Series.fill_missing(s, :min)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([1, 2, nil, 4])\n    iex> Explorer.Series.fill_missing(s, :mean)\n    #Explorer.Series \n\nValues that belong to the series itself can also be added as missing:\n\n    iex> s = Explorer.Series.from_list([1, 2, nil, 4])\n    iex> Explorer.Series.fill_missing(s, 3)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([\"a\", \"b\", nil, \"d\"])\n    iex> Explorer.Series.fill_missing(s, \"c\")\n    #Explorer.Series \n\nMismatched types will raise:\n\n    iex> s = Explorer.Series.from_list([1, 2, nil, 4])\n    iex> Explorer.Series.fill_missing(s, \"foo\")\n    ** (ArgumentError) cannot invoke Explorer.Series.fill_missing/2 with mismatched dtypes: {:s, 64} and \"foo\"\n\nFloats in particular accept missing values to be set to NaN, Inf, and -Inf:\n\n    iex> s = Explorer.Series.from_list([1.0, 2.0, nil, 4.0])\n    iex> Explorer.Series.fill_missing(s, :nan)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([1.0, 2.0, nil, 4.0])\n    iex> Explorer.Series.fill_missing(s, :infinity)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([1.0, 2.0, nil, 4.0])\n    iex> Explorer.Series.fill_missing(s, :neg_infinity)\n    #Explorer.Series","ref":"Explorer.Series.html#fill_missing/2-examples"},{"type":"macro","title":"Explorer.Series.filter/2","doc":"Picks values based on an `Explorer.Query`.\n\nThe query is compiled and runs efficiently against the series.\nThe query must return a boolean expression or a list of boolean expressions.\nWhen a list is returned, they are joined as `and` expressions.\n\n> #### Notice {: .notice}\n>\n> This is a macro.\n>\n>   * You must `require Explorer.Series` before using it.\n>   * You must use the special `_` syntax. See the moduledoc for details.\n\nBesides element-wise series operations, you can also use window functions\nand aggregations inside comparisons.\n\nSee `filter_with/2` for a callback version of this function without\n`Explorer.Query`.\nSee `mask/2` if you want to filter values based on another series.","ref":"Explorer.Series.html#filter/2"},{"type":"macro","title":"Examples - Explorer.Series.filter/2","doc":"iex> s = Explorer.Series.from_list([\"a\", \"b\", \"c\"])\n    iex> Explorer.Series.filter(s, _ == \"b\")\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([1, 2, 3])\n    iex> Explorer.Series.filter(s, remainder(_, 2) == 1)\n    #Explorer.Series \n\nReturning a non-boolean expression errors:\n\n    iex> s = Explorer.Series.from_list([1, 2, 3])\n    iex> Explorer.Series.filter(s, cumulative_max(_))\n    ** (ArgumentError) expecting the function to return a boolean LazySeries, but instead it returned a LazySeries of type {:s, 64}\n\nWhich can be addressed by converting it to boolean:\n\n    iex> s = Explorer.Series.from_list([1, 2, 3])\n    iex> Explorer.Series.filter(s, cumulative_max(_) == 1)\n    #Explorer.Series","ref":"Explorer.Series.html#filter/2-examples"},{"type":"function","title":"Explorer.Series.filter_with/2","doc":"Filters a series with a callback function.\n\nSee `mask/2` if you want to filter values based on another series.","ref":"Explorer.Series.html#filter_with/2"},{"type":"function","title":"Examples - Explorer.Series.filter_with/2","doc":"iex> series = Explorer.Series.from_list([1, 2, 3])\n    iex> is_odd = fn s -> s |> Explorer.Series.remainder(2) |> Explorer.Series.equal(1) end\n    iex> Explorer.Series.filter_with(series, is_odd)\n    #Explorer.Series","ref":"Explorer.Series.html#filter_with/2-examples"},{"type":"function","title":"Explorer.Series.first/1","doc":"Returns the first element of the series.","ref":"Explorer.Series.html#first/1"},{"type":"function","title":"Examples - Explorer.Series.first/1","doc":"iex> s = 1..100 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.first(s)\n    1","ref":"Explorer.Series.html#first/1-examples"},{"type":"function","title":"Explorer.Series.floor/1","doc":"Floor floating point series to lowest integers smaller or equal to the float value.","ref":"Explorer.Series.html#floor/1"},{"type":"function","title":"Examples - Explorer.Series.floor/1","doc":"iex> s = Explorer.Series.from_list([1.124993, 2.555321, 3.995001])\n    iex> Explorer.Series.floor(s)\n    #Explorer.Series","ref":"Explorer.Series.html#floor/1-examples"},{"type":"function","title":"Explorer.Series.format/1","doc":"Returns a string series with all values concatenated.","ref":"Explorer.Series.html#format/1"},{"type":"function","title":"Examples - Explorer.Series.format/1","doc":"iex> s1 = Explorer.Series.from_list([\"a\", \"b\", \"c\"])\n    iex> s2 = Explorer.Series.from_list([\"d\", \"e\", \"f\"])\n    iex> s3 = Explorer.Series.from_list([\"g\", \"h\", \"i\"])\n    iex> Explorer.Series.format([s1, s2, s3])\n    #Explorer.Series \n\n    iex> s1 = Explorer.Series.from_list([\"a\", \"b\", \"c\", \"d\"])\n    iex> s2 = Explorer.Series.from_list([1, 2, 3, 4])\n    iex> s3 = Explorer.Series.from_list([1.5, :nan, :infinity, :neg_infinity])\n    iex> Explorer.Series.format([s1, \"/\", s2, \"/\", s3])\n    #Explorer.Series \n\n    iex> s1 = Explorer.Series.from_list([<<1>>, <<239, 191, 19>>], dtype: :binary)\n    iex> s2 = Explorer.Series.from_list([<<3>>, <<4>>], dtype: :binary)\n    iex> Explorer.Series.format([s1, s2])\n    ** (RuntimeError) Polars Error: invalid utf8","ref":"Explorer.Series.html#format/1-examples"},{"type":"function","title":"Explorer.Series.frequencies/1","doc":"Creates a new dataframe with unique values and the frequencies of each.","ref":"Explorer.Series.html#frequencies/1"},{"type":"function","title":"Examples - Explorer.Series.frequencies/1","doc":"iex> s = Explorer.Series.from_list([\"a\", \"a\", \"b\", \"c\", \"c\", \"c\"])\n    iex> Explorer.Series.frequencies(s)\n    #Explorer.DataFrame","ref":"Explorer.Series.html#frequencies/1-examples"},{"type":"function","title":"Explorer.Series.from_binary/3","doc":"Builds a series of `dtype` from `binary`.\n\nAll binaries must be in native endianness.","ref":"Explorer.Series.html#from_binary/3"},{"type":"function","title":"Options - Explorer.Series.from_binary/3","doc":"* `:backend` - The backend to allocate the series on.\n  * `:node` - The Erlang node to allocate the series on.","ref":"Explorer.Series.html#from_binary/3-options"},{"type":"function","title":"Examples - Explorer.Series.from_binary/3","doc":"Integers and floats follow their native encoding:\n\n    iex> Explorer.Series.from_binary(<<1.0::float-64-native, 2.0::float-64-native>>, {:f, 64})\n    #Explorer.Series \n\n    iex> Explorer.Series.from_binary(<<-1::signed-64-native, 1::signed-64-native>>, :integer)\n    #Explorer.Series \n\nBooleans are unsigned integers:\n\n    iex> Explorer.Series.from_binary(<<1, 0, 1>>, :boolean)\n    #Explorer.Series \n\nDates are encoded as s32 representing days from the Unix epoch (1970-01-01):\n\n    iex> binary = <<-719162::signed-32-native, 0::signed-32-native, 6129::signed-32-native>>\n    iex> Explorer.Series.from_binary(binary, :date)\n    #Explorer.Series \n\nTimes are encoded as s64 representing nanoseconds from midnight:\n\n    iex> binary = <<0::signed-64-native, 86399999999000::signed-64-native>>\n    iex> Explorer.Series.from_binary(binary, :time)\n    #Explorer.Series \n\nDatetimes are encoded as s64 representing microseconds from the Unix epoch (1970-01-01):\n\n    iex> binary = <<0::signed-64-native, 529550625987654::signed-64-native>>\n    iex> Explorer.Series.from_binary(binary, {:naive_datetime, :microsecond})\n    #Explorer.Series","ref":"Explorer.Series.html#from_binary/3-examples"},{"type":"function","title":"Explorer.Series.from_list/2","doc":"Creates a new series from a list.\n\nThe list must consist of a single data type and nils. It is possible to have\na list of only nil values. In this case, the list will have the `:dtype` of `:null`.","ref":"Explorer.Series.html#from_list/2"},{"type":"function","title":"Options - Explorer.Series.from_list/2","doc":"* `:backend` - The backend to allocate the series on.\n  * `:node` - The Erlang node to allocate the series on.\n  * `:dtype` - Create a series of a given `:dtype`. By default this is `nil`, which means\n    that Explorer will infer the type from the values in the list.\n    See the module docs for the list of valid dtypes and aliases.","ref":"Explorer.Series.html#from_list/2-options"},{"type":"function","title":"Examples - Explorer.Series.from_list/2","doc":"Explorer will infer the type from the values in the list.\nIntegers are always treated as `s64` and floats are always\ntreated as `f64`:\n\n    iex> Explorer.Series.from_list([1, 2, 3])\n    #Explorer.Series \n    iex> Explorer.Series.from_list([1.0, 2.0, 3.0])\n    #Explorer.Series \n\nSeries are nullable, so you may also include nils:\n\n    iex> Explorer.Series.from_list([1.0, nil, 2.5, 3.1])\n    #Explorer.Series \n\nA mix of integers and floats will be cast to a float:\n\n    iex> Explorer.Series.from_list([1, 2.0])\n    #Explorer.Series \n\nFloats series can accept NaN, Inf, and -Inf values:\n\n    iex> Explorer.Series.from_list([1.0, 2.0, :nan, 4.0])\n    #Explorer.Series \n\n    iex> Explorer.Series.from_list([1.0, 2.0, :infinity, 4.0])\n    #Explorer.Series \n\n    iex> Explorer.Series.from_list([1.0, 2.0, :neg_infinity, 4.0])\n    #Explorer.Series \n\nTrying to create an empty series or a series of nils will, by default,\nresult in a series of `:null` type:\n\n    iex> Explorer.Series.from_list([])\n    #Explorer.Series \n    iex> Explorer.Series.from_list([nil, nil])\n    #Explorer.Series \n\nA list of `Date`, `Time`, `NaiveDateTime`, `DateTime`, and\n`Explorer.Duration` structs are also supported, and they will become series\nwith the respective dtypes: `:date`, `:time`, `{:naive_datetime, :microsecond}`,\n`{:datetime, :microsecond, time_zone}` and `{:duration, precision}`.\nFor example:\n\n    iex> Explorer.Series.from_list([~D[0001-01-01], ~D[1970-01-01], ~D[1986-10-13]])\n    #Explorer.Series \n\nYou can specify the desired `dtype` for a series with the `:dtype` option.\n\n    iex> Explorer.Series.from_list([nil, nil], dtype: :integer)\n    #Explorer.Series \n\n    iex> Explorer.Series.from_list([1, 2], dtype: :f32)\n    #Explorer.Series \n\n    iex> Explorer.Series.from_list([1, nil, 2], dtype: :float)\n    #Explorer.Series \n\nThe `dtype` option is particulary important if a `:binary` series\nis desired, because by default binary series will have the dtype\nof `:string`:\n\n    iex> Explorer.Series.from_list([<<228, 146, 51>>, <<42, 209, 236>>], dtype: :binary)\n    #Explorer.Series >, <<42, 209, 236>>]\n    >\n\nA series mixing UTF8 strings and binaries is possible:\n\n    iex> Explorer.Series.from_list([<<228, 146, 51>>, \"Elixir\"], dtype: :binary)\n    #Explorer.Series >, \"Elixir\"]\n    >\n\nAnother option is to create a categorical series from a list of strings:\n\n    iex> Explorer.Series.from_list([\"EUA\", \"Brazil\", \"Poland\"], dtype: :category)\n    #Explorer.Series \n\nIt is possible to create a series of `:date` from a list of days since Unix Epoch.\n\n    iex> Explorer.Series.from_list([1, nil], dtype: :date)\n    #Explorer.Series \n\nIt is possible to create a series of `:datetime` from a list of microseconds since Unix Epoch.\n\n    iex> Explorer.Series.from_list([1649883642 * 1_000 * 1_000], dtype: {:naive_datetime, :microsecond})\n    #Explorer.Series \n\nIt is possible to create a series of `:time` from a list of nanoseconds since midnight.\n\n    iex> Explorer.Series.from_list([123 * 1_000 * 1_000 * 1_000], dtype: :time)\n    #Explorer.Series \n\nMixing non-numeric data types will raise an ArgumentError:\n\n    iex> Explorer.Series.from_list([1, \"a\"])\n    ** (ArgumentError) the value \"a\" does not match the inferred dtype {:s, 64}\n\nBut mixing integers and some of the types for `:date`, `:datetime`, `:time`, or `:duration`\nwill work if the desired dtype is given:\n\n    iex> Explorer.Series.from_list([1, nil, ~D[2024-06-13]], dtype: :date)\n    #Explorer.Series","ref":"Explorer.Series.html#from_list/2-examples"},{"type":"function","title":"Explorer.Series.from_tensor/2","doc":"Converts a `t:Nx.Tensor.t/0` to a series.\n\n> #### Warning {: .warning}\n>\n> `Nx` is an optional dependency. You will need to ensure it's installed to use this function.","ref":"Explorer.Series.html#from_tensor/2"},{"type":"function","title":"Options - Explorer.Series.from_tensor/2","doc":"* `:backend` - The backend to allocate the series on.\n  * `:node` - The Erlang node to allocate the series on.\n  * `:dtype` - The dtype of the series that must match the underlying tensor type.\n\n    The series can have a different dtype if the tensor is compatible with it.\n    For example, a tensor of `{:u, 8}` can represent a series of `:boolean` dtype.\n\n    Here are the list of compatible tensor types and dtypes:\n\n    * `{:u, 8}` tensor as a `:boolean` series.\n    * `{:s, 32}` tensor as a `:date` series.\n    * `{:s, 64}` tensor as a `:time` series.\n    * `{:s, 64}` tensor as a `{:naive_datetime, unit}` or `{:duration, unit}` series.","ref":"Explorer.Series.html#from_tensor/2-options"},{"type":"function","title":"Examples - Explorer.Series.from_tensor/2","doc":"Integers and floats:\n\n    iex> tensor = Nx.tensor([1, 2, 3])\n    iex> Explorer.Series.from_tensor(tensor)\n    #Explorer.Series \n\n    iex> tensor = Nx.tensor([1.0, 2.0, 3.0], type: :f64)\n    iex> Explorer.Series.from_tensor(tensor)\n    #Explorer.Series \n\n    iex> tensor = Nx.tensor([1, 0, 1], type: :u8)\n    iex> Explorer.Series.from_tensor(tensor)\n    #Explorer.Series \n\n    iex> tensor = Nx.tensor([-719162, 0, 6129], type: :s32)\n    iex> Explorer.Series.from_tensor(tensor)\n    #Explorer.Series \n\nBooleans can be read from a tensor of `{:u, 8}` type if the dtype is explicitly given:\n\n    iex> tensor = Nx.tensor([1, 0, 1], type: :u8)\n    iex> Explorer.Series.from_tensor(tensor, dtype: :boolean)\n    #Explorer.Series \n\nTimes are signed 64-bit representing nanoseconds from midnight and\ntherefore must have their dtype explicitly given:\n\n    iex> tensor = Nx.tensor([0, 86399999999000])\n    iex> Explorer.Series.from_tensor(tensor, dtype: :time)\n    #Explorer.Series \n\nDatetimes are signed 64-bit and therefore must have their dtype explicitly given:\n\n    iex> tensor = Nx.tensor([0, 529550625987654])\n    iex> Explorer.Series.from_tensor(tensor, dtype: {:naive_datetime, :microsecond})\n    #Explorer.Series","ref":"Explorer.Series.html#from_tensor/2-examples"},{"type":"function","title":"Explorer.Series.greater/2","doc":"Returns boolean mask of `left > right`, element-wise.\n\nAt least one of the arguments must be a series. If both\nsizes are series, the series must have the same size or\nat last one of them must have size of 1.","ref":"Explorer.Series.html#greater/2"},{"type":"function","title":"Supported dtypes - Explorer.Series.greater/2","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`\n  * `:date`\n  * `:time`\n  * `:datetime`\n  * `:duration`","ref":"Explorer.Series.html#greater/2-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.greater/2","doc":"iex> s1 = Explorer.Series.from_list([1, 2, 3])\n    iex> s2 = Explorer.Series.from_list([1, 2, 4])\n    iex> Explorer.Series.greater(s1, s2)\n    #Explorer.Series","ref":"Explorer.Series.html#greater/2-examples"},{"type":"function","title":"Explorer.Series.greater_equal/2","doc":"Returns boolean mask of `left >= right`, element-wise.\n\nAt least one of the arguments must be a series. If both\nsizes are series, the series must have the same size or\nat last one of them must have size of 1.","ref":"Explorer.Series.html#greater_equal/2"},{"type":"function","title":"Supported dtypes - Explorer.Series.greater_equal/2","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`\n  * `:date`\n  * `:time`\n  * `:datetime`\n  * `:duration`","ref":"Explorer.Series.html#greater_equal/2-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.greater_equal/2","doc":"iex> s1 = Explorer.Series.from_list([1, 2, 3])\n    iex> s2 = Explorer.Series.from_list([1, 2, 4])\n    iex> Explorer.Series.greater_equal(s1, s2)\n    #Explorer.Series","ref":"Explorer.Series.html#greater_equal/2-examples"},{"type":"function","title":"Explorer.Series.head/2","doc":"Returns the first N elements of the series.","ref":"Explorer.Series.html#head/2"},{"type":"function","title":"Examples - Explorer.Series.head/2","doc":"iex> s = 1..100 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.head(s)\n    #Explorer.Series","ref":"Explorer.Series.html#head/2-examples"},{"type":"function","title":"Explorer.Series.hour/1","doc":"Returns the hour number from 0 to 23.","ref":"Explorer.Series.html#hour/1"},{"type":"function","title":"Examples - Explorer.Series.hour/1","doc":"iex> s = Explorer.Series.from_list([~N[2023-01-15 00:00:00], ~N[2022-02-16 23:59:59.999999], ~N[2021-03-20 12:00:00], nil])\n    iex> Explorer.Series.hour(s)\n    #Explorer.Series","ref":"Explorer.Series.html#hour/1-examples"},{"type":"function","title":"Explorer.Series.in/2","doc":"Checks if each element of the series in the left exists in the series\non the right, returning a boolean mask.\n\nThe series sizes do not have to match.\n\nSee `member?/2` if you want to check if a literal belongs to a list.","ref":"Explorer.Series.html#in/2"},{"type":"function","title":"Examples - Explorer.Series.in/2","doc":"iex> left = Explorer.Series.from_list([1, 2, 3])\n    iex> right = Explorer.Series.from_list([1, 2])\n    iex> Series.in(left, right)\n    #Explorer.Series \n\n    iex> left = Explorer.Series.from_list([~D[1970-01-01], ~D[2000-01-01], ~D[2010-04-17]])\n    iex> right = Explorer.Series.from_list([~D[1970-01-01], ~D[2010-04-17]])\n    iex> Series.in(left, right)\n    #Explorer.Series","ref":"Explorer.Series.html#in/2-examples"},{"type":"function","title":"Explorer.Series.iotype/1","doc":"Returns the type of the underlying fixed-width binary representation.\n\nIt returns something in the shape of `{atom(), bits_size}` or `:none`.\nIt is often used in conjunction with `to_iovec/1` and `to_binary/1`.\n\nThe possible iotypes are:\n\n* `:u` for unsigned integers.\n* `:s` for signed integers.\n* `:f` for floats.","ref":"Explorer.Series.html#iotype/1"},{"type":"function","title":"Examples - Explorer.Series.iotype/1","doc":"iex> s = Explorer.Series.from_list([1, 2, 3, 4])\n    iex> Explorer.Series.iotype(s)\n    {:s, 64}\n\n    iex> s = Explorer.Series.from_list([~D[1999-12-31], ~D[1989-01-01]])\n    iex> Explorer.Series.iotype(s)\n    {:s, 32}\n\n    iex> s = Explorer.Series.from_list([~T[00:00:00.000000], ~T[23:59:59.999999]])\n    iex> Explorer.Series.iotype(s)\n    {:s, 64}\n\n    iex> s = Explorer.Series.from_list([1.2, 2.3, 3.5, 4.5])\n    iex> Explorer.Series.iotype(s)\n    {:f, 64}\n\n    iex> s = Explorer.Series.from_list([true, false, true])\n    iex> Explorer.Series.iotype(s)\n    {:u, 8}\n\nThe operation returns `:none` for strings and binaries, as they do not\nprovide a fixed-width binary representation:\n\n    iex> s = Explorer.Series.from_list([\"a\", \"b\", \"c\"])\n    iex> Explorer.Series.iotype(s)\n    :none\n\nHowever, if appropriate, you can convert them to categorical types,\nwhich will then return the index of each category:\n\n    iex> s = Explorer.Series.from_list([\"a\", \"b\", \"c\"], dtype: :category)\n    iex> Explorer.Series.iotype(s)\n    {:u, 32}","ref":"Explorer.Series.html#iotype/1-examples"},{"type":"function","title":"Explorer.Series.is_finite/1","doc":"Returns a mask of finite values.","ref":"Explorer.Series.html#is_finite/1"},{"type":"function","title":"Examples - Explorer.Series.is_finite/1","doc":"iex> s1 = Explorer.Series.from_list([1, 2, 0, nil])\n    iex> s2 = Explorer.Series.from_list([0, 2, 0, nil])\n    iex> s3 = Explorer.Series.divide(s1, s2)\n    iex> Explorer.Series.is_finite(s3)\n    #Explorer.Series","ref":"Explorer.Series.html#is_finite/1-examples"},{"type":"function","title":"Explorer.Series.is_infinite/1","doc":"Returns a mask of infinite values.","ref":"Explorer.Series.html#is_infinite/1"},{"type":"function","title":"Examples - Explorer.Series.is_infinite/1","doc":"iex> s1 = Explorer.Series.from_list([1, -1, 2, 0, nil])\n    iex> s2 = Explorer.Series.from_list([0, 0, 2, 0, nil])\n    iex> s3 = Explorer.Series.divide(s1, s2)\n    iex> Explorer.Series.is_infinite(s3)\n    #Explorer.Series","ref":"Explorer.Series.html#is_infinite/1-examples"},{"type":"function","title":"Explorer.Series.is_nan/1","doc":"Returns a mask of nan values.","ref":"Explorer.Series.html#is_nan/1"},{"type":"function","title":"Examples - Explorer.Series.is_nan/1","doc":"iex> s1 = Explorer.Series.from_list([1, 2, 0, nil])\n    iex> s2 = Explorer.Series.from_list([0, 2, 0, nil])\n    iex> s3 = Explorer.Series.divide(s1, s2)\n    iex> Explorer.Series.is_nan(s3)\n    #Explorer.Series","ref":"Explorer.Series.html#is_nan/1-examples"},{"type":"function","title":"Explorer.Series.is_nil/1","doc":"Returns a mask of nil values.","ref":"Explorer.Series.html#is_nil/1"},{"type":"function","title":"Examples - Explorer.Series.is_nil/1","doc":"iex> s = Explorer.Series.from_list([1, 2, nil, 4])\n    iex> Explorer.Series.is_nil(s)\n    #Explorer.Series","ref":"Explorer.Series.html#is_nil/1-examples"},{"type":"function","title":"Explorer.Series.is_not_nil/1","doc":"Returns a mask of not nil values.","ref":"Explorer.Series.html#is_not_nil/1"},{"type":"function","title":"Examples - Explorer.Series.is_not_nil/1","doc":"iex> s = Explorer.Series.from_list([1, 2, nil, 4])\n    iex> Explorer.Series.is_not_nil(s)\n    #Explorer.Series","ref":"Explorer.Series.html#is_not_nil/1-examples"},{"type":"function","title":"Explorer.Series.join/2","doc":"Join all string items in a sublist and place a separator between them.","ref":"Explorer.Series.html#join/2"},{"type":"function","title":"Examples - Explorer.Series.join/2","doc":"iex> s = Series.from_list([[\"1\"], [\"1\", \"2\"]])\n    iex> Series.join(s, \"|\")\n    #Explorer.Series","ref":"Explorer.Series.html#join/2-examples"},{"type":"function","title":"Explorer.Series.json_decode/2","doc":"Decodes a string series containing valid JSON according to `dtype`.","ref":"Explorer.Series.html#json_decode/2"},{"type":"function","title":"Examples - Explorer.Series.json_decode/2","doc":"iex> s = Series.from_list([\"1\"])\n    iex> Series.json_decode(s, {:s, 64})\n    #Explorer.Series \n\n    iex> s = Series.from_list([\"{\\\"a\\\":1}\"])\n    iex> Series.json_decode(s, {:struct, [{\"a\", {:s, 64}}]})\n    #Explorer.Series \n\nIf the decoded value does not match the given `dtype`,\nnil is returned for the given entry:\n\n    iex> s = Series.from_list([\"\\\"1\\\"\"])\n    iex> Series.json_decode(s, {:s, 64})\n    #Explorer.Series \n\nIt raises an exception if the string is invalid JSON.","ref":"Explorer.Series.html#json_decode/2-examples"},{"type":"function","title":"Explorer.Series.json_path_match/2","doc":"Extracts a string series using a [`json_path`](https://goessner.net/articles/JsonPath/) from a series.","ref":"Explorer.Series.html#json_path_match/2"},{"type":"function","title":"Examples - Explorer.Series.json_path_match/2","doc":"iex> s = Series.from_list([\"{\\\"a\\\":1}\", \"{\\\"a\\\":2}\"])\n    iex> Series.json_path_match(s, \"$.a\")\n    #Explorer.Series \n\nIf `json_path` is not found or if the string is invalid JSON or `nil`,\nnil is returned for the given entry:\n\n    iex> s = Series.from_list([\"{\\\"a\\\":1}\", nil, \"{\\\"a\\\":2}\"])\n    iex> Series.json_path_match(s, \"$.b\")\n    #Explorer.Series \n\nIt raises an exception if the `json_path` is invalid.","ref":"Explorer.Series.html#json_path_match/2-examples"},{"type":"function","title":"Explorer.Series.last/1","doc":"Returns the last element of the series.","ref":"Explorer.Series.html#last/1"},{"type":"function","title":"Examples - Explorer.Series.last/1","doc":"iex> s = 1..100 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.last(s)\n    100","ref":"Explorer.Series.html#last/1-examples"},{"type":"function","title":"Explorer.Series.lengths/1","doc":"Calculates the length of each list in a list series.","ref":"Explorer.Series.html#lengths/1"},{"type":"function","title":"Examples - Explorer.Series.lengths/1","doc":"iex> s = Series.from_list([[1], [1, 2]])\n    iex> Series.lengths(s)\n    #Explorer.Series","ref":"Explorer.Series.html#lengths/1-examples"},{"type":"function","title":"Explorer.Series.less/2","doc":"Returns boolean mask of `left < right`, element-wise.\n\nAt least one of the arguments must be a series. If both\nsizes are series, the series must have the same size or\nat last one of them must have size of 1.","ref":"Explorer.Series.html#less/2"},{"type":"function","title":"Supported dtypes - Explorer.Series.less/2","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`\n  * `:date`\n  * `:time`\n  * `:datetime`\n  * `:duration`","ref":"Explorer.Series.html#less/2-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.less/2","doc":"iex> s1 = Explorer.Series.from_list([1, 2, 3])\n    iex> s2 = Explorer.Series.from_list([1, 2, 4])\n    iex> Explorer.Series.less(s1, s2)\n    #Explorer.Series","ref":"Explorer.Series.html#less/2-examples"},{"type":"function","title":"Explorer.Series.less_equal/2","doc":"Returns boolean mask of `left <= right`, element-wise.\n\nAt least one of the arguments must be a series. If both\nsizes are series, the series must have the same size or\nat last one of them must have size of 1.","ref":"Explorer.Series.html#less_equal/2"},{"type":"function","title":"Supported dtypes - Explorer.Series.less_equal/2","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`\n  * `:date`\n  * `:time`\n  * `:datetime`\n  * `:duration`","ref":"Explorer.Series.html#less_equal/2-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.less_equal/2","doc":"iex> s1 = Explorer.Series.from_list([1, 2, 3])\n    iex> s2 = Explorer.Series.from_list([1, 2, 4])\n    iex> Explorer.Series.less_equal(s1, s2)\n    #Explorer.Series","ref":"Explorer.Series.html#less_equal/2-examples"},{"type":"function","title":"Explorer.Series.log/1","doc":"Calculates the natural logarithm.\n\nThe resultant series is going to be of dtype `{:f, 64}`.\nSee `log/2` for passing a custom base.","ref":"Explorer.Series.html#log/1"},{"type":"function","title":"Supported dtypes - Explorer.Series.log/1","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`","ref":"Explorer.Series.html#log/1-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.log/1","doc":"iex> s = Explorer.Series.from_list([1, 2, 3, nil, 4])\n    iex> Explorer.Series.log(s)\n    #Explorer.Series","ref":"Explorer.Series.html#log/1-examples"},{"type":"function","title":"Explorer.Series.log/2","doc":"Calculates the logarithm on a given base.\n\nThe resultant series is going to be of dtype `{:f, 64}`.","ref":"Explorer.Series.html#log/2"},{"type":"function","title":"Supported dtypes - Explorer.Series.log/2","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`","ref":"Explorer.Series.html#log/2-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.log/2","doc":"iex> s = Explorer.Series.from_list([8, 16, 32])\n    iex> Explorer.Series.log(s, 2)\n    #Explorer.Series","ref":"Explorer.Series.html#log/2-examples"},{"type":"function","title":"Explorer.Series.lstrip/1","doc":"Returns a string series where all leading Unicode whitespaces have been removed.","ref":"Explorer.Series.html#lstrip/1"},{"type":"function","title":"Examples - Explorer.Series.lstrip/1","doc":"iex> s = Explorer.Series.from_list([\"  abc\", \"def  \", \"  bcd\"])\n    iex> Explorer.Series.lstrip(s)\n    #Explorer.Series","ref":"Explorer.Series.html#lstrip/1-examples"},{"type":"function","title":"Explorer.Series.lstrip/2","doc":"Returns a string series where all leading examples of the provided string\nhave been removed.\n\nWhere multiple characters are provided, all combinations of this set of characters will be stripped","ref":"Explorer.Series.html#lstrip/2"},{"type":"function","title":"Examples - Explorer.Series.lstrip/2","doc":"iex> s = Explorer.Series.from_list([\"$1\", \"$$200$$\", \"$$$3000$\"])\n    iex> Explorer.Series.lstrip(s, \"$\")\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([\"abc\", \"adefa\", \"bcda\"])\n    iex> Explorer.Series.lstrip(s, \"ab\")\n    #Explorer.Series","ref":"Explorer.Series.html#lstrip/2-examples"},{"type":"macro","title":"Explorer.Series.map/2","doc":"Maps values based on an `Explorer.Query`.\n\nThe query is compiled and runs efficiently against the series.\n\n> #### Notice {: .notice}\n>\n> This is a macro.\n>\n>   * You must `require Explorer.Series` before using it.\n>   * You must use the special `_` syntax. See the moduledoc for details.\n\nSee `map_with/2` for a callback version of this function without `Explorer.Query`.\n\nThis function only works with lazy computations.\nSee `transform/2` for a version that works with any Elixir function.","ref":"Explorer.Series.html#map/2"},{"type":"macro","title":"Examples - Explorer.Series.map/2","doc":"Basic example:\n\n    iex> s = Explorer.Series.from_list([1, 2, 3])\n    iex> Explorer.Series.map(s, _ * 2)\n    #Explorer.Series \n\nYou can also use window functions and aggregations:\n\n    iex> s = Explorer.Series.from_list([2, 3, 4])\n    iex> Explorer.Series.map(s, _ - min(_))\n    #Explorer.Series","ref":"Explorer.Series.html#map/2-examples"},{"type":"function","title":"Explorer.Series.map_with/2","doc":"Maps a series with a callback function.\n\nThis function only works with lazy computations.\nSee `transform/2` for a version that works with any Elixir function.","ref":"Explorer.Series.html#map_with/2"},{"type":"function","title":"Examples - Explorer.Series.map_with/2","doc":"iex> series = Explorer.Series.from_list([2, 3, 4])\n    iex> shift_left = fn s -> Explorer.Series.subtract(s, Explorer.Series.min(s)) end\n    iex> Explorer.Series.map_with(series, shift_left)\n    #Explorer.Series","ref":"Explorer.Series.html#map_with/2-examples"},{"type":"function","title":"Explorer.Series.mask/2","doc":"Filters a series with a mask.","ref":"Explorer.Series.html#mask/2"},{"type":"function","title":"Examples - Explorer.Series.mask/2","doc":"iex> s1 = Explorer.Series.from_list([1,2,3])\n    iex> s2 = Explorer.Series.from_list([true, false, true])\n    iex> Explorer.Series.mask(s1, s2)\n    #Explorer.Series","ref":"Explorer.Series.html#mask/2-examples"},{"type":"function","title":"Explorer.Series.max/1","doc":"Gets the maximum value of the series.","ref":"Explorer.Series.html#max/1"},{"type":"function","title":"Supported dtypes - Explorer.Series.max/1","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`\n  * `:date`\n  * `:time`\n  * `:naive_datetime`\n  * `:datetime`\n  * `:duration`","ref":"Explorer.Series.html#max/1-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.max/1","doc":"iex> s = Explorer.Series.from_list([1, 2, nil, 3])\n    iex> Explorer.Series.max(s)\n    3\n\n    iex> s = Explorer.Series.from_list([1.0, 2.0, nil, 3.0])\n    iex> Explorer.Series.max(s)\n    3.0\n\n    iex> s = Explorer.Series.from_list([~D[2021-01-01], ~D[1999-12-31]])\n    iex> Explorer.Series.max(s)\n    ~D[2021-01-01]\n\n    iex> s = Explorer.Series.from_list([~N[2021-01-01 00:00:00], ~N[1999-12-31 00:00:00]])\n    iex> Explorer.Series.max(s)\n    ~N[2021-01-01 00:00:00.000000]\n\n    iex> s = Explorer.Series.from_list([~T[00:02:03.000212], ~T[00:05:04.000456]])\n    iex> Explorer.Series.max(s)\n    ~T[00:05:04.000456]\n\n    iex> s = Explorer.Series.from_list([\"a\", \"b\", \"c\"])\n    iex> Explorer.Series.max(s)\n    ** (ArgumentError) Explorer.Series.max/1 not implemented for dtype :string. Valid dtypes are :date, :time, {:duration, :microsecond}, {:duration, :millisecond}, {:duration, :nanosecond}, {:f, 32}, {:f, 64}, {:naive_datetime, :microsecond}, {:naive_datetime, :millisecond}, {:naive_datetime, :nanosecond}, {:s, 8}, {:s, 16}, {:s, 32}, {:s, 64}, {:u, 8}, {:u, 16}, {:u, 32} and {:u, 64}","ref":"Explorer.Series.html#max/1-examples"},{"type":"function","title":"Explorer.Series.mean/1","doc":"Gets the mean value of the series.","ref":"Explorer.Series.html#mean/1"},{"type":"function","title":"Supported dtypes - Explorer.Series.mean/1","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`","ref":"Explorer.Series.html#mean/1-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.mean/1","doc":"iex> s = Explorer.Series.from_list([1, 2, nil, 3])\n    iex> Explorer.Series.mean(s)\n    2.0\n\n    iex> s = Explorer.Series.from_list([1.0, 2.0, nil, 3.0])\n    iex> Explorer.Series.mean(s)\n    2.0\n\n    iex> s = Explorer.Series.from_list([~D[2021-01-01], ~D[1999-12-31]])\n    iex> Explorer.Series.mean(s)\n    ** (ArgumentError) Explorer.Series.mean/1 not implemented for dtype :date. Valid dtypes are {:f, 32}, {:f, 64}, {:s, 8}, {:s, 16}, {:s, 32}, {:s, 64}, {:u, 8}, {:u, 16}, {:u, 32} and {:u, 64}","ref":"Explorer.Series.html#mean/1-examples"},{"type":"function","title":"Explorer.Series.median/1","doc":"Gets the median value of the series.","ref":"Explorer.Series.html#median/1"},{"type":"function","title":"Supported dtypes - Explorer.Series.median/1","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`","ref":"Explorer.Series.html#median/1-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.median/1","doc":"iex> s = Explorer.Series.from_list([1, 2, nil, 3])\n    iex> Explorer.Series.median(s)\n    2.0\n\n    iex> s = Explorer.Series.from_list([1.0, 2.0, nil, 3.0])\n    iex> Explorer.Series.median(s)\n    2.0\n\n    iex> s = Explorer.Series.from_list([~D[2021-01-01], ~D[1999-12-31]])\n    iex> Explorer.Series.median(s)\n    ** (ArgumentError) Explorer.Series.median/1 not implemented for dtype :date. Valid dtypes are {:f, 32}, {:f, 64}, {:s, 8}, {:s, 16}, {:s, 32}, {:s, 64}, {:u, 8}, {:u, 16}, {:u, 32} and {:u, 64}","ref":"Explorer.Series.html#median/1-examples"},{"type":"function","title":"Explorer.Series.member?/2","doc":"Checks for the presence of a value in a list series.","ref":"Explorer.Series.html#member?/2"},{"type":"function","title":"Examples - Explorer.Series.member?/2","doc":"iex> s = Series.from_list([[1], [1, 2]])\n    iex> Series.member?(s, 2)\n    #Explorer.Series","ref":"Explorer.Series.html#member?/2-examples"},{"type":"function","title":"Explorer.Series.min/1","doc":"Gets the minimum value of the series.","ref":"Explorer.Series.html#min/1"},{"type":"function","title":"Supported dtypes - Explorer.Series.min/1","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`\n  * `:date`\n  * `:time`\n  * `:datetime`\n  * `:duration`","ref":"Explorer.Series.html#min/1-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.min/1","doc":"iex> s = Explorer.Series.from_list([1, 2, nil, 3])\n    iex> Explorer.Series.min(s)\n    1\n\n    iex> s = Explorer.Series.from_list([1.0, 2.0, nil, 3.0])\n    iex> Explorer.Series.min(s)\n    1.0\n\n    iex> s = Explorer.Series.from_list([~D[2021-01-01], ~D[1999-12-31]])\n    iex> Explorer.Series.min(s)\n    ~D[1999-12-31]\n\n    iex> s = Explorer.Series.from_list([~N[2021-01-01 00:00:00], ~N[1999-12-31 00:00:00]])\n    iex> Explorer.Series.min(s)\n    ~N[1999-12-31 00:00:00.000000]\n\n    iex> s = Explorer.Series.from_list([~T[00:02:03.000451], ~T[00:05:04.000134]])\n    iex> Explorer.Series.min(s)\n    ~T[00:02:03.000451]\n\n    iex> s = Explorer.Series.from_list([\"a\", \"b\", \"c\"])\n    iex> Explorer.Series.min(s)\n    ** (ArgumentError) Explorer.Series.min/1 not implemented for dtype :string. Valid dtypes are :date, :time, {:duration, :microsecond}, {:duration, :millisecond}, {:duration, :nanosecond}, {:f, 32}, {:f, 64}, {:naive_datetime, :microsecond}, {:naive_datetime, :millisecond}, {:naive_datetime, :nanosecond}, {:s, 8}, {:s, 16}, {:s, 32}, {:s, 64}, {:u, 8}, {:u, 16}, {:u, 32} and {:u, 64}","ref":"Explorer.Series.html#min/1-examples"},{"type":"function","title":"Explorer.Series.minute/1","doc":"Returns the minute number from 0 to 59.","ref":"Explorer.Series.html#minute/1"},{"type":"function","title":"Examples - Explorer.Series.minute/1","doc":"iex> s = Explorer.Series.from_list([~N[2023-01-15 00:00:00], ~N[2022-02-16 23:59:59.999999], ~N[2021-03-20 12:00:00], nil])\n    iex> Explorer.Series.minute(s)\n    #Explorer.Series","ref":"Explorer.Series.html#minute/1-examples"},{"type":"function","title":"Explorer.Series.mode/1","doc":"Gets the most common value(s) of the series.\n\nThis function will return multiple values when there's a tie.","ref":"Explorer.Series.html#mode/1"},{"type":"function","title":"Supported dtypes - Explorer.Series.mode/1","doc":"All except `:list` and `:struct`.","ref":"Explorer.Series.html#mode/1-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.mode/1","doc":"iex> s = Explorer.Series.from_list([1, 2, 2, nil])\n    iex> Explorer.Series.mode(s)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([\"a\", \"b\", \"b\", \"c\"])\n    iex> Explorer.Series.mode(s)\n    #Explorer.Series \n\nThis function can return multiple entries, but the order is not guaranteed.\nYou may sort the series if desired.\n\n    iex> s = Explorer.Series.from_list([1.0, 2.0, 2.0, 3.0, 3.0])\n    iex> Explorer.Series.mode(s) |> Explorer.Series.sort()\n    #Explorer.Series","ref":"Explorer.Series.html#mode/1-examples"},{"type":"function","title":"Explorer.Series.month/1","doc":"Returns the month number starting from 1. The return value ranges from 1 to 12.","ref":"Explorer.Series.html#month/1"},{"type":"function","title":"Examples - Explorer.Series.month/1","doc":"iex> s = Explorer.Series.from_list([~D[2023-01-15], ~D[2023-02-16], ~D[2023-03-20], nil])\n    iex> Explorer.Series.month(s)\n    #Explorer.Series \n\nIt can also be called on a datetime series.\n\n    iex> s = Explorer.Series.from_list([~N[2023-01-15 00:00:00], ~N[2023-02-16 23:59:59.999999], ~N[2023-03-20 12:00:00], nil])\n    iex> Explorer.Series.month(s)\n    #Explorer.Series","ref":"Explorer.Series.html#month/1-examples"},{"type":"function","title":"Explorer.Series.multiply/2","doc":"Multiplies left and right, element-wise.\n\nWhen mixing floats and integers, the resulting series will have dtype `{:f, 64}`.\n\nAt least one of the arguments must be a series. If both\nsizes are series, the series must have the same size or\nat last one of them must have size of 1.","ref":"Explorer.Series.html#multiply/2"},{"type":"function","title":"Supported dtypes - Explorer.Series.multiply/2","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`","ref":"Explorer.Series.html#multiply/2-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.multiply/2","doc":"iex> s1 = 1..10 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> s2 = 11..20 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.multiply(s1, s2)\n    #Explorer.Series \n\n    iex> s1 = 1..5 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.multiply(s1, 2)\n    #Explorer.Series","ref":"Explorer.Series.html#multiply/2-examples"},{"type":"function","title":"Explorer.Series.n_distinct/1","doc":"Returns the number of unique values in the series.","ref":"Explorer.Series.html#n_distinct/1"},{"type":"function","title":"Examples - Explorer.Series.n_distinct/1","doc":"iex> s = Explorer.Series.from_list([\"a\", \"b\", \"a\", \"b\"])\n    iex> Explorer.Series.n_distinct(s)\n    2","ref":"Explorer.Series.html#n_distinct/1-examples"},{"type":"function","title":"Explorer.Series.nil_count/1","doc":"Counts the number of `nil` elements in a series.\n\nWhen used in a query on grouped data, `count_nil/1` is a per-group operation.\n\nSee also:\n\n  * `count/1` - counts only the non-`nil` elements.\n  * `size/1` - counts all elements.","ref":"Explorer.Series.html#nil_count/1"},{"type":"function","title":"Examples - Explorer.Series.nil_count/1","doc":"Without `nil`s:\n\n    iex> s = Explorer.Series.from_list([\"a\", \"b\", \"c\"])\n    iex> Explorer.Series.nil_count(s)\n    0\n\nWith `nil`s:\n\n    iex> s = Explorer.Series.from_list([\"a\", nil, \"c\"])\n    iex> Explorer.Series.nil_count(s)\n    1\n\nWith `:nan`s (`:nan` does not count as `nil`):\n\n    iex> s = Explorer.Series.from_list([1, :nan, 3])\n    iex> Explorer.Series.nil_count(s)\n    0","ref":"Explorer.Series.html#nil_count/1-examples"},{"type":"function","title":"Explorer.Series.not/1","doc":"Negate the elements of a boolean series.","ref":"Explorer.Series.html#not/1"},{"type":"function","title":"Examples - Explorer.Series.not/1","doc":"iex> s1 = Explorer.Series.from_list([true, false, false])\n    iex> Explorer.Series.not(s1)\n    #Explorer.Series","ref":"Explorer.Series.html#not/1-examples"},{"type":"function","title":"Explorer.Series.not_equal/2","doc":"Returns boolean mask of `left != right`, element-wise.\n\nAt least one of the arguments must be a series. If both\nsizes are series, the series must have the same size or\nat last one of them must have size of 1.","ref":"Explorer.Series.html#not_equal/2"},{"type":"function","title":"Examples - Explorer.Series.not_equal/2","doc":"iex> s1 = Explorer.Series.from_list([1, 2, 3])\n    iex> s2 = Explorer.Series.from_list([1, 2, 4])\n    iex> Explorer.Series.not_equal(s1, s2)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([1, 2, 3])\n    iex> Explorer.Series.not_equal(s, 1)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([true, true, false])\n    iex> Explorer.Series.not_equal(s, true)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([\"a\", \"b\", \"c\"])\n    iex> Explorer.Series.not_equal(s, \"a\")\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([~D[2021-01-01], ~D[1999-12-31]])\n    iex> Explorer.Series.not_equal(s, ~D[1999-12-31])\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([~N[2022-01-01 00:00:00], ~N[2022-01-01 23:00:00]])\n    iex> Explorer.Series.not_equal(s, ~N[2022-01-01 00:00:00])\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([\"a\", \"b\", \"c\"])\n    iex> Explorer.Series.not_equal(s, false)\n    ** (ArgumentError) cannot invoke Explorer.Series.not_equal/2 with mismatched dtypes: :string and false","ref":"Explorer.Series.html#not_equal/2-examples"},{"type":"function","title":"Explorer.Series.or/2","doc":"Returns a boolean mask of `left or right`, element-wise.\n\nBoth sizes must be series, the series must have the same\nsize or at last one of them must have size of 1.","ref":"Explorer.Series.html#or/2"},{"type":"function","title":"Examples - Explorer.Series.or/2","doc":"iex> s1 = Explorer.Series.from_list([1, 2, 3])\n    iex> mask1 = Explorer.Series.less(s1, 2)\n    iex> mask2 = Explorer.Series.greater(s1, 2)\n    iex> Explorer.Series.or(mask1, mask2)\n    #Explorer.Series","ref":"Explorer.Series.html#or/2-examples"},{"type":"function","title":"Explorer.Series.peaks/2","doc":"Returns a boolean mask with `true` where the 'peaks' (series max or min, default max) are.","ref":"Explorer.Series.html#peaks/2"},{"type":"function","title":"Supported dtypes - Explorer.Series.peaks/2","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`\n  * `:date`\n  * `:time`\n  * `:datetime`\n  * `:duration`","ref":"Explorer.Series.html#peaks/2-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.peaks/2","doc":"iex> s = Explorer.Series.from_list([1, 2, 4, 1, 4])\n    iex> Explorer.Series.peaks(s)\n    #Explorer.Series \n\n    iex> s = [~T[03:00:02.000000], ~T[13:24:56.000000], ~T[02:04:19.000000]] |> Explorer.Series.from_list()\n    iex> Explorer.Series.peaks(s)\n    #Explorer.Series","ref":"Explorer.Series.html#peaks/2-examples"},{"type":"function","title":"Explorer.Series.pow/2","doc":"Raises a numeric series to the power of the exponent.\n\nAt least one of the arguments must be a series. If both\nsides are series, the series must have the same size or\nat last one of them must have size of 1.\n\nNote that this operation can fail if the exponent is a\nsigned integer series or scalar containing negative values,\nand the base is also of an integer type.","ref":"Explorer.Series.html#pow/2"},{"type":"function","title":"Supported dtypes - Explorer.Series.pow/2","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`","ref":"Explorer.Series.html#pow/2-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.pow/2","doc":"iex> s = [8, 16, 32] |> Explorer.Series.from_list()\n    iex> Explorer.Series.pow(s, 2.0)\n    #Explorer.Series \n\n    iex> s = [2, 4, 6] |> Explorer.Series.from_list()\n    iex> Explorer.Series.pow(s, 3)\n    #Explorer.Series \n\n    iex> s = [2, 4, 6] |> Explorer.Series.from_list()\n    iex> Explorer.Series.pow(s, -3.0)\n    #Explorer.Series \n\n    iex> s = [1.0, 2.0, 3.0] |> Explorer.Series.from_list()\n    iex> Explorer.Series.pow(s, 3.0)\n    #Explorer.Series \n\n    iex> s = [2.0, 4.0, 6.0] |> Explorer.Series.from_list()\n    iex> Explorer.Series.pow(s, 2)\n    #Explorer.Series","ref":"Explorer.Series.html#pow/2-examples"},{"type":"function","title":"Explorer.Series.product/1","doc":"Reduce this Series to the product value.\n\nNote that an empty series is going to result in a\nproduct of `1`. Values that are `nil` are ignored.","ref":"Explorer.Series.html#product/1"},{"type":"function","title":"Supported dtypes - Explorer.Series.product/1","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`","ref":"Explorer.Series.html#product/1-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.product/1","doc":"iex> s = Explorer.Series.from_list([1, 2, 3])\n    iex> Explorer.Series.product(s)\n    6\n\n    iex> s = Explorer.Series.from_list([], dtype: :float)\n    iex> Explorer.Series.product(s)\n    1.0\n\n    iex> s = Explorer.Series.from_list([true, false, true])\n    iex> Explorer.Series.product(s)\n    ** (ArgumentError) Explorer.Series.product/1 not implemented for dtype :boolean. Valid dtypes are {:f, 32}, {:f, 64}, {:s, 8}, {:s, 16}, {:s, 32}, {:s, 64}, {:u, 8}, {:u, 16}, {:u, 32} and {:u, 64}","ref":"Explorer.Series.html#product/1-examples"},{"type":"function","title":"Explorer.Series.qcut/3","doc":"Bins values into discrete values base on their quantiles.\n\nGiven a `quantiles` length of N, there will be N+1 categories. Each\nelement of `quantiles` is expected to be between 0.0 and 1.0.","ref":"Explorer.Series.html#qcut/3"},{"type":"function","title":"Options - Explorer.Series.qcut/3","doc":"* `:labels` - The labels assigned to the bins. Given `bins` of\n    length N, `:labels` must be of length N+1. Defaults to the bin\n    bounds (e.g. `(-inf -1.0]`, `(-1.0, 1.0]`, `(1.0, inf]`)\n\n  * `:break_point_label` - The name given to the breakpoint column.\n    Defaults to `break_point`.\n\n  * `:category_label` - The name given to the category column.\n    Defaults to `category`.","ref":"Explorer.Series.html#qcut/3-options"},{"type":"function","title":"Examples - Explorer.Series.qcut/3","doc":"iex> s = Explorer.Series.from_list([1.0, 2.0, 3.0, 4.0, 5.0])\n    iex> Explorer.Series.qcut(s, [0.25, 0.75])\n    #Explorer.DataFrame","ref":"Explorer.Series.html#qcut/3-examples"},{"type":"function","title":"Explorer.Series.quantile/2","doc":"Gets the given quantile of the series.","ref":"Explorer.Series.html#quantile/2"},{"type":"function","title":"Supported dtypes - Explorer.Series.quantile/2","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`\n  * `:date`\n  * `:time`\n  * `:datetime`\n  * `:duration`","ref":"Explorer.Series.html#quantile/2-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.quantile/2","doc":"iex> s = Explorer.Series.from_list([1, 2, nil, 3])\n    iex> Explorer.Series.quantile(s, 0.2)\n    1\n\n    iex> s = Explorer.Series.from_list([1.0, 2.0, nil, 3.0])\n    iex> Explorer.Series.quantile(s, 0.5)\n    2.0\n\n    iex> s = Explorer.Series.from_list([~D[2021-01-01], ~D[1999-12-31]])\n    iex> Explorer.Series.quantile(s, 0.5)\n    ~D[2021-01-01]\n\n    iex> s = Explorer.Series.from_list([~N[2021-01-01 00:00:00], ~N[1999-12-31 00:00:00]])\n    iex> Explorer.Series.quantile(s, 0.5)\n    ~N[2021-01-01 00:00:00.000000]\n\n    iex> s = Explorer.Series.from_list([~T[01:55:00], ~T[15:35:00], ~T[23:00:00]])\n    iex> Explorer.Series.quantile(s, 0.5)\n    ~T[15:35:00]\n\n    iex> s = Explorer.Series.from_list([true, false, true])\n    iex> Explorer.Series.quantile(s, 0.5)\n    ** (ArgumentError) Explorer.Series.quantile/2 not implemented for dtype :boolean. Valid dtypes are any subtype of [:u, :s, :f, :date, :time, :naive_datetime, :datetime]","ref":"Explorer.Series.html#quantile/2-examples"},{"type":"function","title":"Explorer.Series.quotient/2","doc":"Element-wise integer division.\n\nAt least one of the arguments must be a series. If both\nsizes are series, the series must have the same size or\nat last one of them must have size of 1.","ref":"Explorer.Series.html#quotient/2"},{"type":"function","title":"Supported dtype - Explorer.Series.quotient/2","doc":"* integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`\n\nReturns `nil` if there is a zero in the right-hand side.","ref":"Explorer.Series.html#quotient/2-supported-dtype"},{"type":"function","title":"Examples - Explorer.Series.quotient/2","doc":"iex> s1 = [10, 11, 10] |> Explorer.Series.from_list()\n    iex> s2 = [2, 2, 2] |> Explorer.Series.from_list()\n    iex> Explorer.Series.quotient(s1, s2)\n    #Explorer.Series \n\n    iex> s1 = [10, 11, 10] |> Explorer.Series.from_list()\n    iex> s2 = [2, 2, 0] |> Explorer.Series.from_list()\n    iex> Explorer.Series.quotient(s1, s2)\n    #Explorer.Series \n\n    iex> s1 = [10, 12, 15] |> Explorer.Series.from_list()\n    iex> Explorer.Series.quotient(s1, 3)\n    #Explorer.Series","ref":"Explorer.Series.html#quotient/2-examples"},{"type":"function","title":"Explorer.Series.rank/2","doc":"Assign ranks to data with appropriate handling of tied values.","ref":"Explorer.Series.html#rank/2"},{"type":"function","title":"Options - Explorer.Series.rank/2","doc":"* `:method` - Determine how ranks are assigned to tied elements. The following methods are available:\n  - `:average` : Each value receives the average rank that would be assigned to all tied values. (default)\n  - `:min` : Tied values are assigned the minimum rank. Also known as \"competition\" ranking.\n  - `:max` : Tied values are assigned the maximum of their ranks.\n  - `:dense` : Similar to `:min`, but the rank of the next highest element is assigned the rank immediately after those assigned to the tied elements.\n  - `:ordinal` : Each value is given a distinct rank based on its occurrence in the series.\n  - `:random` : Similar to `:ordinal`, but the rank for ties is not dependent on the order that the values occur in the Series.\n* `:descending` - Rank in descending order.\n* `:seed` - An integer to be used as a random seed. If nil, a random value between 0 and 2^64 − 1 will be used. (default: nil)","ref":"Explorer.Series.html#rank/2-options"},{"type":"function","title":"Examples - Explorer.Series.rank/2","doc":"iex> s = Explorer.Series.from_list([3, 6, 1, 1, 6])\n    iex> Explorer.Series.rank(s)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([1.1, 2.4, 3.2])\n    iex> Explorer.Series.rank(s, method: :ordinal)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([ ~N[2022-07-07 17:44:13.020548], ~N[2022-07-07 17:43:08.473561], ~N[2022-07-07 17:45:00.116337] ])\n    iex> Explorer.Series.rank(s, method: :average)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([3, 6, 1, 1, 6])\n    iex> Explorer.Series.rank(s, method: :min)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([3, 6, 1, 1, 6])\n    iex> Explorer.Series.rank(s, method: :dense)\n    #Explorer.Series \n\n\n    iex> s = Explorer.Series.from_list([3, 6, 1, 1, 6])\n    iex> Explorer.Series.rank(s, method: :random, seed: 42)\n    #Explorer.Series","ref":"Explorer.Series.html#rank/2-examples"},{"type":"function","title":"Explorer.Series.re_contains/2","doc":"Detects whether a string matches a pattern.\n\n>","ref":"Explorer.Series.html#re_contains/2"},{"type":"function","title":"Notice {: .warning} - Explorer.Series.re_contains/2","doc":">\n> This function matches against a regular expression. It does not expect an Elixir regex,\n> but a escaped string and you can use the `~S` sigil for escaping it. Since each Explorer\n> backend may have its own regular expression rules, you must consult their underlying\n> engine. For the default backend (Polars), the rules are outlined in the Rust create named\n> [`regex`](https://docs.rs/regex/latest/regex/).\n>\n> To match literal strings, you can use `contains/2`.","ref":"Explorer.Series.html#re_contains/2-notice-warning"},{"type":"function","title":"Examples - Explorer.Series.re_contains/2","doc":"iex> s = Explorer.Series.from_list([\"abc\", \"def\", \"bcd\"])\n    iex> Explorer.Series.re_contains(s, ~S/(a|e)/)\n    #Explorer.Series","ref":"Explorer.Series.html#re_contains/2-examples"},{"type":"function","title":"Explorer.Series.re_count_matches/2","doc":"Count how many times a pattern matches a string.\n\n>","ref":"Explorer.Series.html#re_count_matches/2"},{"type":"function","title":"Notice {: .warning} - Explorer.Series.re_count_matches/2","doc":">\n> This function matches against a regular expression. It does not expect an Elixir regex,\n> but a escaped string and you can use the `~S` sigil for escaping it. Since each Explorer\n> backend may have its own regular expression rules, you must consult their underlying\n> engine. For the default backend (Polars), the rules are outlined in the Rust create named\n> [`regex`](https://docs.rs/regex/latest/regex/).\n>\n> To count matching literal strings, you can use `count_matches/2`.","ref":"Explorer.Series.html#re_count_matches/2-notice-warning"},{"type":"function","title":"Examples - Explorer.Series.re_count_matches/2","doc":"iex> s = Explorer.Series.from_list([\"abc\", \"def def\", \"bcd\", nil])\n    iex> Explorer.Series.re_count_matches(s, ~S/(a|e)/)\n    #Explorer.Series","ref":"Explorer.Series.html#re_count_matches/2-examples"},{"type":"function","title":"Explorer.Series.re_named_captures/2","doc":"Extract all capture groups as a struct for the given regex pattern.\n\nAll group names are strings. If your pattern contains unnamed groups, their numerical position\nis converted to a string, starting from \"1\".\n\n>","ref":"Explorer.Series.html#re_named_captures/2"},{"type":"function","title":"Notice {: .warning} - Explorer.Series.re_named_captures/2","doc":">\n> This function does not work inside the context of `Explorer.Query`. If you are working on\n> a data frame, you first need to extract the series from it. See `Explorer.DataFrame.pull/2`.\n>\n> This function matches against a regular expression. It does not expect an Elixir regex,\n> but a escaped string and you can use the `~S` sigil for escaping it. Since each Explorer\n> backend may have its own regular expression rules, you must consult their underlying\n> engine. For the default backend (Polars), the rules are outlined in the Rust create named\n> [`regex`](https://docs.rs/regex/latest/regex/).","ref":"Explorer.Series.html#re_named_captures/2-notice-warning"},{"type":"function","title":"Examples - Explorer.Series.re_named_captures/2","doc":"iex> s = Explorer.Series.from_list([\"abc\", \"def def\", \"bcd\", nil])\n    iex> Explorer.Series.re_named_captures(s, ~S/(b|d)/)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([\"alice@service.com\", \"bob@example.com\"])\n    iex> Explorer.Series.re_named_captures(s, ~S/(? [^@]+)@(? .*)/)\n    #Explorer.Series","ref":"Explorer.Series.html#re_named_captures/2-examples"},{"type":"function","title":"Explorer.Series.re_replace/3","doc":"Replaces all occurences of a pattern with replacement in string series.\n\nBoth pattern and replacement must be of type string. The replacement\ncan refer to groups captures by using the `${x}`, where `x` is a number starting from 1.\nIt can also refer to named groups using the same syntax.\n\n>","ref":"Explorer.Series.html#re_replace/3"},{"type":"function","title":"Notice {: .warning} - Explorer.Series.re_replace/3","doc":">\n> This function matches against a regular expression. It does not expect an Elixir regex,\n> but a escaped string and you can use the `~S` sigil for escaping it. Since each Explorer\n> backend may have its own regular expression rules, you must consult their underlying\n> engine. For the default backend (Polars), the rules are outlined in the Rust create named\n> [`regex`](https://docs.rs/regex/latest/regex/).\n>\n> To replace by literal strings, you can use `replace/3`.","ref":"Explorer.Series.html#re_replace/3-notice-warning"},{"type":"function","title":"Examples - Explorer.Series.re_replace/3","doc":"iex> series = Explorer.Series.from_list([\"1.200,45\", \"1.234.567,30\", \"asdf\", nil])\n    iex> Explorer.Series.re_replace(series, ~S/[,.]/, \"\")\n    #Explorer.Series \n\n    iex> series = Explorer.Series.from_list([\"hat\", \"hut\"])\n    iex> Explorer.Series.re_replace(series, ~S/h(.)t/, \"b${1}d\")\n    #Explorer.Series \n\n    iex> series = Explorer.Series.from_list([\"hat\", \"hut\"])\n    iex> Explorer.Series.re_replace(series, ~S/h(? .)t/, \"b${vowel}d\")\n    #Explorer.Series \n\nApply case-insensitive string replacement using the `(?i)` flag - remember, from the `regex` Rust crate.\n\n    iex> series = Explorer.Series.from_list([\"Foggy\", \"Rainy\", \"Sunny\"])\n    iex> Explorer.Series.re_replace(series, ~S/(?i)foggy|rainy/, \"Sunny\")\n    #Explorer.Series \n\nWith an Elixir regex it causes an error:\n\n    iex> series = Explorer.Series.from_list([\"hat\", \"hut\"])\n    iex> Explorer.Series.re_replace(series, ~r/h(.)t/, \"b${1}d\")\n    ** (ArgumentError) standard regexes cannot be used as pattern because it may be incompatible with the backend. Please use the `~S` sigil or extract the source from the regex with `Regex.source/1`","ref":"Explorer.Series.html#re_replace/3-examples"},{"type":"function","title":"Explorer.Series.re_scan/2","doc":"Scan for all matches for the given regex pattern.\n\nExtract each successive non-overlapping regex match in an individual string as a list.\n\n>","ref":"Explorer.Series.html#re_scan/2"},{"type":"function","title":"Notice {: .warning} - Explorer.Series.re_scan/2","doc":">\n> This function matches against a regular expression. It does not expect an Elixir regex,\n> but a escaped string and you can use the `~S` sigil for escaping it. Since each Explorer\n> backend may have its own regular expression rules, you must consult their underlying\n> engine. For the default backend (Polars), the rules are outlined in the Rust create named\n> [`regex`](https://docs.rs/regex/latest/regex/).","ref":"Explorer.Series.html#re_scan/2-notice-warning"},{"type":"function","title":"Examples - Explorer.Series.re_scan/2","doc":"iex> s = Explorer.Series.from_list([\"abc\", \"def def\", \"bcd\", nil])\n    iex> Explorer.Series.re_scan(s, ~S/(b|d)/)\n    #Explorer.Series","ref":"Explorer.Series.html#re_scan/2-examples"},{"type":"function","title":"Explorer.Series.remainder/2","doc":"Computes the remainder of an element-wise integer division.\n\nAt least one of the arguments must be a series. If both\nsizes are series, the series must have the same size or\nat last one of them must have size of 1.","ref":"Explorer.Series.html#remainder/2"},{"type":"function","title":"Supported dtype - Explorer.Series.remainder/2","doc":"* integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`\n\nReturns `nil` if there is a zero in the right-hand side.","ref":"Explorer.Series.html#remainder/2-supported-dtype"},{"type":"function","title":"Examples - Explorer.Series.remainder/2","doc":"iex> s1 = [10, 11, 10] |> Explorer.Series.from_list()\n    iex> s2 = [2, 2, 2] |> Explorer.Series.from_list()\n    iex> Explorer.Series.remainder(s1, s2)\n    #Explorer.Series \n\n    iex> s1 = [10, 11, 10] |> Explorer.Series.from_list()\n    iex> s2 = [2, 2, 0] |> Explorer.Series.from_list()\n    iex> Explorer.Series.remainder(s1, s2)\n    #Explorer.Series \n\n    iex> s1 = [10, 11, 9] |> Explorer.Series.from_list()\n    iex> Explorer.Series.remainder(s1, 3)\n    #Explorer.Series","ref":"Explorer.Series.html#remainder/2-examples"},{"type":"function","title":"Explorer.Series.replace/2","doc":"Replaces the contents of the given series by the one given in\na tensor or list.\n\nThe new series will have the same dtype and backend as the current\nseries, but the size may not necessarily match.","ref":"Explorer.Series.html#replace/2"},{"type":"function","title":"Tensor examples - Explorer.Series.replace/2","doc":"iex> s = Explorer.Series.from_list([0, 1, 2])\n    iex> Explorer.Series.replace(s, Nx.tensor([1, 2, 3]))\n    #Explorer.Series \n\nThis is particularly useful for categorical columns:\n\n    iex> s = Explorer.Series.from_list([\"foo\", \"bar\", \"baz\"], dtype: :category)\n    iex> Explorer.Series.replace(s, Nx.tensor([2, 1, 0]))\n    #Explorer.Series","ref":"Explorer.Series.html#replace/2-tensor-examples"},{"type":"function","title":"List examples - Explorer.Series.replace/2","doc":"Similar to tensors, we can also replace by lists:\n\n    iex> s = Explorer.Series.from_list([0, 1, 2])\n    iex> Explorer.Series.replace(s, [1, 2, 3, 4, 5])\n    #Explorer.Series \n\nThe same considerations as above apply.","ref":"Explorer.Series.html#replace/2-list-examples"},{"type":"function","title":"Explorer.Series.replace/3","doc":"Replaces all occurences of a substring with replacement in string series.\n\nBoth substring and replacement must be of type string.\n\n>","ref":"Explorer.Series.html#replace/3"},{"type":"function","title":"Notice {: .warning} - Explorer.Series.replace/3","doc":">\n> This function replaces only literal strings. For regular expressions, see `re_replace/3`.","ref":"Explorer.Series.html#replace/3-notice-warning"},{"type":"function","title":"Examples - Explorer.Series.replace/3","doc":"iex> series = Explorer.Series.from_list([\"1,200\", \"1,234,567\", \"asdf\", nil])\n    iex> Explorer.Series.replace(series, \",\", \"\")\n    #Explorer.Series","ref":"Explorer.Series.html#replace/3-examples"},{"type":"function","title":"Explorer.Series.reverse/1","doc":"Reverses the series order.","ref":"Explorer.Series.html#reverse/1"},{"type":"function","title":"Example - Explorer.Series.reverse/1","doc":"iex> s = [1, 2, 3] |> Explorer.Series.from_list()\n    iex> Explorer.Series.reverse(s)\n    #Explorer.Series","ref":"Explorer.Series.html#reverse/1-example"},{"type":"function","title":"Explorer.Series.round/2","doc":"Round floating point series to given decimal places.","ref":"Explorer.Series.html#round/2"},{"type":"function","title":"Examples - Explorer.Series.round/2","doc":"iex> s = Explorer.Series.from_list([1.124993, 2.555321, 3.995001])\n    iex> Explorer.Series.round(s, 2)\n    #Explorer.Series","ref":"Explorer.Series.html#round/2-examples"},{"type":"function","title":"Explorer.Series.row_index/1","doc":"Returns a series of indexes for each item (row) in the series, starting from 0.","ref":"Explorer.Series.html#row_index/1"},{"type":"function","title":"Examples - Explorer.Series.row_index/1","doc":"iex> s = Series.from_list([nil, true, true])\n    iex> Series.row_index(s)\n    #Explorer.Series \n\nThis function can be used to add a row index as the first column of a dataframe.\nThe resulting column is a regular column of type `:u32`.\n\n    iex> require Explorer.DataFrame, as: DF\n    iex> df = DF.new(a: [1, 3, 5], b: [2, 4, 6])\n    iex> DF.mutate(df, index: row_index(a)) |> DF.relocate(\"index\", before: 0)\n    #Explorer.DataFrame \n    iex> df = DF.new(a: [1, 3, 5], b: [2, 4, 6])\n    iex> DF.mutate(df, id: row_index(a) + 1000)\n    #Explorer.DataFrame","ref":"Explorer.Series.html#row_index/1-examples"},{"type":"function","title":"Explorer.Series.rstrip/1","doc":"Returns a string series where all trailing Unicode whitespaces have been removed.","ref":"Explorer.Series.html#rstrip/1"},{"type":"function","title":"Examples - Explorer.Series.rstrip/1","doc":"iex> s = Explorer.Series.from_list([\"  abc\", \"def  \", \"  bcd\"])\n    iex> Explorer.Series.rstrip(s)\n    #Explorer.Series","ref":"Explorer.Series.html#rstrip/1-examples"},{"type":"function","title":"Explorer.Series.rstrip/2","doc":"Returns a string series where all trailing examples of the provided string\nhave been removed.\n\nWhere multiple characters are provided, all combinations of this set of characters will be stripped","ref":"Explorer.Series.html#rstrip/2"},{"type":"function","title":"Examples - Explorer.Series.rstrip/2","doc":"iex> s = Explorer.Series.from_list([\"__abc__\", \"def_\", \"__bcd_\"])\n    iex> Explorer.Series.rstrip(s, \"_\")\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([\"abc\", \"adefa\", \"bcdabaaa\"])\n    iex> Explorer.Series.rstrip(s, \"ab\")\n    #Explorer.Series","ref":"Explorer.Series.html#rstrip/2-examples"},{"type":"function","title":"Explorer.Series.sample/3","doc":"Returns a random sample of the series.\n\nIf given an integer as the second argument, it will return N samples. If given a float, it will\nreturn that proportion of the series.\n\nCan sample with or without replace.","ref":"Explorer.Series.html#sample/3"},{"type":"function","title":"Options - Explorer.Series.sample/3","doc":"* `:replace` - If set to `true`, each sample will be independent and therefore values may repeat.\n    Required to be `true` for `n` greater then the number of rows in the series or `frac` > 1.0. (default: `false`)\n  * `:seed` - An integer to be used as a random seed. If nil, a random value between 0 and 2^64 − 1 will be used. (default: nil)\n  * `:shuffle` - In case the sample is equal to the size of the series, shuffle tells if the resultant\n    series should be shuffled or if it should return the same series. (default: `false`).","ref":"Explorer.Series.html#sample/3-options"},{"type":"function","title":"Examples - Explorer.Series.sample/3","doc":"iex> s = 1..100 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.sample(s, 10, seed: 100)\n    #Explorer.Series \n\n    iex> s = 1..100 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.sample(s, 0.05, seed: 100)\n    #Explorer.Series \n\n    iex> s = 1..5 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.sample(s, 7, seed: 100, replace: true)\n    #Explorer.Series \n\n    iex> s = 1..5 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.sample(s, 1.2, seed: 100, replace: true)\n    #Explorer.Series \n\n    iex> s = 0..9 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.sample(s, 1.0, seed: 100, shuffle: false)\n    #Explorer.Series \n\n    iex> s = 0..9 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.sample(s, 1.0, seed: 100, shuffle: true)\n    #Explorer.Series","ref":"Explorer.Series.html#sample/3-examples"},{"type":"function","title":"Explorer.Series.second/1","doc":"Returns the second number from 0 to 59.","ref":"Explorer.Series.html#second/1"},{"type":"function","title":"Examples - Explorer.Series.second/1","doc":"iex> s = Explorer.Series.from_list([~N[2023-01-15 00:00:00], ~N[2022-02-16 23:59:59.999999], ~N[2021-03-20 12:00:00], nil])\n    iex> Explorer.Series.second(s)\n    #Explorer.Series","ref":"Explorer.Series.html#second/1-examples"},{"type":"function","title":"Explorer.Series.select/3","doc":"Returns a series from two series, based on a predicate.\n\nThe resulting series is built by evaluating each element of\n`predicate` and returning either the corresponding element from\n`on_true` or `on_false`.\n\n`predicate` must be a boolean series. `on_true` and `on_false` must be\na series of the same size as `predicate` or a series of size 1.\n\nIt is possible to mix numeric series in the `on_true` and `on_false`,\nand the resultant series will have the dtype of the greater side.\nFor example, `:u8` and `:s16` is going to result in `:s16` series.","ref":"Explorer.Series.html#select/3"},{"type":"function","title":"Explorer.Series.shift/2","doc":"Shifts `series` by `offset` with `nil` values.\n\nPositive offset shifts from first, negative offset shifts from last.","ref":"Explorer.Series.html#shift/2"},{"type":"function","title":"Examples - Explorer.Series.shift/2","doc":"iex> s = 1..5 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.shift(s, 2)\n    #Explorer.Series \n\n    iex> s = 1..5 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.shift(s, -2)\n    #Explorer.Series","ref":"Explorer.Series.html#shift/2-examples"},{"type":"function","title":"Explorer.Series.shuffle/2","doc":"Change the elements order randomly.","ref":"Explorer.Series.html#shuffle/2"},{"type":"function","title":"Options - Explorer.Series.shuffle/2","doc":"* `:seed` - An integer to be used as a random seed. If nil,\n    a random value between 0 and 2^64 − 1 will be used. (default: nil)","ref":"Explorer.Series.html#shuffle/2-options"},{"type":"function","title":"Examples - Explorer.Series.shuffle/2","doc":"iex> s = 1..10 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.shuffle(s, seed: 100)\n    #Explorer.Series","ref":"Explorer.Series.html#shuffle/2-examples"},{"type":"function","title":"Explorer.Series.sin/1","doc":"Computes the the sine of a number (in radians).\nThe resultant series is going to be of dtype `{:f, 64}`, with values between 1 and -1.","ref":"Explorer.Series.html#sin/1"},{"type":"function","title":"Supported dtype - Explorer.Series.sin/1","doc":"* `{:f, 32}`\n  * `{:f, 64}`","ref":"Explorer.Series.html#sin/1-supported-dtype"},{"type":"function","title":"Examples - Explorer.Series.sin/1","doc":"iex> pi = :math.pi()\n    iex> s = [-pi * 3/2, -pi, -pi / 2, -pi / 4, 0, pi / 4, pi / 2, pi, pi * 3/2] |> Explorer.Series.from_list()\n    iex> Explorer.Series.sin(s)\n    #Explorer.Series","ref":"Explorer.Series.html#sin/1-examples"},{"type":"function","title":"Explorer.Series.size/1","doc":"Returns the number of elements in the series.\n\nSee also:\n\n  * `count/1` - counts only the non-`nil` elements.\n  * `nil_count/1` - counts all `nil` elements.","ref":"Explorer.Series.html#size/1"},{"type":"function","title":"Examples - Explorer.Series.size/1","doc":"Basic example:\n\n    iex> s = Explorer.Series.from_list([~D[1999-12-31], ~D[1989-01-01]])\n    iex> Explorer.Series.size(s)\n    2\n\nWith lists:\n\n    iex> s = Explorer.Series.from_list([[1, 2, 3], [4, 5]])\n    iex> Explorer.Series.size(s)\n    2","ref":"Explorer.Series.html#size/1-examples"},{"type":"function","title":"Explorer.Series.skew/2","doc":"Compute the sample skewness of a series.\n\nFor normally distributed data, the skewness should be about zero.\n\nFor unimodal continuous distributions, a skewness value greater\nthan zero means that there is more weight in the right tail of the\ndistribution.","ref":"Explorer.Series.html#skew/2"},{"type":"function","title":"Supported dtypes - Explorer.Series.skew/2","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`","ref":"Explorer.Series.html#skew/2-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.skew/2","doc":"iex> s = Explorer.Series.from_list([1, 2, 3, 4, 5, 23])\n    iex> Explorer.Series.skew(s)\n    1.6727687946848508\n\n    iex> s = Explorer.Series.from_list([1, 2, 3, 4, 5, 23])\n    iex> Explorer.Series.skew(s, bias: false)\n    2.2905330058490514\n\n    iex> s = Explorer.Series.from_list([1, 2, 3, nil, 1])\n    iex> Explorer.Series.skew(s, bias: false)\n    0.8545630383279712\n\n    iex> s = Explorer.Series.from_list([1, 2, 3, nil, 1])\n    iex> Explorer.Series.skew(s)\n    0.49338220021815865\n\n    iex> s = Explorer.Series.from_list([true, false, true])\n    iex> Explorer.Series.skew(s, false)\n    ** (ArgumentError) Explorer.Series.skew/2 not implemented for dtype :boolean. Valid dtypes are {:f, 32}, {:f, 64}, {:s, 8}, {:s, 16}, {:s, 32}, {:s, 64}, {:u, 8}, {:u, 16}, {:u, 32} and {:u, 64}","ref":"Explorer.Series.html#skew/2-examples"},{"type":"function","title":"Explorer.Series.slice/2","doc":"Slices the elements at the given indices as a new series.\n\nThe indices may be either a list (or series) of indices or a range.\nA list or series of indices does not support negative numbers.\nRanges may be negative on either end, which are then\nnormalized. Note ranges in Elixir are inclusive.","ref":"Explorer.Series.html#slice/2"},{"type":"function","title":"Examples - Explorer.Series.slice/2","doc":"iex> s = Explorer.Series.from_list([\"a\", \"b\", \"c\"])\n    iex> Explorer.Series.slice(s, [0, 2])\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([\"a\", \"b\", \"c\"])\n    iex> Explorer.Series.slice(s, 1..2)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([\"a\", \"b\", \"c\"])\n    iex> Explorer.Series.slice(s, -2..-1)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([\"a\", \"b\", \"c\"])\n    iex> Explorer.Series.slice(s, 3..2//1)\n    #Explorer.Series","ref":"Explorer.Series.html#slice/2-examples"},{"type":"function","title":"Explorer.Series.slice/3","doc":"Returns a slice of the series, with `size` elements starting at `offset`.","ref":"Explorer.Series.html#slice/3"},{"type":"function","title":"Examples - Explorer.Series.slice/3","doc":"iex> s = Explorer.Series.from_list([1, 2, 3, 4, 5])\n    iex> Explorer.Series.slice(s, 1, 2)\n    #Explorer.Series \n\nNegative offsets count from the end of the series:\n\n    iex> s = Explorer.Series.from_list([1, 2, 3, 4, 5])\n    iex> Explorer.Series.slice(s, -3, 2)\n    #Explorer.Series \n\nIf the offset runs past the end of the series,\nthe series is empty:\n\n    iex> s = Explorer.Series.from_list([1, 2, 3, 4, 5])\n    iex> Explorer.Series.slice(s, 10, 3)\n    #Explorer.Series \n\nIf the size runs past the end of the series,\nthe result may be shorter than the size:\n\n    iex> s = Explorer.Series.from_list([1, 2, 3, 4, 5])\n    iex> Explorer.Series.slice(s, -3, 4)\n    #Explorer.Series","ref":"Explorer.Series.html#slice/3-examples"},{"type":"function","title":"Explorer.Series.sort/2","doc":"Sorts the series.\n\nSee `sort_by/3` for an expression-based sorting function.\nSee `sort_with/3` for a callback-based sorting function.","ref":"Explorer.Series.html#sort/2"},{"type":"function","title":"Options - Explorer.Series.sort/2","doc":"* `:direction` - `:asc` or `:desc`, meaning \"ascending\" or \"descending\", respectively.\n    By default it sorts in ascending order.\n\n  * `:nils` - `:first` or `:last`.\n    By default it is `:last` if direction is `:asc`, and `:first` otherwise.\n\n  * `:parallel` - boolean.\n    Whether to parallelize the sorting.\n    By default it is `true`.\n\n  * `:stable` - boolean.\n    Determines if the sorting is stable (ties are guaranteed to maintain their order) or not.\n    Unstable sorting may be more performant.\n    By default it is `false`.","ref":"Explorer.Series.html#sort/2-options"},{"type":"function","title":"Examples - Explorer.Series.sort/2","doc":"iex> s = Explorer.Series.from_list([9, 3, 7, 1])\n    iex> Explorer.Series.sort(s)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([9, 3, 7, 1])\n    iex> Explorer.Series.sort(s, direction: :desc)\n    #Explorer.Series","ref":"Explorer.Series.html#sort/2-examples"},{"type":"macro","title":"Explorer.Series.sort_by/3","doc":"Sorts the series based on an expression.\n\n> #### Notice {: .notice}\n>\n> This is a macro.\n>\n>   * You must `require Explorer.Series` before using it.\n>   * You must use the special `_` syntax. See the moduledoc for details.\n\nSee `sort_with/3` for the callback-based version of this function.","ref":"Explorer.Series.html#sort_by/3"},{"type":"macro","title":"Options - Explorer.Series.sort_by/3","doc":"* `:direction` - `:asc` or `:desc`, meaning \"ascending\" or \"descending\", respectively.\n    By default it sorts in ascending order.\n\n  * `:nils` - `:first` or `:last`.\n    By default it is `:last` if direction is `:asc`, and `:first` otherwise.\n\n  * `:parallel` - boolean.\n    Whether to parallelize the sorting.\n    By default it is `true`.\n\n    Parallel sort isn't available on certain lazy operations.\n    In those situations this option is ignored.\n\n  * `:stable` - boolean.\n    Determines if the sorting is stable (ties are guaranteed to maintain their order) or not.\n    Unstable sorting may be more performant.\n    By default it is `false`.","ref":"Explorer.Series.html#sort_by/3-options"},{"type":"macro","title":"Examples - Explorer.Series.sort_by/3","doc":"iex> s = Explorer.Series.from_list([1, 2, 3])\n    iex> Explorer.Series.sort_by(s, remainder(_, 3))\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([1, 2, 3])\n    iex> Explorer.Series.sort_by(s, remainder(_, 3), direction: :desc)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([1, nil, 2, 3])\n    iex> Explorer.Series.sort_by(s, -2 * _, nils: :first)\n    #Explorer.Series","ref":"Explorer.Series.html#sort_by/3-examples"},{"type":"function","title":"Explorer.Series.sort_with/3","doc":"Sorts the series based on a callback that returns a lazy series.\n\nSee `sort_by/3` for the expression-based version of this function.","ref":"Explorer.Series.html#sort_with/3"},{"type":"function","title":"Options - Explorer.Series.sort_with/3","doc":"* `:direction` - `:asc` or `:desc`, meaning \"ascending\" or \"descending\", respectively.\n    By default it sorts in ascending order.\n\n  * `:nils` - `:first` or `:last`.\n    By default it is `:last` if direction is `:asc`, and `:first` otherwise.\n\n  * `:parallel` - boolean.\n    Whether to parallelize the sorting.\n    By default it is `true`.\n\n    Parallel sort isn't available on certain lazy operations.\n    In those situations this option is ignored.\n\n  * `:stable` - boolean.\n    Determines if the sorting is stable (ties are guaranteed to maintain their order) or not.\n    Unstable sorting may be more performant.\n    By default it is `false`.","ref":"Explorer.Series.html#sort_with/3-options"},{"type":"function","title":"Examples - Explorer.Series.sort_with/3","doc":"iex> s = Explorer.Series.from_list([1, 2, 3])\n    iex> Explorer.Series.sort_with(s, &Explorer.Series.remainder(&1, 3))\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([1, 2, 3])\n    iex> Explorer.Series.sort_with(s, &Explorer.Series.remainder(&1, 3), direction: :desc)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([1, nil, 2, 3])\n    iex> Explorer.Series.sort_with(s, &Explorer.Series.multiply(-2, &1), nils: :first)\n    #Explorer.Series","ref":"Explorer.Series.html#sort_with/3-examples"},{"type":"function","title":"Explorer.Series.split/2","doc":"Split the string by a substring.","ref":"Explorer.Series.html#split/2"},{"type":"function","title":"Examples - Explorer.Series.split/2","doc":"iex> s = Series.from_list([\"1\", \"1|2\"])\n    iex> Series.split(s, \"|\")\n    #Explorer.Series","ref":"Explorer.Series.html#split/2-examples"},{"type":"function","title":"Explorer.Series.split_into/3","doc":"Split a string Series into a struct of string `fields`.\n\nThe length of the field names list determines how many times the\nstring will be split at most. If the string cannot be split into that\nmany separate strings, null values will be provided for the\nremaining fields.","ref":"Explorer.Series.html#split_into/3"},{"type":"function","title":"Examples - Explorer.Series.split_into/3","doc":"iex> s = Series.from_list([\"Smith, John\", \"Jones, Jane\"])\n    iex> Series.split_into(s, \", \", [\"Last Name\", \"First Name\"])\n    #Explorer.Series","ref":"Explorer.Series.html#split_into/3-examples"},{"type":"function","title":"Explorer.Series.standard_deviation/2","doc":"Gets the standard deviation of the series.\n\nBy default, this is the sample standard deviation. This function also takes an optional\ndelta degrees of freedom (ddof). Setting this to zero corresponds to the population\nsample standard deviation.","ref":"Explorer.Series.html#standard_deviation/2"},{"type":"function","title":"Supported dtypes - Explorer.Series.standard_deviation/2","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`","ref":"Explorer.Series.html#standard_deviation/2-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.standard_deviation/2","doc":"iex> s = Explorer.Series.from_list([1, 2, nil, 3])\n    iex> Explorer.Series.standard_deviation(s)\n    1.0\n\n    iex> s = Explorer.Series.from_list([1.0, 2.0, nil, 3.0])\n    iex> Explorer.Series.standard_deviation(s)\n    1.0\n\n    iex> s = Explorer.Series.from_list([\"a\", \"b\", \"c\"])\n    iex> Explorer.Series.standard_deviation(s)\n    ** (ArgumentError) Explorer.Series.standard_deviation/1 not implemented for dtype :string. Valid dtypes are {:f, 32}, {:f, 64}, {:s, 8}, {:s, 16}, {:s, 32}, {:s, 64}, {:u, 8}, {:u, 16}, {:u, 32} and {:u, 64}","ref":"Explorer.Series.html#standard_deviation/2-examples"},{"type":"function","title":"Explorer.Series.strftime/2","doc":"Converts a datetime series to a string series.\n\nFor the format string specification, refer to the\n[chrono crate documentation](https://docs.rs/chrono/latest/chrono/format/strftime/index.html).\n\nUse `cast(series, :string)` for the default `\"%Y-%m-%d %H:%M:%S%.6f\"` format.","ref":"Explorer.Series.html#strftime/2"},{"type":"function","title":"Examples - Explorer.Series.strftime/2","doc":"iex> s = Explorer.Series.from_list([~N[2023-01-05 12:34:56], nil])\n    iex> Explorer.Series.strftime(s, \"%Y/%m/%d %H:%M:%S\")\n    #Explorer.Series","ref":"Explorer.Series.html#strftime/2-examples"},{"type":"function","title":"Explorer.Series.strip/1","doc":"Returns a string series where all leading and trailing Unicode whitespaces\nhave been removed.","ref":"Explorer.Series.html#strip/1"},{"type":"function","title":"Examples - Explorer.Series.strip/1","doc":"iex> s = Explorer.Series.from_list([\"  abc\", \"def  \", \"  bcd   \"])\n    iex> Explorer.Series.strip(s)\n    #Explorer.Series","ref":"Explorer.Series.html#strip/1-examples"},{"type":"function","title":"Explorer.Series.strip/2","doc":"Returns a string series where all leading and trailing examples of the provided string\nhave been removed.\n\nWhere multiple characters are provided, all combinations of this set of characters will be stripped","ref":"Explorer.Series.html#strip/2"},{"type":"function","title":"Examples - Explorer.Series.strip/2","doc":"iex> s = Explorer.Series.from_list([\"£123\", \"1.00£\", \"£1.00£\"])\n    iex> Explorer.Series.strip(s, \"£\")\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([\"abc\", \"adefa\", \"bcda\"])\n    iex> Explorer.Series.strip(s, \"ab\")\n    #Explorer.Series","ref":"Explorer.Series.html#strip/2-examples"},{"type":"function","title":"Explorer.Series.strptime/2","doc":"Converts a string series to a datetime series with a given `format_string`.\n\nFor the format string specification, refer to the\n[chrono crate documentation](https://docs.rs/chrono/latest/chrono/format/strftime/).\n\nUse `cast(series, :datetime)` if you prefer the format to be inferred (if possible).","ref":"Explorer.Series.html#strptime/2"},{"type":"function","title":"Examples - Explorer.Series.strptime/2","doc":"iex> s = Explorer.Series.from_list([\"2023-01-05 12:34:56\", \"XYZ\", nil])\n    iex> Explorer.Series.strptime(s, \"%Y-%m-%d %H:%M:%S\")\n    #Explorer.Series","ref":"Explorer.Series.html#strptime/2-examples"},{"type":"function","title":"Explorer.Series.substring/2","doc":"Returns a string sliced from the offset to the end of the string, supporting\nnegative indexing","ref":"Explorer.Series.html#substring/2"},{"type":"function","title":"Examples - Explorer.Series.substring/2","doc":"iex> s = Explorer.Series.from_list([\"earth\", \"mars\", \"neptune\"])\n    iex> Explorer.Series.substring(s, -3)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([\"earth\", \"mars\", \"neptune\"])\n    iex> Explorer.Series.substring(s, 1)\n    #Explorer.Series","ref":"Explorer.Series.html#substring/2-examples"},{"type":"function","title":"Explorer.Series.substring/3","doc":"Returns a string sliced from the offset to the length provided, supporting\nnegative indexing","ref":"Explorer.Series.html#substring/3"},{"type":"function","title":"Examples - Explorer.Series.substring/3","doc":"iex> s = Explorer.Series.from_list([\"earth\", \"mars\", \"neptune\"])\n    iex> Explorer.Series.substring(s, -3, 2)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([\"earth\", \"mars\", \"neptune\"])\n    iex> Explorer.Series.substring(s, 1, 5)\n    #Explorer.Series \n\n    iex> d = Explorer.Series.from_list([\"こんにちは世界\", \"مرحبًا\", \"안녕하세요\"])\n    iex> Explorer.Series.substring(d, 1, 3)\n    #Explorer.Series","ref":"Explorer.Series.html#substring/3-examples"},{"type":"function","title":"Explorer.Series.subtract/2","doc":"Subtracts right from left, element-wise.\n\nWhen mixing floats and integers, the resulting series will have dtype `{:f, 64}`.\nIn case both series are of unsigned integers, we will try to subtract,\nbut an exception is raised if overflow occurs.\n\nAt least one of the arguments must be a series. If both\nsizes are series, the series must have the same size or\nat last one of them must have size of 1.","ref":"Explorer.Series.html#subtract/2"},{"type":"function","title":"Supported dtypes - Explorer.Series.subtract/2","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`","ref":"Explorer.Series.html#subtract/2-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.subtract/2","doc":"iex> s1 = Explorer.Series.from_list([1, 2, 3])\n    iex> s2 = Explorer.Series.from_list([4, 5, 6])\n    iex> Explorer.Series.subtract(s1, s2)\n    #Explorer.Series \n\nYou can also use scalar values on both sides:\n\n    iex> s1 = Explorer.Series.from_list([1, 2, 3])\n    iex> Explorer.Series.subtract(s1, 2)\n    #Explorer.Series \n\n    iex> s1 = Explorer.Series.from_list([1, 2, 3])\n    iex> Explorer.Series.subtract(2, s1)\n    #Explorer.Series","ref":"Explorer.Series.html#subtract/2-examples"},{"type":"function","title":"Explorer.Series.sum/1","doc":"Gets the sum of the series.","ref":"Explorer.Series.html#sum/1"},{"type":"function","title":"Supported dtypes - Explorer.Series.sum/1","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`\n  * `:boolean`","ref":"Explorer.Series.html#sum/1-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.sum/1","doc":"iex> s = Explorer.Series.from_list([1, 2, nil, 3])\n    iex> Explorer.Series.sum(s)\n    6\n\n    iex> s = Explorer.Series.from_list([1.0, 2.0, nil, 3.0])\n    iex> Explorer.Series.sum(s)\n    6.0\n\n    iex> s = Explorer.Series.from_list([true, false, true])\n    iex> Explorer.Series.sum(s)\n    2\n\n    iex> s = Explorer.Series.from_list([~D[2021-01-01], ~D[1999-12-31]])\n    iex> Explorer.Series.sum(s)\n    ** (ArgumentError) Explorer.Series.sum/1 not implemented for dtype :date. Valid dtypes are :boolean, {:f, 32}, {:f, 64}, {:s, 8}, {:s, 16}, {:s, 32}, {:s, 64}, {:u, 8}, {:u, 16}, {:u, 32} and {:u, 64}","ref":"Explorer.Series.html#sum/1-examples"},{"type":"function","title":"Explorer.Series.tail/2","doc":"Returns the last N elements of the series.","ref":"Explorer.Series.html#tail/2"},{"type":"function","title":"Examples - Explorer.Series.tail/2","doc":"iex> s = 1..100 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.tail(s)\n    #Explorer.Series","ref":"Explorer.Series.html#tail/2-examples"},{"type":"function","title":"Explorer.Series.tan/1","doc":"Computes the tangent of a number (in radians).\nThe resultant series is going to be of dtype `{:f, 64}`.","ref":"Explorer.Series.html#tan/1"},{"type":"function","title":"Supported dtype - Explorer.Series.tan/1","doc":"* `{:f, 32}`\n  * `{:f, 64}`","ref":"Explorer.Series.html#tan/1-supported-dtype"},{"type":"function","title":"Examples - Explorer.Series.tan/1","doc":"iex> pi = :math.pi()\n    iex> s = [-pi * 3/2, -pi, -pi / 2, -pi / 4, 0, pi / 4, pi / 2, pi, pi * 3/2] |> Explorer.Series.from_list()\n    iex> Explorer.Series.tan(s)\n    #Explorer.Series","ref":"Explorer.Series.html#tan/1-examples"},{"type":"function","title":"Explorer.Series.to_binary/1","doc":"Returns a series as a fixed-width binary.\n\nThis is a shortcut around `to_iovec/1`. If possible, prefer\nto use `to_iovec/1` as that avoids copying binaries.","ref":"Explorer.Series.html#to_binary/1"},{"type":"function","title":"Examples - Explorer.Series.to_binary/1","doc":"iex> series = Explorer.Series.from_list([1, 2, 3])\n    iex> Explorer.Series.to_binary(series)\n    <<1::signed-64-native, 2::signed-64-native, 3::signed-64-native>>\n\n    iex> series = Explorer.Series.from_list([true, false, true])\n    iex> Explorer.Series.to_binary(series)\n    <<1, 0, 1>>","ref":"Explorer.Series.html#to_binary/1-examples"},{"type":"function","title":"Explorer.Series.to_date/1","doc":"","ref":"Explorer.Series.html#to_date/1"},{"type":"function","title":"Explorer.Series.to_enum/1","doc":"Converts a series to an enumerable.\n\nThe enumerable will lazily traverse the series.\n\n> #### Warning {: .warning}\n>\n> You must avoid converting a series to enum, as that will copy the whole\n> series in memory as you traverse it. Prefer to use the operations in this\n> module rather than the ones in `Enum` whenever possible, as this module is\n> optimized for large series.","ref":"Explorer.Series.html#to_enum/1"},{"type":"function","title":"Examples - Explorer.Series.to_enum/1","doc":"iex> series = Explorer.Series.from_list([1, 2, 3])\n    iex> series |> Explorer.Series.to_enum() |> Enum.to_list()\n    [1, 2, 3]","ref":"Explorer.Series.html#to_enum/1-examples"},{"type":"function","title":"Explorer.Series.to_iovec/1","doc":"Returns a series as a list of fixed-width binaries.\n\nAn io vector (`iovec`) is the Erlang VM term for a flat list of binaries.\nThis is typically a reference to the in-memory representation of the series.\nIf the whole series in contiguous in memory, then the list will have a single\nelement. All binaries are in native endianness.\n\nThis operation fails if the series has `nil` values.\nUse `fill_missing/1` to handle them accordingly.\n\nTo retrieve the type of the underlying io vector, use `iotype/1`.\nTo convert an iovec to a binary, you can use `IO.iodata_to_binary/1`.","ref":"Explorer.Series.html#to_iovec/1"},{"type":"function","title":"Examples - Explorer.Series.to_iovec/1","doc":"Integers and floats follow their native encoding:\n\n    iex> series = Explorer.Series.from_list([-1, 0, 1])\n    iex> Explorer.Series.to_iovec(series)\n    [<<-1::signed-64-native, 0::signed-64-native, 1::signed-64-native>>]\n\n    iex> series = Explorer.Series.from_list([1.0, 2.0, 3.0])\n    iex> Explorer.Series.to_iovec(series)\n    [<<1.0::float-64-native, 2.0::float-64-native, 3.0::float-64-native>>]\n\nBooleans are encoded as 0 and 1:\n\n    iex> series = Explorer.Series.from_list([true, false, true])\n    iex> Explorer.Series.to_iovec(series)\n    [<<1, 0, 1>>]\n\nDates are encoded as s32 representing days from the Unix epoch (1970-01-01):\n\n    iex> series = Explorer.Series.from_list([~D[0001-01-01], ~D[1970-01-01], ~D[1986-10-13]])\n    iex> Explorer.Series.to_iovec(series)\n    [<<-719162::signed-32-native, 0::signed-32-native, 6129::signed-32-native>>]\n\nTimes are encoded as s64 representing nanoseconds from midnight:\n\n    iex> series = Explorer.Series.from_list([~T[00:00:00.000000], ~T[23:59:59.999999]])\n    iex> Explorer.Series.to_iovec(series)\n    [<<0::signed-64-native, 86399999999000::signed-64-native>>]\n\nDatetimes are encoded as s64 representing their precision from the Unix epoch (1970-01-01):\n\n    iex> series = Explorer.Series.from_list([~N[0001-01-01 00:00:00], ~N[1970-01-01 00:00:00], ~N[1986-10-13 01:23:45.987654]])\n    iex> Explorer.Series.to_iovec(series)\n    [<<-62135596800000000::signed-64-native, 0::signed-64-native, 529550625987654::signed-64-native>>]\n\nThe operation raises for binaries and strings, as they do not provide a fixed-width\nbinary representation:\n\n    iex> s = Explorer.Series.from_list([\"a\", \"b\", \"c\", \"b\"])\n    iex> Explorer.Series.to_iovec(s)\n    ** (ArgumentError) cannot convert series of dtype :string into iovec\n\nHowever, if appropriate, you can convert them to categorical types,\nwhich will then return the index of each category:\n\n    iex> series = Explorer.Series.from_list([\"a\", \"b\", \"c\", \"b\"], dtype: :category)\n    iex> Explorer.Series.to_iovec(series)\n    [<<0::unsigned-32-native, 1::unsigned-32-native, 2::unsigned-32-native, 1::unsigned-32-native>>]","ref":"Explorer.Series.html#to_iovec/1-examples"},{"type":"function","title":"Explorer.Series.to_list/1","doc":"Converts a series to a list.\n\n> #### Warning {: .warning}\n>\n> You must avoid converting a series to list, as that requires copying\n> the whole series in memory. Prefer to use the operations in this module\n> rather than the ones in `Enum` whenever possible, as this module is\n> optimized for large series.","ref":"Explorer.Series.html#to_list/1"},{"type":"function","title":"Examples - Explorer.Series.to_list/1","doc":"iex> series = Explorer.Series.from_list([1, 2, 3])\n    iex> Explorer.Series.to_list(series)\n    [1, 2, 3]","ref":"Explorer.Series.html#to_list/1-examples"},{"type":"function","title":"Explorer.Series.to_tensor/2","doc":"Converts a series to a `t:Nx.Tensor.t/0`.\n\nNote that `Explorer.Series` are automatically converted\nto tensors when passed to numerical definitions.\nThe tensor type is given by `iotype/1`.\n\n> #### Warning {: .warning}\n>\n> `Nx` is an optional dependency. You will need to ensure it's installed to use this function.","ref":"Explorer.Series.html#to_tensor/2"},{"type":"function","title":"Options - Explorer.Series.to_tensor/2","doc":"* `:backend` - the Nx backend to allocate the tensor on","ref":"Explorer.Series.html#to_tensor/2-options"},{"type":"function","title":"Examples - Explorer.Series.to_tensor/2","doc":"iex> s = Explorer.Series.from_list([1, 2, 3])\n    iex> Explorer.Series.to_tensor(s)\n    #Nx.Tensor \n\n    iex> s = Explorer.Series.from_list([true, false, true])\n    iex> Explorer.Series.to_tensor(s)\n    #Nx.Tensor","ref":"Explorer.Series.html#to_tensor/2-examples"},{"type":"function","title":"Explorer.Series.to_time/1","doc":"","ref":"Explorer.Series.html#to_time/1"},{"type":"function","title":"Explorer.Series.transform/2","doc":"Returns an `Explorer.Series` where each element is the result of invoking `fun` on each\ncorresponding element of `series`.\n\nThis is an expensive operation meant to enable the use of arbitrary Elixir functions against\nany backend. The implementation will vary by backend but in most (all?) cases will require\nconverting to an `Elixir.List`, applying `Enum.map/2`, and then converting back to an\n`Explorer.Series`.\n\nSee also the version of this function for a dataframe:\n\n  * `Explorer.DataFrame.transform/3`","ref":"Explorer.Series.html#transform/2"},{"type":"function","title":"Examples - Explorer.Series.transform/2","doc":"iex> s = Explorer.Series.from_list([\"this \", \" is\", \"great   \"])\n    iex> Explorer.Series.transform(s, &String.trim/1)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([\"this\", \"is\", \"great\"])\n    iex> Explorer.Series.transform(s, &String.length/1)\n    #Explorer.Series","ref":"Explorer.Series.html#transform/2-examples"},{"type":"function","title":"Explorer.Series.unordered_distinct/1","doc":"Returns the unique values of the series, but does not maintain order.\n\nFaster than `distinct/1`.","ref":"Explorer.Series.html#unordered_distinct/1"},{"type":"function","title":"Examples - Explorer.Series.unordered_distinct/1","doc":"iex> s = [1, 1, 2, 2, 3, 3] |> Explorer.Series.from_list()\n    iex> Explorer.Series.unordered_distinct(s)","ref":"Explorer.Series.html#unordered_distinct/1-examples"},{"type":"function","title":"Explorer.Series.upcase/1","doc":"Converts all characters to uppercase.","ref":"Explorer.Series.html#upcase/1"},{"type":"function","title":"Examples - Explorer.Series.upcase/1","doc":"iex> s = Explorer.Series.from_list([\"abc\", \"def\", \"bcd\"])\n    iex> Explorer.Series.upcase(s)\n    #Explorer.Series","ref":"Explorer.Series.html#upcase/1-examples"},{"type":"function","title":"Explorer.Series.variance/2","doc":"Gets the variance of the series.\n\nBy default, this is the sample variance. This function also takes an optional\ndelta degrees of freedom (ddof). Setting this to zero corresponds to the population\nvariance.","ref":"Explorer.Series.html#variance/2"},{"type":"function","title":"Supported dtypes - Explorer.Series.variance/2","doc":"* floats: `{:f, 32}` and `{:f, 64}`\n  * integers: `{:s, 8}`, `{:s, 16}`, `{:s, 32}`, `{:s, 64}`, `{:u, 8}`, `{:u, 16}`, `{:u, 32}` and `{:u, 64}`","ref":"Explorer.Series.html#variance/2-supported-dtypes"},{"type":"function","title":"Examples - Explorer.Series.variance/2","doc":"iex> s = Explorer.Series.from_list([1, 2, nil, 3])\n    iex> Explorer.Series.variance(s)\n    1.0\n\n    iex> s = Explorer.Series.from_list([1.0, 2.0, nil, 3.0])\n    iex> Explorer.Series.variance(s)\n    1.0\n\n    iex> s = Explorer.Series.from_list([~N[2021-01-01 00:00:00], ~N[1999-12-31 00:00:00]])\n    iex> Explorer.Series.variance(s)\n    ** (ArgumentError) Explorer.Series.variance/1 not implemented for dtype {:naive_datetime, :microsecond}. Valid dtypes are {:f, 32}, {:f, 64}, {:s, 8}, {:s, 16}, {:s, 32}, {:s, 64}, {:u, 8}, {:u, 16}, {:u, 32} and {:u, 64}","ref":"Explorer.Series.html#variance/2-examples"},{"type":"function","title":"Explorer.Series.week_of_year/1","doc":"Returns the week-of-year number.\n\nThe return value ranges from 1 to 53 (the last week of year differs by years).\n\nWeeks start on Monday and end on Sunday. If the final week of a year does not end on Sunday, the\nfirst days of the following year will have a week number of 52 (or 53 for a leap year).","ref":"Explorer.Series.html#week_of_year/1"},{"type":"function","title":"Examples - Explorer.Series.week_of_year/1","doc":"iex> s = Explorer.Series.from_list([~D[2023-01-01], ~D[2023-01-02], ~D[2023-02-01], nil])\n    iex> Explorer.Series.week_of_year(s)\n    #Explorer.Series \n\nIt can also be called on a datetime series.\n\n    iex> s = Explorer.Series.from_list(\n    ...>   [~N[2023-01-01 00:00:00], ~N[2023-01-02 00:00:00], ~N[2023-02-01 23:59:59], nil]\n    ...> )\n    iex> Explorer.Series.week_of_year(s)\n    #Explorer.Series","ref":"Explorer.Series.html#week_of_year/1-examples"},{"type":"function","title":"Explorer.Series.window_max/3","doc":"Calculate the rolling max, given a window size and optional list of weights.","ref":"Explorer.Series.html#window_max/3"},{"type":"function","title":"Options - Explorer.Series.window_max/3","doc":"* `:weights` - An optional list of weights with the same length as the window\n    that will be multiplied elementwise with the values in the window. Defaults to `nil`.\n\n  * `:min_periods` - The number of values in the window that should be non-nil\n    before computing a result. If `nil`, it will be set equal to window size. Defaults to `1`.\n\n  * `:center` - Set the labels at the center of the window. Defaults to `false`.","ref":"Explorer.Series.html#window_max/3-options"},{"type":"function","title":"Examples - Explorer.Series.window_max/3","doc":"iex> s = 1..10 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.window_max(s, 4)\n    #Explorer.Series \n\n    iex> s = 1..10 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.window_max(s, 2, weights: [1.0, 2.0])\n    #Explorer.Series","ref":"Explorer.Series.html#window_max/3-examples"},{"type":"function","title":"Explorer.Series.window_mean/3","doc":"Calculate the rolling mean, given a window size and optional list of weights.","ref":"Explorer.Series.html#window_mean/3"},{"type":"function","title":"Options - Explorer.Series.window_mean/3","doc":"* `:weights` - An optional list of weights with the same length as the window\n    that will be multiplied elementwise with the values in the window. Defaults to `nil`.\n\n  * `:min_periods` - The number of values in the window that should be non-nil\n    before computing a result. If `nil`, it will be set equal to window size. Defaults to `1`.\n\n  * `:center` - Set the labels at the center of the window. Defaults to `false`.","ref":"Explorer.Series.html#window_mean/3-options"},{"type":"function","title":"Examples - Explorer.Series.window_mean/3","doc":"iex> s = 1..10 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.window_mean(s, 4)\n    #Explorer.Series \n\n    iex> s = 1..10 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.window_mean(s, 2, weights: [0.25, 0.75])\n    #Explorer.Series \n\n    iex> s = 1..10 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.window_mean(s, 2, weights: [0.25, 0.75], min_periods: nil)\n    #Explorer.Series","ref":"Explorer.Series.html#window_mean/3-examples"},{"type":"function","title":"Explorer.Series.window_median/3","doc":"Calculate the rolling median, given a window size and optional list of weights.","ref":"Explorer.Series.html#window_median/3"},{"type":"function","title":"Options - Explorer.Series.window_median/3","doc":"* `:weights` - An optional list of weights with the same length as the window\n    that will be multiplied elementwise with the values in the window. Defaults to `nil`.\n\n  * `:min_periods` - The number of values in the window that should be non-nil\n    before computing a result. If `nil`, it will be set equal to window size. Defaults to `1`.\n\n  * `:center` - Set the labels at the center of the window. Defaults to `false`.","ref":"Explorer.Series.html#window_median/3-options"},{"type":"function","title":"Examples - Explorer.Series.window_median/3","doc":"iex> s = 1..10 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.window_median(s, 4)\n    #Explorer.Series \n\n    iex> s = 1..10 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.window_median(s, 2, weights: [0.25, 0.75])\n    #Explorer.Series \n\n    iex> s = 1..10 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.window_median(s, 2, weights: [0.25, 0.75], min_periods: nil)\n    #Explorer.Series","ref":"Explorer.Series.html#window_median/3-examples"},{"type":"function","title":"Explorer.Series.window_min/3","doc":"Calculate the rolling min, given a window size and optional list of weights.","ref":"Explorer.Series.html#window_min/3"},{"type":"function","title":"Options - Explorer.Series.window_min/3","doc":"* `:weights` - An optional list of weights with the same length as the window\n    that will be multiplied elementwise with the values in the window. Defaults to `nil`.\n\n  * `:min_periods` - The number of values in the window that should be non-nil\n    before computing a result. If `nil`, it will be set equal to window size. Defaults to `1`.\n\n  * `:center` - Set the labels at the center of the window. Defaults to `false`.","ref":"Explorer.Series.html#window_min/3-options"},{"type":"function","title":"Examples - Explorer.Series.window_min/3","doc":"iex> s = 1..10 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.window_min(s, 4)\n    #Explorer.Series \n\n    iex> s = 1..10 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.window_min(s, 2, weights: [1.0, 2.0])\n    #Explorer.Series","ref":"Explorer.Series.html#window_min/3-examples"},{"type":"function","title":"Explorer.Series.window_standard_deviation/3","doc":"Calculate the rolling standard deviation, given a window size and optional list of weights.","ref":"Explorer.Series.html#window_standard_deviation/3"},{"type":"function","title":"Options - Explorer.Series.window_standard_deviation/3","doc":"* `:weights` - An optional list of weights with the same length as the window\n    that will be multiplied elementwise with the values in the window. Defaults to `nil`.\n\n  * `:min_periods` - The number of values in the window that should be non-nil\n    before computing a result. If `nil`, it will be set equal to window size. Defaults to `1`.\n\n  * `:center` - Set the labels at the center of the window. Defaults to `false`.","ref":"Explorer.Series.html#window_standard_deviation/3-options"},{"type":"function","title":"Examples - Explorer.Series.window_standard_deviation/3","doc":"iex> s = Explorer.Series.from_list([1, 2, 3, 4, 1])\n    iex> Explorer.Series.window_standard_deviation(s, 2)\n    #Explorer.Series \n\n    iex> s = Explorer.Series.from_list([1, 2, 3, 4, 5, 6])\n    iex> Explorer.Series.window_standard_deviation(s, 2, weights: [0.25, 0.75])\n    #Explorer.Series","ref":"Explorer.Series.html#window_standard_deviation/3-examples"},{"type":"function","title":"Explorer.Series.window_sum/3","doc":"Calculate the rolling sum, given a window size and optional list of weights.","ref":"Explorer.Series.html#window_sum/3"},{"type":"function","title":"Options - Explorer.Series.window_sum/3","doc":"* `:weights` - An optional list of weights with the same length as the window\n    that will be multiplied elementwise with the values in the window. Defaults to `nil`.\n\n  * `:min_periods` - The number of values in the window that should be non-nil\n    before computing a result. If `nil`, it will be set equal to window size. Defaults to `1`.\n\n  * `:center` - Set the labels at the center of the window. Defaults to `false`.","ref":"Explorer.Series.html#window_sum/3-options"},{"type":"function","title":"Examples - Explorer.Series.window_sum/3","doc":"iex> s = 1..10 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.window_sum(s, 4)\n    #Explorer.Series \n\n    iex> s = 1..10 |> Enum.to_list() |> Explorer.Series.from_list()\n    iex> Explorer.Series.window_sum(s, 2, weights: [1.0, 2.0])\n    #Explorer.Series","ref":"Explorer.Series.html#window_sum/3-examples"},{"type":"function","title":"Explorer.Series.year/1","doc":"Returns the year number in the calendar date.","ref":"Explorer.Series.html#year/1"},{"type":"function","title":"Examples - Explorer.Series.year/1","doc":"iex> s = Explorer.Series.from_list([~D[2023-01-15], ~D[2022-02-16], ~D[2021-03-20], nil])\n    iex> Explorer.Series.year(s)\n    #Explorer.Series \n\nIt can also be called on a datetime series.\n\n    iex> s = Explorer.Series.from_list([~N[2023-01-15 00:00:00], ~N[2022-02-16 23:59:59.999999], ~N[2021-03-20 12:00:00], nil])\n    iex> Explorer.Series.year(s)\n    #Explorer.Series","ref":"Explorer.Series.html#year/1-examples"},{"type":"type","title":"Explorer.Series.datetime_dtype/0","doc":"","ref":"Explorer.Series.html#t:datetime_dtype/0"},{"type":"type","title":"Explorer.Series.dtype/0","doc":"","ref":"Explorer.Series.html#t:dtype/0"},{"type":"type","title":"Explorer.Series.duration_dtype/0","doc":"","ref":"Explorer.Series.html#t:duration_dtype/0"},{"type":"type","title":"Explorer.Series.float_dtype/0","doc":"","ref":"Explorer.Series.html#t:float_dtype/0"},{"type":"type","title":"Explorer.Series.float_dtype_alias/0","doc":"","ref":"Explorer.Series.html#t:float_dtype_alias/0"},{"type":"type","title":"Explorer.Series.inferable_scalar/0","doc":"","ref":"Explorer.Series.html#t:inferable_scalar/0"},{"type":"type","title":"Explorer.Series.integer_dtype_alias/0","doc":"","ref":"Explorer.Series.html#t:integer_dtype_alias/0"},{"type":"type","title":"Explorer.Series.lazy_t/0","doc":"","ref":"Explorer.Series.html#t:lazy_t/0"},{"type":"type","title":"Explorer.Series.list_dtype/0","doc":"","ref":"Explorer.Series.html#t:list_dtype/0"},{"type":"type","title":"Explorer.Series.naive_datetime_dtype/0","doc":"","ref":"Explorer.Series.html#t:naive_datetime_dtype/0"},{"type":"type","title":"Explorer.Series.non_finite/0","doc":"","ref":"Explorer.Series.html#t:non_finite/0"},{"type":"type","title":"Explorer.Series.signed_integer_dtype/0","doc":"","ref":"Explorer.Series.html#t:signed_integer_dtype/0"},{"type":"type","title":"Explorer.Series.struct_dtype/0","doc":"","ref":"Explorer.Series.html#t:struct_dtype/0"},{"type":"type","title":"Explorer.Series.t/0","doc":"","ref":"Explorer.Series.html#t:t/0"},{"type":"type","title":"Explorer.Series.time_unit/0","doc":"","ref":"Explorer.Series.html#t:time_unit/0"},{"type":"type","title":"Explorer.Series.time_zone/0","doc":"","ref":"Explorer.Series.html#t:time_zone/0"},{"type":"type","title":"Explorer.Series.unsigned_integer_dtype/0","doc":"","ref":"Explorer.Series.html#t:unsigned_integer_dtype/0"},{"type":"module","title":"Explorer.Duration","doc":"Represents a duration of time.\n\nThe value is represented by an integer, and the precision can be one\nof the following:\n\n* `:millisecond`\n* `:microsecond`\n* `:nanosecond`","ref":"Explorer.Duration.html"},{"type":"function","title":"Explorer.Duration.to_string/1","doc":"","ref":"Explorer.Duration.html#to_string/1"},{"type":"type","title":"Explorer.Duration.precision/0","doc":"","ref":"Explorer.Duration.html#t:precision/0"},{"type":"type","title":"Explorer.Duration.t/0","doc":"","ref":"Explorer.Duration.html#t:t/0"},{"type":"module","title":"Explorer.TensorFrame","doc":"TensorFrame is a representation of `Explorer.DataFrame`\nthat is designed to work inside Nx's `defn` expressions.\n\nFor example, imagine the following `defn`:\n\n    defn add_columns(tf) do\n      tf[:a] + tf[:b]\n    end\n\nWe can now pass a DataFrame as argument:\n\n    iex> add_columns(Explorer.DataFrame.new(a: [11, 12], b: [21, 22]))\n    #Nx.Tensor \n\nPassing an `Explorer.DataFrame` to a `defn` will automatically\nconvert it to a TensorFrame. The TensorFrame will lazily\nbuild tensors out of the used dataframe fields.","ref":"Explorer.TensorFrame.html"},{"type":"module","title":"Stack and concatenating - Explorer.TensorFrame","doc":"Due to the integration with Nx, you can also pass dataframes\ninto `Nx.stack/2` and `Nx.concatenate` and they will be automatically\nconverted to tensors. This makes it easy to pass dataframes into\nneural networks and other computationally intensive algorithms:\n\n    iex> Nx.concatenate(Explorer.DataFrame.new(a: [11, 12], b: [21, 22]))\n    #Nx.Tensor \n\n    iex> Nx.stack(Explorer.DataFrame.new(a: [11, 12], b: [21, 22]))\n    #Nx.Tensor \n\n    iex> Nx.stack(Explorer.DataFrame.new(a: [11, 12], b: [21, 22]), axis: -1)\n    #Nx.Tensor","ref":"Explorer.TensorFrame.html#module-stack-and-concatenating"},{"type":"module","title":"Warning: returning TensorFrames - Explorer.TensorFrame","doc":"It is not recommended to return a TensorFrame from a `defn`,\nas that would force all columns to be sent to the CPU/GPU\nand then copied back. Return only the columns that have been\nmodified during the computation. For example, in the example\nabove we used `Nx` to add two columns, if you want to\nput the result of the computation back into a DataFrame,\nyou can use `Explorer.DataFrame.put/4`, which also accepts\ntensors:\n\n    iex> df = Explorer.DataFrame.new(a: [11, 12], b: [21, 22])\n    iex> Explorer.DataFrame.put(df, \"result\", add_columns(df))\n    #Explorer.DataFrame \n\nOne benefit of using `Explorer.DataFrame.put/4` is that it will\npreserve the type of the column if one already exists. Alternatively,\nuse `Explorer.Series.from_tensor/1` to explicitly convert a tensor\nback to a series.","ref":"Explorer.TensorFrame.html#module-warning-returning-tensorframes"},{"type":"module","title":"Supported dtypes - Explorer.TensorFrame","doc":"The following dtypes can be converted to tensors:\n\n  * `:integer`\n  * `{:f, 32}`\n  * `{:f, 64}`\n  * `:boolean`\n  * `:date`\n  * `{:naive_datetime, :millisecond}`\n  * `{:naive_datetime, :microsecond}`\n  * `{:naive_datetime, :nanosecond}`\n\nSee `Explorer.Series.to_iovec/1` and `Explorer.Series.to_tensor/1`\nto learn more about their internal representation.","ref":"Explorer.TensorFrame.html#module-supported-dtypes"},{"type":"function","title":"Explorer.TensorFrame.pull/2","doc":"Pulls a tensor from the TensorFrame.\n\nThis is equivalent to using the `tf[name]` to access\na tensor.","ref":"Explorer.TensorFrame.html#pull/2"},{"type":"function","title":"Examples - Explorer.TensorFrame.pull/2","doc":"Explorer.TensorFrame.pull(tf, \"some_column\")","ref":"Explorer.TensorFrame.html#pull/2-examples"},{"type":"function","title":"Explorer.TensorFrame.put/3","doc":"Puts a tensor in the TensorFrame.\n\nThis function can be invoked from within `defn`.","ref":"Explorer.TensorFrame.html#put/3"},{"type":"function","title":"Examples - Explorer.TensorFrame.put/3","doc":"Explorer.TensorFrame.put(tf, \"result\", some_tensor)","ref":"Explorer.TensorFrame.html#put/3-examples"},{"type":"type","title":"Explorer.TensorFrame.t/0","doc":"","ref":"Explorer.TensorFrame.html#t:t/0"},{"type":"module","title":"Explorer.Backend","doc":"The behaviour for Explorer backends and associated functions.\n\nEach backend is a module with `DataFrame` and `Series` submodules that match the \nrespective behaviours for each.\n\nThe default backend is read from the application environment. Currently, the only \nbackend is an in-memory, eager one based on\n[`Polars`](https://github.com/pola-rs/polars). When alternatives are\navailable, you can use them by configuring your runtime:\n\n    # config/runtime.exs\n    import Config\n    config :explorer, default_backend: Lib.CustomBackend","ref":"Explorer.Backend.html"},{"type":"function","title":"Explorer.Backend.get/0","doc":"Gets the default backend for the current process.","ref":"Explorer.Backend.html#get/0"},{"type":"function","title":"Explorer.Backend.put/1","doc":"Sets the current process default backend to `backend`.\n\nThe default backend is stored only in the process dictionary. This means if\nyou start a separate process, such as `Task`, the default backend must be set\non the new process too.","ref":"Explorer.Backend.html#put/1"},{"type":"function","title":"Examples - Explorer.Backend.put/1","doc":"iex> Explorer.Backend.put(Lib.CustomBackend)\n    Explorer.PolarsBackend\n    iex> Explorer.Backend.get()\n    Lib.CustomBackend","ref":"Explorer.Backend.html#put/1-examples"},{"type":"behaviour","title":"Explorer.Backend.DataFrame","doc":"The behaviour for DataFrame backends.","ref":"Explorer.Backend.DataFrame.html"},{"type":"callback","title":"Explorer.Backend.DataFrame.compute/1","doc":"","ref":"Explorer.Backend.DataFrame.html#c:compute/1"},{"type":"callback","title":"Explorer.Backend.DataFrame.concat_columns/2","doc":"","ref":"Explorer.Backend.DataFrame.html#c:concat_columns/2"},{"type":"callback","title":"Explorer.Backend.DataFrame.concat_rows/2","doc":"","ref":"Explorer.Backend.DataFrame.html#c:concat_rows/2"},{"type":"callback","title":"Explorer.Backend.DataFrame.correlation/4","doc":"","ref":"Explorer.Backend.DataFrame.html#c:correlation/4"},{"type":"callback","title":"Explorer.Backend.DataFrame.covariance/3","doc":"","ref":"Explorer.Backend.DataFrame.html#c:covariance/3"},{"type":"callback","title":"Explorer.Backend.DataFrame.distinct/3","doc":"","ref":"Explorer.Backend.DataFrame.html#c:distinct/3"},{"type":"callback","title":"Explorer.Backend.DataFrame.drop_nil/2","doc":"","ref":"Explorer.Backend.DataFrame.html#c:drop_nil/2"},{"type":"callback","title":"Explorer.Backend.DataFrame.dummies/3","doc":"","ref":"Explorer.Backend.DataFrame.html#c:dummies/3"},{"type":"callback","title":"Explorer.Backend.DataFrame.dump_csv/3","doc":"","ref":"Explorer.Backend.DataFrame.html#c:dump_csv/3"},{"type":"callback","title":"Explorer.Backend.DataFrame.dump_ipc/2","doc":"","ref":"Explorer.Backend.DataFrame.html#c:dump_ipc/2"},{"type":"callback","title":"Explorer.Backend.DataFrame.dump_ipc_stream/2","doc":"","ref":"Explorer.Backend.DataFrame.html#c:dump_ipc_stream/2"},{"type":"callback","title":"Explorer.Backend.DataFrame.dump_ndjson/1","doc":"","ref":"Explorer.Backend.DataFrame.html#c:dump_ndjson/1"},{"type":"callback","title":"Explorer.Backend.DataFrame.dump_parquet/2","doc":"","ref":"Explorer.Backend.DataFrame.html#c:dump_parquet/2"},{"type":"callback","title":"Explorer.Backend.DataFrame.explode/3","doc":"","ref":"Explorer.Backend.DataFrame.html#c:explode/3"},{"type":"callback","title":"Explorer.Backend.DataFrame.filter_with/3","doc":"","ref":"Explorer.Backend.DataFrame.html#c:filter_with/3"},{"type":"callback","title":"Explorer.Backend.DataFrame.from_csv/13","doc":"","ref":"Explorer.Backend.DataFrame.html#c:from_csv/13"},{"type":"callback","title":"Explorer.Backend.DataFrame.from_ipc/2","doc":"","ref":"Explorer.Backend.DataFrame.html#c:from_ipc/2"},{"type":"callback","title":"Explorer.Backend.DataFrame.from_ipc_stream/2","doc":"","ref":"Explorer.Backend.DataFrame.html#c:from_ipc_stream/2"},{"type":"callback","title":"Explorer.Backend.DataFrame.from_ndjson/3","doc":"","ref":"Explorer.Backend.DataFrame.html#c:from_ndjson/3"},{"type":"callback","title":"Explorer.Backend.DataFrame.from_parquet/4","doc":"","ref":"Explorer.Backend.DataFrame.html#c:from_parquet/4"},{"type":"callback","title":"Explorer.Backend.DataFrame.from_query/3","doc":"","ref":"Explorer.Backend.DataFrame.html#c:from_query/3"},{"type":"callback","title":"Explorer.Backend.DataFrame.from_series/1","doc":"","ref":"Explorer.Backend.DataFrame.html#c:from_series/1"},{"type":"callback","title":"Explorer.Backend.DataFrame.from_tabular/2","doc":"","ref":"Explorer.Backend.DataFrame.html#c:from_tabular/2"},{"type":"callback","title":"Explorer.Backend.DataFrame.head/2","doc":"","ref":"Explorer.Backend.DataFrame.html#c:head/2"},{"type":"callback","title":"Explorer.Backend.DataFrame.inspect/2","doc":"","ref":"Explorer.Backend.DataFrame.html#c:inspect/2"},{"type":"function","title":"Explorer.Backend.DataFrame.inspect/5","doc":"Default inspect implementation for backends.","ref":"Explorer.Backend.DataFrame.html#inspect/5"},{"type":"callback","title":"Explorer.Backend.DataFrame.join/4","doc":"","ref":"Explorer.Backend.DataFrame.html#c:join/4"},{"type":"callback","title":"Explorer.Backend.DataFrame.lazy/0","doc":"","ref":"Explorer.Backend.DataFrame.html#c:lazy/0"},{"type":"callback","title":"Explorer.Backend.DataFrame.lazy/1","doc":"","ref":"Explorer.Backend.DataFrame.html#c:lazy/1"},{"type":"callback","title":"Explorer.Backend.DataFrame.load_csv/13","doc":"","ref":"Explorer.Backend.DataFrame.html#c:load_csv/13"},{"type":"callback","title":"Explorer.Backend.DataFrame.load_ipc/2","doc":"","ref":"Explorer.Backend.DataFrame.html#c:load_ipc/2"},{"type":"callback","title":"Explorer.Backend.DataFrame.load_ipc_stream/2","doc":"","ref":"Explorer.Backend.DataFrame.html#c:load_ipc_stream/2"},{"type":"callback","title":"Explorer.Backend.DataFrame.load_ndjson/3","doc":"","ref":"Explorer.Backend.DataFrame.html#c:load_ndjson/3"},{"type":"callback","title":"Explorer.Backend.DataFrame.load_parquet/1","doc":"","ref":"Explorer.Backend.DataFrame.html#c:load_parquet/1"},{"type":"callback","title":"Explorer.Backend.DataFrame.mask/2","doc":"","ref":"Explorer.Backend.DataFrame.html#c:mask/2"},{"type":"callback","title":"Explorer.Backend.DataFrame.mutate_with/3","doc":"","ref":"Explorer.Backend.DataFrame.html#c:mutate_with/3"},{"type":"callback","title":"Explorer.Backend.DataFrame.n_rows/1","doc":"","ref":"Explorer.Backend.DataFrame.html#c:n_rows/1"},{"type":"function","title":"Explorer.Backend.DataFrame.new/3","doc":"Creates a new DataFrame for a given backend.","ref":"Explorer.Backend.DataFrame.html#new/3"},{"type":"callback","title":"Explorer.Backend.DataFrame.nil_count/1","doc":"","ref":"Explorer.Backend.DataFrame.html#c:nil_count/1"},{"type":"callback","title":"Explorer.Backend.DataFrame.owner_export/1","doc":"","ref":"Explorer.Backend.DataFrame.html#c:owner_export/1"},{"type":"callback","title":"Explorer.Backend.DataFrame.owner_import/1","doc":"","ref":"Explorer.Backend.DataFrame.html#c:owner_import/1"},{"type":"callback","title":"Explorer.Backend.DataFrame.owner_reference/1","doc":"","ref":"Explorer.Backend.DataFrame.html#c:owner_reference/1"},{"type":"callback","title":"Explorer.Backend.DataFrame.pivot_longer/6","doc":"","ref":"Explorer.Backend.DataFrame.html#c:pivot_longer/6"},{"type":"callback","title":"Explorer.Backend.DataFrame.pivot_wider/5","doc":"","ref":"Explorer.Backend.DataFrame.html#c:pivot_wider/5"},{"type":"callback","title":"Explorer.Backend.DataFrame.pull/2","doc":"","ref":"Explorer.Backend.DataFrame.html#c:pull/2"},{"type":"callback","title":"Explorer.Backend.DataFrame.put/4","doc":"","ref":"Explorer.Backend.DataFrame.html#c:put/4"},{"type":"callback","title":"Explorer.Backend.DataFrame.re_dtype/1","doc":"","ref":"Explorer.Backend.DataFrame.html#c:re_dtype/1"},{"type":"callback","title":"Explorer.Backend.DataFrame.rename/3","doc":"","ref":"Explorer.Backend.DataFrame.html#c:rename/3"},{"type":"callback","title":"Explorer.Backend.DataFrame.sample/5","doc":"","ref":"Explorer.Backend.DataFrame.html#c:sample/5"},{"type":"callback","title":"Explorer.Backend.DataFrame.select/2","doc":"","ref":"Explorer.Backend.DataFrame.html#c:select/2"},{"type":"callback","title":"Explorer.Backend.DataFrame.slice/2","doc":"","ref":"Explorer.Backend.DataFrame.html#c:slice/2"},{"type":"callback","title":"Explorer.Backend.DataFrame.slice/3","doc":"","ref":"Explorer.Backend.DataFrame.html#c:slice/3"},{"type":"callback","title":"Explorer.Backend.DataFrame.sort_with/6","doc":"","ref":"Explorer.Backend.DataFrame.html#c:sort_with/6"},{"type":"callback","title":"Explorer.Backend.DataFrame.sql/3","doc":"","ref":"Explorer.Backend.DataFrame.html#c:sql/3"},{"type":"callback","title":"Explorer.Backend.DataFrame.summarise_with/3","doc":"","ref":"Explorer.Backend.DataFrame.html#c:summarise_with/3"},{"type":"callback","title":"Explorer.Backend.DataFrame.tail/2","doc":"","ref":"Explorer.Backend.DataFrame.html#c:tail/2"},{"type":"callback","title":"Explorer.Backend.DataFrame.to_csv/5","doc":"","ref":"Explorer.Backend.DataFrame.html#c:to_csv/5"},{"type":"callback","title":"Explorer.Backend.DataFrame.to_ipc/4","doc":"","ref":"Explorer.Backend.DataFrame.html#c:to_ipc/4"},{"type":"callback","title":"Explorer.Backend.DataFrame.to_ipc_stream/3","doc":"","ref":"Explorer.Backend.DataFrame.html#c:to_ipc_stream/3"},{"type":"callback","title":"Explorer.Backend.DataFrame.to_ndjson/2","doc":"","ref":"Explorer.Backend.DataFrame.html#c:to_ndjson/2"},{"type":"callback","title":"Explorer.Backend.DataFrame.to_parquet/4","doc":"","ref":"Explorer.Backend.DataFrame.html#c:to_parquet/4"},{"type":"callback","title":"Explorer.Backend.DataFrame.to_rows/2","doc":"","ref":"Explorer.Backend.DataFrame.html#c:to_rows/2"},{"type":"callback","title":"Explorer.Backend.DataFrame.to_rows_stream/3","doc":"","ref":"Explorer.Backend.DataFrame.html#c:to_rows_stream/3"},{"type":"callback","title":"Explorer.Backend.DataFrame.transpose/4","doc":"","ref":"Explorer.Backend.DataFrame.html#c:transpose/4"},{"type":"callback","title":"Explorer.Backend.DataFrame.unnest/3","doc":"","ref":"Explorer.Backend.DataFrame.html#c:unnest/3"},{"type":"type","title":"Explorer.Backend.DataFrame.basic_types/0","doc":"","ref":"Explorer.Backend.DataFrame.html#t:basic_types/0"},{"type":"type","title":"Explorer.Backend.DataFrame.column_name/0","doc":"","ref":"Explorer.Backend.DataFrame.html#t:column_name/0"},{"type":"type","title":"Explorer.Backend.DataFrame.columns_for_io/0","doc":"","ref":"Explorer.Backend.DataFrame.html#t:columns_for_io/0"},{"type":"type","title":"Explorer.Backend.DataFrame.compression/0","doc":"","ref":"Explorer.Backend.DataFrame.html#t:compression/0"},{"type":"type","title":"Explorer.Backend.DataFrame.df/0","doc":"","ref":"Explorer.Backend.DataFrame.html#t:df/0"},{"type":"type","title":"Explorer.Backend.DataFrame.dtype/0","doc":"","ref":"Explorer.Backend.DataFrame.html#t:dtype/0"},{"type":"type","title":"Explorer.Backend.DataFrame.dtypes/0","doc":"","ref":"Explorer.Backend.DataFrame.html#t:dtypes/0"},{"type":"type","title":"Explorer.Backend.DataFrame.fs_entry/0","doc":"","ref":"Explorer.Backend.DataFrame.html#t:fs_entry/0"},{"type":"type","title":"Explorer.Backend.DataFrame.io_result/1","doc":"","ref":"Explorer.Backend.DataFrame.html#t:io_result/1"},{"type":"type","title":"Explorer.Backend.DataFrame.lazy_frame/0","doc":"","ref":"Explorer.Backend.DataFrame.html#t:lazy_frame/0"},{"type":"type","title":"Explorer.Backend.DataFrame.lazy_series/0","doc":"","ref":"Explorer.Backend.DataFrame.html#t:lazy_series/0"},{"type":"type","title":"Explorer.Backend.DataFrame.mutate_value/0","doc":"","ref":"Explorer.Backend.DataFrame.html#t:mutate_value/0"},{"type":"type","title":"Explorer.Backend.DataFrame.ok_result/0","doc":"","ref":"Explorer.Backend.DataFrame.html#t:ok_result/0"},{"type":"type","title":"Explorer.Backend.DataFrame.option/1","doc":"","ref":"Explorer.Backend.DataFrame.html#t:option/1"},{"type":"type","title":"Explorer.Backend.DataFrame.result/1","doc":"","ref":"Explorer.Backend.DataFrame.html#t:result/1"},{"type":"type","title":"Explorer.Backend.DataFrame.series/0","doc":"","ref":"Explorer.Backend.DataFrame.html#t:series/0"},{"type":"type","title":"Explorer.Backend.DataFrame.t/0","doc":"","ref":"Explorer.Backend.DataFrame.html#t:t/0"},{"type":"module","title":"Explorer.Backend.LazyFrame","doc":"Represents a lazy dataframe for building query expressions.\n\nThe LazyFrame is available inside `filter_with`, `mutate_with`, and\nsimilar. You cannot perform any operation on them except accessing\nits underlying series.","ref":"Explorer.Backend.LazyFrame.html"},{"type":"type","title":"Explorer.Backend.LazyFrame.t/0","doc":"","ref":"Explorer.Backend.LazyFrame.html#t:t/0"},{"type":"module","title":"Explorer.Backend.LazySeries","doc":"This is an opaque implementation of a Series.\n\nIt represents an operation with its arguments.","ref":"Explorer.Backend.LazySeries.html"},{"type":"type","title":"Explorer.Backend.LazySeries.t/0","doc":"","ref":"Explorer.Backend.LazySeries.html#t:t/0"},{"type":"behaviour","title":"Explorer.Backend.Series","doc":"The behaviour for series backends.","ref":"Explorer.Backend.Series.html"},{"type":"callback","title":"Explorer.Backend.Series.abs/1","doc":"","ref":"Explorer.Backend.Series.html#c:abs/1"},{"type":"callback","title":"Explorer.Backend.Series.acos/1","doc":"","ref":"Explorer.Backend.Series.html#c:acos/1"},{"type":"callback","title":"Explorer.Backend.Series.add/3","doc":"","ref":"Explorer.Backend.Series.html#c:add/3"},{"type":"callback","title":"Explorer.Backend.Series.all?/1","doc":"","ref":"Explorer.Backend.Series.html#c:all?/1"},{"type":"callback","title":"Explorer.Backend.Series.all_equal/2","doc":"","ref":"Explorer.Backend.Series.html#c:all_equal/2"},{"type":"callback","title":"Explorer.Backend.Series.any?/1","doc":"","ref":"Explorer.Backend.Series.html#c:any?/1"},{"type":"callback","title":"Explorer.Backend.Series.argmax/1","doc":"","ref":"Explorer.Backend.Series.html#c:argmax/1"},{"type":"callback","title":"Explorer.Backend.Series.argmin/1","doc":"","ref":"Explorer.Backend.Series.html#c:argmin/1"},{"type":"callback","title":"Explorer.Backend.Series.argsort/5","doc":"","ref":"Explorer.Backend.Series.html#c:argsort/5"},{"type":"callback","title":"Explorer.Backend.Series.asin/1","doc":"","ref":"Explorer.Backend.Series.html#c:asin/1"},{"type":"callback","title":"Explorer.Backend.Series.at/2","doc":"","ref":"Explorer.Backend.Series.html#c:at/2"},{"type":"callback","title":"Explorer.Backend.Series.at_every/2","doc":"","ref":"Explorer.Backend.Series.html#c:at_every/2"},{"type":"callback","title":"Explorer.Backend.Series.atan/1","doc":"","ref":"Explorer.Backend.Series.html#c:atan/1"},{"type":"callback","title":"Explorer.Backend.Series.binary_and/2","doc":"","ref":"Explorer.Backend.Series.html#c:binary_and/2"},{"type":"callback","title":"Explorer.Backend.Series.binary_in/2","doc":"","ref":"Explorer.Backend.Series.html#c:binary_in/2"},{"type":"callback","title":"Explorer.Backend.Series.binary_or/2","doc":"","ref":"Explorer.Backend.Series.html#c:binary_or/2"},{"type":"callback","title":"Explorer.Backend.Series.cast/2","doc":"","ref":"Explorer.Backend.Series.html#c:cast/2"},{"type":"callback","title":"Explorer.Backend.Series.categories/1","doc":"","ref":"Explorer.Backend.Series.html#c:categories/1"},{"type":"callback","title":"Explorer.Backend.Series.categorise/2","doc":"","ref":"Explorer.Backend.Series.html#c:categorise/2"},{"type":"callback","title":"Explorer.Backend.Series.ceil/1","doc":"","ref":"Explorer.Backend.Series.html#c:ceil/1"},{"type":"callback","title":"Explorer.Backend.Series.clip/3","doc":"","ref":"Explorer.Backend.Series.html#c:clip/3"},{"type":"callback","title":"Explorer.Backend.Series.coalesce/2","doc":"","ref":"Explorer.Backend.Series.html#c:coalesce/2"},{"type":"callback","title":"Explorer.Backend.Series.concat/1","doc":"","ref":"Explorer.Backend.Series.html#c:concat/1"},{"type":"callback","title":"Explorer.Backend.Series.contains/2","doc":"","ref":"Explorer.Backend.Series.html#c:contains/2"},{"type":"callback","title":"Explorer.Backend.Series.correlation/4","doc":"","ref":"Explorer.Backend.Series.html#c:correlation/4"},{"type":"callback","title":"Explorer.Backend.Series.cos/1","doc":"","ref":"Explorer.Backend.Series.html#c:cos/1"},{"type":"callback","title":"Explorer.Backend.Series.count/1","doc":"","ref":"Explorer.Backend.Series.html#c:count/1"},{"type":"callback","title":"Explorer.Backend.Series.count_matches/2","doc":"","ref":"Explorer.Backend.Series.html#c:count_matches/2"},{"type":"callback","title":"Explorer.Backend.Series.covariance/3","doc":"","ref":"Explorer.Backend.Series.html#c:covariance/3"},{"type":"callback","title":"Explorer.Backend.Series.cumulative_max/2","doc":"","ref":"Explorer.Backend.Series.html#c:cumulative_max/2"},{"type":"callback","title":"Explorer.Backend.Series.cumulative_min/2","doc":"","ref":"Explorer.Backend.Series.html#c:cumulative_min/2"},{"type":"callback","title":"Explorer.Backend.Series.cumulative_product/2","doc":"","ref":"Explorer.Backend.Series.html#c:cumulative_product/2"},{"type":"callback","title":"Explorer.Backend.Series.cumulative_sum/2","doc":"","ref":"Explorer.Backend.Series.html#c:cumulative_sum/2"},{"type":"callback","title":"Explorer.Backend.Series.cut/5","doc":"","ref":"Explorer.Backend.Series.html#c:cut/5"},{"type":"callback","title":"Explorer.Backend.Series.day_of_week/1","doc":"","ref":"Explorer.Backend.Series.html#c:day_of_week/1"},{"type":"callback","title":"Explorer.Backend.Series.day_of_year/1","doc":"","ref":"Explorer.Backend.Series.html#c:day_of_year/1"},{"type":"callback","title":"Explorer.Backend.Series.distinct/1","doc":"","ref":"Explorer.Backend.Series.html#c:distinct/1"},{"type":"callback","title":"Explorer.Backend.Series.divide/3","doc":"","ref":"Explorer.Backend.Series.html#c:divide/3"},{"type":"callback","title":"Explorer.Backend.Series.downcase/1","doc":"","ref":"Explorer.Backend.Series.html#c:downcase/1"},{"type":"callback","title":"Explorer.Backend.Series.equal/2","doc":"","ref":"Explorer.Backend.Series.html#c:equal/2"},{"type":"callback","title":"Explorer.Backend.Series.ewm_mean/5","doc":"","ref":"Explorer.Backend.Series.html#c:ewm_mean/5"},{"type":"callback","title":"Explorer.Backend.Series.ewm_standard_deviation/6","doc":"","ref":"Explorer.Backend.Series.html#c:ewm_standard_deviation/6"},{"type":"callback","title":"Explorer.Backend.Series.ewm_variance/6","doc":"","ref":"Explorer.Backend.Series.html#c:ewm_variance/6"},{"type":"callback","title":"Explorer.Backend.Series.exp/1","doc":"","ref":"Explorer.Backend.Series.html#c:exp/1"},{"type":"callback","title":"Explorer.Backend.Series.field/2","doc":"","ref":"Explorer.Backend.Series.html#c:field/2"},{"type":"callback","title":"Explorer.Backend.Series.fill_missing_with_strategy/2","doc":"","ref":"Explorer.Backend.Series.html#c:fill_missing_with_strategy/2"},{"type":"callback","title":"Explorer.Backend.Series.fill_missing_with_value/2","doc":"","ref":"Explorer.Backend.Series.html#c:fill_missing_with_value/2"},{"type":"callback","title":"Explorer.Backend.Series.first/1","doc":"","ref":"Explorer.Backend.Series.html#c:first/1"},{"type":"callback","title":"Explorer.Backend.Series.floor/1","doc":"","ref":"Explorer.Backend.Series.html#c:floor/1"},{"type":"callback","title":"Explorer.Backend.Series.format/1","doc":"","ref":"Explorer.Backend.Series.html#c:format/1"},{"type":"callback","title":"Explorer.Backend.Series.frequencies/1","doc":"","ref":"Explorer.Backend.Series.html#c:frequencies/1"},{"type":"callback","title":"Explorer.Backend.Series.from_binary/2","doc":"","ref":"Explorer.Backend.Series.html#c:from_binary/2"},{"type":"callback","title":"Explorer.Backend.Series.from_list/2","doc":"","ref":"Explorer.Backend.Series.html#c:from_list/2"},{"type":"callback","title":"Explorer.Backend.Series.greater/2","doc":"","ref":"Explorer.Backend.Series.html#c:greater/2"},{"type":"callback","title":"Explorer.Backend.Series.greater_equal/2","doc":"","ref":"Explorer.Backend.Series.html#c:greater_equal/2"},{"type":"callback","title":"Explorer.Backend.Series.head/2","doc":"","ref":"Explorer.Backend.Series.html#c:head/2"},{"type":"callback","title":"Explorer.Backend.Series.hour/1","doc":"","ref":"Explorer.Backend.Series.html#c:hour/1"},{"type":"callback","title":"Explorer.Backend.Series.inspect/2","doc":"","ref":"Explorer.Backend.Series.html#c:inspect/2"},{"type":"function","title":"Explorer.Backend.Series.inspect/5","doc":"Default inspect implementation for backends.","ref":"Explorer.Backend.Series.html#inspect/5"},{"type":"callback","title":"Explorer.Backend.Series.is_finite/1","doc":"","ref":"Explorer.Backend.Series.html#c:is_finite/1"},{"type":"callback","title":"Explorer.Backend.Series.is_infinite/1","doc":"","ref":"Explorer.Backend.Series.html#c:is_infinite/1"},{"type":"callback","title":"Explorer.Backend.Series.is_nan/1","doc":"","ref":"Explorer.Backend.Series.html#c:is_nan/1"},{"type":"callback","title":"Explorer.Backend.Series.is_nil/1","doc":"","ref":"Explorer.Backend.Series.html#c:is_nil/1"},{"type":"callback","title":"Explorer.Backend.Series.is_not_nil/1","doc":"","ref":"Explorer.Backend.Series.html#c:is_not_nil/1"},{"type":"callback","title":"Explorer.Backend.Series.join/2","doc":"","ref":"Explorer.Backend.Series.html#c:join/2"},{"type":"callback","title":"Explorer.Backend.Series.json_decode/2","doc":"","ref":"Explorer.Backend.Series.html#c:json_decode/2"},{"type":"callback","title":"Explorer.Backend.Series.json_path_match/2","doc":"","ref":"Explorer.Backend.Series.html#c:json_path_match/2"},{"type":"callback","title":"Explorer.Backend.Series.last/1","doc":"","ref":"Explorer.Backend.Series.html#c:last/1"},{"type":"callback","title":"Explorer.Backend.Series.lengths/1","doc":"","ref":"Explorer.Backend.Series.html#c:lengths/1"},{"type":"callback","title":"Explorer.Backend.Series.less/2","doc":"","ref":"Explorer.Backend.Series.html#c:less/2"},{"type":"callback","title":"Explorer.Backend.Series.less_equal/2","doc":"","ref":"Explorer.Backend.Series.html#c:less_equal/2"},{"type":"callback","title":"Explorer.Backend.Series.log/1","doc":"","ref":"Explorer.Backend.Series.html#c:log/1"},{"type":"callback","title":"Explorer.Backend.Series.log/2","doc":"","ref":"Explorer.Backend.Series.html#c:log/2"},{"type":"callback","title":"Explorer.Backend.Series.lstrip/2","doc":"","ref":"Explorer.Backend.Series.html#c:lstrip/2"},{"type":"callback","title":"Explorer.Backend.Series.mask/2","doc":"","ref":"Explorer.Backend.Series.html#c:mask/2"},{"type":"callback","title":"Explorer.Backend.Series.max/1","doc":"","ref":"Explorer.Backend.Series.html#c:max/1"},{"type":"callback","title":"Explorer.Backend.Series.mean/1","doc":"","ref":"Explorer.Backend.Series.html#c:mean/1"},{"type":"callback","title":"Explorer.Backend.Series.median/1","doc":"","ref":"Explorer.Backend.Series.html#c:median/1"},{"type":"callback","title":"Explorer.Backend.Series.member?/2","doc":"","ref":"Explorer.Backend.Series.html#c:member?/2"},{"type":"callback","title":"Explorer.Backend.Series.min/1","doc":"","ref":"Explorer.Backend.Series.html#c:min/1"},{"type":"callback","title":"Explorer.Backend.Series.minute/1","doc":"","ref":"Explorer.Backend.Series.html#c:minute/1"},{"type":"callback","title":"Explorer.Backend.Series.mode/1","doc":"","ref":"Explorer.Backend.Series.html#c:mode/1"},{"type":"callback","title":"Explorer.Backend.Series.month/1","doc":"","ref":"Explorer.Backend.Series.html#c:month/1"},{"type":"callback","title":"Explorer.Backend.Series.multiply/3","doc":"","ref":"Explorer.Backend.Series.html#c:multiply/3"},{"type":"callback","title":"Explorer.Backend.Series.n_distinct/1","doc":"","ref":"Explorer.Backend.Series.html#c:n_distinct/1"},{"type":"function","title":"Explorer.Backend.Series.new/2","doc":"Create a new `Series`.","ref":"Explorer.Backend.Series.html#new/2"},{"type":"callback","title":"Explorer.Backend.Series.nil_count/1","doc":"","ref":"Explorer.Backend.Series.html#c:nil_count/1"},{"type":"callback","title":"Explorer.Backend.Series.not_equal/2","doc":"","ref":"Explorer.Backend.Series.html#c:not_equal/2"},{"type":"callback","title":"Explorer.Backend.Series.owner_export/1","doc":"","ref":"Explorer.Backend.Series.html#c:owner_export/1"},{"type":"callback","title":"Explorer.Backend.Series.owner_import/1","doc":"","ref":"Explorer.Backend.Series.html#c:owner_import/1"},{"type":"callback","title":"Explorer.Backend.Series.owner_reference/1","doc":"","ref":"Explorer.Backend.Series.html#c:owner_reference/1"},{"type":"callback","title":"Explorer.Backend.Series.peaks/2","doc":"","ref":"Explorer.Backend.Series.html#c:peaks/2"},{"type":"callback","title":"Explorer.Backend.Series.pow/3","doc":"","ref":"Explorer.Backend.Series.html#c:pow/3"},{"type":"callback","title":"Explorer.Backend.Series.product/1","doc":"","ref":"Explorer.Backend.Series.html#c:product/1"},{"type":"callback","title":"Explorer.Backend.Series.qcut/5","doc":"","ref":"Explorer.Backend.Series.html#c:qcut/5"},{"type":"callback","title":"Explorer.Backend.Series.quantile/2","doc":"","ref":"Explorer.Backend.Series.html#c:quantile/2"},{"type":"callback","title":"Explorer.Backend.Series.quotient/2","doc":"","ref":"Explorer.Backend.Series.html#c:quotient/2"},{"type":"callback","title":"Explorer.Backend.Series.rank/4","doc":"","ref":"Explorer.Backend.Series.html#c:rank/4"},{"type":"callback","title":"Explorer.Backend.Series.re_contains/2","doc":"","ref":"Explorer.Backend.Series.html#c:re_contains/2"},{"type":"callback","title":"Explorer.Backend.Series.re_count_matches/2","doc":"","ref":"Explorer.Backend.Series.html#c:re_count_matches/2"},{"type":"callback","title":"Explorer.Backend.Series.re_named_captures/2","doc":"","ref":"Explorer.Backend.Series.html#c:re_named_captures/2"},{"type":"callback","title":"Explorer.Backend.Series.re_replace/3","doc":"","ref":"Explorer.Backend.Series.html#c:re_replace/3"},{"type":"callback","title":"Explorer.Backend.Series.re_scan/2","doc":"","ref":"Explorer.Backend.Series.html#c:re_scan/2"},{"type":"callback","title":"Explorer.Backend.Series.remainder/2","doc":"","ref":"Explorer.Backend.Series.html#c:remainder/2"},{"type":"callback","title":"Explorer.Backend.Series.replace/3","doc":"","ref":"Explorer.Backend.Series.html#c:replace/3"},{"type":"callback","title":"Explorer.Backend.Series.reverse/1","doc":"","ref":"Explorer.Backend.Series.html#c:reverse/1"},{"type":"callback","title":"Explorer.Backend.Series.round/2","doc":"","ref":"Explorer.Backend.Series.html#c:round/2"},{"type":"callback","title":"Explorer.Backend.Series.row_index/1","doc":"","ref":"Explorer.Backend.Series.html#c:row_index/1"},{"type":"callback","title":"Explorer.Backend.Series.rstrip/2","doc":"","ref":"Explorer.Backend.Series.html#c:rstrip/2"},{"type":"callback","title":"Explorer.Backend.Series.sample/5","doc":"","ref":"Explorer.Backend.Series.html#c:sample/5"},{"type":"callback","title":"Explorer.Backend.Series.second/1","doc":"","ref":"Explorer.Backend.Series.html#c:second/1"},{"type":"callback","title":"Explorer.Backend.Series.select/3","doc":"","ref":"Explorer.Backend.Series.html#c:select/3"},{"type":"callback","title":"Explorer.Backend.Series.shift/3","doc":"","ref":"Explorer.Backend.Series.html#c:shift/3"},{"type":"callback","title":"Explorer.Backend.Series.sin/1","doc":"","ref":"Explorer.Backend.Series.html#c:sin/1"},{"type":"callback","title":"Explorer.Backend.Series.size/1","doc":"","ref":"Explorer.Backend.Series.html#c:size/1"},{"type":"callback","title":"Explorer.Backend.Series.skew/2","doc":"","ref":"Explorer.Backend.Series.html#c:skew/2"},{"type":"callback","title":"Explorer.Backend.Series.slice/2","doc":"","ref":"Explorer.Backend.Series.html#c:slice/2"},{"type":"callback","title":"Explorer.Backend.Series.slice/3","doc":"","ref":"Explorer.Backend.Series.html#c:slice/3"},{"type":"callback","title":"Explorer.Backend.Series.sort/5","doc":"","ref":"Explorer.Backend.Series.html#c:sort/5"},{"type":"callback","title":"Explorer.Backend.Series.split/2","doc":"","ref":"Explorer.Backend.Series.html#c:split/2"},{"type":"callback","title":"Explorer.Backend.Series.split_into/3","doc":"","ref":"Explorer.Backend.Series.html#c:split_into/3"},{"type":"callback","title":"Explorer.Backend.Series.standard_deviation/2","doc":"","ref":"Explorer.Backend.Series.html#c:standard_deviation/2"},{"type":"callback","title":"Explorer.Backend.Series.strftime/2","doc":"","ref":"Explorer.Backend.Series.html#c:strftime/2"},{"type":"callback","title":"Explorer.Backend.Series.strip/2","doc":"","ref":"Explorer.Backend.Series.html#c:strip/2"},{"type":"callback","title":"Explorer.Backend.Series.strptime/2","doc":"","ref":"Explorer.Backend.Series.html#c:strptime/2"},{"type":"callback","title":"Explorer.Backend.Series.substring/3","doc":"","ref":"Explorer.Backend.Series.html#c:substring/3"},{"type":"callback","title":"Explorer.Backend.Series.subtract/3","doc":"","ref":"Explorer.Backend.Series.html#c:subtract/3"},{"type":"callback","title":"Explorer.Backend.Series.sum/1","doc":"","ref":"Explorer.Backend.Series.html#c:sum/1"},{"type":"callback","title":"Explorer.Backend.Series.tail/2","doc":"","ref":"Explorer.Backend.Series.html#c:tail/2"},{"type":"callback","title":"Explorer.Backend.Series.tan/1","doc":"","ref":"Explorer.Backend.Series.html#c:tan/1"},{"type":"callback","title":"Explorer.Backend.Series.to_iovec/1","doc":"","ref":"Explorer.Backend.Series.html#c:to_iovec/1"},{"type":"callback","title":"Explorer.Backend.Series.to_list/1","doc":"","ref":"Explorer.Backend.Series.html#c:to_list/1"},{"type":"callback","title":"Explorer.Backend.Series.transform/2","doc":"","ref":"Explorer.Backend.Series.html#c:transform/2"},{"type":"callback","title":"Explorer.Backend.Series.unary_not/1","doc":"","ref":"Explorer.Backend.Series.html#c:unary_not/1"},{"type":"callback","title":"Explorer.Backend.Series.unordered_distinct/1","doc":"","ref":"Explorer.Backend.Series.html#c:unordered_distinct/1"},{"type":"callback","title":"Explorer.Backend.Series.upcase/1","doc":"","ref":"Explorer.Backend.Series.html#c:upcase/1"},{"type":"callback","title":"Explorer.Backend.Series.variance/2","doc":"","ref":"Explorer.Backend.Series.html#c:variance/2"},{"type":"callback","title":"Explorer.Backend.Series.week_of_year/1","doc":"","ref":"Explorer.Backend.Series.html#c:week_of_year/1"},{"type":"callback","title":"Explorer.Backend.Series.window_max/5","doc":"","ref":"Explorer.Backend.Series.html#c:window_max/5"},{"type":"callback","title":"Explorer.Backend.Series.window_mean/5","doc":"","ref":"Explorer.Backend.Series.html#c:window_mean/5"},{"type":"callback","title":"Explorer.Backend.Series.window_median/5","doc":"","ref":"Explorer.Backend.Series.html#c:window_median/5"},{"type":"callback","title":"Explorer.Backend.Series.window_min/5","doc":"","ref":"Explorer.Backend.Series.html#c:window_min/5"},{"type":"callback","title":"Explorer.Backend.Series.window_standard_deviation/5","doc":"","ref":"Explorer.Backend.Series.html#c:window_standard_deviation/5"},{"type":"callback","title":"Explorer.Backend.Series.window_sum/5","doc":"","ref":"Explorer.Backend.Series.html#c:window_sum/5"},{"type":"callback","title":"Explorer.Backend.Series.year/1","doc":"","ref":"Explorer.Backend.Series.html#c:year/1"},{"type":"type","title":"Explorer.Backend.Series.df/0","doc":"","ref":"Explorer.Backend.Series.html#t:df/0"},{"type":"type","title":"Explorer.Backend.Series.dtype/0","doc":"","ref":"Explorer.Backend.Series.html#t:dtype/0"},{"type":"type","title":"Explorer.Backend.Series.io_result/1","doc":"","ref":"Explorer.Backend.Series.html#t:io_result/1"},{"type":"type","title":"Explorer.Backend.Series.lazy_s/0","doc":"","ref":"Explorer.Backend.Series.html#t:lazy_s/0"},{"type":"type","title":"Explorer.Backend.Series.non_finite/0","doc":"","ref":"Explorer.Backend.Series.html#t:non_finite/0"},{"type":"type","title":"Explorer.Backend.Series.option/1","doc":"","ref":"Explorer.Backend.Series.html#t:option/1"},{"type":"type","title":"Explorer.Backend.Series.s/0","doc":"","ref":"Explorer.Backend.Series.html#t:s/0"},{"type":"type","title":"Explorer.Backend.Series.t/0","doc":"","ref":"Explorer.Backend.Series.html#t:t/0"},{"type":"type","title":"Explorer.Backend.Series.valid_types/0","doc":"","ref":"Explorer.Backend.Series.html#t:valid_types/0"},{"type":"module","title":"Explorer.PolarsBackend","doc":"The Explorer backend for Polars.","ref":"Explorer.PolarsBackend.html"},{"type":"extras","title":"Ten Minutes to Explorer","doc":"<!-- livebook:{\"persist_outputs\":true} -->\n\n# Ten Minutes to Explorer\n\n```elixir\nMix.install([\n  {:explorer, \"~> 0.9.0\"},\n  {:kino, \"~> 0.13.0\"}\n])\n```","ref":"exploring_explorer.html"},{"type":"extras","title":"Introduction - Ten Minutes to Explorer","doc":"Explorer is a dataframe library for Elixir. A dataframe is a common data structure used in data analysis. It is a two-dimensional table composed of columns and rows similar to a SQL table or a spreadsheet.\n\nExplorer's aim is to provide a simple and powerful API for manipulating dataframes. It takes influences mainly\n from the [`tidyverse`](https://www.tidyverse.org), but if you've used other dataframe libraries like `pandas` you shouldn't have too much trouble working with Explorer.\n\nThis document is meant to give you a crash course in using Explorer. More in-depth documentation can be found in the relevant sections of the docs.\n\nWe strongly recommend you run this livebook locally so you can see the outputs and play with the inputs!","ref":"exploring_explorer.html#introduction"},{"type":"extras","title":"Reading and writing data - Ten Minutes to Explorer","doc":"Data can be read from delimited files (like CSV), NDJSON, Parquet, and the Arrow IPC (feather) format. You can also load in data from a map or keyword list of columns with `Explorer.DataFrame.new/1`.\n\nFor CSV, your 'usual suspects' of options are available:\n\n* `delimiter` - A single character used to separate fields within a record. (default: `\",\"`)\n* `dtypes` - A keyword list of `[column_name: dtype]`. If a type is not specified for a column, it is imputed from the first 1000 rows. (default: `[]`)\n* `header` - Does the file have a header of column names as the first row or not? (default: `true`)\n* `max_rows` - Maximum number of lines to read. (default: `nil`)\n* `nil_values` - A list of strings that should be interpreted as a nil values. (default: `[]`)\n* `skip_rows` - The number of lines to skip at the beginning of the file. (default: `0`)\n* `skip_rows_after_header` - The number of lines to skip at the after the header row. (default: `0`)\n* `columns` - A list of column names to keep. If present, only these columns are read into the dataframe. (default: `nil`)\n\n`Explorer` also has multiple example datasets built in, which you can load from the `Explorer.Datasets` module like so:\n\n```elixir\ndf = Explorer.Datasets.fossil_fuels()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\nYou'll notice that the output looks slightly different than many dataframe libraries. `Explorer` takes inspiration on this front from [`glimpse`](https://pillar.r-lib.org/reference/glimpse.html) in R. A benefit to this approach is that you will rarely need to elide columns.\n\nIf you'd like to see a table with your data, take a look at [Kino Explorer](https://hexdocs.pm/kino_explorer), that provides a rich table with filtering and sorting.\n\n<!-- livebook:{\"break_markdown\":true} -->\n\nWriting files is very similar to reading them. The options are a little more limited:\n\n* `header` - Should the column names be written as the first line of the file? (default: `true`)\n* `delimiter` - A single character used to separate fields within a record. (default: `\",\"`)\n\nFirst, let's add some useful aliases:\n\n```elixir\nalias Explorer.DataFrame\nalias Explorer.Series\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\nExplorer.Series\n```\n\nAnd then write to a file of your choosing:\n\n```elixir\ninput = Kino.Input.text(\"Filename\")\n```\n\n```elixir\nfilename = Kino.Input.read(input)\nDataFrame.to_csv(df, filename)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n:ok\n```","ref":"exploring_explorer.html#reading-and-writing-data"},{"type":"extras","title":"Working with Series - Ten Minutes to Explorer","doc":"`Explorer`, like `Polars`, works up from the concept of a `Series`. In many ways, you can think of a dataframe as a row-aligned map of `Series`. These are like `vectors` in R or `series` in Pandas.\n\n`Explorer` supports the following `Series` `dtypes`:\n\n* `:null` - Null\n* `:binary` - Binaries (sequences of bytes)\n* `:boolean` - Boolean\n* `:category` - Strings but represented internally as integers\n* `:date` - Date type that unwraps to `Elixir.Date`\n* `{:datetime, precision}` - DateTime type with millisecond/microsecond/nanosecond precision that unwraps to `Elixir.NaiveDateTime`\n* `{:duration, precision}` - Duration type with millisecond/microsecond/nanosecond precision that unwraps to `Explorer.Duration`\n* `{:f, size}` - a 32-bit or 64-bit floating point number\n* `{:s, size}` - a 8-bit or 16-bit or 32-bit or 64-bit signed integer\n* `{:u, size}` - a 8-bit or 16-bit or 32-bit or 64-bit unsigned integer\n* `:string` - UTF-8 encoded binary\n* `:time` - Time type that unwraps to `Elixir.Time`\n* `:list` - a nested data type like `Elixir.List`. It is a sequence of values having the same data type.\n* `:struct` - a nested data type like a `Elixir.Map`, but with the same keys and matching data types.\n\n`Series` can be constructed from Elixir basic types. For example:\n\n```elixir\ns1 = Series.from_list([1, 2, 3])\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.Series \n```\n\n```elixir\ns2 = Series.from_list([\"a\", \"b\", \"c\"])\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.Series \n```\n\n```elixir\ns3 = Series.from_list([~D[2011-01-01], ~D[1965-01-21]])\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.Series \n```\n\nYou'll notice that the `dtype` and size of the `Series` are at the top of the printed value. You can get those programmatically as well.\n\n```elixir\nSeries.dtype(s3)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n:date\n```\n\n```elixir\nSeries.size(s3)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n2\n```\n\nAnd the printed values max out at 50:\n\n```elixir\n1..100 |> Enum.to_list() |> Series.from_list()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.Series \n```\n\nSeries are also nullable.\n\n```elixir\ns = Series.from_list([1.0, 2.0, nil, nil, 5.0])\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.Series \n```\n\nAnd you can fill in those missing values using one of the following strategies:\n\n* `:forward` - replace nil with the previous value\n* `:backward` - replace nil with the next value\n* `:max` - replace nil with the series maximum\n* `:min` - replace nil with the series minimum\n* `:mean` - replace nil with the series mean\n\n```elixir\nSeries.fill_missing(s, :forward)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.Series \n```\n\nIn the case of mixed numeric types (i.e. integers and floats), `Series` will downcast to a float:\n\n```elixir\nSeries.from_list([1, 2.0])\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.Series \n```\n\nIn all other cases, `Series` must all be of the same `dtype` or else you'll get an `ArgumentError`.\n\n```elixir\nSeries.from_list([1, 2, 3, \"a\"])\n```\n\nOne of the goals of `Explorer` is *useful error messages*. If you look at the error above, you get:\n\n> the value \"a\" does not match the inferred dtype {:s, 64}\n\nHopefully this makes abundantly clear what's going on.\n\n`Series` also implements the `Access` protocol. You can slice and dice in many ways:\n\n```elixir\ns = 1..10 |> Enum.to_list() |> Series.from_list()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.Series \n```\n\n```elixir\ns[1]\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n2\n```\n\n```elixir\ns[-1]\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n10\n```\n\n```elixir\ns[0..4]\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.Series \n```\n\n```elixir\ns[[0, 4, 4]]\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.Series \n```\n\nAnd of course, you can convert back to an Elixir list.\n\n```elixir\nSeries.to_list(s)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[1, 2, 3, 4, 5, 6, 7, 8, 9, 10]\n```\n\n`Explorer` comparisons return boolean series. We will talk more\nabout boolean series later.\n\n```elixir\ns = 1..11 |> Enum.to_list() |> Series.from_list()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.Series \n```\n\n```elixir\ns1 = 11..1//-1 |> Enum.to_list() |> Series.from_list()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.Series \n```\n\n```elixir\nSeries.equal(s, s1)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.Series \n```\n\n```elixir\nSeries.equal(s, 5)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.Series \n```\n\n```elixir\nSeries.not_equal(s, 10)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.Series \n```\n\n```elixir\nSeries.greater_equal(s, 4)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.Series \n```\n\n`Explorer` supports arithmetic.\n\n```elixir\nSeries.add(s, s1)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.Series \n```\n\n```elixir\nSeries.subtract(s, 4)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.Series \n```\n\n```elixir\nSeries.multiply(s, s1)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.Series \n```\n\nRemember those helpful errors? We've tried to add those throughout. So if you try to do arithmetic with mismatching dtypes:\n\n```elixir\ns = Series.from_list([1, 2, 3])\ns1 = Series.from_list([1.0, 2.0, 3.0])\nSeries.add(s, s1)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.Series \n```\n\nJust kidding! Integers and floats will downcast to floats. Let's try again:\n\n```elixir\ns = Series.from_list([1, 2, 3])\ns1 = Series.from_list([\"a\", \"b\", \"c\"])\nSeries.add(s, s1)\n```\n\nYou can flip them around.\n\n```elixir\ns = Series.from_list([1, 2, 3, 4])\nSeries.reverse(s)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.Series \n```\n\nAnd sort.\n\n```elixir\n1..100 |> Enum.to_list() |> Enum.shuffle() |> Series.from_list() |> Series.sort()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.Series \n```\n\nOr argsort.\n\n```elixir\ns = 1..100 |> Enum.to_list() |> Enum.shuffle() |> Series.from_list()\nids = Series.argsort(s) |> Series.to_list()\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[84, 8, 3, 96, 76, 91, 36, 87, 42, 94, 69, 27, 52, 80, 34, 13, 28, 38, 89, 61, 21, 56, 55, 82, 22,\n 31, 4, 45, 49, 12, 30, 41, 92, 70, 83, 77, 79, 1, 26, 50, 75, 97, 71, 64, 68, 47, 44, 95, 2, 51,\n ...]\n```\n\nWhich you can pass to `Explorer.Series.slice/2` if you want the sorted values.\n\n```elixir\nSeries.slice(s, ids)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.Series \n```\n\nYou can calculate cumulative values.\n\n```elixir\ns = 1..100 |> Enum.to_list() |> Series.from_list()\nSeries.cumulative_sum(s)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.Series \n```\n\nOr rolling ones.\n\n```elixir\nSeries.window_sum(s, 4)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.Series \n```\n\nYou can count and list unique values.\n\n```elixir\ns = Series.from_list([\"a\", \"b\", \"b\", \"c\", \"c\", \"c\"])\nSeries.distinct(s)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.Series \n```\n\n```elixir\nSeries.n_distinct(s)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n3\n```\n\nAnd you can even get a dataframe showing the frequencies for each distinct value.\n\n```elixir\nSeries.frequencies(s)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\nBack to those boolean series returned by comparison functions like `equal` and `not_equal`.\n\nThese boolean series can be combined with other functions to perform conditional operations.\n\n```elixir\ns1 = Series.from_list([\"It\", \"was\", \"the\", \"best\", \"of\", \"times\"])\ns1 |> Series.equal(\"best\") |> Series.select(\"worst\", s1)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.Series \n```","ref":"exploring_explorer.html#working-with-series"},{"type":"extras","title":"Working with DataFrames - Ten Minutes to Explorer","doc":"A `DataFrame` is really just a collection of `Series` of the same size. Which is why you can create a `DataFrame` from a `Keyword` list.\n\n```elixir\nDataFrame.new(a: [1, 2, 3], b: [\"a\", \"b\", \"c\"])\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\nSimilarly to `Series`, the `Inspect` implementation prints some info at the top and to the left. At the top we see the shape of the dataframe (rows and columns) and then for each column we see the name, dtype, and first five values. We can see a bit more from that built-in dataset we loaded in earlier.\n\n```elixir\ndf\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\nYou will also see grouping information there, but we'll get to that later. You can get the info yourself directly:\n\n```elixir\nDataFrame.names(df)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[\"year\", \"country\", \"total\", \"solid_fuel\", \"liquid_fuel\", \"gas_fuel\", \"cement\", \"gas_flaring\",\n \"per_capita\", \"bunker_fuels\"]\n```\n\n```elixir\nDataFrame.dtypes(df)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%{\n  \"bunker_fuels\" => {:s, 64},\n  \"cement\" => {:s, 64},\n  \"country\" => :string,\n  \"gas_flaring\" => {:s, 64},\n  \"gas_fuel\" => {:s, 64},\n  \"liquid_fuel\" => {:s, 64},\n  \"per_capita\" => {:f, 64},\n  \"solid_fuel\" => {:s, 64},\n  \"total\" => {:s, 64},\n  \"year\" => {:s, 64}\n}\n```\n\n```elixir\nDataFrame.shape(df)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{1094, 10}\n```\n\n```elixir\n{DataFrame.n_rows(df), DataFrame.n_columns(df)}\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{1094, 10}\n```\n\nWe can grab the head.\n\n```elixir\nDataFrame.head(df)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\nOr the tail. Let's get a few more values from the tail.\n\n```elixir\nDataFrame.tail(df, 10)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```","ref":"exploring_explorer.html#working-with-dataframes"},{"type":"extras","title":"Verbs and macros - Ten Minutes to Explorer","doc":"In `Explorer`, like in `dplyr`, we have five main verbs to work with dataframes:\n\n* select\n* filter\n* mutate\n* sort\n* summarise\n\nWe are going to explore then in this notebook, but first we need to \"require\"\nthe `Explorer.DataFrame` module in order to load the macros needed for these verbs.\n\nI want to take the opportunity to create a shorter alias for the `DataFrame` module,\ncalled `DF`:\n\n```elixir\nrequire DataFrame, as: DF\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\nExplorer.DataFrame\n```\n\nFrom now on we are using the shorter version, `DF`, to refer to the required `Explorer.DataFrame`\nmodule.","ref":"exploring_explorer.html#verbs-and-macros"},{"type":"extras","title":"Select - Ten Minutes to Explorer","doc":"Let's jump right into it. We can select columns pretty simply.\n\n```elixir\nDF.select(df, [\"year\", \"country\"])\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\nBut Elixir gives us some superpowers. In R there's [`tidy-select`](https://dplyr.tidyverse.org/reference/dplyr_tidy_select.html). I don't think we need that in Elixir. Anywhere in `Explorer` where you need to pass a list of column names, you can also execute a filtering callback on the column names. It's just an anonymous function passed to `df |> DataFrame.names() |> Enum.filter(callback_here)`.\n\n```elixir\nDF.select(df, &String.ends_with?(&1, \"fuel\"))\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\nWant all but some columns? `discard/2` performs the opposite of `select/2`.\n\n```elixir\nDF.discard(df, &String.ends_with?(&1, \"fuel\"))\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```","ref":"exploring_explorer.html#select"},{"type":"extras","title":"Filter - Ten Minutes to Explorer","doc":"The next verb we'll look at is `filter`.\n\nThis is implemented using a macro, so it's possible to use expressions\nlike you would if comparing variables in Elixir:\n\n```elixir\nDF.filter(df, country == \"BRAZIL\")\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\nUsing complex filters is also possible:\n\n```elixir\nDF.filter(df, country == \"ALGERIA\" and year > 2012)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\nYou can also write the same filter without the macro, by using the callback version function which is `filter_with/2`:\n\n```elixir\nDF.filter_with(df, fn ldf ->\n  ldf[\"country\"]\n  |> Series.equal(\"ALGERIA\")\n  |> Series.and(Series.greater(ldf[\"year\"], 2012))\nend)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\nBy the way, all the `Explorer.DataFrame` macros have a correspondent function that accepts a callback.\nIn fact, our macros are implemented using those functions.\n\nThe `filter_with/2` function is going to use a virtual representation of the dataframe\nthat we call a \"lazy frame\". With lazy frames you can´t access the\nseries contents, but every operation will be optimized and run only once.\n\nRemember those helpful error messages?\n\n```elixir\nDF.filter(df, cuontry == \"BRAZIL\")\n```","ref":"exploring_explorer.html#filter"},{"type":"extras","title":"Mutate - Ten Minutes to Explorer","doc":"A common task in data analysis is to add columns or change existing ones. Mutate is a handy verb.\n\n```elixir\nDF.mutate(df, new_column: solid_fuel + cement)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\nDid you catch that? You can pass in new columns as keyword arguments. It also works to transform existing columns.\n\n```elixir\nDF.mutate(df,\n  gas_fuel: Series.cast(gas_fuel, :float),\n  gas_and_liquid_fuel: gas_fuel + liquid_fuel\n)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\n`DataFrame.mutate/2` is flexible though. You may not always want to use keyword arguments. Given that column names are `String.t()`, it may make more sense to use a map.\n\n```elixir\nDF.mutate(df, %{\"gas_fuel\" => gas_fuel - 10})\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\n`DF.transmute/2`, which is `DF.mutate/2` that only retains the specified columns, is forthcoming.","ref":"exploring_explorer.html#mutate"},{"type":"extras","title":"Sort - Ten Minutes to Explorer","doc":"Sorting the dataframe is pretty straightforward.\n\n```elixir\nDF.sort_by(df, year)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\nBut it comes with some tricks up its sleeve.\n\n```elixir\nDF.sort_by(df, asc: total, desc: year)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\nAs the examples show, `sort_by/2` is a macro, and therefore you can use some Series functions to sort your dataframe:\n\n```elixir\nDF.sort_by(df, asc: window_sum(total, 2))\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\nSort operations happen left to right. And keyword list args permit specifying the direction.","ref":"exploring_explorer.html#sort"},{"type":"extras","title":"Distinct - Ten Minutes to Explorer","doc":"Okay, as expected here too. Very straightforward.\n\n```elixir\nDF.distinct(df, [\"year\", \"country\"])\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\nYou can specify whether to keep the other columns as well, so the first row of each distinct value is kept:\n\n```elixir\nDF.distinct(df, [\"country\"], keep_all: true)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```","ref":"exploring_explorer.html#distinct"},{"type":"extras","title":"Rename - Ten Minutes to Explorer","doc":"Rename can take either a list of new names or a callback that is passed to `Enum.map/2` against the names. You can also use a map or keyword args to rename specific columns.\n\n```elixir\nDF.rename(df, year: \"year_test\")\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\n```elixir\nDF.rename_with(df, &(&1 <> \"_test\"))\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```","ref":"exploring_explorer.html#rename"},{"type":"extras","title":"Dummies - Ten Minutes to Explorer","doc":"This is fun! We can get dummy variables for unique values.\n\n```elixir\nDF.dummies(df, [\"year\"])\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\n```elixir\nDF.dummies(df, [\"country\"])\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```","ref":"exploring_explorer.html#dummies"},{"type":"extras","title":"Sampling - Ten Minutes to Explorer","doc":"Random samples can give us a percent or a specific number of samples, with or without replacement, and the function is seedable.\n\n```elixir\nDF.sample(df, 10)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\n```elixir\nDF.sample(df, 0.4)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\nTrying for those helpful error messages again.\n\n```elixir\nDF.sample(df, 10000)\n```\n\n```elixir\nDF.sample(df, 10000, replace: true)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```","ref":"exploring_explorer.html#sampling"},{"type":"extras","title":"Pull and slice - Ten Minutes to Explorer","doc":"Slicing and dicing can be done with the `Access` protocol or with explicit pull/slice/take functions.\n\n```elixir\ndf[\"year\"]\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.Series \n```\n\n```elixir\nDF.pull(df, \"year\")\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.Series \n```\n\n```elixir\ndf[[\"year\", \"country\"]]\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\n```elixir\nDF.slice(df, [1, 20, 50])\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\nNegative offsets work for slice!\n\n```elixir\nDF.slice(df, -10, 5)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\n```elixir\nDF.slice(df, 10, 5)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\nSlice also works with ranges:\n\n```elixir\nDF.slice(df, 12..42)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```","ref":"exploring_explorer.html#pull-and-slice"},{"type":"extras","title":"Pivot - Ten Minutes to Explorer","doc":"We can `pivot_longer/3` and `pivot_wider/4`. These are inspired by [tidyr](https://tidyr.tidyverse.org/articles/pivot.html).\n\nThere are some shortcomings in `pivot_wider/4` related to `polars`. The `select` option must select only columns of numeric type.\n\n```elixir\nDF.pivot_longer(df, [\"year\", \"country\"], select: &String.ends_with?(&1, \"fuel\"))\n```\n\n```elixir\nDF.pivot_wider(df, \"country\", \"total\", id_columns: [\"year\"])\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\nLet's make those names look nicer!\n\n```elixir\ntidy_names = fn name ->\n  name\n  |> String.downcase()\n  |> String.replace(~r/\\s/, \" \")\n  |> String.replace(~r/[^A-Za-z\\s]/, \"\")\n  |> String.replace(\" \", \"_\")\nend\n\ndf\n|> DF.pivot_wider(\"country\", \"total\", id_columns: [\"year\"])\n|> DF.rename_with(tidy_names)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```","ref":"exploring_explorer.html#pivot"},{"type":"extras","title":"Joins - Ten Minutes to Explorer","doc":"Joining is *fast* and easy. You can specify the columns to join on and how to join. Polars even supports cartesian (cross) joins, so `Explorer` does too.\n\n```elixir\ndf1 = DF.select(df, [\"year\", \"country\", \"total\"])\ndf2 = DF.select(df, [\"year\", \"country\", \"cement\"])\n\nDF.join(df1, df2)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\n```elixir\ndf3 = df |> DF.select([\"year\", \"cement\"]) |> DF.slice(0, 500)\n\nDF.join(df1, df3, how: :left)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```","ref":"exploring_explorer.html#joins"},{"type":"extras","title":"Grouping - Ten Minutes to Explorer","doc":"`Explorer` supports *groupby* operations. They're limited based on what's possible in Polars, but they do most of what you need to do.\n\n```elixir\ngrouped = DF.group_by(df, [\"country\"])\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\nNotice that the `Inspect` call now shows `groups` as well as `rows` and `columns`. You can, of course, get them explicitly.\n\n```elixir\nDF.groups(grouped)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[\"country\"]\n```\n\nAnd you can ungroup explicitly.\n\n```elixir\nDF.ungroup(grouped)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\nBut what we care about the most is aggregating! Let's see which country has the max `per_capita` value.\n\n```elixir\ngrouped\n|> DF.summarise(max_per_capita: max(per_capita))\n|> DF.sort_by(desc: max_per_capita)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\nQatar it is.\n\nYou may noticed that we are using `max/1` inside the `summarise` macro. This is possible because we expose all functions from the `Series` module. You can use the following aggregations inside summarise:\n\n* `min/1` - Take the minimum value within the group. See `Explorer.Series.min/1`.\n* `max/1` - Take the maximum value within the group. See `Explorer.Series.max/1`.\n* `sum/1` - Take the sum of the series within the group. See `Explorer.Series.sum/1`.\n* `mean/1` - Take the mean of the series within the group. See `Explorer.Series.mean/1`.\n* `median/1` - Take the median of the series within the group. See `Explorer.Series.median/1`.\n* `first/1` - Take the first value within the group. See `Explorer.Series.first/1`.\n* `last/1` - Take the last value within the group. See `Explorer.Series.last/1`.\n* `count/1` - Count the number of rows per group.\n* `n_unique/1` - Count the number of unique rows per group.\n\nThe API is similar to `mutate`: you can use keyword args or a map and specify aggregations to use.\n\n```elixir\nDF.summarise(grouped, min_per_capita: min(per_capita), min_total: min(total))\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\nSpeaking of `mutate`, it's 'group-aware'. As are `sort_by`, `distinct`, and `n_rows`.\n\n```elixir\nDF.mutate(grouped, total_window_sum: window_sum(total, 3), rows_in_group: count(country))\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```\n\nIt's also possible to use aggregations inside other functions:\n\n```elixir\ngrouped\n|> DF.summarise(greater_than_9: greater(max(per_capita), 9.0), per_capita_max: max(per_capita))\n|> DataFrame.sort_by(desc: per_capita_max)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Explorer.DataFrame \n```","ref":"exploring_explorer.html#grouping"},{"type":"extras","title":"That's it! - Ten Minutes to Explorer","doc":"And not. This is certainly not exhaustive, but I hope it gives you a good idea of what can be done and what the 'flavour' of the API is like. I'd love contributions and issues raised where you find them!","ref":"exploring_explorer.html#that-s-it"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.1.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [Unreleased]\n\n## [v0.9.1] - 2024-08-15","ref":"changelog.html"},{"type":"extras","title":"Added - Changelog","doc":"- Add support for saving to the cloud using streaming and the IPC format.\n  This will enable saving a lazy frame to the cloud without loading it\n  entirely in memory. It only supports saves to S3-compatible storage services.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Force garbage collection on remote gc.","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Re-enable support for saving to the cloud using streaming and the Parquet format.\n  It's a fix from the release of `v0.9.0` that disabled this feature.\n\n- Fix overwrite of dtypes for `Explorer.DataFrame.load_csv/2`.\n  This was a regression introduced in `v0.9.0`.\n\n## [v0.9.0] - 2024-07-26","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add initial support for SQL queries.\n\n  The `Explorer.DataFrame.sql/3` is a function that accepts a dataframe and\n  a SQL query. The SQL is not validated by Explorer, so the queries will be\n  backend dependent. Right now we have only Polars as the backend.\n\n- Add support for remote series and dataframes.\n\n  Automatically transfer data between nodes for remote series\n  and dataframes and perform distributed garbage collection.\n\n  The functions in `Explorer.DataFrame` and `Explorer.Series`\n  will automatically move operations on remote dataframes to\n  the nodes they belong to. The `Explorer.Remote` module provides\n  additional conveniences for manual placement.\n\n- Add FLAME integration, so we automatically track remote series and\n  dataframes returned from `FLAME` calls when the `:track_resources`\n  option is enabled.\n  See [FLAME](https://github.com/phoenixframework/flame) for more.\n\n- Add `Explorer.DataFrame.transform/3` that applies an Elixir function to\n  each row. This function is similar to `Explorer.Series.transform/2`, and\n  as such, it's considered an expensive operation. So it's recommended only if\n  there is no similar dataframe or series operation available.\n\n- Improve performance of `Explorer.Series.from_list/2` for most of the cases\n  where the `:dtype` option is given. This is specially true for when the\n  dtype is `:binary`.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Stop inference of _dtypes_ if the `:dtype` option is given by the user.\n  The main goal of this change is to improve performance. We are now delegating\n  the job of decoding the terms as the given `:dtype` to the backend.\n\n- `Explorer.Series.pow/2` no longer casts to float when the exponent is a signed\n  integer. We are following the way Polars works now, which is to try to execute\n  the operation or raise an exception in case the exponent is negative.\n\n- `Explorer.Series.pivot_wider/4` no longer includes the `names_from` column\n  name in the new columns when `values_from` is a list of columns. This is more\n  consistent with its behaviour when `values_from` is a single column.\n\n- `Explorer.Series.substring/3` no longer cycles to the end of the string if the\n  negative offset surpasses the beginning of that string. In that case, an empty\n  string is returned.\n\n- The `Explorer.Series.ewm_*` functions no longer replace `nil` values with the\n  value at the previous index. They now propogate `nil` values through to the\n  result series.\n\n- Saving a dataframe as a Parquet file to S3 services no longer works when\n  streaming is enabled. This is temporary due to a bug in Polars. An exception\n  should be raised instead.\n\n## [v0.8.3] - 2024-06-10","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Add new data type for datetimes with timezones: `{:datetime, precision, time_zone}`\n  The old dtype is now `{:naive_datetime, precision}`.\n\n- Add option to rechunk the dataframes when using `Explorer.DataFrame.from_parquet/3`","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Change the `{:datetime, precision}` dtype to `{:naive_datetime, precision}`.\n  The idea is to mirror Elixir's datetime, and introduce support for time zones.\n  Please note: `{:datetime, precision}` will work as an alias for `{:naive_datetime, precision}` for now but will raise a warning.\n  The alias will be removed in a future release.\n\n- Literal `%NaiveDateTime{}` structs used in expressions will now have `:microsecond` precision.\n  Previously they defaulted to `:nanosecond` precision.\n  This was incorrect because `%NaiveDateTime{}` structs only have `:microsecond` precision.","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix regression in `Explorer.DataFrame.concat_rows/2`.\n  It's possible to concat dataframes that are not aligned again.\n\n- Fix \"is_finite\" and \"is_infinite\" from `Series` to work in the context of a `Explorer.Query`.\n\n## [v0.8.2] - 2024-04-22","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add functions to work with strings and regexes.\n\n  Some of the functions have the prefix \"re_\", because they accept a string that\n  represents a regular expression.\n\n  There is an important detail: we do not accept Elixir regexes, because we cannot\n  guarantee that the backend supports it. Instead we accept a plain string that\n  is \"escaped\". This means that you can use the `~S` sigil to build that string.\n  Example: `~S/(a|b)/`.\n\n  The added functions are the following:\n\n  - `Explorer.Series.split_into/3` - split a string series into a struct\n    of string fields. This function accepts a string as a separator.\n\n  - `Explorer.Series.re_contains/2` - check is the string series matches the regex\n    pattern. Like the \"non regex\" counterpart, it returns a boolean series.\n\n  - `Explorer.Series.re_replace/3` - replaces all occurences of a pattern with\n    replacement in string series. The replacement can refer to groups captures\n    by using the `${x}`, where `x` is the group index (starts with 1) or name.\n\n  - `Explorer.Series.count_matches/2` - count how many times a substring appears\n    in a string series.\n\n  - `Explorer.Series.re_count_matches/2` - count how many times a pattern matches\n    in a string series.\n\n  - `Explorer.Series.re_scan/2` - scan for all matches for the given regex pattern.\n    This is going to result in a series of lists of strings - `{:list, :string}`.\n\n  - `Explorer.Series.re_named_captures/2` - extract all capture groups as a struct\n    for the given regex pattern. In case the groups are not named, their positions\n    are used as names.\n\n- Enable the usage of system certificates if OTP version 25 or above.\n\n- Add support for the `:streaming` option in `Explorer.DataFrame.to_csv/3`.\n\n- Support operations with groups in the Lazy Polars backend. This change makes\n  the lazy frame implementation more useful, by supporting the usage of groups in\n  following functions:\n\n  - `Explorer.DataFrame.slice/3`\n\n  - `Explorer.DataFrame.head/2`\n\n  - `Explorer.DataFrame.tail/2`\n\n  - `Explorer.DataFrame.filter_with/2` and the macro version of it, `filter/2`.\n\n  - `Explorer.DataFrame.sort_with/3`, although it ignores \"maintain order\"\n    and \"nulls last\" options when used with groups.\n\n  - `Explorer.DataFrame.mutate_with/2` and its macro version, `mutate/2`.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- We now avoid raising an exception if a non existent column is used in\n  `Explorer.DataFrame.discard/2`.\n\n- Make the dependency of `cacerts` optional. This is because people using\n  Erlang/OTP 25 or above can use the certificates provided by the system.\n  So you may need to add the dependency of `cacerts` if your OTP version\n  is older than that.\n\n- Some precision differences in float operations may appear. This is due to\n  an update in the Polars version to \"v0.38.1\". Polars is our default backend.","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix `Explorer.Series.split/2` inside the context of `Explorer.Query`.\n\n- Add optional `X-Amz-Security-Token` header to S3 request. This is needed\n  in case the user is passing down a `token` for authentication.\n\n- Fix `Explorer.DataFrame.sort_by/3` with groups to respect `:nils` option.\n  This is considering only the eager implementation.\n\n- Fix inspection of lazy frames in remote nodes.\n\n## [v0.8.1] - 2024-02-24","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add `Explorer.Series.field/2` to extract a field from a struct series.\n  It returns a new series with the field's dtype.\n\n- Add `Explorer.Series.json_decode/2` that can decode a string series containing\n  valid JSON objects according to `dtype`.\n\n- Add eager `count/1` and lazy `size/1` to `Explorer.Series`.\n\n- Add support for maps as expressions inside `Explorer.Query`. They are \"converted\"\n  to structs.\n\n- Add `json_path_match/2` to extract a string series from a string containing valid\n  JSON objects. See the article [JSONPath - XPath for JSON](https://goessner.net/articles/JsonPath/)\n  for details about JSON paths.\n\n- Add `Explorer.Series.row_index/1` to retrieve the index of rows starting from 0.\n\n- Add support for passing the `:on` column directly (instead of inside a list)\n  in `Explorer.DataFrame.join/3`.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Remove some deprecated functions from documentation.\n\n- Change internal representation of the `:struct` dtype to use list of tuples instead of a map\n  to represent the dtypes of each field. This shouldn't break because we normalise maps to lists\n  when a struct dtype is passed in `from_list/2` or `cast/2`.\n\n- Update Rustler minimum version to `~> 0.31`. Since Rustler is optional, this shouldn't affect\n  most of the users.","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix float overflow error to avoid crashing the VM, and instead it returns an argument error.\n\n- Fix `Explorer.DataFrame.print/2` for when the DF contains structs.\n\n## [v0.8.0] - 2024-01-20","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add `explode/2` to `Explorer.DataFrame`. This function is useful to expand\n  the contents of a `{:list, inner_dtype}` series into a \"`inner_dtype`\" series.\n\n- Add the new series functions `all?/1` and `any?/1`, to work with boolean series.\n\n- Add support for the \"struct\" dtype. This new dtype represents the struct\n  dtype from Polars/Arrow.\n\n- Add `map/2` and `map_with/2` to the `Explorer.Series` module.\n  This change enables the usage of the `Explore.Query` features in a series.\n\n- Add `sort_by/2` and `sort_with/2` to the `Explorer.Series` module.\n  This change enables the usage of the lazy computations and the `Explorer.Query`\n  module.\n\n- Add `unnest/2` to `Explorer.DataFrame`. It works by taking the fields of a \"struct\" -\n  the new dtype - and transform them into columns.\n\n- Add pairwise correlation - `Explorer.DataFrame.correlation/2` - to calculate the\n  correlation between numeric columns inside a data frame.\n\n- Add pairwise covariance - `Explorer.DataFrame.covariance/2` - to calculate the\n  covariance between numeric columns inside a data frame.\n\n- Add support for more integer dtypes. This change introduces new signed and\n  unsigned integer dtypes:\n  - `{:s, 8}`, `{:s, 16}`, `{:s, 32}`\n  - `{:u, 8}`, `{:u, 16}`, `{:u, 32}`, `{:u, 64}`.\n\n  The existing `:integer` dtype is now represented as `{:s, 64}`, and it's still\n  the default dtype for integers. But series and data frames can now work with the\n  new dtypes. Short names for these new dtypes can be used in functions like\n  `Explorer.Series.from_list/2`. For example, `{:u, 32}` can be represented with\n  the atom `:u32`.\n\n  This may bring more interoperability with Nx, and with Arrow related things, like\n  ADBC and Parquet.\n\n- Add `ewm_standard_deviation/2` and `ewm_variance/2` to `Explorer.Series`.\n  They calculate the \"exponentially weighted moving\" variance and standard deviation.\n\n- Add support for `:skip_rows_after_header` option for the CSV reader functions.\n\n- Support `{:list, numeric_dtype}` for `Explorer.Series.frequencies/1`.\n\n- Support pins in `cond`, inside the context of `Explorer.Query`.\n\n- Introduce the `:null` dtype. This is a special dtype from Polars and Apache Arrow\n  to represent \"all null\" series.\n\n- Add `Explorer.DataFrame.transpose/2` to transpose a data frame.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Rename the functions related to sorting/arranging of the `Explorer.DataFrame`.\n  Now `arrange_with` is named `sort_with`, and `arrange` is `sort_by`.\n\n  The `sort_by/3` is a macro and it is going to work using the `Explorer.Query`\n  module. On the other side, the `sort_with/2` uses a callback function.\n\n- Remove unnecessary casts to `{:s, 64}` now that we support more integer dtypes.\n  It affects some functions, like the following in the `Explorer.Series` module:\n\n  - `argsort`\n  - `count`\n  - `rank`\n  - `day_of_week`, `day_of_year`, `week_of_year`, `month`, `year`, `hour`, `minute`, `second`\n  - `abs`\n  - `clip`\n  - `lengths`\n  - `slice`\n  - `n_distinct`\n  - `frequencies`\n\n  And also some functions from the `Explorer.DataFrame` module:\n\n  - `mutate` - mostly because of series changes\n  - `summarise` - mostly because of series changes\n  - `slice`","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix inspection of series and data frames between nodes.\n\n- Fix cast of `:string` series to `{:datetime, any()}`\n\n- Fix mismatched types in `Explorer.Series.pow/2`, making it more consistent.\n\n- Normalize sorting options.\n\n- Fix functions with dtype mismatching the result from Polars.\n  This fix is affecting the following functions:\n\n  - `quantile/2` in the context of a lazy series\n  - `mode/1` inside a summarisation\n  - `strftime/2` in the context of a lazy series\n  - `mutate_with/2` when creating a column from a `NaiveDateTime` or `Explorer.Duration`.\n\n## [v0.7.2] - 2023-11-30","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add the functions `day_of_year/1` and `week_of_year/1` to `Explorer.Series`.\n\n- Add `filter/2` - a macro -, and `filter_with/2` to `Explorer.Series`.\n\n  This change enables the usage of queries - using `Explorer.Query` - when\n  filtering a series. The main difference is that series does not have a\n  name when used outside a dataframe. So to refer to itself inside the\n  query, we can use the special `_` variable.\n\n        iex> s = Explorer.Series.from_list([1, 2, 3])\n        iex> Explorer.Series.filter(s, _ > 2)\n        #Explorer.Series \n\n- Add support for the `{:list, any()}` dtype, where `any()` can be any other\n  valid dtype. This is a recursive dtype, that can represent nested lists.\n  It's useful to group data together in the same series.\n\n- Add `Explorer.Series.mode/2` to get the most common value(s) of the series.\n\n- Add `split/2` and `join/2` to the `Explorer.Series` module.\n  These functions are useful to split string series into `{:list, :string}`,\n  or to join parts of a `{:list, :string}` and return a `:string` series.\n\n- Expose `ddof` option for variance, covariance and standard deviation.\n\n- Add a new `{:f, 32}` dtype to represent 32 bits float series.\n  It's also possible to use the atom `:f32` to create this type of series.\n  The atom `:f64` can be used as an alias for `{:f, 64}`, just like the\n  `:float` atom.\n\n- Add `lengths/1` and `member?/2` to `Explorer.Series`.\n  These functions work with `{:list, any()}`, where `any()` is any valid dtype.\n  The idea is to count the members of a \"list\" series, and check if a given\n  value is member of a list series, respectively.\n\n- Add support for streaming parquet files from a lazy dataframe to AWS S3\n  compatible services.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Remove restriction on `pivot_wider` dtypes.\n  In the early days, Polars only supported numeric dtypes for the \"first\"\n  aggregation. This is not true anymore, and we can lift this restriction.\n\n- Change `:float` dtype to be represented as `{:f, 64}`. It's still possible\n  to use the atom `:float` to create float series, but now `Explorer.Series.dtype/1`\n  returns `{:f, 64}` for float 64 bits series.","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Add missing implementation of `Explorer.Series.replace/3` for lazy series.\n\n- Fix inspection of DFs and series when `limit: :infinity` is used.","ref":"changelog.html#fixed"},{"type":"extras","title":"Removed - Changelog","doc":"- Drop support for the `riscv64gc-unknown-linux-gnu` target.\n\n  We decided to stop precompiling to this target because it's been hard to maintain it.\n  Ideally we should support it again in the future.\n\n## [v0.7.1] - 2023-09-25","ref":"changelog.html#removed"},{"type":"extras","title":"Added - Changelog","doc":"- Add more temporal arithmetic operations. This change makes possible\n  to mix some datatypes, like `date`, `duration` and scalar types like\n  `integers` and `floats`.\n\n  The following operations are possible now:\n\n  * `date - date`\n  * `date + duration`\n  * `date - duration`\n  * `duration + date`\n  * `duration * integer`\n  * `duration * float`\n  * `duration / integer`\n  * `duration / float`\n  * `integer * duration`\n  * `float * duration`\n\n- Support lazy dataframes on `Explorer.DataFrame.print/2`.\n\n- Add support for strings as the \"indexes\" of `Explorer.Series.categorise/2`.\n  This makes possible to categorise a string series with a categories series.\n\n- Introduce `cond/1` support in queries, which enables multi-clause conditions.\n  Example of usage:\n\n          iex> df = DF.new(a: [10, 4, 6])\n          iex> DF.mutate(df,\n          ...>   b:\n          ...>     cond do\n          ...>       a > 9 -> \"Exceptional\"\n          ...>       a > 5 -> \"Passed\"\n          ...>       true -> \"Failed\"\n          ...>     end\n          ...> )\n          #Explorer.DataFrame \n\n- Similar to `cond/1`, this version also introduces support for the `if/2`\n  and `unless/2` macros inside queries.\n\n- Allow the usage of scalar booleans inside queries.\n\n- Add `Explorer.Series.replace/3` for string series.\n  This enables the replacement of patterns inside string series.","ref":"changelog.html#added"},{"type":"extras","title":"Deprecated - Changelog","doc":"- Deprecate `Explorer.DataFrame.to_lazy/1` in favor of just `lazy/1`.","ref":"changelog.html#deprecated"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix the `Explorer.Series.in/2` function to work with series of the\n  `:category` dtype.\n\n  Now, if both series shares the same categories, we can compare them.\n  To make sure that a categorical series shares the same categories from\n  another series, you must create that series using the\n  `Explorer.Series.categorise/2` function.\n\n- Display the dtype of duration series correctly in `Explorer.DataFrame.print/2`.\n\n## [v0.7.0] - 2023-08-28","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Enable reads and writes of dataframes from/to external file systems.\n\n  It supports HTTP(s) URLs or AWS S3 locations.\n\n  This feature introduces the [FSS](https://github.com/elixir-explorer/fss) abstraction,\n  which is also going to be present in newer versions of Kino. This is going to make the integration\n  of Livebook files with Explorer much easier.\n\n  The implementation is done differently, depending on which file format is used, and if\n  it's a read or write. All the writes to AWS S3 are done in the Rust side - using an abstraction\n  called `CloudWriter` -, and most of the readers are implemented in Elixir, by doing a download\n  of the files, and then loading the dataframe from it. The only exception is the reads of\n  parquet files, which are done in Rust, using Polars' `scan_parquet` with streaming.\n\n  We want to give a special thanks to [Qqwy / Marten](https://github.com/Qqwy) for the\n  `CloudWriter` implementation!\n\n- Add ADBC: Arrow Database Connectivity.\n\n  Continuing with improvements in the IO area, we added support for reading dataframes from\n  databases using ADBC, which is similar in idea to ODBC, but integrates much better with\n  Apache Arrow, that is the backbone of Polars - our backend today.\n\n  The function `Explorer.DataFrame.from_query/1` is the entrypoint for this feature, and it\n  allows quering databases like PostgreSQL, SQLite and Snowflake.\n\n  Check the [Elixir ADBC bindings docs](https://hexdocs.pm/adbc) for more information.\n\n  For the this feature, we had a fundamental contribution from [Cocoa](https://github.com/cocoa-xu)\n  in the ADBC bindings, so we want to say a special thanks to her!\n\n  We want to thank the people that joined José in [his live streamings on Twitch](https://www.twitch.tv/josevalim),\n  and helped to build this feature!\n\n- Add the following functions to `Explorer.Series`:\n  - [`window_median/3`](https://hexdocs.pm/explorer/Explorer.Series.html#window_median/3)\n  - [`substring/3`](https://hexdocs.pm/explorer/Explorer.Series.html#substring/3)\n\n- Add duration dtypes. This is adds the following dtypes:\n  - `{:duration, :nanosecond}`\n  - `{:duration, :microsecond}`\n  - `{:duration, :millisecond}`\n\n  This feature was a great contribution from [Billy Lanchantin](https://github.com/billylanchantin),\n  and we want to thank him for this!","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Return exception structs instead of strings for all IO operation errors, and for anything\n  that returns an error from the NIF integration.\n\n  This change makes easier to define which type of error we want to raise.\n\n- Update Polars to v0.32.\n\n  With that we made some minor API changes, like changing some options for `cut/qcut` operations\n  in the `Explorer.Series` module.\n\n- Use `nil_values` instead of `null_character` for IO operations.\n\n- Never expect `nil` for CSV IO dtypes.\n\n- Rename `Explorer.DataFrame.table/2` to `Explorer.DataFrame.print/2`.\n\n- Change `:datetime` dtype to be `{:datetime, time_unit}`, where time unit can be\n  the following:\n  - `:millisecond`\n  - `:microsecond`\n  - `:nanosecond`\n\n- Rename the following `Series` functions:\n  - `trim/1` to `strip/2`\n  - `trim_leading/1` to `lstrip/2`\n  - `trim_trailing/1` to `rstrip/2`\n\n  These functions now support a string argument.","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix warnings for the upcoming Elixir v1.16.\n\n- Fix `Explorer.Series.abs/1` type specs.\n\n- Allow comparison of strings with categories.\n\n- Fix `Explorer.Series.is_nan/1` inside the context of `Explorer.Query`.\n  The NIF function was not being exported.\n\n## [v0.6.1] - 2023-07-06","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix summarise without groups for lazy frames.\n\n## [v0.6.0] - 2023-07-05","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add support for OTP 26 and Elixir 1.15.\n\n- Allow `Explorer.DataFrame.summarise/2` to work without groups.\n  The aggregations can work considering the entire dataframe.\n\n- Add the following series functions: `product/1`, `cummulative_product/1`, `abs/1`,\n  `skew/2`, `window_standard_deviation/3`, `rank/2`, `year/1`, `mounth/1`, `day/1`,\n  `hour/1`, `minute/1`, `second/1`, `strptime/2`, `strftime/2`, `argmin/1`, `argmax/1`,\n  `cut/3`, `qcut/3`, `correlation/3`, `covariance/2` and `clip/3`.\n\n  They cover a lot in terms of functionality, so please check the `Explorer.Series`\n  docs for further details.\n\n- Add `Explorer.DataFrame.nil_count/1` that counts the number of nil elements in each\n  column.\n\n- Add `Explorer.DataFrame.frequencies/2` that creates a new dataframe with unique rows\n  and the frequencies of each.\n\n- Add `Explorer.DataFrame.relocate/3` that enables changing order of columns from a df.\n\n- Add precompiled NIFs for FreeBSD.\n\n- Support scalar values in the `on_true` and `on_false` arguments of `Explore.Series.select/3`.","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix `Series.day_of_week/1` and `Series.round/2` for operations using a lazy frame.\n\n- Fix upcasted date to datetime for literal expressions. It allows to use scalar dates\n  in expressions like this: `DF.mutate(a: ~D[2023-01-01])`. This also fixes the support\n  for naive datetimes.\n\n- Improve error messages returned from the NIF to be always strings. Now we add more\n  context to the string returned, instead of having `{:context, error_message}`.\n\n- Fix the `:infer_schema_length` option of `Explorer.DataFrame.from_csv/2` when passing `nil`.\n  Now it's possible to take into account the entire file to infer the schema.","ref":"changelog.html#fixed"},{"type":"extras","title":"Deprecated - Changelog","doc":"- Deprecate `Explorer.Series.to_date/1` and `Explorer.Series.to_time/1` in favor of\n  using `Explorer.Series.cast(s, :date)` and `Explorer.Series.cast(s, :time)` respectively.\n\n## [v0.5.7] - 2023-05-10","ref":"changelog.html#deprecated"},{"type":"extras","title":"Added - Changelog","doc":"- Allow `Explorer.Series.select/3` to receive series of size 1 in both sides.\n\n- Add trigonometric functions `sin/1`, `cos/1`, `tan/1`, `asin/1`, `acos/1` and\n  `atan/1` to `Explorer.Series`.\n\n- Add `Explorer.DataFrame.to_rows_stream/2` function. This is useful to traverse\n  dataframes with large series, but is not recommended since it can be an expensive\n  operation.\n\n- Add LazyFrame version of `Explorer.DataFrame.to_ipc/3`.\n\n- Add options to control streaming when writing lazy dataframes. Now users can\n  toggle streaming for the `to_ipc/3` and `to_parquet/3` functions.\n\n- Add `Explorer.DataFrame.from_ipc_stream/2` lazy, but using the eager implementation\n  underneath.\n\n- Add option to control the end of line (EOF) char when reading CSV files.\n  We call this new option `:eol_delimiter`, and it's available for the `from_csv/2`\n  and `load_csv/2` functions in the `Explorer.DataFrame` module.\n\n- Allow `Explorer.DataFrame.pivot_wider/4` to use category fields.","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix `nif_not_loaded` error when `Explorer.Series.ewm_mean/2` is called from query.\n\n- Type check arguments for boolean series operations, only allowing series of\n  the `boolean` dtype.\n\n- Do not use `../0` in order to keep compatible with Elixir 1.13","ref":"changelog.html#fixed"},{"type":"extras","title":"Removed - Changelog","doc":"- Temporarely remove support for ARM 32 bits computers in the precompilation\n  workflow.\n\n## [v0.5.6] - 2023-03-24","ref":"changelog.html#removed"},{"type":"extras","title":"Added - Changelog","doc":"- Add the following functions to the `Explorer.Series` module: `log/1`, `log/2`\n  and `exp/1`. They compute the logarithm and exponential of a series.","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Allow `Explorer.Series.select/3` to receive series of size 1 for both the\n  `on_true` and `on_false` arguments.\n\n- Fix the encoding of special float values that may return from some series\n  functions. This is going to encode the atoms for NaN and infinity values.\n\n## [v0.5.5] - 2023-03-13","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add support for multiple value columns in pivot wider.\n  The resultant dataframe that is created from this type of pivoting is going\n  to have columns with the names prefixed by the original value column, followed\n  by an underscore and the name of the variable.\n\n- Add `Explorer.Series.ewm_mean/2` for calculating exponentially weighted moving\n  average.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Change the `Explorer.Backend.DataFrame`'s `pivot_wider` callback to work with\n  multiple columns instead of only one.\n\n- Change the `Explorer.Backend.DataFrame`'s `window_*` callbacks to work with\n  variables instead of keyword args. This is needed to make explicit when a backend\n  is not implementing an option.\n\n- Change the `Explorer.Backend.DataFrame`'s `describe` callback and remove the\n  need for an \"out df\", since we won't have a lazy version of that funcion.\n\n- This shouldn't affect the API, but we had an update in Polars.\n  It is now using `v0.27.2`. For further details, see:\n  [Rust Polars 0.27.0](https://github.com/pola-rs/polars/releases/tag/rs-0.27.0).","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Provide hints when converting string/binary series to tensors.\n\n- Add libatomic as a link to the generated NIF. This is needed to fix the load\n  of the Explorer NIF when running on ARM 32 bits machines like the Pi 3.\n  See the [original issue](https://github.com/philss/rustler_precompiled/issues/53)\n\n## [v0.5.4] - 2023-03-09","ref":"changelog.html#fixed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix missing \"README.md\" file in the list of package files.\n  Our readme is now required in compilation, because it contains the moduledoc for\n  the main `Explorer` module.\n\n## [v0.5.3] - 2023-03-08","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add the `Explorer.Series.format/1` function that concatenates multiple series together,\n  always returning a string series.\n\n- With the addition of `format/1`, we also have a new operator for string concatenation\n  inside `Explorer.Query`. It is the `<>` operator, that is similar to what the `Kernel.<>/2`\n  operator does, but instead of concatenating strings, it concatenates two series, returning\n  a string series - it is using `format/1` underneath.\n\n- Add support for slicing by series in dataframes and other series.\n\n- Add support for 2D tensors in `Explorer.DataFrame.new/2`.","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix `Explorer.DataFrame.new/2` to respect the selected dtype when an entire series is nil.\n\n- Improve error message for mismatched dtypes in series operations.\n\n- Fix lazy series operations of binary series and binary values. This is going to wrap binary\n  values in the correct dtype, in order to pass down to Polars.\n\n- Fix two bugs in `Explorer.DataFrame.pivot_wider/3`: `nil` values in the series that is\n  used for new column names is now correctly creating a `nil` column. We also fixed the problem\n  of a duplicated column created after pivoting, and possibly conflicting with an existing\n  ID column. We add a suffix for these columns.\n\n## [v0.5.2] - 2023-02-28","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add `across` and comprehensions to `Explorer.Query`. These features allow a\n  more flexible and elegant way to work with multiple columns at once. Example:\n\n  ```elixir\n  iris = Explorer.Datasets.iris()\n  Explorer.DataFrame.mutate(iris,\n   for col <- across([\"sepal_width\", \"sepal_length\", \"petal_length\", \"petal_width\"]) do\n     {col.name, (col - mean(col)) / variance(col)}\n   end\n  )\n  ```\n\n  See the `Explorer.Query` documentation for further details.\n\n- Add support for regexes to select columns of a dataframe. Example:\n\n  ```elixir\n  df = Explorer.Datasets.wine()\n  df[~r/(class|hue)/]\n  ```\n\n- Add the `:max_rows` and `:columns` options to `Explorer.DataFrame.from_parquet/2`. This mirrors\n  the `from_csv/2` function.\n\n- Allow `Explorer.Series` functions that accept floats to work with `:nan`, `:infinity`\n  and `:neg_infinity` values.\n\n- Add `Explorer.DataFrame.shuffle/2` and `Explorer.Series.shuffle/2`.\n\n- Add support for a list of filters in `Explorer.DataFrame.filter/2`. These filters are\n  joined as `and` expressions.","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Add `is_integer/1` guard to `Explorer.Series.shift/2`.\n- Raise if series sizes do not match for binary operations.","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- Rename the option `:replacement` to `:replace` for `Explorer.DataFrame.sample/3` and\n  `Explorer.Series.sample/3`.\n\n- Change the default behaviour of sampling to not shuffle by default. A new option\n  named `:shuffle` was added to control that.\n\n## [v0.5.1] - 2023-02-17","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Add boolean dtype to `Series.in/2`.\n- Add binary dtype to `Series.in/2`.\n- Add `Series.day_of_week/1`.\n\n- Allow `Series.fill_missing/2` to:\n  - receive `:infinity` and `:neg_infinity` values.\n  - receive date and datetime values.\n  - receive binary values.\n\n- Add support for `time` dtype.\n- Add version of `Series.pow/2` that accepts series on both sides.\n- Allow `Series.from_list/2` to receive `:nan`, `:infinity` and `:neg_infinity` atoms.\n- Add `Series.to_date/1` and `Series.to_time/1` for datetime series.\n- Allow casting of string series to category.\n- Accept tensors when creating a new dataframe.\n- Add compatibility with Nx v0.5.\n- Add support for Nx's serialize and deserialize.\n\n- Add the following function implementations for the Polars' Lazy dataframe backend:\n  - `arrange_with`\n  - `concat_columns`\n  - `concat_rows`\n  - `distinct`\n  - `drop_nil`\n  - `filter_with`\n  - `join`\n  - `mutate_with`\n  - `pivot_longer`\n  - `rename`\n  - `summarise_with`\n  - `to_parquet`\n\n  Only `summarise_with` supports groups for this version.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Require version of Rustler to be `~> 0.27.0`, which mirrors the NIF requirement.","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Casting to an unknown dtype returns a better error message.\n\n## [v0.5.0] - 2023-01-12","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add `DataFrame.describe/2` to gather some statistics from a dataframe.\n- Add `Series.nil_count/1` to count nil values.\n- Add `Series.in/2` to check if a given value is inside a series.\n- Add `Series` float predicates: `is_finite/1`, `is_infinite/1` and `is_nan/1`.\n- Add `Series` string functions: `contains/2`, `trim/1`, `trim_leading/1`, `trim_trailing/1`,\n  `upcase/1` and `downcase/1`.\n\n- Enable slicing of lazy frames (`LazyFrame`).\n- Add IO operations \"from/load\" to the lazy frame implementation.\n- Add support for the `:lazy` option in the `DataFrame.new/2` function.\n- Add `Series` float rounding methods: `round/2`, `floor/1` and `ceil/1`.\n- Add support for precompiling to Linux running on RISCV CPUs.\n- Add support for precompiling to Linux - with musl - running on AARCH64 computers.\n- Allow `DataFrame.new/1` to receive the `:dtypes` option.\n- Accept `:nan` as an option for `Series.fill_missing/2` with float series.\n- Add basic support for the categorical dtype - the `:category` dtype.\n- Add `Series.categories/1` to return categories from a categorical series.\n- Add `Series.categorise/2` to categorise a series of integers using predefined categories.\n- Add `Series.replace/2` to replace the contents of a series.\n- Support selecting columns with unusual names (like with spaces) inside `Explorer.Query`\n  with `col/1`.\n\n  The usage is like this:\n\n  ```elixir\n  Explorer.DataFrame.filter(df, col(\"my col\") > 42)\n  ```","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix `DataFrame.mutate/2` using a boolean scalar value.\n- Stop leaking `UInt32` series to Elixir.\n- Cast numeric columns to our supported dtypes after IO read.\n  This fix is only applied for the eager implementation for now.","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- Rename `Series.bintype/1` to `Series.iotype/1`.\n\n## [v0.4.0] - 2022-11-29","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Add `Series.quotient/2` and `Series.remainder/2` to work with integer division.\n- Add `Series.iotype/1` to return the underlying representation type.\n- Allow series on both sides of binary operations, like: `add(series, 1)`\n  and `add(1, series)`.\n\n- Allow comparison, concat and coalesce operations on \"(series, lazy series)\".\n- Add lazy version of `Series.sample/3` and `Series.size/1`.\n- Add support for Arrow IPC Stream files.\n- Add `Explorer.Query` and the macros that allow a simplified query API.\n  This is a huge improvement to some of the main functions, and allow refering to\n  columns as they were variables.\n\n  Before this change we would need to write a filter like this:\n\n  ```elixir\n  Explorer.DataFrame.filter_with(df, &Explorer.Series.greater(&1[\"col1\"], 42))\n  ```\n\n  But now it's also possible to write this operation like this:\n\n  ```elixir\n  Explorer.DataFrame.filter(df, col1 > 42)\n  ```\n\n  This operation is going to use `filter_with/2` underneath, which means that\n  is going to use lazy series and compute the results at once.\n  Notice that is mandatory to \"require\" the DataFrame module, since these operations\n  are implemented as macros.\n\n  The following new macros were added:\n  - `filter/2`\n  - `mutate/2`\n  - `summarise/2`\n  - `arrange/2`\n\n  They substitute older versions that did not accept the new query syntax.\n\n- Add `DataFrame.put/3` to enable adding or replacing columns in a eager manner.\n  This works similar to the previous version of `mutate/2`.\n\n- Add `Series.select/3` operation that enables selecting a value\n  from two series based on a predicate.\n\n- Add \"dump\" and \"load\" functions to IO operations. They are useful to load\n  or dump dataframes from/to memory.\n\n- Add `Series.to_iovec/2` and `Series.to_binary/1`. They return the underlying\n  representation of series as binary. The first one returns a list of binaries,\n  possibly with one element if the series is contiguous in memory. The second one\n  returns a single binary representing the series.\n\n- Add `Series.shift/2` that shifts the series by an offset with nil values.\n- Rename `Series.fetch!/2` and `Series.take_every/2` to `Series.at/2`\n  and `Series.at_every/2`.\n\n- Add `DataFrame.discard/2` to drop columns. This is the opposite of `select/2`.\n\n- Implement `Nx.LazyContainer` for `Explorer.DataFrame` and `Explorer.Series`\n  so data can be passed into Nx.\n\n- Add `Series.not/1` that negates values in a boolean series.\n- Add the `:binary` dtype for Series. This enables the usage of arbitrary binaries.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Change DataFrame's `to_*` functions to return only `:ok`.\n- Change series inspect to resamble the dataframe inspect with the backend name.\n- Rename `Series.var/1` to `Series.variance/1`\n- Rename `Series.std/1` to `Series.standard_deviation/1`\n- Rename `Series.count/2` to `Series.frequencies/1` and add a new `Series.count/1`\n  that returns the size of an \"eager\" series, or the count of members in a group\n  for a lazy series.\n  In case there is no groups, it calculates the size of the dataframe.\n- Change the option to control direction in `Series.sort/2` and `Series.argsort/2`.\n  Instead of a boolean, now we have a new option called `:direction` that accepts\n  `:asc` or `:desc`.","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fix the following DataFrame functions to work with groups:\n  - `filter_with/2`\n  - `head/2`\n  - `tail/2`\n  - `slice/2`\n  - `slice/3`\n  - `pivot_longer/3`\n  - `pivot_wider/4`\n  - `concat_rows/1`\n  - `concat_columns/1`\n- Improve the documentation of functions that behave differently with groups.\n- Fix `arrange_with/2` to use \"group by\" stable, making results more predictable.\n- Add `nil` as a possible return value of aggregations.\n- Fix the behaviour of `Series.sort/2` and `Series.argsort/2` to add nils at the\n  front when direction is descending, or at the back when the direction is ascending.\n  This also adds an option to control this behaviour.","ref":"changelog.html#fixed"},{"type":"extras","title":"Removed - Changelog","doc":"- Remove support for `NDJSON` read and write for ARM 32 bits targets.\n  This is due to a limitation of a dependency of Polars.\n\n## [v0.3.1] - 2022-09-09","ref":"changelog.html#removed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Define `multiply` inside `*_with` operations.\n- Fix column types in several operations, such as `n_distinct`.\n\n## [v0.3.0] - 2022-09-01","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Add `DataFrame.concat_columns/1` and `DataFrame.concat_columns/2` for horizontally stacking\n  dataframes.\n- Add compression as an option to write parquet files.\n- Add count metadata to `DataFrame` table reader.\n- Add `DataFrame.filter_with/2`, `DataFrame.summarise_with/2`, `DataFrame.mutate_with/2` and\n`DataFrame.arrange_with/2`. They all accept a `DataFrame` and a function, and they all work with\n  a new concept called \"lazy series\".\n\n  Lazy Series is an opaque representation of a series that can be\n  used to perform complex operations without pulling data from the series. This is faster than\n  using masks. There is no big difference from the API perspective compared to the functions that were\n  accepting callbacks before (eg. `filter/2` and the new `filter_with/2`), with the exception being\n  `DataFrame.summarise_with/2` that now accepts a lot more operations.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Bump version requirement of the `table` dependency to `~> 0.1.2`, and raise for non-tabular values.\n- Normalize how columns are handled. This changes some functions to accept one column or\na list of columns, ranges, indexes and callbacks selecting columns.\n- Rename `DataFrame.filter/2` to `DataFrame.mask/2`.\n- Rename `Series.filter/2` to `Series.mask/2`.\n- Rename `take/2` from both `Series` and `DataFrame` to `slice/2`. `slice/2` now they accept ranges as well.\n- Raise an error if `DataFrame.pivot_wider/4` has float columns as IDs. This is because we can´t\nproperly compare floats.\n- Change `DataFrame.distinct/2` to accept columns as argument instead of receiving it as option.","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Ensure that we can compare boolean series in functions like `Series.equal/2`.\n- Fix rename of columns after summarise.\n- Fix inspect of float series containing `NaN` or `Infinity` values. They are represented as atoms.","ref":"changelog.html#fixed"},{"type":"extras","title":"Deprecated - Changelog","doc":"- Deprecate `DataFrame.filter/2` with a callback in favor of `DataFrame.filter_with/2`.\n\n## [v0.2.0] - 2022-06-22","ref":"changelog.html#deprecated"},{"type":"extras","title":"Added - Changelog","doc":"- Consistently support ranges throughout the columns API\n- Support negative indexes throughout the columns API\n- Integrate with the `table` package\n- Add `Series.to_enum/1` for lazily traversing the series\n- Add `Series.coalesce/1` and `Series.coalesce/2` for finding the first non-null value in a list of series","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- `Series.length/1` is now `Series.size/1` in keeping with Elixir idioms\n- `Nx` is now an optional dependency\n- Minimum Elixir version is now 1.13\n- `DataFrame.to_map/2` is now `DataFrame.to_columns/2` and `DataFrame.to_series/2`\n- `Rustler` is now an optional dependency\n- `read_` and `write_` IO functions are now `from_` and `to_`\n- `to_binary` is now `dump_csv`\n- Now uses `polars`'s \"simd\" feature\n- Now uses `polars`'s \"performant\" feature\n- `Explorer.default_backend/0` is now `Explorer.Backend.get/0`\n- `Explorer.default_backend/1` is now `Explorer.Backend.put/1`\n- `Series.cum_*` functions are now `Series.cumulative_*` to mirror `Nx`\n- `Series.rolling_*` functions are now `Series.window_*` to mirror `Nx`\n- `reverse?` is now an option instead of an argument in `Series.cumulative_*` functions\n- `DataFrame.from_columns/2` and `DataFrame.from_rows/2` is now `DataFrame.new/2`\n- Rename \"col\" to \"column\" throughout the API\n- Remove \"with\\_\" prefix in options throughout the API\n- `DataFrame.table/2` accepts options with `:limit` instead of single integer\n- `rename/2` no longer accepts a function, use `rename_with/2` instead\n- `rename_with/3` now expects the function as the last argument","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Explorer now works on Linux with musl\n\n## [v0.1.1] - 2022-04-27","ref":"changelog.html#fixed"},{"type":"extras","title":"Security - Changelog","doc":"- Updated Rust dependencies to address Dependabot security alerts: [1](https://github.com/elixir-explorer/explorer/security/dependabot/1), [2](https://github.com/elixir-explorer/explorer/security/dependabot/3), [3](https://github.com/elixir-explorer/explorer/security/dependabot/4)\n\n## [v0.1.0] - 2022-04-26\n\nFirst release.\n\n[Unreleased]: https://github.com/elixir-explorer/explorer/compare/v0.9.1...HEAD\n[v0.9.1]: https://github.com/elixir-explorer/explorer/compare/v0.9.0...v0.9.1\n[v0.9.0]: https://github.com/elixir-explorer/explorer/compare/v0.8.3...v0.9.0\n[v0.8.3]: https://github.com/elixir-explorer/explorer/compare/v0.8.2...v0.8.3\n[v0.8.2]: https://github.com/elixir-explorer/explorer/compare/v0.8.1...v0.8.2\n[v0.8.1]: https://github.com/elixir-explorer/explorer/compare/v0.8.0...v0.8.1\n[v0.8.0]: https://github.com/elixir-explorer/explorer/compare/v0.7.2...v0.8.0\n[v0.7.2]: https://github.com/elixir-explorer/explorer/compare/v0.7.1...v0.7.2\n[v0.7.1]: https://github.com/elixir-explorer/explorer/compare/v0.7.0...v0.7.1\n[v0.7.0]: https://github.com/elixir-explorer/explorer/compare/v0.6.1...v0.7.0\n[v0.6.1]: https://github.com/elixir-explorer/explorer/compare/v0.6.0...v0.6.1\n[v0.6.0]: https://github.com/elixir-explorer/explorer/compare/v0.5.7...v0.6.0\n[v0.5.7]: https://github.com/elixir-explorer/explorer/compare/v0.5.6...v0.5.7\n[v0.5.6]: https://github.com/elixir-explorer/explorer/compare/v0.5.5...v0.5.6\n[v0.5.5]: https://github.com/elixir-explorer/explorer/compare/v0.5.4...v0.5.5\n[v0.5.4]: https://github.com/elixir-explorer/explorer/compare/v0.5.3...v0.5.4\n[v0.5.3]: https://github.com/elixir-explorer/explorer/compare/v0.5.2...v0.5.3\n[v0.5.2]: https://github.com/elixir-explorer/explorer/compare/v0.5.1...v0.5.2\n[v0.5.1]: https://github.com/elixir-explorer/explorer/compare/v0.5.0...v0.5.1\n[v0.5.0]: https://github.com/elixir-explorer/explorer/compare/v0.4.0...v0.5.0\n[v0.4.0]: https://github.com/elixir-explorer/explorer/compare/v0.3.1...v0.4.0\n[v0.3.1]: https://github.com/elixir-explorer/explorer/compare/v0.3.0...v0.3.1\n[v0.3.0]: https://github.com/elixir-explorer/explorer/compare/v0.2.0...v0.3.0\n[v0.2.0]: https://github.com/elixir-explorer/explorer/compare/v0.1.1...v0.2.0\n[v0.1.1]: https://github.com/elixir-explorer/explorer/compare/v0.1.0...v0.1.1\n[v0.1.0]: https://github.com/elixir-explorer/explorer/releases/tag/v0.1.0","ref":"changelog.html#security"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}